/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import java.util.logging.Logger;

import de.fhstralsund.catsit.base.eventactions.localeventactions.LocalEventAction;
import de.fhstralsund.catsit.machine.EventList;

/**
 * Abstract base class for DiscreteCompartment and ContinuousCompartment
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 * 
 */
public abstract class Compartment {
    protected static final Logger logger = Logger.getLogger("metastase.base.compartment");

    private EventList eventList;
    
    private int myId;
    private String name;
    
    private boolean isPrimaryTumor;
	private boolean isBloodStream;
    
    
    //yields information about the organ the tumor is situated in
  	//necessary for radionuclide therapy
  	private HostOrgan hostOrgan;
    
    private DiscreteCompartment successor;
       
    //if isDormant is true, getNumberOfCells() will return the value 1 
    private boolean isDormant;
    
    //if isInLateDormancy is true, the tumor won't grow for a some time (as defined 
    //in lateDormancyDuration)
    private boolean isInLateDormancy;
    private long lateDormancyStartSize; //in cells
    private double lateDormancyDuration; //in minutes
    
    
    /**
     * 
     * 
     * @param name the name of the compartment
     * @param hostOrgan the organ compartment is situated in (e.g. lung, liver, blood stream)
     * @param isPrimaryTumor true if this compartment is the primary tumor. false, otherwise.
     * @throws MetastaseException
     */
    public Compartment(String name, HostOrgan hostOrgan, boolean isPrimaryTumor) {
    	
    	if(hostOrgan == null){
			throw new IllegalArgumentException("Host organ must not be 'null'");
		}
    	
    	this.myId = CompartmentInventory.createNewID();
    	this.hostOrgan = hostOrgan;
    	this.name = name;
    	this.isPrimaryTumor = isPrimaryTumor;
        this.eventList = EventList.getInstance();
        
        //compute late dormancy start size and time 
        if(hostOrgan.hasLateDormancy()){
        	this.lateDormancyStartSize = hostOrgan.computeLateDormancySize();
        	this.lateDormancyDuration = hostOrgan.computeLateDormancyDuration();
        }
        
        CompartmentInventory.registerCompartment(this);
    }        

    public abstract long getNumberOfCells(double time);

    /**
     * Generates the first event for this compartment.
     * Note, the bloodstream does not need this activation since it will get its events by other
     * compartments.
     * 
     * @param time the actual global simulation time
     */
    public abstract void createFirstEvent(double time);
 
    /**
     * Create the next event
     * @param time the actual global simulation time
     */
    public abstract void createNextEvent(double time) throws MetastaseException;
   
    public void delayEvent(LocalEventAction ea, double time){
    	LocalEvent event = new LocalEvent(ea, time + lateDormancyDuration, this);
    	getEventList().addEvent(event);
    }
    
    public String getHostOrganName(){
    	return hostOrgan.getName();
    }
    
	public HostOrgan getHostOrgan(){
		return hostOrgan;
	}
	
	public boolean hasDormancy(){
		return hostOrgan.hasDormancy();
	}
    	
	public boolean hasLateDormancy(){
		return hostOrgan.hasLateDormancy();
	}	
    
    public double getLateDormancyDuration() {
		return lateDormancyDuration;
	}
    
    public long getLateDormancyStartSize() {
		return lateDormancyStartSize;
	}


	/**
     * If dormant is true, getNumberOfCells will return 1.
     * @return true, if the tumor is dormant at the moment
     */
    public boolean isDormant() {
		return isDormant;
	}

	public void setDormant(boolean isDormant) {
		this.isDormant = isDormant;
	}

		
	public boolean isInLateDormancy() {
		return isInLateDormancy;
	}

	public void setInLateDormancy(boolean isInLateDormancy) {
		this.isInLateDormancy = isInLateDormancy;
	}

	/**
     * Only used for testing!
     * @return the compartment id
     */
    public int getMyId() {
        return myId;
    }
        
    public String getName() {
		return name;
	}

	public DiscreteCompartment getSuccessor() {
        return successor;
    }

    public void setSuccessor(DiscreteCompartment theSuccessor) {
    	if(this == theSuccessor){
    		throw new IllegalArgumentException("a compartment must not be the succesor of itself. ");
    	}
        successor = theSuccessor;
    }    
    
    public boolean isPrimaryTumor() {
		return isPrimaryTumor;
	}
    
	public boolean isBloodStream() {
		return isBloodStream;
	}

	public void setBloodStream(boolean isBloodStream) {
		this.isBloodStream = isBloodStream;
	}

	protected EventList getEventList() {
        return eventList;
    }
    
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	@Override
	public String toString() {
		return "myId=" + myId 
				+ ", name=" + name
				+ ", isPrimaryTumor=" + isPrimaryTumor 
				+ ", isBloodStream=" + isBloodStream 
				+ ", hostOrgan=" + hostOrgan 
				+ ", successor=" + successor + ", isDormant=" + isDormant
				+ ", isInLateDormancy=" + isInLateDormancy
				+ ", lateDormancyStartSize=" + lateDormancyStartSize
				+ ", lateDormancyDuration=" + lateDormancyDuration;
	}
	

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Compartment other = (Compartment) obj;
		
		return (name.equals(other.name) &&
				isPrimaryTumor == other.isPrimaryTumor &&
				isBloodStream == other.isBloodStream &&
				((successor == null)?other.successor == null:successor.equals(other.successor)) &&
				isDormant == other.isDormant &&
				isInLateDormancy == other.isInLateDormancy && 
				lateDormancyStartSize == other.lateDormancyStartSize &&
				Double.doubleToLongBits(lateDormancyDuration) == Double.doubleToLongBits(other.lateDormancyDuration) &&
				hostOrgan.equals(other.hostOrgan)
			);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + name.hashCode();
		result = prime * result + (isPrimaryTumor ? 1231 : 1237);
		result = prime * result + (isBloodStream ? 1231 : 1237);
		result = prime * result	+ hostOrgan.hashCode();
		result = prime * result	+ ((successor == null) ? 0 : successor.hashCode());
		result = prime * result + (isDormant ? 1231 : 1237);
		result = prime * result + (isInLateDormancy ? 1231 : 1237);
		result = prime * result + (int) (lateDormancyStartSize ^ (lateDormancyStartSize >>> 32));
		long temp = Double.doubleToLongBits(lateDormancyDuration);
		result = prime * result + (int) (temp ^ (temp >>> 32));		
		return result;
	}


	public boolean sameAs(Compartment c){
		return this.myId == c.myId;
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
}
