/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import java.util.ArrayList;
import java.util.List;

import de.fhstralsund.catsit.base.eventactions.globaleventactions.PrintStatus;

/**
 * Keeps track of all instances of {@link Compartment}. This is used 
 * for global treatments and for analyzing the running compartment {@link PrintStatus}. 
 * 
 * @author Gero Wedemann, Anja Bethge
 */
public class CompartmentInventory {
	
	private static int COMPARTMENT_COUNTER = 0;
	private static List<Compartment> compartments = new ArrayList<Compartment>();
	
	
    
    /**
     * Create a new id from the global counter. This is synchronized to avoid
     * problems if we start threading one day.
     * 
     * @return a new id.
     */
    public static synchronized int createNewID() {
        COMPARTMENT_COUNTER++;
        
      //check whether the id is unique
        while (!isIDFree(COMPARTMENT_COUNTER)) {
        	COMPARTMENT_COUNTER++;
		}
        
        return COMPARTMENT_COUNTER;
    }
    
    /**
     * Check, whether on of the existing compartments already uses the given id
     * @param id the id to check upon
     * @return true, if no other compartment uses the given id, false otherwise
     */
    private static synchronized boolean isIDFree(int id){
    	
    	for (Compartment c : compartments) {
			if(c.getMyId() == id){
				//This id is already taken
				return false;
			}
		}
    	return true;
    }
	
    /**
     * Sets the global compartments counter to 0. Only for test reasons.
     */
    public static void resetCompartmentCounter() {
        COMPARTMENT_COUNTER = 0;
    }
    
	/** 
	 * Register a new Compartment. We have to make it synchronized
	 * since ArrayList is not synchronized by itself.
	 * 
	 * @param newCompartment
	 */
	public static synchronized void registerCompartment(Compartment newCompartment) {
		compartments.add(newCompartment);
	}
	
	public static List<Compartment> getCompartments(){
		return compartments;
	}
	
	/**
	 * only for tests
	 */
	public static void reset() {
		compartments = new ArrayList<Compartment>();
	}
}
