/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StartLateDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopLateDormancy;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.ExternalBeamTreatment;
import de.fhstralsund.catsit.base.treatments.RadiationTreatment;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.ResectionTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.base.treatments.TreatmentList.TreatmentEntry;
import de.fhstralsund.catsit.machine.AdaptiveIntegrationCalculator;
import de.fhstralsund.catsit.machine.EventTimeCalculator;
import de.fhstralsund.catsit.machine.TumorIsDeadException;
import de.fhstralsund.catsit.support.SpecialMath;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.FunctionUtil;
import de.fhstralsund.catsit.support.functions.PiecewiseGrowthFunction;
import de.fhstralsund.catsit.support.functions.chemo.BasicChemoFunction;

/**
 * Compartments, that are non discrete. This is a very simple variant, where growth
 * and rate of generated events are described by a function.
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class ContinuousCompartment extends Compartment {

	private PiecewiseGrowthFunction growthFunction;
	private Function initialStatedGrowthFunction;
	private ColonizationRate colonizationRate;
    private EventTimeCalculator eventTimeCalculator;
    
    //the colonization event
    private AddCellToDiscreteCompartment addCellToBloodStream;
    
    private double lateDormancyStartTime;

    /**
     * 
     * @param creationTime the creation time of the new compartment
     * @param name the name of the compartment/tumor
     * @param hostOrgan the host organ of the compartment
	 * @param initialGrowthFunction the initial growth function of the tumor (without influence of treatment, dormancy ...)
	 * @param colonizationRate the colonization rate of the tumor. Can be null if the tumor shall not be able to spread malignant cells
	 * @param isPrimaryTumor true, if the new compartment represents the primary tumor. false, otherwise
     * 
     * @throws MetastaseException if something goes wrong while creating the compartment
     */
    public ContinuousCompartment(double creationTime, String name, HostOrgan hostOrgan, Function initialGrowthFunction, ColonizationRate colonizationRate, boolean isPrimaryTumor) 
    		throws MetastaseException{
    	super(name, hostOrgan, isPrimaryTumor);
    	
    	//convert the time dependent values from days to minute
    	this.initialStatedGrowthFunction = FunctionUtil.convertToMinutes(initialGrowthFunction);    	
    	this.colonizationRate = FunctionUtil.convertToMinutes(colonizationRate);
    	
    	//create the piece wise growth function and update the colonization rate
    	this.buildPiecewiseGrowthFunction(creationTime);
    	
    	if(colonizationRate != null){
    		this.colonizationRate.setGrowthFunction(growthFunction);
    	}
    	
        this.addCellToBloodStream = new AddCellToDiscreteCompartment("addCellToBloodStream", this);
    }
    
    /**
     * Used for non primary tumors
     * 
     * @param creationTime the creation time of the new compartment
     * @param name the name of the compartment/tumor
     * @param hostOrgan the host organ of the compartment
	 * @param initialGrowthFunction the normal growth function of the tumor (without influence of treatment, dormancy ...)
	 * @param colonizationRate the colonization rate of the tumor. Can be null if the tumor shall not be able to spread malignant cells
     * 
     * @throws MetastaseException if something goes wrong while creating the compartment
     */
    public ContinuousCompartment(double creationTime, String name, HostOrgan hostOrgan, Function initialGrowthFunction, ColonizationRate colonizationRate) 
    		throws MetastaseException{
    	this(creationTime, name, hostOrgan, initialGrowthFunction, colonizationRate, false);
    }
    
    
    @Override
    public long getNumberOfCells(double currentTime) {    	    	
    	//return the actual number of cells
    	return SpecialMath.round(growthFunction.f(currentTime));
    }


    @Override
    public void createFirstEvent(double time) {
    	createNextEvent(time); 
    }
    
    @Override
    public void createNextEvent(double time) {
    	
    	//this method should not be called if the compartment is in (late) dormancy 
    	if(isDormant() || isInLateDormancy()){ 
        	throw new IllegalStateException("No new event should be created while in (late) dormany phase.");
        } 
    	
    	//return, if no event rate function is specified (so no intravasation events will occur)   	 
    	if (colonizationRate == null) return;
    	
    	//if the eventGenerator wasn't created yet => create a new instance
    	if(eventTimeCalculator == null ){
    		eventTimeCalculator = new AdaptiveIntegrationCalculator(colonizationRate);
    	}

    	
    	// compute time for the event
        double eventTime;
		try {
			eventTime = eventTimeCalculator.computeTimeToNextEvent(time);
			
		} catch (TumorIsDeadException e) {
			//if the tumor dies before the next event can occur we can stop and return
			return;
		}        
		
        //create the event and add it to the event list
		addCellToBloodStream.setTimeOfLastExecutedEventAction(time);
        Event e = new LocalEvent(addCellToBloodStream, eventTime, getSuccessor());
        getEventList().addEvent(e);
	}
    
	public void updateDormancySize(double time, long numberOfCellsAfterTreatment) throws MetastaseException{
		if(!(growthFunction.getCurrentValidFunction(time) instanceof ConstantFunction)){
			throw new MetastaseException("You tried to update the dormancy size but the compartment is not dormant at time "+ time+"!");
		}
		
		//their should be one more function after this one (the normal growth function)
		double nextFunctionTime = growthFunction.getTimeOfNextValidFunction(time);
		//add the adapted dormancy function (constant function with the new number of cells)
		growthFunction.addFunction(time, new ConstantFunction(numberOfCellsAfterTreatment));
		//delete the old following normal growth function
		growthFunction.deleteAllFollowingSections(time);
		//add the new following normal growth function (with adapted number of cells)
		growthFunction.addFunction(nextFunctionTime, FunctionUtil.copyGrowthFunctionWithNewStartSize(initialStatedGrowthFunction, numberOfCellsAfterTreatment));
		// ... and rebuild the piecewise function from this time point on
    	buildTreatments(time, getCreationTime());
	}

	/**
	 * Updates the tumor size. Uses the growth function that was initially stated or a constant function if the given number of 
	 * cells is zero.
	 * @param time The time from which on the new tumor size is valid
	 * @param numberOfCellsAfterTreatment the new tumor size
	 * @throws MetastaseException
	 */
	public void updateTumorSize(double time, long numberOfCellsAfterTreatment) throws MetastaseException{
		
		if(numberOfCellsAfterTreatment == 0){
			// the tumor is dead
			growthFunction.addFunction(time, new ConstantFunction(0));
			// delete all further functions from the piecewise function
			growthFunction.deleteAllFollowingSections(time);
			
		}else{
			// add the normal growth function with the adapted start size to the piecewise function
	    	growthFunction.addFunction(time, FunctionUtil.copyGrowthFunctionWithNewStartSize(initialStatedGrowthFunction, numberOfCellsAfterTreatment));
	    	// since the newly added function messes up the in advance computed piecewise function, we have to recompute it from this time point on 
	    	// delete all former functions from the piecewise function that occur after the just added function
	    	growthFunction.deleteAllFollowingSections(time);
	    	// ... and rebuild the piecewise function from this time point on
	    	buildTreatments(time, getCreationTime());
	    	
	    	if(getHostOrgan().hasLateDormancy() && lateDormancyStartTime > time){
	    		//late dormancy phase didn't take place yet, so we have to recompute the start time
	    		addLateDormancy(getCreationTime());
	    		//rebuild the treatments from this time point on, since the old treatments aren't valid anymore
				buildTreatments(lateDormancyStartTime, getCreationTime());
	    	}
		}
	}
    
	/**
	 * Creates the piecewise growth function, including all characteristics and treatments.
	 * @param creationTime the creation time of the compartment
	 * @throws MetastaseException
	 */
	private void buildPiecewiseGrowthFunction(double creationTime) throws MetastaseException{
		
		growthFunction = null;
		
		//if the tumor has a dormancy phase, start with a constant function and add the normal growth function after the dormancy phase
		if(getHostOrgan().hasDormancy()){
			double dormancyDuration = getHostOrgan().computeDormancyTime();
			long startSize = SpecialMath.round(initialStatedGrowthFunction.f(0)); //get the startsize of the tumor
			growthFunction = new PiecewiseGrowthFunction(creationTime, new ConstantFunction(startSize));
			growthFunction.addFunction(creationTime + dormancyDuration, initialStatedGrowthFunction);
			
			//set the compartment dormant
			this.setDormant(true);
			//add an events that resets the dormant state and calls the createFirstEvent() method to creation of the first intravsation event
			LocalEvent stopDormancyEvent = new LocalEvent(new StopDormancy(), creationTime + dormancyDuration, this);
			getEventList().addEvent(stopDormancyEvent);
		
			//else just add the growth function as the first piece
		}else{
			Function firstGrowthFunction = checkLastChemoCycleInfluence(creationTime);
			growthFunction = new PiecewiseGrowthFunction(creationTime, firstGrowthFunction);
		}
		
		buildTreatments(creationTime, creationTime);
		
		//check if the tumor has a late dormancy phase
		if(getHostOrgan().hasLateDormancy()){
			//add the late dormancy
			addLateDormancy(creationTime);
			//rebuild the treatments from this time point on, since the old treatments aren't valid anymore
			buildTreatments(lateDormancyStartTime, creationTime);
		}
		
		return;
	}
	
	/**
	 * Checks whether the newly created metastasis is still influenced by the last chemo therapy cycle (is there was one before the creation of the metastasis).
	 * @param creationTime the creation time of the new metastasis
	 * @return returns the corresponding chemo growth function, if the new metastasis is influenced by the last chemo cycle. If not, the initial growth function 
	 * will be returned.
	 */
	private Function checkLastChemoCycleInfluence(double creationTime){
		
		TreatmentEntry lastChemoBeforeCreation = null;
		//get the last active chemo treatment before the creation of the compartment
		for (TreatmentEntry treatmentEntry : TreatmentList.getInstance()) {
			if(treatmentEntry.time < creationTime && treatmentEntry.treatment instanceof ChemoTreatment) lastChemoBeforeCreation = treatmentEntry;
			else continue;
		}
		
		if(lastChemoBeforeCreation != null){
			ChemoTreatment chemoTreatment = (ChemoTreatment) lastChemoBeforeCreation.treatment;
			//check whether the drug concentration is still high enough (I chose a value of 0.01) to influence the new compartment
			double decayTime = Math.log(0.01)/(-1.*chemoTreatment.getDrugDecayRate());
			double delay = creationTime - lastChemoBeforeCreation.time;
			
			if(decayTime > delay){
				//the drug concentration is still high enough to have an impact on the new compartment
				BasicChemoFunction chemoFunction = chemoTreatment.getChemoFunction(initialStatedGrowthFunction, 1);
				chemoFunction.setDelay(delay);				
				return chemoFunction;
			}
		}
		
		return initialStatedGrowthFunction;
	}
	
	private void addLateDormancy(double creationTime) throws MetastaseException{
		//find time for the given late dormancy start size
		lateDormancyStartTime = findStartTimeForLateDormancy(creationTime);
		
		//add the constant function that represents the dormancy phase
		growthFunction.addFunction(lateDormancyStartTime, new ConstantFunction(getLateDormancyStartSize()));
		//delete all former following functions this they aren't valid anymore
		growthFunction.deleteAllFollowingSections(lateDormancyStartTime);
		//add the normal growth function after the dormancy phase (in our model dormancy cannot occur during chemo therapy)
		growthFunction.addFunction(lateDormancyStartTime + getLateDormancyDuration(), 
				FunctionUtil.copyGrowthFunctionWithNewStartSize(initialStatedGrowthFunction, getLateDormancyStartSize()));
		
		//create the events and add them to the list
		LocalEvent startLateDormancy = new LocalEvent(new StartLateDormancy(), lateDormancyStartTime, this);
		LocalEvent stopLateDormancy = new LocalEvent(new StopLateDormancy(), lateDormancyStartTime + getLateDormancyDuration(), this);
		
		getEventList().addEvent(startLateDormancy);
		getEventList().addEvent(stopLateDormancy);
	}
	
	/**
	 * Numerically computes the time value for the late dormancy size.
	 * @param creationTime 
	 * @return returns the first matching time value after the creation time 
	 */
	private double findStartTimeForLateDormancy(double creationTime) throws MetastaseException{
		Function function = growthFunction;
		
		double size = getLateDormancyStartSize();
		if(size < growthFunction.f(creationTime)){
			throw new MetastaseException("The start size of the late dormancy phase ("+size+" cells) is smaller than the start size of "
					+ "the compartment ("+growthFunction.f(creationTime)+").");
		}
		
		double time = creationTime;
		double interval = 1.; //we are starting with an interval of one minute TODO can we make this value dynamic?		
		
		while (Math.abs(size - function.f(time)) > 1e-10){
			if((size - function.f(time)) > 0){
				time += interval;
			}else{
				interval /= 2;
				time -= interval;
			}
		}
		
		return time;
	}
	
	/**
	 * Adds treatments to the piecewise function starting from the given time. Treatments that lay before that given time are ignored (used to rebuild the 
	 * piecewise function after a function has been added after the initial creation of the piecewise function)
	 * @param time the time from which on treatments shall be added to the piecewise function
	 * @param creationTime the creation time of the compartment
	 * @param normalGrowthFunction the initial stated growth function
	 * @throws MetastaseException
	 */
	private void buildTreatments(double time, double creationTime) throws MetastaseException{
		TreatmentList treatmentList = TreatmentList.getInstance();
		
		//check the treatments
		//the treatments are sorted by their occurrence in time
		for (TreatmentEntry treatmentEntry : treatmentList) {
			
			if (treatmentEntry.treatment instanceof RadioimmunoTreatment){
				// radioimmunotherapy needs the total tumor mass of all tumors in the system to compute distribution of the injected activity 
				// to the singular tumors and therefore their number of surviving cells. Since we don't know how many metastases will be created 
				// until the treatment time we cannot compute the total tumor mass in advance. Hence, we have to use an event for this treatment,
				// which was already added to the event list in the simulation controller at the start of the simulation
				
				continue; //continue with the next treatment
			}			
			
			double treatmentTime = treatmentEntry.time;
			
			//if the treatment lies before the given time continue with the next
			if(treatmentTime < time) continue;
			
			long numberOfCellsBeforeTreatment = SpecialMath.round(growthFunction.f(treatmentTime));
			// if the tumor doesn't contain any cells then its already dead and treatments have no effect on it any more
			if (numberOfCellsBeforeTreatment == 0) return;
			
			
			///////////////// resection of the primary tumor //////////////////////////////////////////////////////////////////////////////////////////////////
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// at the moment the model only allows the resection of the primary tumor 
			if(treatmentEntry.treatment instanceof ResectionTreatment && isPrimaryTumor()){
				
				ResectionTreatment rt = (ResectionTreatment)treatmentEntry.treatment;
				
				if(rt.getRemnantFraction() == 0.0){
					//the tumor is completely removed
					growthFunction.addFunction(treatmentTime, new ConstantFunction(0));
					// delete all following functions (if there are any)
					growthFunction.deleteAllFollowingSections(treatmentTime);
					// since the tumor is dead after the resection further treatments have no influence on the it. hence, we can return.
					return;
				}else{
					//compute the number of remaining cells
					long numberOfRemainingCells = (long)(numberOfCellsBeforeTreatment * rt.getRemnantFraction());
					//add the new following normal growth function (with adapted number of cells)
					growthFunction.addFunction(treatmentTime, FunctionUtil.copyGrowthFunctionWithNewStartSize(initialStatedGrowthFunction, numberOfRemainingCells));				
				}				
			
			
			///////////////// external beam radiation of the primary tumor ////////////////////////////////////////////////////////////////////////////////////
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			// at the moment the model only allows the radiation of the primary tumor 
			} else if(treatmentEntry.treatment instanceof RadiationTreatment){
				
				Function currentFunction = growthFunction.getCurrentValidFunction(treatmentTime);
				double currentFunctionStartTime = growthFunction.getTimeOfCurrentValidSection(treatmentTime);
				//check whether the actual growth function is a chemo function, hence, whether a chemo therapy was applied before the radiation therapy
				if(currentFunction instanceof BasicChemoFunction){
					//get the time of drug decay
					double timeOfDrugDecay = currentFunctionStartTime + ((BasicChemoFunction)currentFunction).getTimeOfDrugDecay();
					//check whether both therapies overlap
					if(treatmentTime < timeOfDrugDecay){
						//The compartment is still effected by the chemo therapy. Hence, we are not able to apply radiation therapy
						throw new MetastaseException("An error occurred while computing radiation therapy: The interval between " +
								"chemo therapy and radiation therapy is to short. The drug concentration in the compartment with the id "+ 
								getMyId() +" ("+getName()+" is still to high. Current time (d) = "+treatmentTime/24./60. +" - time of drug " +
								"decay (d) = "+timeOfDrugDecay/24./60. +"\n" +
								"The model is currently not able to simultaneously compute the effect of chemo therapy and radiation therapy.");
					}
				}

				long numberOfCellsAfterTreatment = -1;
				
				if (treatmentEntry.treatment instanceof ExternalBeamTreatment){
					// ExternalBeamTreatment is only available for the primary tumor right now
					if(!isPrimaryTumor()) continue; //continue with the next treatment
					
					ExternalBeamTreatment ebt = (ExternalBeamTreatment)treatmentEntry.treatment;
					numberOfCellsAfterTreatment = ebt.computeNumberOfSurvivingCells(numberOfCellsBeforeTreatment);
					
				} else if (treatmentEntry.treatment instanceof RadioembolizationTreatment){
					RadioembolizationTreatment ret = (RadioembolizationTreatment)treatmentEntry.treatment;
					
					//if the tumor isn't situated in the treated organ continue with the next one
					if(!getHostOrgan().equals(ret.getTreatedOrgan())) continue;
					//if the tumor is smaller than the given min size at the treatment time than it is not detectable and therefore not treated  
					if(numberOfCellsBeforeTreatment < ret.getMinSize()) continue; //continue with the next treatment					
					
					numberOfCellsAfterTreatment = ret.computeNumberOfSurvivingCells(numberOfCellsBeforeTreatment);				
				}
				
				if(numberOfCellsAfterTreatment == -1){
					//something went wrong computing the number of surviving cells
					throw new MetastaseException("An error occured while calculating the number of surving cells after the radiation therapy.");
				}
				
				if(numberOfCellsAfterTreatment < 1){
					//the tumor is dead, add a new Constant function to the piecewise function
					growthFunction.addFunction(treatmentTime, new ConstantFunction(0));
					//and delete all following functions (if there are any)
					growthFunction.deleteAllFollowingSections(treatmentTime);
					//since the tumor is dead further treatments have no influence on the it. hence, we can return.
					return;
				}else{
					
					if( currentFunction instanceof ConstantFunction){
						//the tumor is dormant (dead tumors (x=0) were already sorted out some lines above)
						//their should be one more function after this one (the normal growth function)
						double nextFunctionTime = growthFunction.getTimeOfNextValidFunction(treatmentTime);
						//add the adapted dormancy function (constant function with the new number of cells)
						growthFunction.addFunction(treatmentTime, new ConstantFunction(numberOfCellsAfterTreatment));
						//delete the old following normal growth function
						growthFunction.deleteAllFollowingSections(treatmentTime);
						//add the new following normal growth function (with adapted number of cells)
						growthFunction.addFunction(nextFunctionTime, FunctionUtil.copyGrowthFunctionWithNewStartSize(initialStatedGrowthFunction, numberOfCellsAfterTreatment));
					
					} else {
						//add the normal growth function with an adapted value for the start size
						//since radiation therapy can only be applied if any prior applied chemo therapy has no influence on the tumor any more we can return 
						//to the normal growth function (calculation of cells is faster with normal growth function than with chemo functions)
						growthFunction.addFunction(treatmentTime, FunctionUtil.copyGrowthFunctionWithNewStartSize(initialStatedGrowthFunction, numberOfCellsAfterTreatment));
					}
				}				
				
				
			///////////////// chemo therapy ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
			} else if(treatmentEntry.treatment instanceof ChemoTreatment){
				
				ChemoTreatment ct = (ChemoTreatment)treatmentEntry.treatment;
				
				//get the new growth function (chemo cycle function)
				BasicChemoFunction chemoFunction = ct.getChemoFunction(initialStatedGrowthFunction, numberOfCellsBeforeTreatment);
				
				if(chemoFunction == null){
					//if the chemoFunction is null, the tumor is either dead or no specific chemo function is available for the given growth rate function
					//but since we already checked if the tumor is dead some lines above it will probably mean that no specific chemo function is available
					throw new MetastaseException("No corresponding chemo therapy availabe for the given growth function: "+initialStatedGrowthFunction);
				}
				
				if(growthFunction.getCurrentValidFunction(treatmentTime) instanceof ConstantFunction){
					//tumor is dormant at the start of the chemo. get the time the dormancy ends
					double dormancyEndTime = growthFunction.getTimeOfNextValidFunction(treatmentTime);
					double timeOfDrugDecay = treatmentTime + chemoFunction.getTimeOfDrugDecay();
					
					if(timeOfDrugDecay < dormancyEndTime) {
						//the influence of the chemo drug ends before the dormancy. hence, no need to apply the chemo function
						//continue with the next treatment
						continue;
					}
					//add the delay (start of chemo till end of dormancy)
					chemoFunction.setDelay(dormancyEndTime-treatmentTime);
					
					//remove the former growth function that would have been applied after the dormancy
					growthFunction.deleteAllFollowingSections(treatmentTime);
					//and add the chemo function as the new function after dormancy
					growthFunction.addFunction(dormancyEndTime, chemoFunction);
				}			
				
				
				//add the chemo function to the piecewise function
				growthFunction.addFunction(treatmentTime, chemoFunction);
				
				///// check, whether the tumor dies during this chemo cycle ////////////////////////////////
				Double t_min = chemoFunction.getMinimum(); //If no minimum exists t_min will be null
				if(t_min != null){					
					Double t_min_global = treatmentTime + t_min;
					if((treatmentEntry.next == null || t_min_global < treatmentEntry.next.time) && chemoFunction.f(t_min) < 1){
						//if the number of cells becomes smaller than 1 in this chemo cycle, 
						//add a new Constant function to the piecewise function and return
						growthFunction.addFunction(treatmentTime, new ConstantFunction(0));
						//and delete all following functions (if there are any)
						growthFunction.deleteAllFollowingSections(treatmentTime);
						//since the tumor is dead further treatments have no influence on the it. hence, we can return.
						return;
					}	
				}			
				
			}//end if different treatments		
			
		}//end for all treatments
	}	
	
	public double getCreationTime(){
		return growthFunction.getFirstTimePoint();
	}
	
	public Function getCurrentValidFunction(double time){
		return growthFunction.getCurrentValidFunction(time);
	}
	
	public double getStartTimeOfCurrentValidFunction(double time){
		return growthFunction.getTimeOfCurrentValidSection(time);
	}
	
	/**
	 * Deletes all local events from the event list that are related to this continuous compartment.
	 */
	public void deleteAllEvents() {
		// delete all events that belong to this compartment
		for (Event e : getEventList()) {
			//look for any dormancy events and remove them
			// It has to be a local event related to the current compartment
			if ((e instanceof LocalEvent && ((LocalEvent) e).getCompartment().sameAs(this))) {				
				getEventList().removeEvent(e);				
			}
			
			//look for an AddCell event that originates from this compartment
			if (
				// it has to be a local event
				e instanceof LocalEvent  &&
				// it has to be an AddCellToDiscreteCompartment event ...
				(((LocalEvent) e).getEventAction() instanceof AddCellToDiscreteCompartment) &&
				//... that originated from this compartment
				(((AddCellToDiscreteCompartment)((LocalEvent)e).getEventAction()).getTumorCompartment().sameAs(this))
				) {
				//remove this event from the event list
				getEventList().removeEvent(e);
			}
		}
	}


	public AddCellToDiscreteCompartment deleteLastIntravasationEvent() {
		// delete the AddCell event that belongs to this compartment (there should only be one)
		for (Event e : getEventList()) {
			
			if (
				// it has to be a local event
				e instanceof LocalEvent  &&
				// it has to be an AddCellToDiscreteCompartment event ...
				(((LocalEvent) e).getEventAction() instanceof AddCellToDiscreteCompartment) &&
				//... that originated from this compartment
				(((AddCellToDiscreteCompartment)((LocalEvent)e).getEventAction()).getTumorCompartment().sameAs(this))
				) {
				//remove this event from the event list and return the event action
				getEventList().removeEvent(e);				
				return (AddCellToDiscreteCompartment)((LocalEvent) e).getEventAction();
			}
		}
		
		// no appropriate event found in the event list
		// this may happen if the compartment has no colonization rate or is already dead
		return null;
	}

	////////////// toString(), equals() and hashCode() ///////////////////////////////////////////////////////////////////////////////////////
	@Override
	public String toString() {
		return "ContinuousCompartment ["+super.toString()
				+", growthFunction=" + growthFunction
				+ ", colonizationRate=" + colonizationRate
				+ ", eventTimeCalculator=" + eventTimeCalculator + "]";
	}


	@Override
	public int hashCode() {
		final int prime = 95;
		int result = super.hashCode();
		result = prime * result	+ ((colonizationRate == null) ? 0 : colonizationRate.hashCode());
		result = prime * result	+ growthFunction.hashCode();
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;		
		
		ContinuousCompartment other = (ContinuousCompartment) obj;

		return ((colonizationRate == null)? other.colonizationRate == null : colonizationRate.equals(other.colonizationRate)) &&
				growthFunction.equals(other.growthFunction);

	}
}
