/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.fhstralsund.catsit.base.conditions.Condition;
import de.fhstralsund.catsit.base.conditions.SizeCondition;
import de.fhstralsund.catsit.base.conditions.TimeCondition;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDeath;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDivision;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.LocalEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopLateDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNextCompartment;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.randoms.RandomG;

/**
 * This class describes compartments as e.g. bloodstream, lymphatic vessels that 
 * progress by discrete events such as cell division, cell death, cell transfer...
 * 
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class DiscreteCompartment extends Compartment {

    private long numberOfCells;
    
    
    // to prevent that the tumor undergoes a second late dormancy phase, if due to treatments 
    // the number of cells falls under the late dormancy start size again
    private boolean alreadyUndergoneLateDormancy;    
    
    private List<ScheduledCompartmentType> timeScheduledCompartmentTypes;
    private List<ScheduledCompartmentType> sizeScheduledCompartmentTypes; 
    private int actualSizeSCTIndex; //the index of the actual ScheduledCompartmentType
    
    //saves the time points at which the size conditions became valid
    private Map<String, Double> timeSnapshots;
    private long[] sizesToObserveGreaterEquals;
    private long[] sizesToObserveLessThan;
   

    /**
     * Constructor for discrete compartments as new compartment
     * 
     * @param time the creation time of the new compartment
     * @param hostOrgan the organ compartment is situated in (e.g. lung, liver, blood stream)
     * @param type the first DiscreteCompartmentType
     */
    public DiscreteCompartment(double time, HostOrgan hostOrgan, DiscreteCompartmentType type) {
        this(time, hostOrgan, type, 0, false);
    }
    
    /**
     * Constructor for discrete compartments as new compartment
     * 
     * @param time the creation time of the new compartment
     * @param hostOrgan the organ compartment is situated in (e.g. lung, liver, blood stream)
     * @param type the first DiscreteCompartmentType
     * @param isPrimaryTumor true, if the new compartment represents the primary tumor. false, otherwise
     */
    public DiscreteCompartment(double time, HostOrgan hostOrgan, DiscreteCompartmentType type, boolean isPrimaryTumor) {
        this(time, hostOrgan, type, 0, isPrimaryTumor);
    }
        
    /**
     * Constructor for discrete compartments as new compartment
     * 
     * @param creationTime the creation time of the new compartment
     * @param hostOrgan the organ compartment is situated in (e.g. lung, liver, blood stream)
     * @param type the first DiscreteCompartmentType
     * @param startSize the number of cells at the creation of the compartment (e.g. used for modeling experimental mouse data)
     * @param isPrimaryTumor true, if the new compartment represents the primary tumor. false, otherwise
     */
    public DiscreteCompartment(double creationTime, HostOrgan hostOrgan, DiscreteCompartmentType type, long startSize, boolean isPrimaryTumor){
        super(type.getName(), hostOrgan, isPrimaryTumor);
        
        this.numberOfCells = startSize;
        
        this.timeScheduledCompartmentTypes = new ArrayList<ScheduledCompartmentType>();
        this.sizeScheduledCompartmentTypes = new ArrayList<ScheduledCompartmentType>();
        this.actualSizeSCTIndex = -1;
        
        //create the first ScheduledCompartmentType and add it to the time list
        ScheduledCompartmentType firstCompartmentType = new ScheduledCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 0.), type);
        addScheduledCompartmentType(firstCompartmentType);

        this.alreadyUndergoneLateDormancy = false;
        
        this.timeSnapshots = new HashMap<String, Double>();
        this.sizesToObserveGreaterEquals = new long[0];
        this.sizesToObserveLessThan = new long[0];
        
        if(hostOrgan.hasDormancy()){        	
        	//set the tumor dormant, compute the dormancy time and create an event that stops the dormancy
        	this.setDormant(true);
    		this.createDormancyStopEvent(creationTime, hostOrgan.computeDormancyTime());
        } 
    }  


    @Override
    public long getNumberOfCells(double time) {
    	return numberOfCells;
    }

    
    /**
     * This method is only to be used in treatments, e.g. resection or radiation therapy
     * @param newNumberOfCells the new number of cells
     */
    public void setNumberOfCells(long newNumberOfCells){
    	numberOfCells = newNumberOfCells;
    	
    }
    
    /**
     * Adds a cell to the compartment. 
     * @param currentTime
     */
    public void addCell(double currentTime) {
    	if(isDormant() || isInLateDormancy()){
    		//this should not happen ...
    		logger.severe("The method addCell() was called while in dormancy phase. \nCompartment: "+getMyId()+"-"+getName()+" at time="+currentTime);
    		throw new IllegalStateException("The method addCell() was called while in dormancy phase. This should not happen!\n" +
    				"Compartment: "+getMyId()+"-"+getName()+" at time="+currentTime);
    	}
        numberOfCells++;
        
        //start size of late dormancy reached? => set the compartment dormant
        //currently late dormancy can occur only once. If due to treatments the number of cells drops under the late dormancy start size again no new 
        //late dormancy phase will occur if the start size is reached again (thats what the alreadyUndergoneLateDormancy parameter is for)
        if(!alreadyUndergoneLateDormancy && numberOfCells == getLateDormancyStartSize()){
        	setInLateDormancy(true);
        	alreadyUndergoneLateDormancy = true;
        	//create a stop event to stop the late dormancy
        	createLateDormancyStopEvent(currentTime);
    	}
        
        //check whether the next size condition became valid
        if(sizesToObserveGreaterEquals.length != 0 ){
        	//if there are still sizes to check upon in the list than check the next one
        	if(sizesToObserveGreaterEquals[0] == numberOfCells){
        		timeSnapshots.put(Condition.GREATER_EQUALS + "_" + sizesToObserveGreaterEquals[0], currentTime);
        		//copy the array without the first value
        		if(sizesToObserveGreaterEquals.length == 1){
        			//empty the array
        			sizesToObserveGreaterEquals = new long[0];
        		}else{
        			//copy the array without the first value
        			sizesToObserveGreaterEquals = Arrays.copyOfRange(sizesToObserveGreaterEquals, 1, sizesToObserveGreaterEquals.length);
        		}       		
        	}
        }
    }

    /**
     * Deletes a cell from the compartment
     * @param currentTime
     */
    public void deleteCell(double currentTime) {
    	if(numberOfCells<=0){
    		throw new IllegalStateException("The compartment contains no cells anymore. Therefore no cells can be deleted.");
    	}
        numberOfCells--;
        
        //check whether the next size condition became valid
        if(sizesToObserveLessThan.length != 0 ){
        	//if there are still sizes to check upon in the list than check the next one
        	if(sizesToObserveLessThan[0]-1 == numberOfCells){
        		timeSnapshots.put(Condition.LESSER + "_" + sizesToObserveLessThan[0], currentTime);
        		if(sizesToObserveLessThan.length == 1){
        			//empty the array
        			sizesToObserveLessThan = new long[0];
        		}else{
        			//copy the array without the first value
        			sizesToObserveLessThan = Arrays.copyOfRange(sizesToObserveLessThan, 1, sizesToObserveLessThan.length);
        		}    
        	}
        }
    }

    /**
     * Transfers a cell from this compartment to the next compartment, the successor
     */
    public void transferCellToNextCompartment(double currentTime) {
    	if(numberOfCells<=0){
    		throw new IllegalStateException("The compartment contains no cells anymore. Therefore no cells can be transfered to the next compartment.");
    	}
    	if(getSuccessor() == null){
    		throw new IllegalStateException("The successor must not be 'null' when calling transferCellToNextCompartment.");
    	}
        deleteCell(currentTime);
        getSuccessor().addCell(currentTime);
    }

    /**
     * Creates a new discrete Compartment of type newCompartmentType (the parameter). Transfer a
     * cell from this compartment to the new compartment. This describes mainly formation
     * of a new metastases.
     * 
     * N.B. The new compartment is <bf>not</bf> registered as a successor of this
     * compartment.
     * 
     * @param currentTime the current time
     * @param newHostOrgan the host organ where the new compartment is situated in
     * @param newCompartmentType the type of the new formed compartment
     * @return a reference to the new compartment
     */
    public DiscreteCompartment transferCellToNewDiscreteCompartment(double currentTime, HostOrgan newHostOrgan, DiscreteCompartmentType newCompartmentType) {
    	if(numberOfCells<=0){
    		throw new IllegalStateException("The compartment contains no cells anymore. Therefore no cells can be transfered to a new compartment.");
    	}
        
    	deleteCell(currentTime);  
    	
        DiscreteCompartment newCompartment = new DiscreteCompartment(currentTime, newHostOrgan, newCompartmentType, 1, false);//a new Compartment can never be the primary tumor
        return newCompartment;
    }

    /**
     * Creates a new ContinuousCompartment with the given growth function and colonization rate. 
     * Transfers a cell from this compartment to the new compartment. This describes mainly formation
     * of a new metastases.
     * 
     * N.B. The new compartment is <bf>not</bf> registered as a successor of this
     * compartment.
     * 
     * @param currentTime the current time
     * @param newName the name of the new compartment
     * @param newHostOrgan the host organ of the new compartment
	 * @param growthFunction the initial growth function of the new compartment (without influence of treatment, dormancy ...)
	 * @param colonizationRate the colonization rate of the new compartment. Can be null if the tumor shall not be able to spread malignant cells
     * @return a reference to the new compartment
     */
    public ContinuousCompartment transferCellToNewContinuousCompartment(double time, String newName, HostOrgan newHostOrgan, 
    		Function growthFunction, ColonizationRate colonizationRate) throws MetastaseException{
    	if(numberOfCells<=0){
    		throw new IllegalStateException("The compartment contains no cells anymore. Therefore no cells can be transfered to a new compartment.");
    	}
   
        deleteCell(time);
        
        ContinuousCompartment newCompartment = new ContinuousCompartment(time, newName, newHostOrgan, growthFunction, colonizationRate);
        return newCompartment;
    }
    
    /**
     * Adds the given discrete compartment type with the given condition to the schedule list.
     * @param condition the condition, from which the compartment type is valid
     * @param compartmentType the discrete compartment type to schedule
     */
    public void addCompartmentType(Condition condition, DiscreteCompartmentType compartmentType) {
        addScheduledCompartmentType(new ScheduledCompartmentType(condition, compartmentType));
    }
    
    /**
     * Adds the given ScheduledCompartmentType to the list
     * @param sct the ScheduledCompartmentType to add
     */
    private void addScheduledCompartmentType(ScheduledCompartmentType sct){
    	
    	if(sct.getCondition() instanceof SizeCondition){
    		//size conditions are just added to the list. 
    		//They are not sorted since the size of a tumor can shrink over time due to treatments
    		sizeScheduledCompartmentTypes.add(sct);
    		//add the compare size to one of the observer lists 
    		if(sct.getCondition().getConditionTest() == Condition.GREATER_EQUALS){
    			//copy the array and add an empty slot to the end
    			sizesToObserveGreaterEquals = Arrays.copyOf(sizesToObserveGreaterEquals, sizesToObserveGreaterEquals.length+1);
    			//fill the new empty slot with the new size value
    			sizesToObserveGreaterEquals[sizesToObserveGreaterEquals.length-1]=((SizeCondition)sct.getCondition()).getCompareSize();
    		}else{
    			//copy the array and add an empty slot to the end
    			sizesToObserveLessThan = Arrays.copyOf(sizesToObserveLessThan, sizesToObserveLessThan.length+1);
    			//fill the new empty slot with the new size value
    			sizesToObserveLessThan[sizesToObserveLessThan.length-1]=((SizeCondition)sct.getCondition()).getCompareSize();
    		}
    	}else{
    		double time = ((TimeCondition)sct.getCondition()).getCompareTime();
    		
    		if(timeScheduledCompartmentTypes.size() == 0){
    			//if the list is empty we don't have to find the right insert point but just add it to the list
    			timeScheduledCompartmentTypes.add(sct);
    			
    		}else{
	    		int maxIndex = timeScheduledCompartmentTypes.size()-1;
	    		
	    		for (int i = 0; i <= maxIndex; i++) {
					double t = ((TimeCondition)timeScheduledCompartmentTypes.get(i).getCondition()).getCompareTime();
					if(time < t){
						//if the new SCTs condition time is smaller than the one of the actual SCTs in the list, the new SCT is inserted 
						//into the place of the actual SCT. The actual SCT and all following SCTs are shifted one position to the right
						timeScheduledCompartmentTypes.add(i, sct);
						break;
					}
					if(i == maxIndex){
						//if the new SCT wasn't inserted into the list after the last iteration just add it to the end of the list
						timeScheduledCompartmentTypes.add(sct);
					}
				}
    		}
    	}
    }

    /**
     * Returns the actual valid compartment type
     * @param currentTime the current time point for which the discrete compartment type shall be returned
     * @return the compartment type that is valid at the given time
     */
    public DiscreteCompartmentType getActualCompartmentType(double currentTime){
    
    	//get the actual valid SCT from the time list
    	///////////////////////////////////////////////////////////////////////////////////////////////
    	ScheduledCompartmentType timeSCT = null;
    	for (ScheduledCompartmentType type : timeScheduledCompartmentTypes) {
			if(type.isConditionTrue(currentTime, 0)){
				timeSCT = type;
			}
		}    	
    	
    	//get the actual valid SCT from the size list
    	///////////////////////////////////////////////////////////////////////////////////////////////
    	ScheduledCompartmentType sizeSCT = null;
    	
    	//check whether there are even SCTs in the size list
    	if(sizeScheduledCompartmentTypes.size() > 0){
    		//we cannot start at the lists beginning to find the actual valid SCT since the tumor can shrink over time due to treatments
    		//So, since we don't want to accidentally go backwards in the list we have to start looking for the new SCT starting from the current one
    		
    		if(actualSizeSCTIndex == -1){
    			//if actualSizeSCTIndex == -1 it means that the first SCT in the list wasn't valid yet. So, get the first element in the list.
    			//and check whether it is valid now    			
    			if(sizeScheduledCompartmentTypes.get(0).isConditionTrue(currentTime, numberOfCells)){
    				sizeSCT = sizeScheduledCompartmentTypes.get(0);
    				actualSizeSCTIndex = 0;
    			}
    			
    		}else{
    			sizeSCT = sizeScheduledCompartmentTypes.get(actualSizeSCTIndex);
    			
    			//if its not the last one in the list, check whether the next one is valid yet
        		if(actualSizeSCTIndex < (sizeScheduledCompartmentTypes.size()-1)){
        	    	if(sizeScheduledCompartmentTypes.get(actualSizeSCTIndex+1).isConditionTrue(currentTime, numberOfCells)){
    					sizeSCT = sizeScheduledCompartmentTypes.get(actualSizeSCTIndex+1);
    					actualSizeSCTIndex++;
    				}    	
        		}//end if not last in the list
    		}    		
    	}//end if elements in the list    	

    	//check which SCT is valid right now
    	///////////////////////////////////////////////////////////////////////////////////////////////
    	if(sizeSCT == null) return timeSCT.getCompartmentType();    	
    	
    	double timeOfTimeSCT = ((TimeCondition)timeSCT.getCondition()).getCompareTime();
    	double timeOfSizeSCT = timeSnapshots.get(sizeSCT.getCondition().getConditionTest()+"_"+((SizeCondition)sizeSCT.getCondition()).getCompareSize());
    	
    	//to decide which SCT is valid right now we compare the times at which both SCTs became valid
    	if(timeOfTimeSCT > timeOfSizeSCT) return timeSCT.getCompartmentType();
    	else return sizeSCT.getCompartmentType();

    }

    @Override
    public void createFirstEvent(double currentTime) {
        createNextEvent(currentTime);
    }
    
    /**
     * Creates the next Event for this Compartment. The chosen EventAction is
     * depending on the EventActions specified in the actual DiscreteCompartmentType.
     * 
     * @param currentTime the current time
     */
    public void createNextEvent(double currentTime) {
    	//this method should not be called if the compartment is in (late) dormancy 
    	if(isDormant() || isInLateDormancy()){ 
        	throw new IllegalStateException("No new event should be created while in (late) dormany phase.");
        }    	
    	
        // compute with help of the probability, which EventAction is the next
        CellProgressionEventAction eventAction = getActualCompartmentType(currentTime).getNextEventAction();
       
        // compute the time of the eventAction, generate the event and add it to the event list
        double deltaTime = eventAction.computeTimeToThisEvent(numberOfCells);                     
        
        Event event = new LocalEvent(eventAction, currentTime + deltaTime, this);
        getEventList().addEvent(event);      
    }
    
    
	public void deleteEvents(double rate) {
		// we need a list for each kind of event action
		List<Event> potentiallyInvalidCellDivisionEvents = new ArrayList<Event>();
		List<Event> potentiallyInvalidCellDeathEvents = new ArrayList<Event>();
		List<Event> potentiallyInvalidTransferToNextCompartmentEvents = new ArrayList<Event>();
		List<Event> potentiallyInvalidTransferToNewCCompartmentEvents = new ArrayList<Event>();
		List<Event> potentiallyInvalidTransferToNewDCompartmentEvents = new ArrayList<Event>();
		List<Event> potentiallyInvalidAddCellToDCompartmentEvents = new ArrayList<Event>();

		// find all potentially invalid local events (global events cannot
		// become invalid) and
		// add them to the corresponding list
		for (Event e : getEventList()) {
			if ((e instanceof LocalEvent && // it has to be a local event					
					//and it has to be an event related to the current compartment
					((LocalEvent) e).getCompartment().sameAs(this))) {

				LocalEventAction ea = ((LocalEvent) e).getEventAction();

				if (ea instanceof CellDivision) {
					potentiallyInvalidCellDivisionEvents.add((LocalEvent) e);

				} else if (ea instanceof CellDeath) {
					potentiallyInvalidCellDeathEvents.add((LocalEvent) e);

				} else if (ea instanceof TransferToNextCompartment) {
					potentiallyInvalidTransferToNextCompartmentEvents.add((LocalEvent) e);

				} else if (ea instanceof TransferToNewContinuousCompartment) {
					potentiallyInvalidTransferToNewCCompartmentEvents.add((LocalEvent) e);

				} else if (ea instanceof TransferToNewDiscreteCompartment) {
					potentiallyInvalidTransferToNewDCompartmentEvents.add((LocalEvent) e);
					
				} else if (ea instanceof AddCellToDiscreteCompartment) {
					potentiallyInvalidAddCellToDCompartmentEvents.add((LocalEvent) e);
				}
			}
		}

		removeEvents(rate, potentiallyInvalidCellDivisionEvents);
		removeEvents(rate, potentiallyInvalidCellDeathEvents);
		removeEvents(rate, potentiallyInvalidTransferToNextCompartmentEvents);
		removeEvents(rate, potentiallyInvalidTransferToNewDCompartmentEvents);
		removeEvents(rate, potentiallyInvalidTransferToNewCCompartmentEvents);
		
		//AddCell events are only removed when the compartment is dead, since only one AddCell event 
		//from each continuous compartment is ever present in the event list
		//deleting AddCell events when the compartment is still alive would result in aborting spreading 
		//behavior of the respective continuous compartment
		if(rate == 1.0){
			removeEvents(1.0, potentiallyInvalidAddCellToDCompartmentEvents);
		}
	}
	
    private void removeEvents(double rate, List<Event> potentiallyInvalidEvents){
		
		if(rate==1.0){
			//if the rate is 1, the compartment does not contain any cells anymore. 
			//Hence, delete all events that are related to the compartment, 
			//since all events are invalid now
			for (Event event : potentiallyInvalidEvents) {
				getEventList().removeEvent(event);
			}
		}else if(rate<1.0 && rate >= 0) {
			RandomG rnd = RandomG.getInstance();
			//if the rate is smaller 1, remove only some of the events depending on the rate 
			//compute the number of events that have to be deleted
			long numberOfEventsToDelete = Math.round(rate * potentiallyInvalidEvents.size());	
			
			while(numberOfEventsToDelete > 0){
				//get a random event from the list (and remove it, so we don't try to remove it 
				//twice from the event list)
				Event e = potentiallyInvalidEvents.remove((int)(rnd.getDouble() * potentiallyInvalidEvents.size()));
				//delete the event from the event list
				getEventList().removeEvent(e);
				//decrease the number of events to delete
				numberOfEventsToDelete--;				
			}			
		}else{
			throw new IllegalArgumentException("The value of 'rate' has to be in the interval [0,1]");
		}
	}

    /**
     * 
     * @param creationTime the creation time of the compartment
     */
	private void createDormancyStopEvent(double creationTime, double dormancyTime) {
		//the dormancy is given in minutes.
		double eventTime = creationTime + dormancyTime;
		StopDormancy dormancyStop = new StopDormancy();
		getEventList().addEvent(new LocalEvent(dormancyStop, eventTime, this));
	}
	
	private void createLateDormancyStopEvent(double time) {
		//the late dormancy phase time is given in minutes.
		double eventTime = time + getLateDormancyDuration();
		StopLateDormancy stopLateDormancy = new StopLateDormancy();
		getEventList().addEvent(new LocalEvent(stopLateDormancy, eventTime, this));
	}


    
	////////////toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

	@Override
	public String toString() {
		return "DiscreteCompartment [" + super.toString() 
				+ ", numberOfCells=" + numberOfCells
				+ ", alreadyUndergoneLateDormancy=" + alreadyUndergoneLateDormancy
				+ ", timeScheduledCompartmentTypes=" + timeScheduledCompartmentTypes
				+ ", sizeScheduledCompartmentTypes=" + sizeScheduledCompartmentTypes + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 16;
		int result = super.hashCode();
		result = prime * result + actualSizeSCTIndex;
		result = prime * result + (alreadyUndergoneLateDormancy ? 1231 : 1237);
		result = prime * result	+ sizeScheduledCompartmentTypes.hashCode();
		result = prime * result	+ timeScheduledCompartmentTypes.hashCode();
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;		
		
		DiscreteCompartment other = (DiscreteCompartment) obj;		
		
		return (actualSizeSCTIndex == other.actualSizeSCTIndex && alreadyUndergoneLateDormancy == other.alreadyUndergoneLateDormancy &&
				numberOfCells == other.numberOfCells &&
				sizeScheduledCompartmentTypes.equals(other.sizeScheduledCompartmentTypes) &&
				timeScheduledCompartmentTypes.equals(other.timeScheduledCompartmentTypes)
				);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

}
