/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import java.util.ArrayList;

import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.support.randoms.RandomG;

/**
 * Possible types of {@link DiscreteCompartment} with the {@link CellProgressionEventAction}s
 * that are possible here.
 * 
 * @author Gero Wedemann, Michael Suhr, Anja Bethge
 */
public class DiscreteCompartmentType {

	//the compartment name
	private String name;
	private ArrayList<CellProgressionEventAction> possibleEventActions; 

	/**
	 * 
	 * @param name the name of the discreteCompartmentType e.g. "liver metastasis"
	 */
	public DiscreteCompartmentType(String name) {
		this.name = name;
		this.possibleEventActions = new ArrayList<CellProgressionEventAction>();
	}
	
	
	public String getName() {
		return name;
	}
	
	public void addPossibleEventAction(CellProgressionEventAction eventAction){
		possibleEventActions.add(eventAction);
	}
	
	/**
	 * we check here, if the sum of all probabilities is 1
	 */
	public void checkPossibleEventActions() throws DiscreteCompartmentTypeException {
		if(possibleEventActions.size()<=0){
			throw new DiscreteCompartmentTypeException("The discrete compartment type " +
					"has to contain at least one EventAction");
		}
		double sum = 0;
		for (CellProgressionEventAction eventAction : possibleEventActions){
			sum += eventAction.getProbability();
		}
		if( Math.abs(1-sum)>1e-8) {
			throw new DiscreteCompartmentTypeException("Sum of probabilities of " +
					"all EventActions must be 1 but is " + sum);
		}
	}
	
	public CellProgressionEventAction getNextEventAction(){
		RandomG rng = RandomG.getInstance();
		double zeta = rng.getDouble();

		double probabilitySum=0;
		for (CellProgressionEventAction eventAction : possibleEventActions) {
			probabilitySum += eventAction.getProbability();
			if (probabilitySum >= zeta) return eventAction;
		}
		// we get here only by numerical errors ... 
		// then we deliver the last
		return possibleEventActions.get(possibleEventActions.size()-1);
	}
	
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

	@Override
	public String toString() {
		return "DiscreteCompartmentType [name="+name+", possibleEventActions=" + possibleEventActions + "]";
	}


	@Override
	public int hashCode() {
		final int prime = 72;
		int result = 1;
		result = prime * result + name.hashCode();
		result = prime * result	+ possibleEventActions.hashCode();
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		
		DiscreteCompartmentType other = (DiscreteCompartmentType) obj;
		
		return (name.equals(other.name) && possibleEventActions.equals(other.possibleEventActions));
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   	
	
}
