/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import java.util.logging.Logger;

import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.support.SpecialMath;
import de.fhstralsund.catsit.support.functions.FunctionUtil;

/**
 * This class provides information about the organ, in which the tumor is situated.
 * Is specifies the name and dormancy distributions.
 * @author Anja Bethge
 *
 */
public class HostOrgan {
	protected static final Logger logger = Logger.getLogger("metastase.base.hostorgan");
	
	private String name;
	
	//the distribution to calculate the dormancy time
	private Distribution dormancy;
	
	//distributions to calculate the late dormancy
	//dormancy can also occur after the micro metastases proliferated for some time
	//(e.g. It has to make room for itself and start neo angiogenesis to ensure nutrient supply)
	//both starting size and duration are given as a gaussian distribution
	private Distribution lateDormancySize;
	private Distribution lateDormancyDuration;

	//Defines, whether an own file shall be printed with only the metastases that occur in this organ.
	private boolean printFileForHostOrgan;
	
	/**
	 * 
	 * @param name the name of the host organ
	 * @param dormancy specifies the duration of normal dormancy (Unit: days)
	 * @param lateDormancySize specifies the size when late dormancy applies (given in number of cells)
	 * @param lateDormancyDuration specifies the duration of late dormancy (Unit: days)
	 * @param printFileForHostOrgan if true an excel file will be printed for this organ with information about the metastases in this organ
	 * @throws MetastaseException 
	 */
	public HostOrgan(String name, Distribution dormancy, Distribution lateDormancySize, 
			Distribution lateDormancyDuration, boolean printFileForHostOrgan) {
		this.name = name;
		//convert values for duration from days to minutes 
		this.dormancy = FunctionUtil.convertToMinutes(dormancy);
		
		
		if(!((lateDormancyDuration == null && lateDormancySize == null) || 
				(lateDormancyDuration != null && lateDormancySize != null))){
			throw new IllegalArgumentException("lateDormancySize and lateDormancyDuration must both be either stated or null");
		}
		
		this.lateDormancySize = lateDormancySize;
		//convert values for duration from days to minutes 
		this.lateDormancyDuration = FunctionUtil.convertToMinutes(lateDormancyDuration);
		this.printFileForHostOrgan = printFileForHostOrgan;
	}
	
	/**
	 * 
	 * @param name the name of the host organ
	 * @param dormancy specifies the duration of normal dormancy (Unit: days)
	 * @param printFileForHostOrgan if true an excel file will be printed for this organ with information about the metastases in this organ
	 * @throws MetastaseException 
	 */
	public HostOrgan(String name, Distribution dormancy, boolean printFileForHostOrgan) {
		this(name, dormancy, null, null, printFileForHostOrgan);
	}
	
	/**
	 * 
	 * @param name the name of the host organ
	 * @param printFileForHostOrgan if true an excel file will be printed for this organ with information about the metastases in this organ
	 * @throws MetastaseException 
	 */
	public HostOrgan(String name, boolean printFileForHostOrgan) {
		this(name, null, null, null, printFileForHostOrgan);
	}
		
	
	public String getName() {
		return name;
	}

	/**
	 * 
	 * @return true, if a dormancy was specified or false, if dormancy is null
	 */
	public boolean hasDormancy(){
		if (dormancy == null) return false;
		else return true;
	}

	/**
     * Computes a random dormancy time depending on the given distribution.
     * Call {@link hasDormancy()} to check, whether a dormancy was specified before calling this method.
     * @return a random dormancy time in minutes
     * @throws NullPointerException if no dormancy was specified (call {@link hasDormancy()} to check)
     */
	public double computeDormancyTime() {
		if (dormancy == null) {
			throw new NullPointerException("No dormancy was specified");
		}
		
		return dormancy.getDouble();
	}
	
	/**
	 * 
	 * @return true, if a late dormancy was specified or false, otherwise
	 */
	public boolean hasLateDormancy(){
		if (lateDormancySize == null || lateDormancyDuration == null) return false;
		else return true;
	}

	/**
     * Computes a random size, when the late dormancy starts, depending on the given distribution.
     * Call {@link hasLateDormancy()} to check, whether a late dormancy was specified before calling this method.
     * @return a random size, when the late dormancy starts in minutes
     * @throws NullPointerException if no late dormancy was specified (call {@link hasLateDormancy()} to check)
     */
	public long computeLateDormancySize() {
		if (lateDormancySize == null) {
			throw new NullPointerException("No late dormancy was specified");
		}
		
		long size;
		do{
			size = SpecialMath.round(lateDormancySize.getDouble());
			//due to casting into long, the size can become zero. Though, this is not a valid value, since a 
			//metastasis cannot consist of zero cells (if so, it's no metastasis anymore ;) )
		}while(size<1);
		
		return size;		
	}
	
	/**
     * Computes a random duration of the late dormancy depending on the given distribution.
     * Call {@link hasLateDormancy()} to check, whether a late dormancy was specified before calling this method.
     * @return a random duration of the late dormancy in minutes
     * @throws NullPointerException if no late dormancy was specified (call {@link hasLateDormancy()} to check)
     */
	public double computeLateDormancyDuration() {
		if (lateDormancySize == null) {
			throw new NullPointerException("No late dormancy was specified");
		}

		return lateDormancyDuration.getDouble();
	}

	public boolean isPrintFileForHostOrgan() {
		return printFileForHostOrgan;
	}
	
	

	@Override
	public String toString() {
		return "HostOrgan [name=" + name 
				+ ", dormancy=" + dormancy + ", lateDormancySize="
				+ lateDormancySize + ", lateDormancyDuration="
				+ lateDormancyDuration + ", printFileForHostOrgan="
				+ printFileForHostOrgan + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 29;
		int result = 1;
		result = prime * result	+ ((dormancy == null) ? 0 : dormancy.hashCode());
		result = prime * result	+ ((lateDormancyDuration == null) ? 0 : lateDormancyDuration.hashCode());
		result = prime * result	+ ((lateDormancySize == null) ? 0 : lateDormancySize.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + (printFileForHostOrgan ? 1371 : 1537);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		
		HostOrgan other = (HostOrgan) obj;
		
		return(((name == null)? other.name==null: name.equals(other.name))&&
				((dormancy == null)? other.dormancy == null: dormancy.equals(other.dormancy)) &&
				((lateDormancyDuration == null)? other.lateDormancyDuration==null: lateDormancyDuration.equals(other.lateDormancyDuration)) &&
				((lateDormancySize == null)? other.lateDormancySize==null: lateDormancySize.equals(other.lateDormancySize)) &&
				printFileForHostOrgan == other.printFileForHostOrgan);
	}	
	
}
