/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import de.fhstralsund.catsit.base.conditions.Condition;

/**
 * A DiscreteCompartmentType with a {@link Condition}, when it begins. Used to schedule
 * local treatments in a discrete compartment. It cannot be derived from DiscreteCompartmentType, since the same
 * DiscreteCompartmentType (e.g. "BloodStream" or "BloodStreamWithDrug") might occur
 * several times. E.g. first "BloodStream" then "BloodStreamWithDrug" and again
 * "BloodStream".
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class ScheduledCompartmentType {
    
	private DiscreteCompartmentType compartmentType;

    // the condition, that specifies when the DiscreteCompartmentType is valid
    private Condition condition;
    

    /**
     * @param condition the condition that specifies when this CompartmentType is valid
     * @param compartmentType the compartmentType
     */
    public ScheduledCompartmentType(Condition condition, DiscreteCompartmentType compartmentType) {
    	if(condition == null || compartmentType == null){
    		throw new IllegalArgumentException("Condition and CompartmentsType must not be 'null'.");
    	}
    	
        this.compartmentType = compartmentType;
        this.condition = condition;
    }

    public DiscreteCompartmentType getCompartmentType() {
        return compartmentType;
    }
    
    public Condition getCondition(){
    	return condition;
    }

    public boolean isConditionTrue(double currentTime, long currentSize) {
        return condition.isConditionTrue(currentTime, currentSize);
    }
    
    
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
    @Override
	public String toString() {
		return "ScheduledCompartmentType [condition=" + condition + ", compartmentType=" + compartmentType + " ]";
	}

	@Override
	public int hashCode() {
		final int prime = 33;
		int result = 1;
		result = prime * result	+ compartmentType.hashCode();
		result = prime * result + condition.hashCode();
		return result;
	}
   
	@Override
    public boolean equals(Object obj) {
    	if (obj == null) return false;
		if (this == obj) return true;
		if (this.getClass() != obj.getClass()) return false;
		
		ScheduledCompartmentType other = (ScheduledCompartmentType)obj;
		
		return ((condition.equals(other.condition))&&
				(compartmentType.equals(other.compartmentType)));
    }
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

}
