/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions;

import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.support.functions.Function;

/**
 * This class stores the information that are needed to create a new
 * EventAction. This includes the probability and the distribution and meanTime
 * function to compute the time of the new Event.
 * 
 * @author Anja Bethge, Michael Suhr
 * 
 */
public class GenerationInfo {

    private double probability;

    /**
     * x is the size of the compartment
     */
    private Function meanTimeToNextEvent;
    private Distribution distribution;

    /**
     * @param probability
     *            probability of this action for this DiscreteCompartmentType. The
     *            sum of all probabilities for a compartmentType must be 1.
     * @param meanTimeToNextEvent
     *            a Function to compute the mean time to the next event. It is
     *            multiplied with the distribution
     * @param distribution
     *            the Distribution for the time for the this event-type
     */
    public GenerationInfo(double probability, Function meanTimeToNextEvent, Distribution distribution) {

    	if(probability<=0){
    		throw new IllegalArgumentException("the value of 'probability' must not be smaller or equal than '0'.");
    	}
    	if((meanTimeToNextEvent == null || distribution == null)){
    		throw new IllegalArgumentException("'meanTimeToNextEvent' and 'distribution' must not be 'null'.");
    	}
    	
        this.probability = probability;
        this.meanTimeToNextEvent = meanTimeToNextEvent;
        this.distribution = distribution;
    }

    public double getProbability() {
        return probability;
    }
    
    public Distribution getDistribution(){
    	return distribution;
    }
    
    public Function getMeanTimeToNextEventFunction(){
    	return meanTimeToNextEvent;
    }

    public double computeTimeToThisEvent(long numberOfCells) {
        return meanTimeToNextEvent.f(numberOfCells) * distribution.getDouble();
    }

    
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
	@Override
	public String toString() {
		return "GenerationInfo [probability=" + probability	+ ", meanTimeToNextEvent=" + meanTimeToNextEvent + ", distribution=" + distribution + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 85;
		int result = 1;
		result = prime * result + distribution.hashCode();
		result = prime * result	+ meanTimeToNextEvent.hashCode();
		long temp;
		temp = Double.doubleToLongBits(probability);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		
		GenerationInfo gi = (GenerationInfo) obj;		
		
		return (distribution.equals(gi.distribution) &&
				meanTimeToNextEvent.equals(gi.meanTimeToNextEvent) &&
				(probability == gi.probability));
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

}
