/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.globaleventactions;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.io.excel.writer.ExcelRecordMain;
import de.fhstralsund.catsit.io.excel.writer.ExcelRecordSingleHostOrgan;
import de.fhstralsund.catsit.io.excel.writer.ExcelWriter;
import de.fhstralsund.catsit.support.LogSampleHistogram;

/**
 * Buffers the actual status of the simulation in an ExcelRecord which is written into a spreadsheet at the end of the simulation.
 * 
 * @author Gero Wedemann, Anja Bethge
 */
public class PrintStatus extends GlobalEventAction {

	//primary tumor and blood stream
	private Compartment primaryTumor;
	private DiscreteCompartment bloodStream;
	
	//the list of defined host organs
	private List<HostOrgan> hostOrgans;
	
	private Map<String, Long> numberOfMetas;
	private Map<String, Long> numberCellsInAllMetas;
	private Map<String, LogSampleHistogram> sizeHistograms;
	
	//the excel writer, where to save the excel records
	private ExcelWriter excelWriter;
	
	/**
	 * 
	 * @param name
	 * @param primary the primary tumor
	 * @param blood the bloodstream
	 * @param hostOrgans the list of host organs
	 * @param excelWriter the excel writer where the excel records are saved
	 */
	public PrintStatus(String name, Compartment primary, DiscreteCompartment blood, List<HostOrgan> hostOrgans, ExcelWriter excelWriter) {
		super(name);
		this.primaryTumor = primary;
		this.bloodStream = blood;
		this.hostOrgans = hostOrgans;
	
		this.numberOfMetas = new HashMap<String, Long>();
		this.numberCellsInAllMetas = new HashMap<String, Long>();
		this.sizeHistograms = new HashMap<String, LogSampleHistogram>();
	
		this.excelWriter = excelWriter;
	}
	
	/**
	 * to compute the mean and standard derivation of multiple simulations, we need the data from 
	 * the exact same time points of every simulation run. so we just save the data at the 
	 * PrintStatus events, that are placed at the desired time points in every simulation run
	 */
	@Override
	public void execute(double currentTime) {
		
		//compute number of metasases, number of all cells in all metastases and the metastases size histogram
		computeStatistics(currentTime);
		
		//save the actual status of the whole system in an excel record
		ExcelRecordMain mainRecord = new ExcelRecordMain(
				(long) currentTime,							//time in minutes
				(currentTime / 60 / 24), 					//time in days
				primaryTumor.getNumberOfCells(currentTime),	//# cells in primary
				bloodStream.getNumberOfCells(currentTime),	//# cells in blood
				numberOfMetas.get("all"),					//# of metastases
				numberCellsInAllMetas.get("all"),			//# of cells in all metastases
				sizeHistograms.get("all"));					//size histogram of all metastases
		
		excelWriter.bufferMainRecord(mainRecord);
		
		for (HostOrgan organ : hostOrgans) {
			if(organ.isPrintFileForHostOrgan()){
				String organName = organ.getName();
				//save the actual status of the host organ in an excel record
				ExcelRecordSingleHostOrgan organRecord = new ExcelRecordSingleHostOrgan(
						(long) currentTime,						//time in minutes
						(currentTime / 60 / 24), 				//time in days
						numberOfMetas.get(organName),				//# of metastases in the host organ
						numberCellsInAllMetas.get(organName),		//# of cells in all metastases in the host organ
						sizeHistograms.get(organName));				//size histogram of all metastases in the host organ
				
				excelWriter.bufferHostOrganRecord(organ.getName(), organRecord);
			}
		}
	}
	
	
	private void computeStatistics(double time)	{
		resetMaps();
		
		for (Compartment compartment : (CompartmentInventory.getCompartments())) {
			if (compartment == bloodStream || compartment == primaryTumor) continue;
			String organName = compartment.getHostOrganName();
			long number = compartment.getNumberOfCells(time);
			
			//ignore "dead" compartments which contain no cells
			if (number > 0) {
				
				numberOfMetas.put(organName, numberOfMetas.get(organName)+1);
				numberCellsInAllMetas.put(organName, numberCellsInAllMetas.get(organName)+number);
				sizeHistograms.get(organName).add(number);
				
				numberOfMetas.put("all", numberOfMetas.get("all")+1);
				numberCellsInAllMetas.put("all", numberCellsInAllMetas.get("all")+number);
				sizeHistograms.get("all").add(number);
			}
		}
	}

	private void resetMaps(){
		int BUCKETS_PER_DECADE = 10;
		int decades = 12;

		//reset each host organ
		for (HostOrgan organ : hostOrgans) {
			String organName=organ.getName();
			numberOfMetas.put(organName, 0l);
			numberCellsInAllMetas.put(organName, 0l);
			sizeHistograms.put(organName, new LogSampleHistogram(decades, BUCKETS_PER_DECADE));
		}
		
		//reset the whole system
		numberOfMetas.put("all", 0l);
		numberCellsInAllMetas.put("all", 0l);
		sizeHistograms.put("all", new LogSampleHistogram(decades, BUCKETS_PER_DECADE));
	}
}
