/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.globaleventactions;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;

/**
 * Models radioembolization in discrete compartments. 
 * Computes and resets the number of surviving cells in each discrete compartment that exceeds the given minimum size.
 * 
 * @author Anja Bethge
 *
 */
public class Radioembolization extends RadioTherapy {
	
	private RadioembolizationTreatment treatment;

	/**
	 * 
	 * @param name
	 * @param treatmentData treatment object that holds all relevant data of the treatment
	 */
	public Radioembolization(String name, RadioembolizationTreatment treatmentData) {
		super(name);
		this.treatment = treatmentData;
	}


	@Override
	public void execute(double currentTime) throws MetastaseException{
		
		for (Compartment compartment : CompartmentInventory.getCompartments()) {
			
			//we only need to deal with discrete compartments since continuous compartments handle treatments at their creation to build the piecewise function
			if(compartment instanceof DiscreteCompartment){				
				
				//if the tumor isn't situated in the treated organ continue with the next compartment
				if(!compartment.getHostOrgan().equals(treatment.getTreatedOrgan())) continue;
				
				long numberOfCellsBeforeTreatment = compartment.getNumberOfCells(currentTime);
				
				//if the tumor is smaller than the given min size than it is not detectable and therefore not treated  
				if(numberOfCellsBeforeTreatment < treatment.getMinSize()) continue; //continue with the next compartment				
				
				long numberOfCellsAfterTreatment = treatment.computeNumberOfSurvivingCells(numberOfCellsBeforeTreatment);
				double survivalRate = (double)numberOfCellsAfterTreatment / (double)numberOfCellsBeforeTreatment;
				
				handleDiscreteCompartment((DiscreteCompartment)compartment, numberOfCellsAfterTreatment, survivalRate);				
			}
		}
		
		//continuous compartments are handled via the piecewise growth function which is computed in advance at the creation of the compartment
	}
	
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	@Override
	public String toString() {
		return "Radioembolization [name=" + getName() +", treatmentData=" + treatment + "]";
	}	
	

	@Override
	public int hashCode() {
		final int prime = 36;
		int result = super.hashCode();
		result = prime * result	+ treatment.hashCode();
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		Radioembolization other = (Radioembolization) obj;
		
		return treatment.equals(other.treatment);
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
	
}
