/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.globaleventactions;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.chemo.BasicChemoFunction;

/**
 * Models rdioimmunotherapy in discrete and continuous compartments. 
 * Computes and resets the number of surviving cells in each compartment.
 * 
 * @author Anja Bethge
 *
 */
public class Radioimmunotherapy extends RadioTherapy {
	
	private RadioimmunoTreatment treatment;
	
	/**
	 * 
	 * 
	 * @param name name of the event
	 * @param treatmentData treatment object that holds all relevant data of the treatment
	 * @param excludeBloodStream if true, tumor cells in the blood stream are not effected by the therapy
	 */
	public Radioimmunotherapy(String name, RadioimmunoTreatment treatmentData) { 
		super(name);
		this.treatment = treatmentData;
	}
	


	@Override
	public void execute(double currentTime) throws MetastaseException{		
			
		// compute the number of cells in all tumors
		long totalTumorBurden = 0;			
		for (Compartment compartment : CompartmentInventory.getCompartments()){
			//get number of cells and add it to the total tumor burden
			totalTumorBurden += compartment.getNumberOfCells(currentTime);
		}	
				
		
		for (Compartment compartment : CompartmentInventory.getCompartments()) {
			
			long numberOfCellsBeforeTreatment = compartment.getNumberOfCells(currentTime);
			//if the compartment doesn't contain any cells then there is nothing to kill with this treatment
			//just continue with the next compartment
			if(numberOfCellsBeforeTreatment == 0) continue;							
			
			//get the number of surviving cells
			long numberOfCellsAfterTreatment = treatment.computeNumberOfSurvivingCells(numberOfCellsBeforeTreatment, totalTumorBurden);
			
			///////////// handle discrete compartments ////////////////////////////////////////////////////
			if(compartment instanceof DiscreteCompartment){
				
				//if the actual compartment is the bloodstream and the blood shall be excluded, continue with the next compartment
				if(treatment.isExcludeBloodStream() && compartment.isBloodStream()) continue;				
				
				double survivalRate = (double)numberOfCellsAfterTreatment / (double)numberOfCellsBeforeTreatment;
				handleDiscreteCompartment((DiscreteCompartment)compartment, numberOfCellsAfterTreatment, survivalRate);
			
			} else if(compartment instanceof ContinuousCompartment){
				
				ContinuousCompartment cc = (ContinuousCompartment) compartment;
				
				Function currentFunction = cc.getCurrentValidFunction(currentTime);
				
				//check whether the actual growth function is a chemo function, hence, whether a chemo therapy was applied before the radiation therapy
				if(currentFunction instanceof BasicChemoFunction){
					//get the time of drug decay
					double timeOfDrugDecay = cc.getStartTimeOfCurrentValidFunction(currentTime) + ((BasicChemoFunction)currentFunction).getTimeOfDrugDecay();
					
					//check whether both therapies overlap
					if(currentTime < timeOfDrugDecay){
						//The compartment is still effected by the chemo therapy. Hence, we are not able to apply radiation therapy
						throw new IllegalStateException("An error occurred while simulation radiation therapy: The interval between " +
								"chemo therapy and radiation therapy is to short. The drug concentration in the compartment with the id "+ 
								cc.getMyId() +" ("+getName()+" is still to high. Current time (d) = "+currentTime/24./60. +" - time of drug " +
								"decay (d) = "+timeOfDrugDecay/24./60. +"\n" +
								"The model is currently not able to simultaneously compute the effect of chemo therapy and radiation therapy.");
					}
				}
				
				
				if(numberOfCellsAfterTreatment < 1){
					//the tumor is dead, reset the number of cells to zero ...
					cc.updateTumorSize(currentTime, 0);
					//... and delete all Events related to this compartment
					cc.deleteAllEvents();
					//reset the dormant information (since it's dead it isn't dormant anymore)
					cc.setDormant(false);
					cc.setInLateDormancy(false);
				
				}else{
					
					if(cc.isDormant() || cc.isInLateDormancy()){
						//the tumor is dormant
						//update the number of cells in the compartment
						cc.updateDormancySize(currentTime, numberOfCellsAfterTreatment);
						
					}else{
						//reset the number of cells in the compartment
						cc.updateTumorSize(currentTime, numberOfCellsAfterTreatment);
					}					
					
					//delete the last intravsation event and create a new one that follows the updated piecewise function
					AddCellToDiscreteCompartment intravasationEA = cc.deleteLastIntravasationEvent();
					if(intravasationEA != null){
						double timeOfLastExecutedItravasationEvent = intravasationEA.getTimeOfLastExecutedEventAction();
						if(timeOfLastExecutedItravasationEvent == -1){
							//no intravasation event was executed for this compartment yet => start at the creation time
							cc.createNextEvent(cc.getCreationTime());
						}else{
							cc.createNextEvent(timeOfLastExecutedItravasationEvent);
						}
						
					}//no intravasation was event found => the compartment has no colonization rate => no need to create a new one					
				}
				
			}else{
				throw new IllegalStateException("Unknown Compartment: "+compartment.getClass().getSimpleName());
			}
		}
	}

	
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	@Override
	public String toString() {
		return "Radioimmunotherapy [treatmentData=" + treatment + "]";
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result	+ treatment.hashCode();
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;

		Radioimmunotherapy other = (Radioimmunotherapy) obj;

		return treatment.equals(other.treatment);
	}
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
}
