/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;

/**
 * Adds a cell to a discrete compartment. This is used by continuous compartments.
 * 
 * @author Gero Wedemann, Anja Bethge
 */
public class AddCellToDiscreteCompartment extends LocalEventAction {

	private ContinuousCompartment tumorCompartment;
	private double timeOfLastExecutedEventAction;
	
	/**
	 * 
	 * @param name the name of the event action
	 * @param compartment the continuous compartment that spreads the cell to the discrete compartment
	 */
	public AddCellToDiscreteCompartment(String name, ContinuousCompartment compartment) {
		super(name);
		this.tumorCompartment = compartment;
		this.timeOfLastExecutedEventAction = -1;
	}

	@Override
	public void execute(Compartment compartment, double currentTime) {
		
		if(compartment.isInLateDormancy()){
			//no events should occur in late dormancy. 
			//This event had been created before the late dormancy started. 
			//Instead of executing the event now, we just update the time of the event to the duration of the late dormancy
			compartment.delayEvent(this, currentTime);
			return;
		}
		
		
		if(compartment instanceof DiscreteCompartment){
			DiscreteCompartment dc = (DiscreteCompartment)compartment;
			//add a cell to the compartment and create the next event for the added cell
			dc.addCell(currentTime);
			dc.createNextEvent(currentTime);
			
			//create the next AddCellToDiscreteCompartmentEvent
			tumorCompartment.createNextEvent(currentTime);
			
		}else {
			throw new IllegalArgumentException("Only Discrete Compartments are allowed in the AddCellToDiscreteCompartment event action."
					+ "Following type was given: " +compartment.getClass().getName());
		}			
		
	}
	
	public ContinuousCompartment getTumorCompartment(){
		return tumorCompartment;
	}

	public void setTimeOfLastExecutedEventAction(double time){
		timeOfLastExecutedEventAction = time;
	}
	
	public double getTimeOfLastExecutedEventAction(){
		return timeOfLastExecutedEventAction;
	}
	
	
	@Override
	public String toString() {
		return "AddCellToDiscreteCompartment [name="+ getName() +", tumorCompartment="	+ tumorCompartment + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 59;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(timeOfLastExecutedEventAction);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result	+ tumorCompartment.hashCode();
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		AddCellToDiscreteCompartment other = (AddCellToDiscreteCompartment) obj;
		
		return tumorCompartment.equals(other.tumorCompartment) && timeOfLastExecutedEventAction == other.timeOfLastExecutedEventAction;
	}
}
