/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
*/

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.support.functions.Function;

/**
 * Base class for all event actions that model the progression of a discrete compartment, such as 
 * cell division, apoptosis, cell transfer.
 * Cell progression event actions can be applied only on discrete compartments.
 * 
 * @author Anja Bethge
 *
 */
public abstract class CellProgressionEventAction extends LocalEventAction {
	
	private GenerationInfo genererationInfo;
	
	/**
     * @param name name of this action
     * @param generationInfo the generation informations for this event action
     */
	public CellProgressionEventAction(String name, GenerationInfo generationInfo) {
		super(name);
        this.genererationInfo = generationInfo;
	}


    public double getProbability() {
        return genererationInfo.getProbability();
    }
    
    public Distribution getDistribution(){
    	return genererationInfo.getDistribution();
    }
    
    public Function getMeanTimeToNextEventFunction(){
    	return genererationInfo.getMeanTimeToNextEventFunction();
    }
   
    public double computeTimeToThisEvent(long numberOfCells) {
        return genererationInfo.computeTimeToThisEvent(numberOfCells);
    }
    
    @Override
    public void execute(Compartment compartment, double currentTime) throws MetastaseException {
    	
    	if(compartment instanceof DiscreteCompartment){
    		
    		if(compartment.isInLateDormancy()){
    			//no events should occur in late dormancy. 
    			//This event had been created before the late dormancy started. 
    			//Instead of executing the event now, we just update the time of the event to the duration of the late dormancy
    			compartment.delayEvent(this, currentTime);
    			return;
    		}else{
    			//cast to DiscreteCompartment
    			execute((DiscreteCompartment)compartment, currentTime);
    		}
    		
    	}else{
    		throw new IllegalArgumentException("Only Discrete Compartments can be handled in a CellProgressionEventAction. "
					+ "Following type was given: " +compartment.getClass().getName());
    	}
    	
    }

    /**
     * Restricts the execute method to discrete compartments.
     * @param compartment the discrete compartment that shall be influenced
     * @param currentTime the current time
     * @throws MetastaseException
     */
    protected abstract void execute(DiscreteCompartment compartment, double currentTime) throws MetastaseException;
    
    @Override
    public String toString() {
    	return "name=" + getName() +", genererationInfo=" + genererationInfo;
    }

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime	* result + ((genererationInfo == null) ? 0 : genererationInfo.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		CellProgressionEventAction other = (CellProgressionEventAction) obj;
		
		return ((genererationInfo == null)? other.genererationInfo == null: genererationInfo.equals(other.genererationInfo));
	}	
}
