/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.treatments.ResectionTreatment;

/**
 * Models the resection of a discrete compartment by setting the number of cells in the corresponding compartment to 0 
 * and deleting all events from the vent list that belonged to the compartment and have not been executed yet.
 * 
 * @author Anja Bethge
 *
 */
public class Resection extends LocalEventAction {

	private ResectionTreatment treatment;
	
	/**
     * @param name name of this action
     */
	public Resection(String name, ResectionTreatment treatmentData) {
		super(name);
		treatment = treatmentData;
	}

	@Override
	public void execute(Compartment compartment, double currentTime) {		
		
		if(compartment instanceof DiscreteCompartment){
			DiscreteCompartment dc = (DiscreteCompartment)compartment;
			
			long numberOfRemainingCells = (long)(dc.getNumberOfCells(currentTime) * treatment.getRemnantFraction());
			
			if(numberOfRemainingCells < 1){
				//the tumor is dead, reset the number of cells to zero ...
				dc.setNumberOfCells(0);			
				//... and delete all Events related to this compartment
				dc.deleteEvents(1.0);
				//reset the dormant information (since its dead it isn't dormant anymore)
				dc.setDormant(false);
				dc.setInLateDormancy(false);
			}else{
			
				//update the number of cells in the compartment
				dc.setNumberOfCells(numberOfRemainingCells);
				//delete some of the events related to this compartment, according to the survival rate
				dc.deleteEvents(1-treatment.getRemnantFraction());
			}
			
			
		}else{
			throw new IllegalArgumentException("The resction event is only available for discrete compartments");
		}
		
		//the resection of continuous compartments is handled via a constant growth function in the compartment itself			
	}
}
