/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.Function;

/**
 * Transfers a cell from a discrete compartment to a new continuous one
 * e.g. when a cell leaves the bloodstream and forms a new metastasis
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class TransferToNewContinuousCompartment extends CellProgressionEventAction {

	private String newName;
	private HostOrgan newHostOrgan;
    private Function newGrowthFunction;
    private ColonizationRate newColonizationRate;   
    private DiscreteCompartment successorOfNewCompartment;

    /**
     * 
     * @param name name of this action
     * @param generationInfo the generation informations for this event action
     * @param newName the name of the new compartment
     * @param newHostOrgan the organ the new compartment is situated in (e.g. lung, liver, blood stream)
     * @param newGrowthFunction the growth function of the new compartment
     * @param newColonizationRate the colonization rate of the new compartment (can be null)
     * @param successorOfNewCompartment the successor of the new compartment (e.g. blood)
     */
    public TransferToNewContinuousCompartment(String name, GenerationInfo generationInfo, String newName, HostOrgan newHostOrgan,
    		Function newGrowthFunction, ColonizationRate newColonizationRate, DiscreteCompartment successorOfNewCompartment) {
        super(name, generationInfo);
        
        this.newName = newName;
        this.newHostOrgan = newHostOrgan;       
        this.newGrowthFunction = newGrowthFunction;
        this.newColonizationRate = newColonizationRate;
        this.successorOfNewCompartment = successorOfNewCompartment;
    }

    @Override
    protected void execute(DiscreteCompartment compartment, double currentTime) throws MetastaseException{
    	    	
    	ContinuousCompartment newCompartment = compartment.transferCellToNewContinuousCompartment(currentTime, newName, newHostOrgan, newGrowthFunction, newColonizationRate);
		newCompartment.setSuccessor(successorOfNewCompartment);		
		
		//check, whether the new metastasis is dormant for some time before its starts to grow out
		if(!newCompartment.isDormant()){
		  	//if it is not dormant, create the first event, to start the metastatis
		    newCompartment.createFirstEvent(currentTime);
		  
		    //if the new compartment is dormant, the first event is created in the dormancy stop event
		}		
    }

    public void setSuccessorOfNewCompartment(DiscreteCompartment successorOfNewCompartment) {
        this.successorOfNewCompartment = successorOfNewCompartment;
    }

  
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

	@Override
	public String toString() {
		return "TransferToNewContinuousCompartment ["+super.toString() 
				+ ", newName=" + newName + ", newHostOrgan=" + newHostOrgan 
				+ ", newGrowthFunction=" + newGrowthFunction + ", newColonizationRate=" + newColonizationRate 
				+", successorOfNewCompartment="	+ (successorOfNewCompartment == null? "null": successorOfNewCompartment.getName()) + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 56;
		int result = super.hashCode();
		result = prime * result	+ newName.hashCode();
		result = prime * result	+ newHostOrgan.hashCode();
		result = prime * result	+ newGrowthFunction.hashCode();
		result = prime * result	+ ((newColonizationRate == null) ? 0 : newColonizationRate.hashCode());		
		//only use the name of the successor. otherwise we might end up in an endless loop (since the successor is a DiscreteCompartment, which also
		//can contain TransferToNew...EventActions)
		result = prime * result	+ ((successorOfNewCompartment == null) ? 0	: successorOfNewCompartment.getName().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		TransferToNewContinuousCompartment other = (TransferToNewContinuousCompartment) obj;
		
		if(successorOfNewCompartment == null && other.successorOfNewCompartment != null) return false;
		if(successorOfNewCompartment != null && other.successorOfNewCompartment == null) return false;
		
		return newName.equals(other.newName) &&
				newHostOrgan.equals(other.newHostOrgan) &&
				newGrowthFunction.equals(other.newGrowthFunction) &&
				((newColonizationRate == null)? other.newColonizationRate == null : newColonizationRate.equals(other.newColonizationRate)) &&
				//only check the name of the successor. otherwise we might end up in an endless loop (since the successor is a DiscreteCompartment, which also
				//can contain TransferToNew...EventActions)
				((successorOfNewCompartment == null)?other.successorOfNewCompartment == null : successorOfNewCompartment.getName().equals(other.successorOfNewCompartment.getName()));
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}
