/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;

/**
 * Transfers a cell from a discrete compartment to a new discrete one
 * e.g. when a cell leaves the bloodstream and forms a new metastasis
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class TransferToNewDiscreteCompartment extends CellProgressionEventAction {
    
	private DiscreteCompartment successorNewCompartment;
    private DiscreteCompartmentType newCompartmentType;
    private HostOrgan newHostOrgan;

    /**
     * @param name name of this action
     * @param generationInfo the generation informations for this event action
     * @param hostOrgan the organ the new compartment is situated in (e.g. lung, liver, blood stream)
     * @param nextCompartmentType the next DiscreteCompartmentType of the new discrete compartment
     * @param successorOfNewCompartment the successor of the new compartment (e.g. blood)
     */
    public TransferToNewDiscreteCompartment(String name, GenerationInfo generationInfo, HostOrgan newHostOrgan, 
    		DiscreteCompartmentType newCompartmentType, DiscreteCompartment successorNewCompartment) {
        super(name, generationInfo);
        this.newHostOrgan = newHostOrgan;
        this.successorNewCompartment = successorNewCompartment;
        this.newCompartmentType = newCompartmentType;
    }

    @Override
    protected void execute(DiscreteCompartment compartment, double currentTime) {
    	
		DiscreteCompartment newCompartment = compartment.transferCellToNewDiscreteCompartment(currentTime, newHostOrgan, newCompartmentType);
        newCompartment.setSuccessor(successorNewCompartment);
	        
        //check, whether the new metastasis is dormant for some time before its starts to grow out
        if(!newCompartment.isDormant()){        		
        	//if it is not dormant, create the first event, to start the metastatis
    		newCompartment.createFirstEvent(currentTime);
    		
    		//if the metastasis is dormant, the first event is created in the dormancy stop event
        }
    }

    public void setNewCompartmentType(DiscreteCompartmentType newCompartmentType) {
        this.newCompartmentType = newCompartmentType;
    }

    public void setSuccessorOfNewCompartment(DiscreteCompartment successorNewCompartment) {
        this.successorNewCompartment = successorNewCompartment;
    }

	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
	@Override
	public String toString() {
		return "TransferToNewDiscreteCompartment ["+super.toString()+", successorNewCompartment="
				+ (successorNewCompartment == null? "null": successorNewCompartment.getName()) 
				+ ", newCompartmentType=" + newCompartmentType.getName() + ", newHostOrgan=" + newHostOrgan + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 17;
		int result = super.hashCode();
		result = prime * result + newHostOrgan.hashCode();
		//only use the name of the compartment type and the successor. otherwise we might end up in an endless loop (since the compartment type and 
		//successor are DiscreteCompartments, which also can contain TransferToNew...EventActions)
		result = prime * result	+ newCompartmentType.getName().hashCode();		
		result = prime* result + ((successorNewCompartment == null) ? 0	: successorNewCompartment.getName().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		TransferToNewDiscreteCompartment other = (TransferToNewDiscreteCompartment) obj;
		
		return  newHostOrgan.equals(other.newHostOrgan) &&				
				//only check the name of the compartment type and the successor. otherwise we might end up in an endless loop (since the compartment type and 
				//successor are DiscreteCompartments, which also can contain TransferToNew...EventActions)
				(newCompartmentType.getName().equals(other.newCompartmentType.getName()) &&
				((successorNewCompartment == null)?	other.successorNewCompartment == null: successorNewCompartment.getName().equals(other.successorNewCompartment.getName()))
			);
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

}
