/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.LogFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;
import de.fhstralsund.catsit.support.functions.chemo.BasicChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.ConstantChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.ExponentialChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.GompertzChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.LinearChemoFunction;

/**
 * Holds all parameter for a chemotherapy.
 * 
 * @author Anja Bethge
 *
 */
public class ChemoTreatment extends Treatment {
	
	private double cyclingCellFraction;
	private double drugSensitivity;
	private double drugDecayRate;
	
	/**
	 * 
	 * @param cyclingCellFraction
	 * @param drugSensitivity Unit: 1/day
	 * @param drugDecayRate Unit: 1/day
	 */
	public ChemoTreatment(double cyclingCellFraction, double drugSensitivity, double drugDecayRate) {
		this.cyclingCellFraction = cyclingCellFraction;
		//convert from days to minutes
		//drugDecay is given in 1/day but is required in 1/min. Therefore we divide with 24 and 60
		this.drugDecayRate = drugDecayRate/60./24.;
		this.drugSensitivity = drugSensitivity/60./24.;		
	}


	public double getDrugDecayRate() {
		return drugDecayRate;
	}

	/**
	 * Returns the proper chemotherapy function for the given growth function.
	 * @param growthFunction
	 * @param numberOfCells the number of cells in the tumor right before the chemotherapy cycle starts
	 * @return the proper chemotherapy growth function for the given growth function
	 */
	public BasicChemoFunction getChemoFunction(Function growthFunction, long numberOfCells){
		
		//at the first chemo cycle the previous growth function is one of the "normal" growth functions
		///////////////////// Gompertz ////////////////////////////////////////////////////////////////////
		if(growthFunction instanceof GompertzFunction){
			GompertzFunction f = (GompertzFunction) growthFunction;
			return new GompertzChemoFunction(f.getA(), f.getB(), numberOfCells, cyclingCellFraction, drugDecayRate, drugSensitivity);
		
		///////////////////// Constant /////////////////////////////////////////////////////////////////////	
		}else if(growthFunction instanceof ConstantFunction){
			ConstantFunction f = (ConstantFunction)growthFunction;
			if(f.getReturnValue() == 0.0){
				//the tumour is already dead, so no more chemo required
				return null;
			}
			return new ConstantChemoFunction(numberOfCells, cyclingCellFraction, drugDecayRate, drugSensitivity);

		///////////////////// Linear /////////////////////////////////////////////////////////////////////	
		}else if(growthFunction instanceof LinearFunction){
			LinearFunction f = (LinearFunction)growthFunction;
			return new LinearChemoFunction(f.getM(), numberOfCells, cyclingCellFraction, drugDecayRate, drugSensitivity);
			
		///////////////////// Exponential ////////////////////////////////////////////////////////////////
		}else if(growthFunction instanceof ExponentialFunction){
			ExponentialFunction f = (ExponentialFunction)growthFunction;
			return new ExponentialChemoFunction(f.getB(), numberOfCells, cyclingCellFraction, drugDecayRate, drugSensitivity);
			
		}else if(growthFunction instanceof LogFunction || growthFunction instanceof PowerFunction){
			//no analytical solution available for Log- and PowerFunction
			return null;
		}
		return null;
	}

	

	@Override
	public String toString() {
		return "ChemoTreatment [cyclingCellFraction=" + cyclingCellFraction	+ ", drugSensitivity=" + drugSensitivity + ", drugDecayRate=" + drugDecayRate + "]";
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(cyclingCellFraction);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(drugDecayRate);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(drugSensitivity);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		
		ChemoTreatment other = (ChemoTreatment) obj;
		
		return (Double.doubleToLongBits(cyclingCellFraction) == Double.doubleToLongBits(other.cyclingCellFraction) &&
				Double.doubleToLongBits(drugDecayRate) == Double.doubleToLongBits(other.drugDecayRate) &&
				Double.doubleToLongBits(drugSensitivity) == Double.doubleToLongBits(other.drugSensitivity));			
	}	
}
