/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import de.fhstralsund.catsit.support.SpecialMath;
import de.fhstralsund.catsit.support.functions.SurvivalRate;

/**
 * Holds all parameter for the external beam radiation therapy and computes the number of surviving cells.
 * @author Anja Bethge
 *
 */
public class ExternalBeamTreatment extends RadiationTreatment{

	private double dose;
	
	/**
	 * 
	 * @param alpha radiosensitivity of the cells (probability for a DSB). Unit: 1/Gy
	 * @param beta radiosensitivity of the cells (probability for two SSB that lead to a DSB). Unit: 1/(Gy*Gy)
	 * @param dose administered dose. Unit: Gy
	 */
	public ExternalBeamTreatment(double alpha, double beta, double dose) {
		super(alpha, beta);
		this.dose = dose;
	}
	
	public long computeNumberOfSurvivingCells(long numberOfCellsBeforeTreatment){
		//compute the survival rate
		double survivalRate = new SurvivalRate(getAlpha(), getBeta()).f(dose);
		
		return SpecialMath.round(numberOfCellsBeforeTreatment * survivalRate);
	}	
	
	@Override
	public String toString() {
		return "ExternalBeamTreatment ["+super.toString()+", dose=" + dose + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(dose);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		ExternalBeamTreatment other = (ExternalBeamTreatment) obj;
		return (Double.doubleToLongBits(dose) == Double.doubleToLongBits(other.dose));
	}	
}
