/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

/**
 * Abstract base class for all internal radiation therapies such as radioembolization and radioimmunotherapy.
 * 
 * @author Anja Bethge
 *
 */
public abstract class InternalRadiationTreatment extends RadiationTreatment {

	private double activity;
	private double energy;
	private double halflife;
	
	/**
	 * 
	 * @param alpha radiosensitivity of the cells (probability for a DSB). Unit: 1/Gy
	 * @param beta radiosensitivity of the cells (probability for two SSB that lead to a DSB). Unit: 1/(Gy*Gy)
	 * @param activity the administered activity. Unit GBq
	 * @param energy the average energy per disintegration. Unit: MeV/(Bq*s)
	 * @param halflife the physical half life of the radionuclide. Unit: h
	 */
	public InternalRadiationTreatment(double alpha, double beta, double activity, double energy, double halflife) {
		super(alpha, beta);
		this.activity = activity;
		this.energy = energy;
		this.halflife = halflife;
	}

	public double getActivity() {
		return activity;
	}

	public double getEnergy() {
		return energy;
	}

	public double getHalflife() {
		return halflife;
	}

	
	
	@Override
	public String toString() {
		return super.toString()+", activity=" + activity + ", energy="+ energy + ", halflife=" + halflife;
	}

	@Override
	public int hashCode() {
		final int prime = 75;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(activity);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(energy);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(halflife);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		InternalRadiationTreatment other = (InternalRadiationTreatment) obj;
		return (Double.doubleToLongBits(activity) == Double.doubleToLongBits(other.activity) &&
				Double.doubleToLongBits(energy) == Double.doubleToLongBits(other.energy) &&
				Double.doubleToLongBits(halflife) == Double.doubleToLongBits(other.halflife));
	}
	
	
}
