/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.support.SpecialMath;
import de.fhstralsund.catsit.support.functions.SurvivalRate;
import de.fhstralsund.catsit.support.functions.dosimetry.MIRDMeanAbsorbedDose;

/**
 * Holds all parameter for the radioembolization therapy and computes the number of surviving cells.
 * @author Anja Bethge
 *
 */
public class RadioembolizationTreatment extends InternalRadiationTreatment {

	private HostOrgan treatedOrgan;
	private double minVolume;
	
	/**
	 * 
	 * @param alpha radiosensitivity of the cells (probability for a DSB). Unit: 1/Gy
	 * @param beta radiosensitivity of the cells (probability for two SSB that lead to a DSB). Unit: 1/(Gy*Gy)
	 * @param activity the administered activity. Unit GBq
	 * @param energy the average energy per disintegration. Unit: MeV/(Bq*s)
	 * @param halflife the physical half life of the radionuclide. Unit: h
	 * @param treatedOrgan the organ that is treated
	 * @param minVolume the minimum volume. only compartments that exceed the minimum volume are treated. 
	 * We assume that a tumor of 1 mm³ contains 1e6 cells (Iwata, 2001 bzw. Akanuma, 1978)
	 */
	public RadioembolizationTreatment(double alpha, double beta, double activity, double energy, double halflife,
			HostOrgan treatedOrgan, double minVolume) {
		super(alpha, beta, activity, energy, halflife);
		this.treatedOrgan = treatedOrgan;
		this.minVolume = minVolume;
	}

	public HostOrgan getTreatedOrgan() {
		return treatedOrgan;
	}

	public double getMinVolume() {
		return minVolume;
	}
	
	public long getMinSize() {
		//we assume that a tumor of 1 mm³ contains 1e6 cells (Iwata, 2001 bzw. Akanuma, 1978)
		return SpecialMath.round(minVolume * 1e6);
	}
	
	public long computeNumberOfSurvivingCells(long numberOfCellsBeforeTreatment){

		//compute the tissue mass
		double tissueMass = numberOfCellsBeforeTreatment * 1e-12; //one cells weighs 1e-12kg
		MIRDMeanAbsorbedDose dose = new MIRDMeanAbsorbedDose(getActivity(), getEnergy(), getHalflife(), tissueMass);
		
		//compute the survival rate
		double survivalRate = new SurvivalRate(getAlpha(), getBeta()).f(dose.getValue());
		
		return SpecialMath.round(numberOfCellsBeforeTreatment * survivalRate);
	}

	
	
	@Override
	public String toString() {
		return "RadioembolizationTreatment ["+super.toString()+", treatedOrgan=" + treatedOrgan	+ ", minVolume=" + minVolume + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 21;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(minVolume);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result	+ ((treatedOrgan == null) ? 0 : treatedOrgan.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		RadioembolizationTreatment other = (RadioembolizationTreatment) obj;
		return (Double.doubleToLongBits(minVolume) == Double.doubleToLongBits(other.minVolume) &&
				((treatedOrgan == null)?other.treatedOrgan == null:treatedOrgan.equals(other.treatedOrgan)));
	}	
}
