/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import de.fhstralsund.catsit.support.SpecialMath;
import de.fhstralsund.catsit.support.functions.SurvivalRate;
import de.fhstralsund.catsit.support.functions.dosimetry.MIRDMeanAbsorbedDose;

/**
 * Holds all parameter for the radioimmunotherapy and computes the number of surviving cells.
 * @author Anja Bethge
 *
 */
public class RadioimmunoTreatment extends InternalRadiationTreatment {
	
	private double biologicalHalflife;
	private double effectiveHalflife;
	private double clearanceFraction;
	private boolean excludeBloodStream;
	
	/**
	 * 
	 * @param alpha radiosensitivity of the cells (probability for a DSB). Unit: 1/Gy
	 * @param beta radiosensitivity of the cells (probability for two SSB that lead to a DSB). Unit: 1/(Gy*Gy)
	 * @param activity the administered activity. Unit GBq
	 * @param energy the average energy per disintegration. Unit: MeV/(Bq*s)
	 * @param halflife the physical half life of the radionuclide. Unit: h
	 * @param biologicalHalflife the biological half life of the radionuclide. Unit: h
	 * @param clearanceFraction the fraction of the administered activity that does not contribute to the treatment because it does not infiltrate the tumor.
	 * @param excludeBloodStream true, if cells in the bloodstream shall not be affected by the treatment
	 */
	public RadioimmunoTreatment(double alpha, double beta, double activity, double energy, double halflife, 
			double biologicalHalflife, double clearanceFraction, boolean excludeBloodStream) {
		super(alpha, beta, activity, energy, halflife);
		
		this.biologicalHalflife = biologicalHalflife;
		this.effectiveHalflife = (getHalflife() * biologicalHalflife) / (getHalflife() + biologicalHalflife);;
		this.clearanceFraction = clearanceFraction;
		this.excludeBloodStream = excludeBloodStream;
	}
	
	public RadioimmunoTreatment(double alpha, double beta, double activity, double energy, double halflife, double clearanceFraction, boolean excludeBloodStream) {
		super(alpha, beta, activity, energy, halflife);
		
		this.biologicalHalflife = Double.NaN;
		this.effectiveHalflife = getHalflife();
		this.clearanceFraction = clearanceFraction;
		this.excludeBloodStream = excludeBloodStream;
	}

	public double getClearanceFraction() {
		return clearanceFraction;
	}


	public boolean isExcludeBloodStream() {
		return excludeBloodStream;
	}
	
	public long computeNumberOfSurvivingCells(long numberOfCellsBeforeTreatment, long totalTumorBurden){
		//compute the effective activity
		double totalEffectiveActivity = (1.-clearanceFraction) * getActivity();
		//compute the activity in the tumor with respect to the size
		double activityInTumor = totalEffectiveActivity * ((double)numberOfCellsBeforeTreatment / (double)totalTumorBurden);
		
		//compute the tissue mass
		double tissueMass = numberOfCellsBeforeTreatment * 1e-12; //one cells weighs 1e-12kg
		
		MIRDMeanAbsorbedDose dose = new MIRDMeanAbsorbedDose(activityInTumor, getEnergy(), effectiveHalflife, tissueMass);
		
		//compute the survival rate
		double survivalRate = new SurvivalRate(getAlpha(), getBeta()).f(dose.getValue());
		
		return SpecialMath.round(numberOfCellsBeforeTreatment * survivalRate);
	}

	
	@Override
	public String toString() {
		return "RadioimmunoTreatment ["+super.toString()+", biologicalHalflife=" + biologicalHalflife +", effectiveHalflife=" + effectiveHalflife
				+ ", clearanceFraction=" + clearanceFraction + ", excludeBloodStream=" + excludeBloodStream + "]";
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(biologicalHalflife);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(clearanceFraction);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + (excludeBloodStream ? 1231 : 1237);
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;
		
		RadioimmunoTreatment other = (RadioimmunoTreatment) obj;
		
		return (Double.doubleToLongBits(biologicalHalflife) == Double.doubleToLongBits(other.biologicalHalflife) &&
				Double.doubleToLongBits(clearanceFraction) == Double.doubleToLongBits(other.clearanceFraction) &&
				excludeBloodStream == other.excludeBloodStream);
	}
	
	
}
