/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import java.util.Iterator;

import de.fhstralsund.catsit.base.treatments.TreatmentList.TreatmentEntry;

/**
 * The treatment list saves all treatments sorted by their time.
 * @author Anja Bethge
 *
 */
public class TreatmentList implements Iterable<TreatmentEntry>{
	
	public static class TreatmentEntry{
		public double time;
		public Treatment treatment;
		public TreatmentEntry next;
		
		public TreatmentEntry(double time, Treatment t, TreatmentEntry next){
			this.time = time;
			this.treatment = t;
			this.next = next;
		}		
	}
	
	
	private static TreatmentList theTreatmentList;	
	private TreatmentEntry firstTreatment;
	
	private TreatmentList(){}
	
	/**
	 * 
	 * @return the instance of the treatment list
	 */
	public static synchronized TreatmentList getInstance(){
		if (theTreatmentList == null){
			theTreatmentList = new TreatmentList();
		}
		return theTreatmentList;
	}

	/**
	 * Adds the given Treatment to the treatment list and sorts it into the list according to the given time
	 * @param treatment the treatment to add
	 */
	public void addTreatment(double time, Treatment treatment){
		
		if (time <= 0.) throw new IllegalArgumentException("The time must be bigger than 0 but was "+ time +".");		
		if (treatment == null) throw new IllegalArgumentException("The Treatment must no be null.");
		
		//convert from day to minutes
		double minTime = time *60.*24.;
		
		//check whether we already have a first treatment in the list
		if(firstTreatment == null) {
			firstTreatment = new TreatmentEntry(minTime, treatment, null);
			return;
		}
		
		//check wether the new treatment lies before the first treatment
		if(minTime < firstTreatment.time){
			TreatmentEntry newEntry = new TreatmentEntry(minTime, treatment, firstTreatment);
			firstTreatment = newEntry;
			return;
		}
		
		//find the right insert point 
		TreatmentEntry currentTreatment = firstTreatment;		
		while(true){
			if(currentTreatment.time <= minTime && (currentTreatment.next == null || currentTreatment.next.time > minTime)){
				//either no treatments are left in the list => add the new Treatment to the end
				//or the next treatment's time is bigger than the given time => sort the new treatment between the current and the next
				TreatmentEntry newEntry = new TreatmentEntry(minTime, treatment, currentTreatment.next);
				currentTreatment.next = newEntry;
				return;
				
			} else{
				//continue with the next section
				currentTreatment = currentTreatment.next;
			}
		}		
	}


	public Iterator<TreatmentEntry> iterator() {
		return new TreatmentIterator(firstTreatment);
	}
	
	/**
	 * only for tests
	 */
	public static void reset() {
		theTreatmentList = null;
	}
}
