/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.excel.merger;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.fhstralsund.catsit.WrongArgumentsException;
import de.fhstralsund.catsit.base.MetastaseException;

/**
 * The ExcelMerger can merge two excel 2007 files, written by the Apache POI.
 * With an surrounding bash/batch script (like runMultipleSims.bat) the merging
 * of multiple files is possible. The ExcelMerger can handle larger spreadsheets
 * without using the Apache POI and bypasses the memory requirements of the
 * Apache library, which made it impossible to open large files.
 * 
 * Steps performed by the ExcelMerger: 
 * - extract source and destination excel as zip 
 * - copy data-xml files from source to destination folder 
 * - register all data-xml files copied in the destination xml-configuration files (xl/workbook.xml , xl/_rels/workbook.xml.rels and [Content_Types].xml) 
 * - create the destination excel file
 * 
 * ExcelMerger destFile srcFile -> merges the two files and doesn't build the destination file * 
 * ExcelMerger destFile -> builds the destinationfile from tempDir
 * 
 * 
 * Restrictions: Only files written by the Apache POI can be used as input
 * files. When a template file from Microsoft Excel is needed, use the VBA macro
 * (mergeMakro.cls) directly from excel to import the data.
 * 
 * @author Sebastian Glöde
 */
public class ExcelMerger {

	private static final String USAGE_STRING = "usage:\n"
			+ "java -jar CaTSiT.jar merge destinationFile sourceFile\n"
			+ "or \n"
			+ "java -jar CaTSiT.jar zip destinationFile\n";

	final int BUFFER = 2048;
	private String destDir;

	int rIDMax = 0;
	private int fileIdMax;

	
	/**
	 * @param args
	 *            args[0] = destination excel file, args[1] = source excel file,
	 *            or if null, then the ExcelMerger only compresses the
	 *            destination excel file
	 * @throws Exception
	 */	
	public ExcelMerger(String[] args) throws MetastaseException {
		
		if (args.length < 1) {
			throw new WrongArgumentsException("Missing arguments. "+USAGE_STRING);
		}
		
		if (args.length > 2) {
			throw new WrongArgumentsException("Too many arguments. "+USAGE_STRING);
		}
		
		
		//get the parameter
		String src = null;
		if (args.length == 2) {
			src = args[1].replace("\"", ""); // remove quotes, if any
		}
		String dest = args[0].replace("\"", ""); // remove quotes, if any

		//set the action
		Actions a = null;
		if (src == null) {
			a = Actions.Compress;
		}else{
			a = Actions.Merge;
		}

		this.merge(a, dest, src);
	}
	
	public ExcelMerger(String destFile, String srcFile) throws ExcelMergerException{
		//set the action
		Actions a = null;
		if (srcFile == null) {
			a = Actions.Compress; 
		}else{
			a = Actions.Merge;
		}
		
		this.merge(a, destFile, srcFile);
	}

	/**
	 * deletes one folder an all child items
	 * 
	 * @param path
	 *            folder to delete
	 * @throws IOException
	 */
	private void doDelete(File path) throws ExcelMergerException {
		if (path.isDirectory()) {
			for (File child : path.listFiles()) {
				doDelete(child);
			}
		}
		if (!path.delete()) {
			throw new ExcelMergerException("Could not delete " + path);
		}
	}

	/**
	 * merges one excel file into another
	 * 
	 * @param action merge-action to use
	 * @param src source excel file to read from
	 * @param dest destination excel file to write
	 * @throws Exception
	 */
	private void merge(Actions a, String destFile, String srcFile) throws ExcelMergerException {
		
		//get file path without the file extension
		destDir = destFile.replaceAll(".xlsx", "");
		File destDirFile = new File(destDir);
		boolean destExists = destDirFile.exists();
		
		if (!destExists){destDirFile.mkdirs();}
		
		if (a == Actions.Merge) {

			//get file path without the file extension
			String srcDir = srcFile.replaceAll(".xlsx", "");
			File srcDirFile = new File(srcDir);
			
			//delete the directory if it already exists
			if (srcDirFile.exists()) {
				doDelete(srcDirFile);
			}
			srcDirFile.mkdirs();

			//extract the source excel file
			extractExcel(srcFile, srcDir);
			
			//if the destination excel file wasn't extracted yet (should only 
			//happen in the first simrun) => extract it
			if (!destExists) {
				extractExcel(destFile, destDir);
			}
			
			List<Sheet> srcSheets;
			List<Sheet> destSheets;
			try{
				// read all sheets
				srcSheets = readSheetFileNames(srcDir);
				destSheets = readSheetFileNames(destDir);
			}catch (Exception e) {
				throw new ExcelMergerException("Unable to read the sheets from the xml files.", e);
			}

			// copy sheets
			copySheetFiles(srcSheets, destSheets, srcDir, destDir);

			// modify xml in destination
			modifyXML(srcSheets, destSheets, destDir);
			
			//the extracted source excel file isn't needed any more
			//=> delete it
			doDelete(srcDirFile);
		}
		
		
		if (a == Actions.Compress) {
			compress(destFile, destDir);
			doDelete(destDirFile);
		}
	}

	/**
	 * compresses the excel file
	 * 
	 * @param destFile destination excel file to write
	 * @param destDir folder to read the destination data
	 */
	private void compress(String destFile, String destDir) {
		try {
			BufferedInputStream origin = null;
			FileOutputStream dest = new FileOutputStream(destFile);
			ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(
					dest));

			byte data[] = new byte[BUFFER];
			// get a list of files from current directory

			// Read all files
			File destFirFile = new File(destDir);
			Collection<File> files = FileUtils.listFiles(destFirFile, null,	true);

			// write all files into zip-stream
			for (File file : files) {
				if (file.isDirectory())
					continue;
				String entryName = file.getAbsolutePath().replace(
						destFirFile.getAbsolutePath() + File.separator, "");

				FileInputStream fi = new FileInputStream(file);
				origin = new BufferedInputStream(fi, BUFFER);

				ZipEntry entry = new ZipEntry(entryName);

				out.putNextEntry(entry);
				int count;
				while ((count = origin.read(data, 0, BUFFER)) != -1) {
					out.write(data, 0, count);
				}
				origin.close();
			}

			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * updates the xml-content-files
	 * 
	 * @param srcSheets sheet list to add
	 * @param destSheets sheets in destination file
	 * @param destDir destination excel directory
	 * @throws TransformerFactoryConfigurationError
	 * @throws TransformerException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	private void modifyXML(List<Sheet> srcSheets, List<Sheet> destSheets,
			String destDir) throws ExcelMergerException {

		try{
			// get the max sheet index
			int maxSheetIndex = 0;
			for (Sheet s : destSheets) {
	
				int sId = Integer.parseInt(s.getSheetId());
				if (sId > maxSheetIndex) {
					maxSheetIndex = sId;
				}
			}
	
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
	
			/*
			 * there are three xml to modify 1) the workbool.xml, the sheet name, id
			 * (for sorting) and rId (reference to data-xml) 2) workbook.xml.rels,
			 * the rId and the filename of the data-xml 3) Content_Types.xml, the
			 * filename of the data-xml and the sheet contenttype
			 */
	
			String workbookFilename = new File(destDir, "xl/workbook.xml")
					.getAbsolutePath();
			String relationsFilename = new File(destDir,
					"xl/_rels/workbook.xml.rels").getAbsolutePath();
	
			new File(destDir, "[Content_Types].xml").renameTo(new File(destDir,
					"Content_Types.xml")); // Fix, because DocumentBuilder don't
											// like [ in path
	
			String contentTypeFilename = new File(destDir, "Content_Types.xml")
					.getAbsolutePath();
	
			// Create document for each file
			Document domWorkbook = db.parse(workbookFilename);
			Document domRelationship = db.parse(relationsFilename);
			Document domContentType = db.parse(contentTypeFilename);
	
			// get the root node
			Element rootWorkbook = (Element) domWorkbook.getDocumentElement()
					.getElementsByTagName("sheets").item(0);
			Element rootRelationship = domRelationship.getDocumentElement();
			Element rootContentType = domContentType.getDocumentElement();
	
			// iterate all sheets to add
			for (Sheet s : srcSheets) {
				// workbook.xml
				Element sheet = domWorkbook.createElement("sheet");
				sheet.setAttribute("r:id", s.getrId());
				sheet.setAttribute("name", s.getName());
				sheet.setAttribute("sheetId", String.valueOf(++maxSheetIndex));
				rootWorkbook.appendChild(sheet);
	
				// relations.xml
				Element rel = domRelationship.createElement("Relationship");
				rel.setAttribute("Id", s.getrId());
				rel.setAttribute("Type","http://schemas.openxmlformats.org/office" +
						"Document/2006/relationships/worksheet");
				rel.setAttribute("Target", s.getFileName().replace("\\", "/")
						.replace("/worksheets", "worksheets"));
				rootRelationship.appendChild(rel);
	
				// contentype.xml
				Element type = domContentType.createElement("Override");
				type.setAttribute("PartName", FilenameUtils.normalize("/xl/"
						+ s.getFileName().replace("\\", "/"), true));
				type.setAttribute("ContentType","application/vnd.openxmlformats-" +
						"officedocument.spreadsheetml.worksheet+xml");
				rootContentType.appendChild(type);
			}
	
			// Use Transformer to write each xml file
			Transformer tr = TransformerFactory.newInstance().newTransformer();
			tr.setOutputProperty(OutputKeys.INDENT, "yes");
			tr.setOutputProperty(OutputKeys.METHOD, "xml");
			tr.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "3");
	
			FileOutputStream fileOutputStreamWorkbook = new FileOutputStream(
					workbookFilename);
			tr.transform(new DOMSource(domWorkbook), new StreamResult(
					fileOutputStreamWorkbook));
			FileOutputStream fileOutputStreamRelations = new FileOutputStream(
					relationsFilename);
			tr.transform(new DOMSource(domRelationship), new StreamResult(
					fileOutputStreamRelations));
			FileOutputStream fileOutputStreamContentType = new FileOutputStream(
					contentTypeFilename);
			tr.transform(new DOMSource(domContentType), new StreamResult(
					fileOutputStreamContentType));
	
			fileOutputStreamWorkbook.close();
			fileOutputStreamRelations.close();
			fileOutputStreamContentType.close();
			new File(destDir, "Content_Types.xml").renameTo(new File(destDir,
					"[Content_Types].xml"));
			
		}catch(Exception e){
			throw new ExcelMergerException("Unable to update xml-content-files.", e);
		}
	}

	/**
	 * copies the sheets from one directory to another
	 * 
	 * @param srcSheets sheet list to copy
	 * @param destSheets sheets in destination file
	 * @param src source excel directory
	 * @param dest destination excel directory
	 * @throws ParserConfigurationException
	 * @throws IOException
	 * @throws SAXException
	 * @throws TransformerFactoryConfigurationError 
	 * @throws TransformerException 
	 */
	private void copySheetFiles(List<Sheet> srcSheets, List<Sheet> destSheets,
			String src, String dest) throws ExcelMergerException {
		try{
			for (Sheet s : srcSheets) {
				int newRID = rIDMax + 1;
				rIDMax++;
				fileIdMax++;
				s.setrId("rId" + String.valueOf(newRID));
	
				File sheetFile = new File(FilenameUtils.concat(FilenameUtils
						.concat(src, "xl"), s.getFileName()));
				String destPrefix = FilenameUtils.concat(dest, "xl");
				File destSheetFile = new File(FilenameUtils.concat(FilenameUtils
						.concat(destPrefix, "worksheets"), "sheet"
						+ String.valueOf(fileIdMax) + ".xml"));
	
				sheetFile.renameTo(destSheetFile);
				s.setFileName(destSheetFile.getAbsolutePath().replace(destPrefix,""));
	
			}
		}catch (Exception e) {
			throw new ExcelMergerException("Unable to copy sheets", e);
		}
	}

	/**
	 * Reads the sheets from the xml files
	 * 
	 * @param srcDir extracted excel directory
	 * @return List of sheets found
	 * @throws SAXException
	 * @throws IOException
	 * @throws ParserConfigurationException
	 */
	private List<Sheet> readSheetFileNames(String srcDir) throws SAXException,
			IOException, ParserConfigurationException {
		List<Sheet> sheets = new ArrayList<Sheet>();

		// Read Workbook.xml
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();

		String workbookFilename = new File(srcDir, "xl/workbook.xml")
				.getAbsolutePath();
		Document dom = db.parse(workbookFilename);
		Element docEle = dom.getDocumentElement();

		// read sheets from workbook.xml
		NodeList nl = ((Element) docEle.getElementsByTagName("sheets").item(0))
				.getElementsByTagName("sheet");
		if (nl != null && nl.getLength() > 0) {
			for (int i = 0; i < nl.getLength(); i++) {
				Element el = (Element) nl.item(i);
				sheets.add(new Sheet(el.getAttribute("name"), el
						.getAttribute("r:id"), el.getAttribute("sheetId")));
			}
		}

		// read corresponding filenames
		String relationFilename = new File(srcDir, "xl/_rels/workbook.xml.rels")
				.getAbsolutePath();
		dom = db.parse(relationFilename);
		docEle = dom.getDocumentElement();
		nl = docEle.getElementsByTagName("Relationship");
		if (nl != null && nl.getLength() > 0) {
			for (int i = 0; i < nl.getLength(); i++) {
				Element el = (Element) nl.item(i);
				String rId = el.getAttribute("Id");
				for (Sheet s : sheets) {
					if (s.getrId().equals(rId)) {
						s.setFileName(el.getAttribute("Target"));
						break;
					}
				}

				// save the highest rId from destination xml
				if (srcDir == destDir) {
					int rIdInt = Integer.parseInt(rId.replaceAll("rId", ""));
					if (rIDMax < rIdInt)
						rIDMax = rIdInt;

					try {
						// Max-Sheet-filename
						String fileIdString = el.getAttribute("Target")
								.replace("worksheets/sheet", "").replace(
										".xml", "");

						int fileId = Integer.parseInt(fileIdString);

						if (fileIdMax < fileId)
							fileIdMax = fileId;
					} catch (NumberFormatException e) {
						// illegal string found, we can ignore this
					}
				}
			}
		}
		return sheets;
	}

	/**
	 * extracts an excelfile to directory
	 * 
	 * @param excelFile excel file to extract
	 * @param toDir destination folder
	 */
	private void extractExcel(String excelFile, String toDir) throws ExcelMergerException{
		try {
			BufferedOutputStream dest = null;
			FileInputStream fis = new FileInputStream(excelFile);
			ZipInputStream zis = new ZipInputStream(new BufferedInputStream(fis));
			ZipEntry entry;
			while ((entry = zis.getNextEntry()) != null) {
				int count;
				byte data[] = new byte[BUFFER];
				String dirName = entry.getName();
				if (dirName.contains("\\")) {
					dirName = dirName.substring(0, dirName.lastIndexOf("\\"));
				}
				if (dirName.contains("/")) {
					dirName = dirName.substring(0, dirName.lastIndexOf("/"));
				}
				if (!dirName.equals(entry.getName())) {
					File entryFile = new File(toDir, dirName);
					entryFile.mkdirs(); // create directories
				}

				if (entry.isDirectory()){
					continue;
				}
				
				// write file
				FileOutputStream fos = new FileOutputStream(new File(toDir,
						entry.getName()).getAbsolutePath());
				dest = new BufferedOutputStream(fos, BUFFER);
				while ((count = zis.read(data, 0, BUFFER)) != -1) {
					dest.write(data, 0, count);
				}
				dest.flush();
				dest.close();
			}
			zis.close();
		} catch (FileNotFoundException e) {
			throw new ExcelMergerException(e);
		} catch (IOException e) {
			throw new ExcelMergerException(e);
		}
	}
}
