/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.excel.writer;

import de.fhstralsund.catsit.support.LogSampleHistogram;

/**
 * Class to bundle status information of the simulated system.
 * @author Sebastian Glöde, Anja Bethge
 * 
 */
public class ExcelRecordMain implements ExcelRecord{
	/**
	 * the event time.
	 */
	private long time;
	/**
	 * the time in days.
	 */
	private double timeInDays;
	/**
	 * the number of cells in primary tumor.
	 */
	private long cellsInPrimaryTumor;
	/**
	 * the number of cells in blood stream.
	 */
	private long cellsInBloodStream;
	/**
	 * the number of metastases.
	 */
	private long numberOfMetastases;
	/**
	 * the number of cells in metastases.
	 */
	private long cellsInAllMetastases;
	/**
	 * the histogram.
	 */
	private LogSampleHistogram histogram;

	/**
	 * @param time the event time (in minutes)
	 * @param timeInDays the time in days
	 * @param cellsInPrimaryTumor the number of cells in the primary tumor.
	 * @param cellsInBloodStream the number of cells in the blood stream.
	 * @param numberMetastasis the number of all metastases.
	 * @param cellsInMetastasis the number of cells in all metastases.
	 * @param histogram the size histogram of all metastases
	 */
	public ExcelRecordMain(long time, double timeInDays, long cellsInPrimaryTumor, long cellsInBloodStream,
			long numberMetastases, long cellsInMetastases, LogSampleHistogram histogram) {

		if (time < 0 || timeInDays < 0 || cellsInPrimaryTumor < 0 || cellsInBloodStream < 0 || numberMetastases < 0	|| cellsInMetastases < 0) {
			throw new IllegalArgumentException("At least one argument is negative");
		}

		this.time = time;
		this.timeInDays = timeInDays;
		this.cellsInPrimaryTumor = cellsInPrimaryTumor;		
		this.cellsInBloodStream = cellsInBloodStream;
		this.numberOfMetastases = numberMetastases;
		this.cellsInAllMetastases = cellsInMetastases;
		this.histogram = histogram;
	}
	
	/**
	 * @return the histogram
	 */
	public LogSampleHistogram getHistogram() {
		return histogram;
	}

	/**
	 * @return the time
	 */
	public long getTime() {
		return time;
	}

	/**
	 * @return the timeInDays
	 */
	public double getTimeInDays() {
		return timeInDays;
	}

	/**
	 * @return the primaryTumorCells
	 */
	public long getCellsInPrimaryTumor() {
		return cellsInPrimaryTumor;
	}

	/**
	 * @return the bloodStreamCells
	 */
	public long getCellsInBloodStream() {
		return cellsInBloodStream;
	}

	/**
	 * @return the numberMetastasis
	 */
	public long getNumberMetastases() {
		return numberOfMetastases;
	}

	/**
	 * @return the cellsInMetastasis
	 */
	public long getCellsInMetastases() {
		return cellsInAllMetastases;
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result	+ (int) (cellsInBloodStream ^ (cellsInBloodStream >>> 32));
		result = prime * result	+ (int) (cellsInAllMetastases ^ (cellsInAllMetastases >>> 32));
		result = prime * result	+ (int) (cellsInPrimaryTumor ^ (cellsInPrimaryTumor >>> 32));
		result = prime * result	+ ((histogram == null) ? 0 : histogram.hashCode());
		result = prime * result	+ (int) (numberOfMetastases ^ (numberOfMetastases >>> 32));
		result = prime * result + (int) (time ^ (time >>> 32));
		long temp;
		temp = Double.doubleToLongBits(timeInDays);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (this.getClass() != obj.getClass()) return false;
		
		ExcelRecordMain other = (ExcelRecordMain) obj;
		return (time == other.time && 
				timeInDays == other.timeInDays && 
				cellsInPrimaryTumor == other.cellsInPrimaryTumor && 
				cellsInBloodStream == other.cellsInBloodStream && 
				numberOfMetastases == other.numberOfMetastases &&
				cellsInAllMetastases == other.cellsInAllMetastases &&
				((histogram == null)? other.histogram == null : histogram.equals(other.histogram)));		
	}
	
}
