/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.excel.writer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.support.HistogramEntry;

/**
 * Class for writing simulation data with histograms into an excel file (xlsx).
 * 
 * @author Sebastian Glöde
 * 
 */
public class ExcelWriter {

	private static final Logger logger = Logger.getLogger("metastase.io");
	
	/**
	 * the internal buffer of excel-data.
	 */
	private List<ExcelRecordMain> mainBuffer;
	
	/**
	 * the internal buffer of excel-data for all host organs that shall be printed.
	 */
	private Map<String, List<ExcelRecordSingleHostOrgan>> hostOrganBuffers;
	
	/**
	 * Number of header elements
	 */
	private int numberOfHeaderElements;

	/**
	 * Initializes the ExcelWriter.
	 * 
	 * @param hostOrgans the list of existing host organs
	 * @throws Exception
	 */
	public ExcelWriter(List<HostOrgan> hostOrgans) {
		if (hostOrgans == null) {
			throw new IllegalArgumentException("hostOrgans must not be null");
		}

		if (hostOrgans.isEmpty()) {
			throw new IllegalArgumentException("hostOrgans must not be empty");
		}
		
		this.mainBuffer = new ArrayList<ExcelRecordMain>();
		this.hostOrganBuffers = new HashMap<String, List<ExcelRecordSingleHostOrgan>>();
		
		for (HostOrgan organ : hostOrgans) {
			if(organ.isPrintFileForHostOrgan()){
				this.hostOrganBuffers.put(organ.getName(), new ArrayList<ExcelRecordSingleHostOrgan>());
			}
		}
	}

	/**
	 * Buffers one record for writing.
	 * 
	 * @param record  the record to buffer
	 */
	public void bufferMainRecord(ExcelRecordMain record) {
		mainBuffer.add(record);
	}
	
	/**
	 * Buffers the record for the given host organ
	 * @param organName the name of the host organ
	 * @param record the record to buffer
	 */
	public void bufferHostOrganRecord(String organName, ExcelRecordSingleHostOrgan record){
		hostOrganBuffers.get(organName).add(record);
	}
	
	
	public int getMainBufferSize(){
		return mainBuffer.size();
	}
	
	public int getOrganBufferSize(String organName){
		return hostOrganBuffers.get(organName).size();
	}
	

	/**
	 * true, if the sheetName isn't in use or the file doesn't exists.
	 * 
	 * @param sheetName the name of the sheet
	 * @return true, if sheetname isn't in use or the file doesn't exists, otherwise false
	 * @throws IOException the excel file could not been read
	 * @throws FileNotFoundException the excel file was not found
	 */
	public boolean isSheetNameAvailable(String filePath, String sheetName) throws FileNotFoundException, IOException {
		File mainFile = new File(filePath);
		
		//get the file path without the extension
		String filePathNoExtension = mainFile.getAbsolutePath().substring(0, mainFile.getAbsolutePath().lastIndexOf('.'));
		
		
		boolean available = true;
		
		//check all host organ files
		for (String hostOrgan : hostOrganBuffers.keySet()) {
			File excelFile = new File(filePathNoExtension+"_"+hostOrgan+".xlsx");
			if (excelFile.exists()) {
				Workbook workbook = new XSSFWorkbook(new FileInputStream(excelFile));
				if( workbook.getSheet(sheetName) != null)
					available = false;
			} 
		}
		
		//check main file		
		if (mainFile.exists()) {
			Workbook workbook = new XSSFWorkbook(new FileInputStream(mainFile));
			if( workbook.getSheet(sheetName) != null)
				available = false;
		}
		
		return available;
	}

	private void checkFile(String fileName){
		if (fileName == null) {
			throw new IllegalArgumentException("fileName must not be null");
		}

		if (fileName.isEmpty()) {
			throw new IllegalArgumentException("fileName must not be empty");
		}

		if (!fileName.endsWith(".xlsx")) {
			throw new IllegalArgumentException("Wrong fileextention");
		}
	}
	

	/**
	 * Writes all buffered data into the given file and sheet
	 * @param filePath file path of the excel file
	 * @param sheetName the sheet name
	 * @throws ExcelWriterException
	 */
	public void writeAllBuffers(String filePath, String sheetName) throws ExcelWriterException{
		checkFile(filePath);		
		File mainFile = new File(filePath);
		//get the file path without the extension
		String filePathNoExtension = mainFile.getAbsolutePath().substring(0, mainFile.getAbsolutePath().lastIndexOf('.'));
		//write the main file
		writeBuffer(filePath, sheetName, mainBuffer, true);
		
		//write all host organ files
		for (String hostOrgan : hostOrganBuffers.keySet()) {
			writeBuffer(filePathNoExtension+"_"+hostOrgan+".xlsx", sheetName, hostOrganBuffers.get(hostOrgan), false);
		}
	}
	
	/**
	 * writes the buffer into the excel-file.
	 * @param fileName the name of the file
	 * @param sheetName name of sheet to write
	 * @param buffer list of buffers to write
	 * @param true, if this is the main file or false, if a host organ shall be written
	 * @throws ExcelWriterException buffer data invalid
	 */
	private void writeBuffer(String fileName, String sheetName, List<? extends ExcelRecord> buffer, boolean mainFile) throws ExcelWriterException{
		checkFile(fileName);
		
		File file = new File(fileName);
		OutputStream outStream = null;
		
		try {
			logger.info("opening/creating workbook");
			Workbook workbook;
			if (file.exists()) {
				workbook = new XSSFWorkbook(new FileInputStream(file));
			} else {
				workbook = new XSSFWorkbook();
			}
	
			logger.info("creating sheet");
			if (workbook.getSheet(sheetName) != null) {
				throw new IllegalArgumentException("sheetname allready in use");
			}
			Sheet sheet = workbook.createSheet(sheetName);
			logger.info("fixing the header");
			sheet.createFreezePane(0, 3, 0, 3); //Fixes/Freezes the header
	
			logger.info("writing header");
			writeHeader(sheet, buffer, mainFile);
			logger.info("writing data");
			writeBody(sheet, buffer, mainFile);
			
			// write the file
			
			logger.info("writing file");
			outStream = new FileOutputStream(file);
			workbook.write(outStream);
			
		}catch(Exception e){
			logger.warning(e.getMessage());
			if(e instanceof ExcelWriterException){
				throw (ExcelWriterException)e;
			}else{
				throw new ExcelWriterException(e);
			}
			
		} finally {
			if (outStream != null) {
				try {
					outStream.close();
				} catch (IOException e) {
					throw new ExcelWriterException(e);
				}
			}
		}
		logger.info("finished writing into excel file");
		buffer.clear();
	}
	
	
	/**
	 * Computes the number of histogram-entries.
	 * 
	 * @return the number
	 * @throws ExcelWriterException 
	 */
	private int checkNumberOfHistogramEntries(List<? extends ExcelRecord> buffer) throws ExcelWriterException {
		
		if( buffer.size() == 0) {
			throw new ExcelWriterException("No records in buffer");
		}
		int comparisionValue = buffer.get(0).getHistogram().getBuckets().size();
		
		
		for (ExcelRecord entry : buffer) {
			if (entry.getHistogram() == null) {
				throw new ExcelWriterException("No histogram found for current entry ("+entry.getTimeInDays()+")");
			}
			int size = entry.getHistogram().getBuckets().size();
			if (comparisionValue != size) {
				throw new ExcelWriterException("Histogram invalid");
			}
		}
		return comparisionValue;
	}

	/**
	 * writes the data.
	 * @param sheet name of sheet to write
	 * @param numberHistogramEntries number of histogram-entries to write
	 * @throws ExcelWriterException 
	 */
	private void writeBody(Sheet sheet, List<? extends ExcelRecord> buffer, boolean mainFile) throws ExcelWriterException {
		int lastRowIndex = sheet.getLastRowNum() + 1;
		if (lastRowIndex != 3) {
			throw new ExcelWriterException("lastRowIndex should be three. Did you forgot to call writeHeader() first?");
		} // the first data-row starts at index 3

		ArrayList<Double> data = new ArrayList<Double>();
		for (ExcelRecord entry : buffer) {
			Row row = sheet.createRow(lastRowIndex++);

			data.clear();
			data.add((double) entry.getTime());
			data.add((double) entry.getTimeInDays());
			if(mainFile){
				data.add((double) ((ExcelRecordMain)entry).getCellsInPrimaryTumor());
				data.add((double) ((ExcelRecordMain)entry).getCellsInBloodStream());
			}
			data.add((double) entry.getNumberMetastases());
			data.add((double) entry.getCellsInMetastases());

			if (entry.getHistogram() != null) {
				for (HistogramEntry he : entry.getHistogram().getBuckets()) {
					data.add((double) he.getUpperLimit());
					data.add((double) he.getN());
				}
			}

			if(numberOfHeaderElements != data.size()) {
				throw new ExcelWriterException("data doesn't fit the header");
			}
			
			for (int cellNumber = 0; cellNumber < numberOfHeaderElements; cellNumber++) {
				Cell cell = row.createCell(cellNumber, Cell.CELL_TYPE_NUMERIC);
				cell.setCellValue(data.get(cellNumber));
			}
		}
	}

	/**
	 * Writes the header into the sheet.
	 * 
	 * @param sheet the sheet to write
	 * @param numberHistogramEntries number of histogram-entries
	 * @param mainFile true, it is the main data file, false otherwise
	 */
	private void writeHeader(Sheet sheet, List<? extends ExcelRecord> buffer, boolean mainFile) throws ExcelWriterException{
		int numberHistogramEntries = checkNumberOfHistogramEntries(buffer);
		List<ExcelHeaderEntry> headers = createHeaderEntryList(numberHistogramEntries, mainFile);

		Row rowTitle = sheet.createRow(0);
		Row rowUnit = sheet.createRow(1);
		Row rowComment = sheet.createRow(2);

		for (int cellNumber = 0; cellNumber < numberOfHeaderElements; cellNumber++) {
			Cell cell = rowTitle.createCell(cellNumber, Cell.CELL_TYPE_STRING); // title
			cell.setCellValue(headers.get(cellNumber).getTitle());

			cell = rowUnit.createCell(cellNumber, Cell.CELL_TYPE_STRING); // unit
			cell.setCellValue(headers.get(cellNumber).getUnit());

			cell = rowComment.createCell(cellNumber, Cell.CELL_TYPE_STRING); // comment
			cell.setCellValue(headers.get(cellNumber).getComment());
		}
	}

	/**
	 * @param numberHistogramEntries number of histogram entries to write
	 * @param mainFile true, it is the main data file, false otherwise
	 */
	private List<ExcelHeaderEntry> createHeaderEntryList(int numberHistogramEntries, boolean mainFile) {
		List<ExcelHeaderEntry> headers = new ArrayList<ExcelHeaderEntry>();

		headers.add(new ExcelHeaderEntry("time", "minutes", ""));
		headers.add(new ExcelHeaderEntry("time", "days", ""));
		
		if(mainFile){
			//this data is only needed in the main file
			headers.add(new ExcelHeaderEntry("# cells primary", "", ""));
			headers.add(new ExcelHeaderEntry("# cells blood stream", "", ""));
		}
		headers.add(new ExcelHeaderEntry("# metastases", "", ""));
		headers.add(new ExcelHeaderEntry("# cells all metastases", "", ""));

		for (int i = 0; i < numberHistogramEntries; i++) {
			headers.add(new ExcelHeaderEntry("uL_" + String.valueOf(i), "", ""));
			headers.add(new ExcelHeaderEntry("n_" + String.valueOf(i), "", ""));
		}
		numberOfHeaderElements = headers.size();
		
		return headers;
	}

}
