/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.conditions.Condition;
import de.fhstralsund.catsit.base.conditions.SizeCondition;
import de.fhstralsund.catsit.base.conditions.TimeCondition;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.base.treatments.ResectionTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.io.xml.ReferenceMap.RefType;
import de.fhstralsund.catsit.io.xml.builder.CompartmentTypeBuilder;
import de.fhstralsund.catsit.io.xml.builder.DOMHelper;
import de.fhstralsund.catsit.io.xml.builder.DistributionBuilder;
import de.fhstralsund.catsit.io.xml.builder.FunctionBuilder;
import de.fhstralsund.catsit.io.xml.builder.HostOrganBuilder;
import de.fhstralsund.catsit.io.xml.builder.GlobalTreatmentBuilder;
import de.fhstralsund.catsit.machine.Technical;
import de.fhstralsund.catsit.support.SpecialMath;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.LogFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;


/**
 * This is the implementation of the main parser to read the simulation setup
 * from the xml file. It's recommend to use the interface {@link MPMLParser}
 * instead of this class. This class should be used only for test cases.
 * 
 * @author Michael Suhr, Anja Bethge
 * 
 */
public class MPMLParserImpl implements MPMLParser {
    private static final Logger logger = Logger.getLogger("metastase.support.xml");
   
    //public -> necessary for unit tests
    public final ObjectBuffer maps = new ObjectBuffer();

    private String startCompartment = "";
    public List<Treatment> treatments = new ArrayList<Treatment>();
    public CONDITION_TYPE type = CONDITION_TYPE.UNKNOWN;

    private enum CONDITION_TYPE {
        TIMECONDITION, SIZECONDITION, UNKNOWN
    }

    private URL schemaLoc = ClassLoader.getSystemClassLoader().getResource("xsd/catsit.xsd");
    ParserErrorHandler errHandler = new ParserErrorHandler();

    public MPMLParserImpl() { 
    	//nothing to do here
    }

    /**
     * Creates the DocumentBuilder, adds the Schema and returns the Document Element.
     * @param filename
     * @return the root element of the given xml file
     * @throws ParserException if something goes wrong while getting the root element
     */
    private Element getRootElement(String filename) throws ParserException{
    	File source = new File(filename);
    	
    	try {
            DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
            builderFactory.setNamespaceAware(true);

            SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            
            Schema schema = schemaFactory.newSchema(schemaLoc);
            builderFactory.setSchema(schema);

            DocumentBuilder builder = builderFactory.newDocumentBuilder();
            builder.setErrorHandler(errHandler);
            
            /* convert DOM into domain classes */
            Document doc = builder.parse(source);
            if (errHandler.errors() != 0) {
                throw new ParserException(errHandler.getErrors());
            }
            return doc.getDocumentElement();
        
    	} catch (ParserException pe) {
            logger.warning(pe.getMessage());
            throw pe;
        } catch (Exception e) {
            logger.log(Level.WARNING, e.getMessage(), e);
            throw new ParserException("Couldn't parse the file: " + source.getPath() +": "+  e.getMessage());
        }
    }
    
    /**
     * Returns the list of all configured host organs. Should not be called before getCompartment().
     */
    public List<HostOrgan> getHostOrgans() throws ParserException {
    	List<HostOrgan> organs = maps.getHostOrganList();
    	
    	if(organs.isEmpty()){
    		throw new ParserException("No host organs found. Did you call getCompartment(), before you called this method?");
    	}
    	return organs;
    }
    

    /**
     * Returns the technical informations
     */
	public Technical getTechnical(String filename) throws ParserException {

		Element element = getRootElement(filename);
		
		if (DOMHelper.existsTag(element, "technical")) {
			Element technicalElement = null;
			if ("technical".equals(element.getTagName())) {
				technicalElement = element;
			} else {
				technicalElement = (Element) element.getElementsByTagName("technical").item(0);
			}
			long duration = DOMHelper.getElementLong(technicalElement,"duration", 0l);
			double interval = DOMHelper.getElementDouble(technicalElement,"timeStepsForSaving", 0d);
			boolean debugmode = DOMHelper.getElementBoolean(technicalElement,"debug-mode", false);
			
			return new Technical(duration, interval, debugmode);
		}
		throw new ParserException("Could not parse technical information (timeStepsForSaving, duration and debugmode).\n"
						+ "No 'technical'-element was found in the configuration file");
	}

    
	/**
     * Parses the whole file and creates the all objects.
     * Returns the primary tumor compartment
     * @param filename the {@link File} of the configuration
     * @see MPMLParser
     */
    public Compartment getCompartment(String filename) throws ParserException {
    	
		try {
			// get the root element
			Element element = getRootElement(filename);
			NodeList nl = element.getChildNodes();
			
			Element compartments = null;

			for (int i = 0; i < nl.getLength(); i++) {

				Node block = nl.item(i);

				if (block instanceof Element) {
					Element elem = (Element) block;
					String blockType = block.getNodeName();
					
					//parse all the functions
					if ("functions".equals(blockType)) {
						logger.fine("generate functions");
						NodeList functionNodes = elem.getChildNodes();
						for (int j = 0; j < functionNodes.getLength(); j++) {
							FunctionBuilder.parse(functionNodes.item(j), maps);
						}
					
					//parse all the distributions
					} else if ("distributions".equals(blockType)) {
						logger.fine("generate distributions");
						NodeList distributionNodes = elem.getChildNodes();
						for (int j = 0; j < distributionNodes.getLength(); j++) {
							DistributionBuilder.parse(distributionNodes.item(j), maps);
						}
					
					//parse all the host organs
					} else if ("hostOrgans".equals(blockType)) {
						logger.fine("generate host organs");
						NodeList hostOrganNodes = elem.getChildNodes();
						for (int j = 0; j < hostOrganNodes.getLength(); j++) {
							HostOrganBuilder.parse(hostOrganNodes.item(j), maps);
						}

					//parse all the compartment types
					} else if ("compartmentTypes".equals(blockType)) {
						logger.fine("generate compartment types");
						NodeList typeNodes = elem.getChildNodes();
						for (int j = 0; j < typeNodes.getLength(); j++) {
							try {
								CompartmentTypeBuilder.parse(typeNodes.item(j),	maps);
							} catch (ParserException pe) {
								throw new ParserException("An error occured while creating a discrete " +
										"compartment type: " + pe.getMessage(), pe);
							}
						}

					//save the compartments block for later parsing
					} else if ("compartments".equals(blockType)) {
						//compartments are parsed at the end, when all other elements have been parsed
						compartments = elem;
						
					//parse all global treatments
					} else if ("globalTreatments".equals(blockType)) {
						logger.fine("generate global treatments");
						NodeList treatmentNodes = elem.getChildNodes();
						
						//parse each stated systemic treatment
						for (int j = 0; j < treatmentNodes.getLength(); j++) {
							GlobalTreatmentBuilder.parse(treatmentNodes.item(j), maps);
						}

					}
				}
			}
			
			//know you can parse the compartments
			logger.fine("generate compartments");
			Compartment currentCompartment;
			NodeList compartmentNodes = compartments.getChildNodes();
			for (int j = 0; j < compartmentNodes.getLength(); j++) {
				Node compartmentNode = compartmentNodes.item(j);
				String compartmentType = compartmentNode.getNodeName(); //get the name of the type
				currentCompartment = null;
				
				if (compartmentNode instanceof Element) {
					Element compartment = (Element) compartmentNode;
					//parse continuous compartment
					if ("continuousCompartment".equals(compartmentType)) {
						logger.fine("generate continuous compartment");
						currentCompartment = continuousCompartment(compartment);
					
					//parse discrete compartment
					} else if ("discreteCompartment".equals(compartmentType)) {
						logger.fine("generate discrete compartment");
						currentCompartment = discreteCompartment(compartment);						
					}
					
					// use the compartmentID id name as key for the map
					String compartmentID = compartment.getAttribute("compartmentID");
					maps.putCompartment(compartmentID, currentCompartment);
					
					//check, whether this compartment is the start compartment (the primary tumor)					
					if (DOMHelper.getElementBoolean(compartment, "isPrimaryTumor", false)) {
						//save the id of the primary tumor
						startCompartment = compartmentID;
					}
				}
			}
			//resolve all open references
			resolveRefs();
			if (maps.getUnreferencedSize() != 0) {
				logger.warning("Couldn't resolve all references.");
			}
			
			return maps.getCompartment(startCompartment);

		} catch (Exception e) {
			logger.log(Level.WARNING, e.getMessage(), e);
			throw new ParserException("Couldn't parse the file: " + filename + "\n" + e.getMessage(), e);
		}
    }

    public Compartment discreteCompartment(Element node) throws ParserException {
        if (!DOMHelper.existsTag(node, "discreteCompartment")) return null;
                
        //get the compartment type id and get the associated compartment type from the compartment types map
        String typeRef = DOMHelper.getElementAttribute(node, "discreteCompartmentType", "typeRef");
        DiscreteCompartmentType compartmentType = (DiscreteCompartmentType) maps.getCompartmentType(typeRef);
        if (compartmentType == null) {
        	//Whoops! Where is the associated compartment type?
            throw new ParserException("Could not find the DiscreteCompartmentType with id: " + typeRef + " Maybe it's a misspelling?");
        }
        
        boolean isPrimaryTumor = DOMHelper.getElementBoolean(node, "isPrimaryTumor", false);
        long startSize = DOMHelper.getElementLong(node, "startSize", 0);
        //a start size bigger than 1 can only be specified for the primary tumor
        if(startSize > 1 && !isPrimaryTumor){
        	throw new ParserException("A start size bigger than 1 can only be specified for the primary tumor. The primary tumor has to be taged by the " +
        			"tag '<isPrimaryTumor>true</isPrimaryTumor>'.");
        }
        
        //check whether the tumor shall be resected at some point in the simulation
        double resectionTime = DOMHelper.getElementDouble(node, "resectionTime", 0.);
        if(resectionTime > 0){        	
        	//if so, check whether some cells shall remain in the tissue
        	double remnantFraction = DOMHelper.getElementDouble(node, "remnantFraction", 0.);
        	
        	TreatmentList.getInstance().addTreatment(resectionTime, new ResectionTreatment(remnantFraction));
        }
        
        //create the compartment
        DiscreteCompartment compartment = new DiscreteCompartment(0, getHostOrgan(node, maps), compartmentType, startSize, isPrimaryTumor);
		        
        DiscreteCompartment successor = getSuccessor(node, compartment);
        compartment.setSuccessor(successor);
        
        addTreatments(node, compartment);
        
        return compartment;
    }

    public void addTreatments(Element node, DiscreteCompartment compartment) throws ParserException {
        if (!DOMHelper.existsTag(node, "localTreatments"))
            return;

        NodeList treatmentsNode = null;
        if ("localTreatments".equals(node.getTagName())) {
            treatmentsNode = node.getChildNodes();
        } else {
            treatmentsNode = node.getElementsByTagName("localTreatments").item(0).getChildNodes();
        }
        for (int i = 0; i < treatmentsNode.getLength(); i++) {
            Node treatmentNode = treatmentsNode.item(i);
            if (treatmentNode instanceof Element) {
                logger.fine("generate one local treatment");
                Condition start = null;
                Condition end = null;
                DiscreteCompartmentType compartmentType = null;
                Element treatment = (Element) treatmentNode;
                NodeList conditionsNode = treatment.getChildNodes();
                for (int j = 0; j < conditionsNode.getLength(); j++) {
                    Node conditionNode = conditionsNode.item(j);
                    String conditionType = conditionNode.getNodeName();
                    if (conditionNode instanceof Element) {
                        Element type = (Element) conditionNode;
                        if ("start".equals(conditionType)) {
                            logger.fine("generate start conditon");
                            start = getCondition(type);
                        } else if ("end".equals(conditionType)) {
                            logger.fine("generate end condition");
                            end = getCondition(type);
                        } else if ("compartmentType".equals(conditionType)) {
                            logger.fine("get compartment type for treatment");
                            String typeRef = type.getAttribute("typeRef");
                            compartmentType = (DiscreteCompartmentType) maps.getCompartmentType(typeRef);
                        }
                    }
                }
                // Treatment zusammensetzen
                if (start == null) {
                    throw new ParserException("No start for local treatment found.");
                }
                if (compartmentType == null) {
                    throw new ParserException("No new compartment type for local treatment found.");
                }
                if (end != null) {
                    double time = 0d;
                    long size = 0l;
                    if (start instanceof TimeCondition) {
                        time = getConditionValue(start);
                    } else if (start instanceof SizeCondition) {
                        size = (long) getConditionValue(start);
                    }
                    if (end.isConditionTrue(time, size)) {
                        throw new ParserException("Invalid <end> in treatment.\n" + "<start>" + start + "</start>"
                                + "<end>" + end + "</end>");
                    }
                }
                treatments.add(new Treatment(start, end, compartmentType));
            }
        }
        
        // all treatments parsed. Check for consistence
        checkConsistentOfTreatments();
        DiscreteCompartmentType defaultType = compartment.getActualCompartmentType(0);
        for (Iterator<Treatment> iterator = treatments.iterator(); iterator.hasNext();) {
            Treatment treat = iterator.next();
           
            compartment.addCompartmentType(treat.getStart(), treat.getTreatedCompartmentType());
            if (treat.getEnd() != null) {
                compartment.addCompartmentType(treat.getEnd(), defaultType);
            }
        }
    }

    public Condition getCondition(Element node) throws ParserException {
        
        if (DOMHelper.existsTag(node, "timeCondition")) {
            // TimeCondition
            if (type == CONDITION_TYPE.UNKNOWN) {
                type = CONDITION_TYPE.TIMECONDITION;
            }
            if (type != CONDITION_TYPE.TIMECONDITION) {
                throw new ParserException("Either timeCondtion or sizeCondition is allowed!");
            }
            double time = Double.parseDouble(node.getTextContent().trim());
            if (time <= 0) {
                throw new ParserException("TimeCondition must have a positive value.");
            }
            return new TimeCondition(time);
            
        } else if (DOMHelper.existsTag(node, "sizeCondition")) {
        	
        	String conditionTest = DOMHelper.getFirstElementAttribute(node, "conditionTest");
            if (conditionTest == null) {
                return null;
            }
            
            // SizeCondition
            if (type == CONDITION_TYPE.UNKNOWN) {
                type = CONDITION_TYPE.SIZECONDITION;
            }
            if (type != CONDITION_TYPE.SIZECONDITION) {
                throw new ParserException("Either timeCondtion or sizeCondition is allowed!");
            }
            long size = Long.parseLong(node.getTextContent().trim());
            if (size <= 0) {
                throw new ParserException("SizeCondition must have a positive value.");
            }
            return new SizeCondition(Condition.valueOf(conditionTest), size);
        }
        return null;
    }

    /**
     * return the condition compareValue
     * 
     * @param condition
     * @return condition compare value
     */
    public double getConditionValue(Condition condition) {
        if (condition == null) {
            return -1;
        }
        if (condition instanceof TimeCondition) {
            return ((TimeCondition) condition).getCompareTime();
        } else if (condition instanceof SizeCondition) {
            return ((SizeCondition) condition).getCompareSize();
        }
        return -1;
    }

    public void checkConsistentOfTreatments() throws ParserException {
        if (!treatments.isEmpty()) {
            Collections.sort(treatments);
            // the actual
            Condition last = treatments.get(0).getEnd();
            for (int i = 1; i < treatments.size(); i++) {
                Treatment current = treatments.get(i);
                double time = 0d;
                long size = 0l;
                if (last instanceof TimeCondition) {
                    time = ((TimeCondition) last).getCompareTime();
                } else if (last instanceof SizeCondition) {
                    size = ((SizeCondition) last).getCompareSize();
                } else {
                    throw new ParserException("A treatment with no <end> was found but it follows other treatments!");
                    // conditions.get(i - 1).setEnd(current.start);
                }
                if (current.getStart().isConditionTrue(time, size)) {
                    // conditions.get(i - 1).setEnd(current.start);
                    throw new ParserException("Inconsistent treatment was found at:\"" + current + "\"");
                }
                last = current.getEnd();
            }
        }
    }

    public Compartment continuousCompartment(Element node) throws ParserException {
    	
        if (!DOMHelper.existsTag(node, "continuousCompartment")) return null;
        
        String name = DOMHelper.getElementString(node, "name");
        
        //get the growth function id and the associated function from the functions map
        String growthFunctionRef = DOMHelper.getElementAttribute(node, "growthFunction", "functionRef");
        Function growthFunction =  maps.getFunction(growthFunctionRef);
        if (growthFunction == null) {
        	//Whoops! Where is the associated growth function?
            throw new ParserException("Could not find the growth function with id: " + growthFunctionRef + ". Maybe a misspelling?");
        }        
        
        //get the colonization rate id and the associated rate from the colonization rate map
    	String colonizationRef = DOMHelper.getElementAttribute(node, "colonizationRate", "functionRef");
    	ColonizationRate colonizationRate =  null;        
        //no need to look up a function that is not there ;)
        if(colonizationRef != null){ colonizationRate =  maps.getColonisationRate(colonizationRef);}        
        
        boolean isPrimaryTumor = DOMHelper.getElementBoolean(node, "isPrimaryTumor", false);
        
        long startSize = checkGrowthFunction(growthFunction);
        //a start size can only be specified for the primary tumor
        if(startSize > 1 && !isPrimaryTumor){
        	throw new ParserException("A start size greater than 1 can only be specified for the primary tumor. The primary tumor has to be tagged by " +
        			"the tag '<isPrimaryTumor>true</isPrimaryTumor>'.");
        }
        
        //check whether the tumor shall be resected at some point in the simulation
        double resectionTime = DOMHelper.getElementDouble(node, "resectionTime", 0.);
        if(resectionTime > 0){
        	//if so, check whether some cells shall remain in the tissue
        	double remnantFraction = DOMHelper.getElementDouble(node, "remnantFraction", 0.);
        	
        	TreatmentList.getInstance().addTreatment(resectionTime, new ResectionTreatment(remnantFraction));
        }
        
        //create the compartment
        ContinuousCompartment compartment;
        
        try {        	
   			compartment = new ContinuousCompartment(0, name, getHostOrgan(node, maps), growthFunction, colonizationRate, isPrimaryTumor);
   			
            DiscreteCompartment successor = getSuccessor(node, compartment);
            compartment.setSuccessor(successor);

		} catch (MetastaseException e) {
			throw new ParserException(e);
		}
        
        return compartment;
    }
    
    /**
     * Checks whether the growth function is correctly parameterized concerning the start size. The start size has to be greater or equal 1. 
     * @param growthFunction the growth function to check
     * @return the start size
     * @throws ParserException if the start size in less than 1
     */
    private long checkGrowthFunction(Function growthFunction) throws ParserException{
    	
    	if(growthFunction instanceof ConstantFunction){
    		ConstantFunction f = (ConstantFunction)growthFunction;
    		if(f.getReturnValue() >= 1) return SpecialMath.round(f.getReturnValue());
    		else {
    			throw new ParserException("");
    		}
    		
    	}else if(growthFunction instanceof LinearFunction){
    		LinearFunction f = (LinearFunction)growthFunction;
    		if(f.getN() >= 1) return SpecialMath.round(f.getN());
    		else {
    			throw new ParserException("The growth function " + f + " is not correctly parametrisized. To be used as a growth function the parameter 'n' " +
    					"(represents the start size of the tumour) has to be an integer equal or greater than 1!");
    		}    		
    	}else if(growthFunction instanceof ExponentialFunction){
    		ExponentialFunction f = (ExponentialFunction)growthFunction;
    		if((f.getA() + f.getC()) >= 1) return SpecialMath.round(f.getA() + f.getC());
    		else {
    			throw new ParserException("The growth function " + f + " is not correctly parametrisized. To be used as a growth function n = a+c " +
    					"(represents the start size of the tumour) has to be equal or greater than 1!");
    		} 
    		
    	}else if(growthFunction instanceof PowerFunction){
    		PowerFunction f = (PowerFunction)growthFunction;
    		if(f.getC() >= 1) return SpecialMath.round(f.getC());
    		else {
    			throw new ParserException("The growth function " + f + " is not correctly parametrisized. To be used as a growth function the parameter 'c' " +
    					"(represents the start size of the tumour) has to be equal or greater than 1!");
    		}
    		
    	}else if(growthFunction instanceof LogFunction){
    		LogFunction f = (LogFunction)growthFunction;
    		if(f.getN()!=null && f.getN() >= 1) return f.getN();
    		else {
    			throw new ParserException("The growth function " + f + " is not correctly parametrisized. To be used as a growth function the parameter 'startSize' " +
    					"has to be stated and has to be equal or greater than 1!");
    		}
    		
    	}else if(growthFunction instanceof GompertzFunction){
    		GompertzFunction f = (GompertzFunction)growthFunction;
    		if(f.getN() >= 1) return SpecialMath.round(f.getN());
    		else {
    			throw new ParserException("The growth function " + f + " is not correctly parametrisized. To be used as a growth function the parameter 'startSize' " +
    					" has to be equal or greater than 1!");
    		}    		
    	}else{
    		throw new ParserException("Growth function of undefined Function Type "+growthFunction.getClass().getName());
    	}
    	
    }

    public DiscreteCompartment getSuccessor(Element node, Compartment compartment) {
        String compartmentRef = DOMHelper.getElementAttribute(node, "successor", "compartmentRef");
        if (compartmentRef == null) {
            return null;
        }
        DiscreteCompartment successor = (DiscreteCompartment) maps.getCompartment(compartmentRef);
        if (successor == null) {
            logger.fine("ID: " + compartmentRef + " dosn't exists!");
            logger.fine("Added " + compartmentRef + " to unreferenced list!");
            maps.putUnreferenced(compartmentRef, RefType.COMPARTMENT, compartment);
        }
        return successor;
    }

    protected void resolveRefs() throws ParserException {
        logger.fine("#resolve refs: " + maps.getUnreferencedSize());
        Iterator<String> keys = maps.getUnreferencedKeySet().iterator();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            String ref = key.split(ReferenceMap.SEPARATOR)[0];
            RefType type = RefType.valueOf(key.split(ReferenceMap.SEPARATOR)[1].toUpperCase());
            switch (type) {
            case COMPARTMENT:
                resolveRefCompartment(maps.getCompartment(ref), maps.getUnreferenced(key));
                break;
            case TYPE:
                resolveRefCompartmentType(maps.getCompartmentType(ref), maps.getUnreferenced(key));
                break;
            default:
                throw new ParserException("Can't resolve reference:" + type);
            }

        }
    }

    public void resolveRefCompartment(Compartment compartment, List<Object> list) {
        if (compartment == null) {
            return;
        }
        logger.fine("#resolve compartments: " + list.size());
        Iterator<Object> items = list.iterator();
        while (items.hasNext()) {
            Object item = (Object) items.next();
            if (item instanceof TransferToNewContinuousCompartment) {
                ((TransferToNewContinuousCompartment) item).setSuccessorOfNewCompartment((DiscreteCompartment) compartment);
                items.remove();
            } else if (item instanceof TransferToNewDiscreteCompartment) {
                ((TransferToNewDiscreteCompartment) item).setSuccessorOfNewCompartment((DiscreteCompartment) compartment);
                items.remove();
            } else if (item instanceof Compartment) {
                ((Compartment) item).setSuccessor((DiscreteCompartment) compartment);
                items.remove();
            }

        }
    }

    public void resolveRefCompartmentType(Object type, List<Object> list) {
        if (type == null) {
            return;
        }
        logger.fine("#resolve compartment types: " + list.size());
        Iterator<Object> items = list.iterator();
        while (items.hasNext()) {
            Object item = (Object) items.next();
           if (item instanceof TransferToNewDiscreteCompartment) {
                ((TransferToNewDiscreteCompartment) item).setNewCompartmentType((DiscreteCompartmentType) type);
                items.remove();
            }

        }
    }
   
    private static HostOrgan getHostOrgan(Element node, ObjectBuffer maps) throws ParserException{
      	 //get the host organ           
          String hostOrganKey = DOMHelper.getElementAttribute(node, "hostOrgan", "organRef");
          HostOrgan organ = maps.getHostOrgan(hostOrganKey);
          if(organ == null) {
        	  throw new ParserException("Can't find the host organ with the id '"+hostOrganKey+"'. Maybe a misspelling?");
          }
          return organ;
    }

    /**
     * Wrap the treatment in one class.
     * 
     * @author Michael Suhr
     * 
     */
    protected class Treatment implements Comparable<Treatment> {
        private Condition start;
        private Condition end;
        private DiscreteCompartmentType treatedCompartmentType;

        public Treatment(Condition start, Condition end, DiscreteCompartmentType treatedCompartmentType) {
            this.start = start;
            this.end = end;
            this.treatedCompartmentType = treatedCompartmentType;
        }

        public Condition getStart() {
            return this.start;
        }

        public Condition getEnd() {
            return this.end;
        }

        public DiscreteCompartmentType getTreatedCompartmentType() {
            return this.treatedCompartmentType;
        }


        public int compareTo(Treatment o) {
            if (start.getClass().equals(o.getStart().getClass())) {
                
                int factor = 1;
                double endValue = getConditionValue(end);
                if (endValue >= 0 && endValue < getConditionValue(start)) {
                    factor *= -1;
                }
                if (start instanceof TimeCondition) {
                    TimeCondition time = (TimeCondition) o.getStart();
                    return factor * (int) (((TimeCondition) start).getCompareTime() - time.getCompareTime());
                } else if (start instanceof SizeCondition) {
                    SizeCondition time = (SizeCondition) o.getStart();
                    return factor * (int) (((SizeCondition) start).getCompareSize() - time.getCompareSize());
                }
            }
            return 0;
        }

        @Override
        public String toString() {
            return "start:\"" + start.toString() + "\" + end:\"" + (end == null ? "null" : end.toString()) + "\"";
        }

    }
}
