/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Helper Class to parse the stated values from String into the corresponding types.
 * 
 * @author Michael Suhr
 * 
 */
public final class DOMHelper {
    
    private DOMHelper() {
    }

    /**
     * Return the content of the node or defaultValue.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @param defaultValue - the default return value if no tag found
     * @return text content as {@link Double} or defaultValue
     */
    public static Double getElementDouble(final Element node, final String tagName, final double defaultValue) {
        Node tag = node.getElementsByTagName(tagName).item(0);
        if (tag != null) {
            return Double.parseDouble(tag.getTextContent().trim());
        }
        return defaultValue;
    }

    /**
     * Return the content of the node or {@link null}.
     * 
     * @param node - the DOM node
     * @param tagName  - name of tag
     * @return text content as <code>double</code> or null
     */
    public static Double getElementDouble(final Element node, final String tagName) {
    	Node tag = node.getElementsByTagName(tagName).item(0);
        if (tag != null) {
            return new Double(tag.getTextContent().trim());
        }else{
        	return null;
        }
    }

    /**
     * Return the content of the node or defaultValue.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @param defaultValue - the default return value if no tag found
     * @return text content as {@link Long} or defaultValue
     */
    public static Long getElementLong(final Element node, final String tagName, final long defaultValue) {
        Node tag = node.getElementsByTagName(tagName).item(0);
        if (tag != null) {
            return Long.parseLong(tag.getTextContent().trim());
        }
        return defaultValue;
    }

    /**
     * Return the content of the node or {@link null}.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @return text content as {@link Long} or null
     */
    public static Long getElementLong(final Element node, final String tagName) {
    	 Node tag = node.getElementsByTagName(tagName).item(0);
         if (tag != null) {
             return Long.parseLong(tag.getTextContent().trim());
         }else{
        	 return null;
         }
    }

    /**
     * Return the content of the node or {@link null}.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @return text content as {@link String} or {@link null}
     */
    public static String getElementString(final Element node, final String tagName) {
        Node tag = node.getElementsByTagName(tagName).item(0);
        if (tag != null) {
            return node.getElementsByTagName(tagName).item(0).getTextContent();
        }
        return null;
    }

    /**
     * Return the content of the node or {@link null}.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @param attributeName - name of attribute
     * @return attribute content as {@link String} or null
     */
    public static String getElementAttribute(final Element node, final String tagName, final String attributeName) {
        Node tag = node.getElementsByTagName(tagName).item(0);
        if (tag != null) {
            return ((Element) node.getElementsByTagName(tagName).item(0)).getAttribute(attributeName);
        }
        return null;
    }

    /**
     * Return the first content of the attribute or {@link null}.
     * 
     * @param node - the DOM node
     * @param attributeName - name of attribute
     * @return attribute content as {@link String}
     */
    public static String getFirstElementAttribute(Element node, final String attributeName) {
        if (node == null) {
            return null;
        }
        String attribute = node.getAttribute(attributeName);
        if (attribute.length() == 0) {
            NodeList nodeList = node.getChildNodes();
            for (int j = 0; j < nodeList.getLength(); j++) {
                Node child = nodeList.item(j);
                String childName = child.getNodeName();
                if (child instanceof Element) {
                    attribute = getElementAttribute(node, childName, attributeName);
                    if (attribute != null) {
                        return attribute;
                    }
                }
            }
        } else {
            return attribute;
        }
        return null;
    }

    /**
     * Return the boolean if the tag exists or not.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @return whether tag exists or not as {@link boolean}
     */
    public static boolean existsTag(final Element node, final String tagName) {
        if (node.getTagName().equals(tagName)) {
            return true;
        }
        return (node.getElementsByTagName(tagName).item(0) != null);
    }


    /**
     * Return the content of the node or defaultValue.
     * 
     * @param node - the DOM node
     * @param tagName - name of tag
     * @param defaultValue - the default return value if no tag found
     * @return attribute content as {@link boolean}
     */
    public static boolean getElementBoolean(final Element node, final String tagName, final boolean defaultValue) {
        Node tag = node.getElementsByTagName(tagName).item(0);
        if (tag != null) {
            return Boolean.parseBoolean(tag.getTextContent().trim());
        }
        return defaultValue;
    }

}
