/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import java.util.logging.Logger;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDeath;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDivision;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNextCompartment;
import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.io.xml.ParserException;
import de.fhstralsund.catsit.io.xml.ReferenceMap.RefType;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.Function;

/**
 * This Builder is used to parse a DOM node to a {@link CellProgressionEventAction}.
 * 
 * @author Michael Suhr
 * 
 */
public abstract class EventActionBuilder {
    private static final Logger logger = Logger.getLogger("metastase.support.xml.builder");
    
    /**
     * 
     * @param node the DOM node
     * @param maps the object buffer where the distributions and functions are buffered
     * @return the event action object
     */
    public static CellProgressionEventAction parse(Node node, ObjectBuffer maps) throws ParserException{

        String type = node.getNodeName();
        CellProgressionEventAction returnEvent = null;
        if (node instanceof Element) {
            Element elem = (Element) node;
            if ("transferToNewContinuousCompartment".equals(type)) {
                logger.fine("generate transfer To New Continuous Compartment event");
                returnEvent = transferToNewContinuousCompartment(elem, maps);
            } else if ("transferToNewDiscreteCompartment".equals(type)) {
                logger.fine("generate transfer To New Discrete Compartment event");
                returnEvent = transferToNewDiscreteCompartment(elem, maps);
            } else if ("cellDeath".equals(type)) {
                logger.fine("generate cell Death event");
                returnEvent = cellDeath(elem, maps);
            } else if ("cellDivision".equals(type)) {
                logger.fine("generate cell division event");
                returnEvent = cellDivision(elem, maps);
            } else if ("transferToNext".equals(type)) {
                logger.fine("generate transfer to next compartment event");
                returnEvent = transferToNext(elem, maps);
            }
        }
        return returnEvent;
    }

    private static CellProgressionEventAction transferToNewContinuousCompartment(Element node, ObjectBuffer maps) throws ParserException {

    	String newName = DOMHelper.getElementString(node, "newName");
    	
    	String growthFunctionRef = DOMHelper.getElementAttribute(node, "newGrowthFunction", "functionRef");
        Function newGrowthFunction =  maps.getFunction(growthFunctionRef);
        
        if(newGrowthFunction == null){
        	throw new ParserException("Could not find the growth function '"+growthFunctionRef+"'. Maybe a mispelling?");
        }
    	
    	String colonizationRef = DOMHelper.getElementAttribute(node, "newColonizationRate", "functionRef");
        ColonizationRate newColonizationRate =  null;        
        //no need to look up a function that is not there ;)
        if(colonizationRef != null){ 
        	newColonizationRate =  maps.getColonisationRate(colonizationRef);
        	if (newColonizationRate == null){
        		throw new ParserException("Could not find the colonization rate '"+colonizationRef+"'. Maybe a mispelling?");
        	}
        	//create a new instance of the colonization rate so the tumors don't all reference the same instance
        	newColonizationRate = newColonizationRate.copy();
        }        
        
        String successorRef = DOMHelper.getElementAttribute(node, "newSuccessor", "compartmentRef");
        DiscreteCompartment successorOfNewCompartment = null; 
        if(successorRef != null){
        	successorOfNewCompartment = (DiscreteCompartment) maps.getCompartment(successorRef);        	
        }
        
        CellProgressionEventAction action = new TransferToNewContinuousCompartment(getName(node), getGenerationInfo(node, maps), 
        		newName, getHostOrgan(node, maps), newGrowthFunction, newColonizationRate, successorOfNewCompartment);

        if (successorOfNewCompartment == null) {
            logger.fine("ID: " + successorRef + " dosn't exists!");
            logger.fine("Added " + successorRef + " to unreferenced list!");
            maps.putUnreferenced(successorRef, RefType.COMPARTMENT, action);
        }
        
        return action;
    }

    private static CellProgressionEventAction transferToNewDiscreteCompartment(Element node, ObjectBuffer maps) {

        String typeRef = DOMHelper.getElementAttribute(node, "newCompartmentType", "typeRef");
        String compartmentRef = DOMHelper.getElementAttribute(node, "newSuccessor", "compartmentRef");
        DiscreteCompartment successorNewCompartment = (DiscreteCompartment) maps.getCompartment(compartmentRef);
        DiscreteCompartmentType newCompartmentType = (DiscreteCompartmentType) maps.getCompartmentType(typeRef);
        
        CellProgressionEventAction action = new TransferToNewDiscreteCompartment(getName(node), 
        		getGenerationInfo(node, maps), getHostOrgan(node, maps), newCompartmentType, successorNewCompartment);

        if (successorNewCompartment == null) {
            logger.fine("ID: " + compartmentRef + " dosn't exists!");
            logger.fine("Added " + compartmentRef + " to unreferenced list!");
            maps.putUnreferenced(compartmentRef, RefType.COMPARTMENT, action);
        }
        if (newCompartmentType == null) {
            logger.fine("ID: " + typeRef + " dosn't exists!");
            logger.fine("Added " + typeRef + " to unreferenced list!");
            maps.putUnreferenced(typeRef, RefType.TYPE, action);
        }
        return action;
    }

    private static CellProgressionEventAction cellDeath(Element node, ObjectBuffer maps) {
        return new CellDeath(getName(node), getGenerationInfo(node, maps));
    }

    private static CellProgressionEventAction cellDivision(Element node, ObjectBuffer maps) {
        return new CellDivision(getName(node), getGenerationInfo(node, maps));
    }

    private static CellProgressionEventAction transferToNext(Element node,ObjectBuffer maps) {
        return new TransferToNextCompartment(getName(node), getGenerationInfo(node, maps));
    }

    private static GenerationInfo getGenerationInfo(Element node, ObjectBuffer maps) {
        String functionRef = DOMHelper.getElementAttribute(node, "meanTimeToNextEvent", "functionRef");
        String distRef = DOMHelper.getElementAttribute(node, "distribution", "distRef");
        double probability = DOMHelper.getElementDouble(node, "probability", .0);
        Function meanTimeToNextEvent = maps.getFunction(functionRef);
        Distribution distribution = maps.getDistribution(distRef);
        
        return new GenerationInfo(probability, meanTimeToNextEvent, distribution);
    }

    private static String getName(Element node) {
        return DOMHelper.getElementString(node, "name");
    }
    
    private static HostOrgan getHostOrgan(Element node, ObjectBuffer maps){
   	 //get the host organ           
       String hostOrganKey = DOMHelper.getElementAttribute(node, "newHostOrgan", "organRef");
       return maps.getHostOrgan(hostOrganKey);
   }

}
