/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import java.util.logging.Logger;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.LogFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;

/**
 * This Builder is used to parse a DOM node to a {@link Function}.
 * 
 * @author Michael Suhr
 * 
 */
public abstract class FunctionBuilder {
    protected static final Logger logger = Logger.getLogger("metastase.support.xml.builder");

    /**
     * 
     * @param node the DOM node
     * @param maps the object buffer where the functions will be saved until they are referenced
     * @return the function object
     */
    public static Function parse(Node node, ObjectBuffer maps) {

        String type = node.getNodeName();
        Function returnFunction = null;
        if (node instanceof Element) {
            Element elem = (Element) node;
            if ("constantFunction".equals(type)) {
                logger.fine("generate Constant Function");
                returnFunction = constantFunction(elem);
            } else if ("gompertzFunction".equals(type)) {
                logger.fine("generate Gompertz Function");
                returnFunction = gompertzFunction(elem);
            } else if ("colonizationRate".equals(type)) {
                logger.fine("generate colonization Rate");
                returnFunction = colonizationFunction(elem, maps);
            } else if ("logFunction".equals(type)) {
                logger.fine("generate log Function");
                returnFunction = logFunction(elem);
            } else if ("powerFunction".equals(type)) {
                logger.fine("generate power Function");
                returnFunction = powerFunction(elem);
            } else if ("linearFunction".equals(type)) {
                logger.fine("generate linear Function");
                returnFunction = linearFunction(elem);
            } else if ("exponentialFunction".equals(type)) {
                logger.fine("generate exponentialFunction Function");
                returnFunction = exponentialFunction(elem);
            }
            // use the function id name as key for the map.
            maps.putFunction(elem.getAttribute("functionID"), returnFunction);
        }
        return returnFunction;
    }

    private static Function constantFunction(Element node) {
        return new ConstantFunction(DOMHelper.getElementDouble(node, "constant"));
    }

    private static Function logFunction(Element node) {
        double a = DOMHelper.getElementDouble(node, "a");
        double b = DOMHelper.getElementDouble(node, "b");
        Long n = DOMHelper.getElementLong(node, "startsize"); 
        
        if(n == null){
        	return new LogFunction(a, b);
        }else{
        	return new LogFunction(a, b, n);
        }        
    }

    private static Function powerFunction(Element node) {
        double a = DOMHelper.getElementDouble(node, "a");
        double b = DOMHelper.getElementDouble(node, "b");
        double c = DOMHelper.getElementDouble(node, "c");
        return new PowerFunction(a,b,c);
    }

    private static Function linearFunction(Element node) {
        double m = DOMHelper.getElementDouble(node, "m");
        double n = DOMHelper.getElementDouble(node, "n");
        return new LinearFunction(m, n);
    }

    private static Function exponentialFunction(Element node) {
        double a = DOMHelper.getElementDouble(node, "a");
        double b = DOMHelper.getElementDouble(node, "b");
        double c = DOMHelper.getElementDouble(node, "c");
        return new ExponentialFunction(a,b,c);
    }

    private static Function gompertzFunction(Element node) {
        double growthRate = DOMHelper.getElementDouble(node, "growthRate");
        double maxTumorSize = DOMHelper.getElementDouble(node, "tumorSize");
        double startsize = DOMHelper.getElementDouble(node, "startSize", 1);

        return new GompertzFunction(growthRate, maxTumorSize, startsize);
    }

    private static Function colonizationFunction(Element node, ObjectBuffer maps) {
        double colonizationCoefficient = DOMHelper.getElementDouble(node, "colonizationCoefficient");
        double fractalDim = DOMHelper.getElementDouble(node, "fractalDim");

        ColonizationRate returnFunction = new ColonizationRate(colonizationCoefficient, fractalDim);        

        return returnFunction;
    }

}
