/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import java.util.logging.Logger;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.ExternalBeamTreatment;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.Treatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.io.xml.ParserException;

/**
 * This Builder is used to parse a DOM node to a {@link Treatment}.
 * 
 * @author Anja Bethge
 *
 */
public abstract class GlobalTreatmentBuilder {

	protected static final Logger logger = Logger.getLogger("metastase.support.xml.builder");

	/**
     * 
     * @param node the DOM node
     * @param maps the object buffer where the host organs, distributions and functions are buffered
     */
	public static void parse(Node node, ObjectBuffer maps) throws ParserException {

		if (node instanceof Element) {
			Element treatmentNode = (Element) node;
			
			//the time and the interval are given in days, but we simulate in minutes. Hence, we have to 
			//convert the time into minutes
			double time = DOMHelper.getElementDouble(treatmentNode, "time");
			long quanity = DOMHelper.getElementLong(treatmentNode, "quantity", 1);
			double interval = DOMHelper.getElementDouble(treatmentNode, "interval", 0);
			
			if(quanity > 1 && interval == 0){
				throw new ParserException("Please specify an interval, if the therapy is applied more than once.");
			}
			
			String treatmentType = treatmentNode.getNodeName();
			Treatment treatment = null;

			if ("radioimmunotherapy".equals(treatmentType)) {
				logger.fine("parsing radioimmunotherpy parameter");
				treatment = GlobalTreatmentBuilder.parseRadioimmunotherapy(treatmentNode);
				
			} else if ("radioembolization".equals(treatmentType)) {
				logger.fine("parsing radioembolization parameter");
				treatment = GlobalTreatmentBuilder.parseRadioembolization(treatmentNode, maps);
				
			} else if ("externalBeamRadiation".equals(treatmentType)) {
				logger.fine("parsing external beam radiation parameter");
				treatment = GlobalTreatmentBuilder.parseExternalBeamRadiation(treatmentNode);	
			
			} else if ("chemotherapy".equals(treatmentType)) {
				logger.fine("parsing chemo therapy parameter");
				treatment = GlobalTreatmentBuilder.parseChemoTherapy(treatmentNode);
			}
			
			if(treatment == null){
				throw new ParserException("Could not parse the given systemic treatment. Please check the spelling of the XML tag.");
			}				
				
			//add all treatments to the list
			TreatmentList treatmentList = TreatmentList.getInstance();
			for (int i = 0; i < quanity; i++) {					
				double t = time + (i*interval);
				treatmentList.addTreatment(t, treatment);
			}	
						
		} 
	}

	/**
	 * 
	 * @param treatment the DOM element
	 * @return the treatment object
	 */
	private static Treatment parseRadioimmunotherapy(Element treatment){
		
		//mandatory fields
		double alpha = DOMHelper.getElementDouble(treatment, "alpha");
		double beta = DOMHelper.getElementDouble(treatment, "beta");
		double activity = DOMHelper.getElementDouble(treatment, "activity");		
		double energy = DOMHelper.getElementDouble(treatment, "energy");		
		double halflife = DOMHelper.getElementDouble(treatment, "halflife");
		
		//optional fields
		double clearanceFraction = DOMHelper.getElementDouble(treatment, "clearanceFraction", 0.);
		boolean excludeBlood = DOMHelper.getElementBoolean(treatment, "excludeBlood", false);
		
		if (DOMHelper.existsTag(treatment, "biologicalHalflife")){
			double biologicalHalflife = DOMHelper.getElementDouble(treatment, "biologicalHalflife");
			return new RadioimmunoTreatment(alpha, beta, activity, energy, halflife, biologicalHalflife, clearanceFraction, excludeBlood);		
		}else{
			return new RadioimmunoTreatment(alpha, beta, activity, energy, halflife, clearanceFraction, excludeBlood);
		}		
	}	
	
	
	/**
	 * @param treatment the DOM element
	 * @param maps the object buffer where the host organs, distributions and functions are buffered
	 * @return the treatment object with all necessary treatment parameter 
	 * @throws ParserException
	 */
	private static Treatment parseRadioembolization(Element treatment, ObjectBuffer maps) throws ParserException{
		
		String hostOrganKey = DOMHelper.getElementAttribute(treatment, "hostOrgan", "organRef");
		if(hostOrganKey == null){
			throw new ParserException("Please specify the host organ in which the radioembolisation is executed.");
		}
		
		HostOrgan hostOrgan = maps.getHostOrgan(hostOrganKey);
		if(hostOrgan == null){
			throw new ParserException("Could not find host organ with the id '"+hostOrganKey+"'. Maybe a misspelling?");
		}
		
		double alpha = DOMHelper.getElementDouble(treatment, "alpha");
		double beta = DOMHelper.getElementDouble(treatment, "beta");
		double activity = DOMHelper.getElementDouble(treatment, "activity");
		double energy = DOMHelper.getElementDouble(treatment, "energy");
		double halflife = DOMHelper.getElementDouble(treatment, "halflife");
		
		Double diameter = DOMHelper.getElementDouble(treatment, "minDiameter");
		Double volume = DOMHelper.getElementDouble(treatment, "minVolume");
		
		if(diameter == null && volume == null){
			throw new ParserException("Please state a minimum size (mm³) or minimum diameter (mm) for metastases to be treated.");
		}
		
		if(diameter != null && volume != null){
			throw new ParserException("Please state either a minimum size (mm³) or minimum diameter (mm) for metastases to be treated. " +
					"You cannot state both at the same time.");
		}
		
		if(diameter != null){
			//we assume the tumor to be a sphere
			volume = 1./6. * Math.PI * Math.pow(diameter, 3);
		}		
		
		return new RadioembolizationTreatment(alpha, beta, activity, energy, halflife, hostOrgan, volume);
	}

	/**
	 * 
	 * @param treatment the DOM element
	 * @return the treatment object
	 */
	private static Treatment parseExternalBeamRadiation(Element treatment){
		
		//mandatory fields
		double alpha = DOMHelper.getElementDouble(treatment, "alpha");
		double beta = DOMHelper.getElementDouble(treatment, "beta");
		double dose = DOMHelper.getElementDouble(treatment, "dose");	
		
		return new ExternalBeamTreatment(alpha, beta, dose);
	}	

	
	/**
	 * 
	 * @param treatment the DOM element
	 * @return the treatment object
	 */
	private static Treatment parseChemoTherapy(Element treatment) {
				
		double drugSensitivity = DOMHelper.getElementDouble(treatment, "drugSensitivity");
		
		double drugDecayRate = DOMHelper.getElementDouble(treatment, "drugDecay");		
		double cyclingCellFraction = DOMHelper.getElementDouble(treatment, "cyclingCellsFraction", 1);		
		
		return new ChemoTreatment(cyclingCellFraction, drugSensitivity, drugDecayRate);
	}

}
