/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.machine;

import java.util.logging.Logger;

import de.fhstralsund.catsit.support.functions.Function;

/**
 * Computes the time of an event according to a rate
 * that follows a given {@link Function}. This Generator integrates 
 * the event rate function using many intervals straight forward.<br>
 * This works fine with monotonically increasing functions, but not
 * so fine with monotonically decreasing functions.
 * 
 * @author Gero Wedemann, Anja Bethge
 */
public class AdaptiveIntegrationCalculator extends EventTimeCalculator {
	private static final Logger logger = Logger.getLogger("metastase.machine");
	
	private int integrationSteps = 1000000;
	
	// the first generated is treated differently
	private static boolean isFirst = true; 
	
	

	/**
	 * the constructor
	 * 
	 * @param eventRateFunction the rate in which the events are created
	 */
	public AdaptiveIntegrationCalculator(Function eventRateFunction) {
		super(eventRateFunction);
	}

	
	/**
	 * Computes the time of the next intravasation event based on the time of the last event.
	 * 
	 * @param currentTime the time of the last event
	 */
	@Override
	public double computeTimeToNextEvent(double currentTime) throws TumorIsDeadException{
		
		if(currentTime < 0){
			throw new IllegalStateException("The time must not be smaller than zero.");
		}
		double time = currentTime;

		double rndSurfaceArea;
		if (isFirst) {
			rndSurfaceArea = getRng().getDouble() / 5. + 0.9; // 0.9 .. 1.1
			isFirst = false;
		} else {
			rndSurfaceArea = 2. * getRng().getDouble(); // 0 .. 2
		}
		
		double integrationTimeStep = computeIntegrationTimeStep(time, rndSurfaceArea);
		
		double timeNextEventLower;
		double sumEventLower;
		double timeNextEventUpper = time;
		double sumEventUpper = 0.;
		
		do {
			
			timeNextEventLower = timeNextEventUpper;
			//if the rate function returns 0, than the tumor is dead. hence, no further intravsation events will occur and we can stop the computation.
			if (getEventRateFunction().f(timeNextEventLower) <= 0){
				throw new TumorIsDeadException("The tumor dies before the next intravasation event!");
			}
			sumEventLower = sumEventUpper;
			sumEventUpper += getEventRateFunction().f(timeNextEventLower) * integrationTimeStep;
			timeNextEventUpper += integrationTimeStep;
		}

		//simple sum-integral
		while(sumEventUpper<rndSurfaceArea);

		if (Math.abs(rndSurfaceArea-sumEventUpper)<Math.abs(rndSurfaceArea-sumEventLower)) {
			time = timeNextEventUpper;
		} else {
			time = timeNextEventLower;
		}
		
		//adjust the integration step. we only need a high number of steps at 
		//the beginning, since the function is very flat and low there
		if (integrationSteps>1000) integrationSteps=(int)(integrationSteps/10);
		
		return time;
	}
	

	/**
	 * Computes the integration time step. To compute the time step, the time of 
	 * the next event is estimated with the help of a taylor polynomial or with 
	 * the linear approach.
	 * @param time the time of the last event
	 * @param surfaceArea the surfaceArea
	 * @return the integration time step
	 */
	private double computeIntegrationTimeStep(double time, double surfaceArea){
		
		// use the taylor polynomial until the time of 11000 days (because the 
		// computation becomes too inexact after this time) and when a derivation 
		// exists 
		if((time/24./60.) <= 11000.0){
			//compute the the integration time step with the help of the taylor polynomial
			double valueFirstDerivation = getEventRateFunction().f(time);
			double valueSecondDerivation = getEventRateFunction().f_FirstDerivation(time);
			
			//because it is a quadratic equation, two potential solutions can be computed
			double upper1 = (valueSecondDerivation * time - valueFirstDerivation + Math.sqrt(
								Math.pow(valueFirstDerivation, 2) + 2. * surfaceArea * valueSecondDerivation))
								/valueSecondDerivation;
			
			double upper2 = (valueSecondDerivation * time - valueFirstDerivation - Math.sqrt(
								Math.pow(valueFirstDerivation, 2) + 2. * surfaceArea * valueSecondDerivation))
								/valueSecondDerivation;			
			
			//check which solution is the right one and return the integration time step 
			if(upper1 > time && upper2 < time){
//				logger.info("### Used taylor polynomial with "+getEventRateFunction().toString());
				return (upper1 - time) / integrationSteps;
				
			}else if(upper1 < time && upper2 > time){
//				logger.info("### Used taylor polynomial with "+getEventRateFunction().toString());
				return (upper2 - time) / integrationSteps;
				
				
			}else if((upper1 == time || upper2 == time)){
//				logger.info("The difference between the computed upper " +
//						"value and the actual time is to small. The linear " +
//						"approach is used instead of the taylor polynom to " +
//						"avoid too high computation time.");
				
			}else {
//				logger.info("The computed values don't match the form " +
//						"upper1 < time < upper2 or upper2 < time < upper1. " +
//						"The values are:" +	"time = " + time + ", upper1 = " + 
//						upper1 + ", upper2 = " + upper2 + ". The linear " +
//						"approach is used instead.");
			}	
		}
		
		
		//if no derivation is given for the eventRateFunction or the computed 
		//difference between the actual time and the computed next events time 
		//is to small, then the linear approach is used to estimate the time 
		//for the next event.
//		logger.info("Used linear approach with "+getEventRateFunction().toString());
		return 1. / (integrationSteps * getEventRateFunction().f(time)); 
	}

}
