/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.machine;

import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.machine.eventlists.HenriksenEventList;

/**
 * This abstract class describes the basic methods of all event lists. 
 * Events lists are modeled as Singletons, that means only one instance is allowed at a time.
 * The instance is created in the getInstance() Method. All constructors are private.
 * 
 * @author Gero Wedemann
 * @author Anja Bethge
 */
public abstract class EventList implements Iterable<Event>{
	
	//the only instance of the event list
	private static EventList theEventList = null;
	
	
	
	public static synchronized EventList getInstance(){
		if (theEventList == null){
			theEventList = new HenriksenEventList();
		}
		return theEventList;
	}

	
	/**
	 * Sets the deadline to the given value.
	 * @param deadLine the time in minutes until which events shall be added to the list. Events with a time greater than 
	 * the deadline are ignored and not sorted into the list.
	 */
	public abstract void setDeadline(double deadLine);
	
	
	/**
	 * add an event to the list and sort it in
	 *   
	 * @param event the event
	 */
	public abstract void addEvent(Event event);
	
	/**
	 * returns next event in time and deletes it from the list
	 * 
	 * @return the event
	 * @throws NoEventLeftException if the event-list is empty
	 */
	public abstract Event nextEvent() throws NoEventLeftException;
	
	/**
	 * is the list empty?
	 * 
	 * @return true, if empty, else false
	 */
	public abstract boolean isEmpty();
	
	/**
	 * deletes the given Event from the event list
	 * @param e the event that shall be removed from the list
	 * @return the removed Event or null, if the given Event could not be removed
	 */
	public abstract Event removeEvent(Event e);
	
	/**
	 * only for tests
	 */
	public static void reset() {
		theEventList = null;
	}
}
