/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.machine;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.fhstralsund.catsit.WrongArgumentsException;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.base.GlobalEvent;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.ExternalBeamRadiation;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.PrintStatus;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.Radioembolization;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.Radioimmunotherapy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.Resection;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.ExternalBeamTreatment;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.ResectionTreatment;
import de.fhstralsund.catsit.base.treatments.Treatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.base.treatments.TreatmentList.TreatmentEntry;
import de.fhstralsund.catsit.io.excel.writer.ExcelWriter;
import de.fhstralsund.catsit.io.excel.writer.ExcelWriterException;
import de.fhstralsund.catsit.io.xml.MPMLParser;
import de.fhstralsund.catsit.io.xml.MPMLParserImpl;

/**
 * Starts and controls the simulation. Here we instantiate all objects which are read
 * from the xml configuration file. The Controller uses the {@link MPMLParser} to parse the file.
 * 
 * @author Michael Suhr, Anja Bethge, Gero Wedemann
 */
public class SimController {
	private static final Logger logger = Logger.getLogger("metastase.machine");

	private static final String USAGE_STRING = "usage: java -jar CaTSiT.jar simulate excelFile excelSheetName configFile technicalFile";

	
	//the config and technical xmls
	private String config;
	private String technical;

	//the excel file and excel sheet name, where to save the results
	private String excelFileName;
	private String excelSheetName;
	
	//the EventList
	private EventList theEventList;
	
	
	/**
	 * This constructor is used, when the simulation is started via the command line.
	 * @param args the command line parameter in the following form: "excelFile excelSheetName configFile [technicalData]"
	 * @throws MetastaseException, if one or more of the obligatory parameter are missing or the configuration and/or 
	 * technical files are not given as an xml file
	 */
	public SimController(String[] args) throws MetastaseException {
//		logger.info("\n\nSW started ");
		parseCommandline(args);
		run();
	}
	
	/**
	 * This constructor is used, when the simulation is started via the sim starter
	 * @param excelFileName the name of the excel file, where the results shall be saved
	 * @param excelSheetName the name of the excel sheet, where the results shall be saved
	 * @param config the configuration file as xml
	 * @param technical the technical file as xml
	 * @throws MetastaseException, if one or more of the obligatory parameter are missing or the configuration and/or 
	 * technical files are not given as an xml file
	 */
	public SimController(String excelFileName, String excelSheetName, String config, String technical) 
			throws MetastaseException{
		
		//check whether all obligatory parameter were specified
		if(excelFileName == null || excelSheetName == null || config == null || technical == null){
			throw new WrongArgumentsException("Missing arguments. "+USAGE_STRING);
		}
		
		//check whether the excel file name is empty
		if(excelFileName.length()<1){
			throw new WrongArgumentsException("Please state an excel file name!");
		}else{
			this.excelFileName = excelFileName;
		}
		
		//check whether the excel sheet name is empty
		if(excelSheetName.length()<1){
			throw new WrongArgumentsException("Please state an excel sheet name!");
		}else{
			this.excelSheetName = excelSheetName;
		}
		
		//check whether the config file is a xml file
		this.config = getXMLFile(config);
		
		if(technical != null){
			//check whether the technical file is a xml file
			this.technical = getXMLFile(technical);
		}
		
	}

	/**
	 * Parses the command line arguments (excelFile, excelSheetName, configFile, technicalData).
	 * 
	 * @param args The parameter shall be specified in the following order: excelFile excelSheetName configFile technicalData
	 * @throws MetastaseException, if one or more of the obligatory parameter are missing or the configuration and/or 
	 * technical files are not given as an xml file
	 */
	private void parseCommandline(String[] args) throws MetastaseException {
		if (args.length != 4) {
			throw new WrongArgumentsException("Missing arguments. "+USAGE_STRING);
		}

		excelFileName = args[0].replace("\"", "");
		if (!excelFileName.endsWith(".xlsx")) {
			excelFileName+=".xlsx"; //add the right ending, if it is missing
		}
		
		excelSheetName = args[1];
		// remove quotes, if any
		config = getXMLFile(args[2].replace("\"", ""));
		technical = getXMLFile(args[3].replace("\"", ""));
		
		//check whether the configuration files and parent folder of the excel file exist
		if(!new File(config).exists()){
			throw new WrongArgumentsException("The configuration file "+config+" does not exists!");
		}
		if(!new File(technical).exists()){
			throw new WrongArgumentsException("The technical configuration file "+technical+" does not exists!");
		}
		
		File folder = new File(excelFileName).getParentFile();
		if(folder == null || !folder.exists()){
			logger.info("The folder "+folder+" does not exists. Will create it now");
			boolean created = folder.mkdirs();
			if (!created){
				throw new MetastaseException("Could not create the folder structure: "+folder+". Please created it manually and restart the simulation.");
			}
			
		}

		logger.info("Parameters are: config=" + config + ", technical="	+ technical);
	}

	/**
	 * Checks whether the given xml file path ends on ".xml". Throws a MetastaseException if not. 
	 * @param file the file to check
	 * @return the correct xml file path
	 * @throws a MetastaseException if the given file does not end on ".xml"
	 */
	private String getXMLFile(String file) throws MetastaseException {
		if (file.endsWith(".xml")) {
			return file;
		}else {
			throw new WrongArgumentsException("The configuration and technical description has to be a xml file!");
		}
	}

	/**
	 * The main method that starts and controls the simulation process.
	 * @throws MetastaseException
	 */
	public void run() throws MetastaseException {
		try {
			
			//get the event list
			theEventList = EventList.getInstance();
			MPMLParser parser = new MPMLParserImpl();
			
			Technical technicalData = null;

			// get the technical data
			technicalData = parser.getTechnical(technical);
			
			if (technicalData.isDebugMode()) {
				logger.getParent().setLevel(Level.INFO);
			} else {
				logger.getParent().setLevel(Level.WARNING);
			}
			logger.info("technicalData: " + technicalData);

			theEventList.setDeadline(technicalData.getDurationInMinutes());
			
			// get the compartment from the config file
			Compartment primaryTumor = parser.getCompartment(config);
			logger.fine("primaryTumor: " + primaryTumor);

			//currently we assume that the successor of the primary tumor is the bloodstream
			DiscreteCompartment bloodStream = primaryTumor.getSuccessor();
			bloodStream.setBloodStream(true);
			logger.fine("bloodStream: " + bloodStream);

			//get the host organs and create the excel writer
			ExcelWriter excelWriter = new ExcelWriter(parser.getHostOrgans());
			
			if (!excelWriter.isSheetNameAvailable(excelFileName, excelSheetName)) {
				logger.warning("Sheetname allready in use");
				throw new ExcelWriterException("Sheetname allready in use");
			}
			
			//create print events, where the actual status shall be plotted
			PrintStatus print = new PrintStatus("print data", primaryTumor, bloodStream, parser.getHostOrgans(), excelWriter);
			createPlotEvents(print, technicalData);
			
			//create treatment events
			createTreatmentEvents(primaryTumor);
			
			if(!primaryTumor.isDormant()){
				//only create the first event, to get the simulation run started, if the primary is not dormant
				primaryTumor.createFirstEvent(0.);
			}
			
			long stepCounter = 0;
			while (true) {

				if (theEventList.isEmpty()) {
					logger.info("eventList empty, terminating");
					break;
				}

				Event event = theEventList.nextEvent();
				double eventTime = event.getTime();
									
				// run simulation for the time specified in the technical data
				if (eventTime > (technicalData.getDurationInMinutes())) {
					break;
				}				
				
				//execute the event action
				event.execute();
				
				if (stepCounter % 10L == 0) {
					logger.info("at step " + stepCounter + " and time "	+ eventTime/24/60);
//					System.out.println("at step " + stepCounter + " and time "	+ eventTime/24/60);
				}
				stepCounter++;
				
			}// end of simulation run
			
			// print an info and close the buffered writer
			logger.info("terminated at step " + stepCounter);
			logger.info("finished sim run\n\n\n");

			excelWriter.writeAllBuffers(excelFileName, excelSheetName);

			logger.info("stopping SW\n\n");
			
			
		} catch (Throwable t) {
			logger.severe(t.getMessage());
			t.printStackTrace();
		}
	}


	/**
	 * Generated empty events at particular points in time. This points in time
	 * are needed for analyzing and plotting the simulation data.
	 * 
	 */
	private void createPlotEvents(PrintStatus printAction, Technical technicalData) {
		
		// we start at day 0
        double startTime = 0; 
        // and end at the specified day
        double endTime = technicalData.getDurationInMinutes(); 
        
        double timeStep = technicalData.getTimeStepsForSavingInMinutes();
        
        for (double t = startTime; t <= endTime; t += timeStep) {
            theEventList.addEvent(new GlobalEvent(printAction, t));
        }
	}
	
	private void createTreatmentEvents(Compartment primaryTumor){
		
		for (TreatmentEntry treatmentEntry : TreatmentList.getInstance()) {
			Treatment treatment = treatmentEntry.treatment;
			
			if(treatment instanceof ResectionTreatment){
				//only the primary tumor is resectable right now
				if(primaryTumor instanceof DiscreteCompartment){
					LocalEvent resection = new LocalEvent(new Resection("resection of the primary tumor", (ResectionTreatment)treatment), treatmentEntry.time, primaryTumor);
					theEventList.addEvent(resection);
				}
				//continuous compartment don't need an event. the resection is handled via the piecewise function at the creation of the compartment
				
			} else if(treatment instanceof ExternalBeamTreatment){
				//external beam radiation is only available for the primary tumor right now
				if(primaryTumor instanceof DiscreteCompartment){
					GlobalEvent ebr = new GlobalEvent(new ExternalBeamRadiation("External beam radiation of the primary tumor", 
														(ExternalBeamTreatment)treatment), treatmentEntry.time);
					theEventList.addEvent(ebr);
				}
				//continuous compartment don't need an event. the external beam radiation is handled via the piecewise function at the creation of the compartment
				
			} else if(treatment instanceof RadioembolizationTreatment){
				GlobalEvent ebr = new GlobalEvent(new Radioembolization("Radioembolization", (RadioembolizationTreatment)treatment), treatmentEntry.time);
				theEventList.addEvent(ebr);
				
			} else if(treatment instanceof RadioimmunoTreatment){
				GlobalEvent ebr = new GlobalEvent(new Radioimmunotherapy("Radioimmunotherapy", (RadioimmunoTreatment)treatment), treatmentEntry.time);
				theEventList.addEvent(ebr);
			
			} else if(treatment instanceof ChemoTreatment){
				//chemo therapy is only available for continuous compartments and they handle chemo therapy via the piecewise function during the creation of the compartment
				continue; 
			}
			
		}
	}
}
