/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.machine.eventlists;

import java.util.Iterator;

import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.machine.NoEventLeftException;

/**
 * Implementation of a Hendrisksen event list
 * @author Gero Wedemann, Anja Bethge
 *
 */
public class HenriksenEventList extends EventList {
	
	protected static class Entry {
		public Event event;
		public Entry left;
		public Entry right;
		public double time;
		
		//if the entry is a pointer, this value represents the index of the entry in the pointer vector
		//if the value is < 0, this entry is not a pointer
		public int pointerIndex = -1; 
		
		Entry(Event event, Entry left, Entry right, double time) {
			this.event = event;
			this.left = left;
			this.right = right;
			this.time = time;
		}
	}
	
	protected static final double MIN_VALUE = -9.9999999999999994E+038D;
	protected static final double MAX_VALUE = 9.9999999999999994E+038D;
	
	// the following is for unit-tests
	private static final int ARRAY_LENGTH_INIT = 2;
	
	private Entry firstEntry; //it's just a dummy entry to mark the start of the linked list
	private Entry lastEntry;
	private Entry pointerVector[]; //all pointers are stored in this array
	private int vectSize;
	private int arrayLength;
	
	// the time until which events shall be added to the list. Events with a time greater than 
	// the deadline are ignored and not sorted into the list.
	private double deadLine; 

	
	/**
	 * Initializes the list with two fake events at maxtime and mintime, marking the start and the end of the list.
	 */
	public HenriksenEventList()	{
		lastEntry = new Entry(null, null, null, MAX_VALUE); // a dummy entry to mark the end of the linked list
		firstEntry = new Entry(null, null, lastEntry, MIN_VALUE);
		lastEntry.left = firstEntry;
		arrayLength = ARRAY_LENGTH_INIT;
		pointerVector = new Entry[arrayLength];
		changeSize(1);
		pointerVector[0] = lastEntry;
		
		this.deadLine = Double.MAX_VALUE; 
	}
	
	
	/**
	 * Sets the deadline to the given value.
	 * @param deadLine the time in minutes until which events shall be added to the list. Events with a time greater than 
	 * the deadline are ignored and not sorted into the list.
	 */
	public void setDeadline(double deadLine){
		this.deadLine = deadLine;
	}

	/**
	 * @return true, if no events are left in the event list. false, otherwise
	 */
	public boolean isEmpty() {
		return firstEntry.right == pointerVector[0];
	}


	/**
	 * Sorts the given event into the event list.
	 */
	public void addEvent(Event event) {
		
		//return, if the time of the event lies after the deadline
		//since we will never process this event, we don't have to sort it into the list
		if(event.getTime() > deadLine) return;
		
		//get the left and right entries for the new event entry
		Entry leftEntry = findLeftEntry(event);
		Entry rightEntry = leftEntry.right;
		Entry newEntry = new Entry(event, leftEntry, rightEntry, event.getTime());
		//update the the left and right entries of the left and right entry of the new entry
		rightEntry.left = newEntry;
		leftEntry.right = newEntry;
	}


	/**
	 * Returns the first event from the event list and removes it from the list afterwards.
	 * @return the next event in line
	 */
	public Event nextEvent() throws NoEventLeftException {
		if (pointerVector[vectSize / 2].time <= firstEntry.right.time && vectSize > 2){
			changeSize(vectSize / 2);
		}
		
		Entry entry = firstEntry.right;
		
		if (entry == pointerVector[0]) {
			throw new NoEventLeftException();
		} else {
			//check whether the entry is a pointer
			if(entry.pointerIndex >= 0){
				//replace the pointer with it's predecessor (which is always the first entry) 
				//and update the pointer index
				pointerVector[entry.pointerIndex] = firstEntry;
				entry.pointerIndex = -1;
			}
			//update the left and right entries
			firstEntry.right = entry.right;
			entry.right.left = firstEntry;
			entry.right = null;
			entry.left = null;
			Event event = entry.event;
			entry.event = null;
			return event;
		}
	}


	/**
	 * Extends the entry vector to the given size. Does not change the array length, if the
	 * given size is smaller than the actual array length.
	 * @param i the new size of the entry vector
	 */
	private void changeSize(int i) {
		if (i > arrayLength) {
			//create a new entry vector and ...
			arrayLength=i;
			Entry newEntryVec[] = new Entry[ arrayLength ];
			//... copy all entries from the old entry vector to the new one
			for (int k = 0; k < vectSize; k++)
				newEntryVec[k] = pointerVector[k];

			//replace the old entry vector with the new one
			pointerVector = newEntryVec;
		}

		//fill up all empty slots with firstEntry (why?)
		for (int j = vectSize; j < i; j++){
			pointerVector[j] = firstEntry;
		}
		
		vectSize = i;
	}

	/**
	 * 
	 * @param d
	 * @return
	 */
	private int findIndex(double d) {
		int i = vectSize / 2;
		for (int j = vectSize / 4; j > 0; j /= 2){
			if (d >= pointerVector[i].time)
				i -= j;
			else
				i += j;
		}
		
		if (d >= pointerVector[i].time)	i--;
		
		return i;
	}

	/**
	 * 
	 * @param event
	 * @return
	 */
	private Entry findLeftEntry(Event event) {
		double d = event.getTime();
		
		int i = findIndex(d);
		Entry entry = pointerVector[i].left;
		int j = 0;
		
		for (; entry.time > d; entry = entry.left) {
			if (++j != 4)
				continue;
			if (i + 1 >= vectSize)
				changeSize(vectSize * 2);
			i++;
			j = 0;
			
			//update the pointer index (-1 = this entry is no longer a pointer)
			pointerVector[i].pointerIndex = -1;
			entry.pointerIndex = i;
			//update the pointer
			pointerVector[i] = entry;
		}
		return entry;
	}
	

	public Iterator<Event> iterator() {
		return new HenriksenIterator(this);
	}


	public Event removeEvent(Event event) {
		Entry entry = findLeftEntry(event);
		
		//if the returned entry has a different time than the given event, 
		//than that given event does not exist in the henriksen list
		//so we cannot remove it
		if(entry.time != event.getTime()){
			return null;
		}
		
		//if the returned entry's event is not the same as the given event, than
		//more than one event occur at the same time
		if(!entry.event.sameAs(event)){
			//go back in the linked list to find the correct entry
			do{
				entry = entry.left;
				if(entry.time != event.getTime()){
					return null;//if the time of the predecessor differs, than the given event does not exist in the henriksen list
				}
			}while(!entry.event.sameAs(event));
			
		}

		
		// check whether the entry is a pointer
		if (entry.pointerIndex >= 0) {
			
			Entry predecessor = entry.left;
			if(predecessor.pointerIndex>=0){
				//budge up all entries in the pointer vector, since we don't need two pointer pointing to the same entry
				budgeUpPointerVector(predecessor.pointerIndex);
			}
			// replace the pointer with it's predecessor (as 
			// described in the henriksen paper) and ...
			pointerVector[entry.pointerIndex] = predecessor;
			// ... update the pointer indexes
			predecessor.pointerIndex = entry.pointerIndex;
			entry.pointerIndex = -1;
		}
		// update the the left and right entries of the left and right entry of
		// the entry to remove
		Entry left = entry.left;
		Entry right = entry.right;
		
		left.right = entry.right;
		right.left = entry.left;
		
		Event e = entry.event;
		entry.event = null;
		
		return e;
	}
	
	private void budgeUpPointerVector(int index){
		
		//budge up all entries beginning with the given index
		for (int i=index; i<pointerVector.length-1; i++) {
			Entry newEntry = pointerVector[i+1];
			pointerVector[i] = newEntry;
			newEntry.pointerIndex = i;
		}
		
		//fill the last stop with the first entry
		pointerVector[pointerVector.length-1]=firstEntry;
	}
	
	
	/**
	 * Only used by the henriksen iterator.
	 * @return the dummy entry marking the start of the linked list
	 */
	protected Entry getFirstEntry() {
		return firstEntry;
	}
	
}