/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support;

import java.util.ArrayList;

/**
 * Generates distribution of numbers in buckets in a logarithmic fashion. 
 * 
 * @author Gero Wedemann
 */
public class LogSampleHistogram extends SampleStatistic{
	private ArrayList<HistogramEntry> buckets; 
	private int decades;
	private int bucketsPerDecade;
	
	public LogSampleHistogram(int decades, int bucketsPerDecade) {
		super();
		
		this.decades = decades;
		if (decades<1) {
			throw new IllegalArgumentException("decades must be >0"); 
		}
		this.bucketsPerDecade = bucketsPerDecade;
		if (bucketsPerDecade<1) {
			throw new IllegalArgumentException("buckets per decade must be >0");
		}
		assert (bucketsPerDecade==10); // ok, this is a bug
		
		buckets = new ArrayList<HistogramEntry>();
		
		double lastLimit = 0;
		double decadestep = 1./((double)bucketsPerDecade);
		for (double decade=0; decade<=decades; decade+=decadestep) {
			double nextLimit = Math.pow(10,decade);
			buckets.add(new HistogramEntry(lastLimit, nextLimit));
			lastLimit = nextLimit;
		}
		buckets.add(new HistogramEntry( lastLimit, Math.pow(10,decades) ) ); 
		buckets.add(new HistogramEntry(Math.pow(10,decades), Double.MAX_VALUE)); // Auffangbecken
	}

	@Override
	public void add(double val) {		
		super.add(val);
		for (HistogramEntry he: buckets) {
			if (he.isInLimits(val)) {
				he.add(val);
				break;
			}
		}
	}
	
	public ArrayList<HistogramEntry> getBuckets() {
		return buckets;
	}

	
	@Override
	public String toString() {
		return "LogSampleHistogram [buckets=" + buckets + ", decades=" + decades + ", bucketsPerDecade=" + bucketsPerDecade + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + buckets.hashCode();
		result = prime * result + bucketsPerDecade;
		result = prime * result + decades;
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj)) return false;	
		
		LogSampleHistogram other = (LogSampleHistogram) obj;
		return(decades == other.decades && 
				bucketsPerDecade == other.bucketsPerDecade &&
				buckets.equals(other.buckets));
	}
}
