/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support;

/**
 * Computes statistical values as mean and stddev from added values.
 * 
 * Adapted from GNU C++ stdlib
 * 
 * @author Gero Wedemann
 *
 */
public class SampleStatistic {
	
	private long n = 0; //number of entries in the bucket
	private double x = 0; //sum of all values in the bucket
	private double x2 = 0;
	private double minValue = 0; //lowest value in the bucket
	private double maxValue = 0; //highest value in the bucket

	public long getN() {
		return n;
	}
	
	public void add(double value) {
		n++;
		x += value;
		x2 += (value * value);
		if ( minValue > value)
			minValue = value;
		if ( maxValue < value)
			maxValue = value;
	}
	
	/**
	 * Computes the mean of all values in the bucket
	 * @return the mean
	 */
	public double mean() {
	    if ( n > 0) {
	        return (x / n);
	    } else {
	        return ( 0.0 );
	    }
	}
	
	/**
	 * Computes the variance.
	 * @return the variance
	 */
	public double var() {
	    if ( n > 1) {
	        return(( x2 - ((x * x) /  n)) / ( n - 1));
	    } else {
	        return ( 0.0 );
	    }
	}

	/**
	 * Computes the standard deviation.
	 * @return the standard deviation
	 */
	public double stdDev() {
	    if ( n <= 0 || var() <= 0) {
	        return(0.);
	    } else {
	        return( (double) Math.sqrt( var() ) );
	    }
	}


	
	@Override
	public String toString() {
		return "SampleStatistic [n=" + n + ", x=" + x + ", x2=" + x2 + ", minValue=" + minValue + ", maxValue=" + maxValue + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(maxValue);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(minValue);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + (int) (n ^ (n >>> 32));
		temp = Double.doubleToLongBits(x);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(x2);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (this.getClass() != obj.getClass()) return false;		
		
		SampleStatistic other = (SampleStatistic) obj;
		return (minValue == other.minValue && maxValue == other.maxValue &&
				n == other.n && x == other.x && x2 == other.x2);
	}
	
	
}
