/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

/**
 * computes g(x) = m * x(t)^alpha
 * (its the colonization rate beta(x) = m * x^alpha from the Iwata Paper)
 * 
 * @author Gero Wedemann
 * @author Anja Bethge
 */
public class ColonizationRate implements Function{
	
	private double m;
	private double alpha;
	private PiecewiseGrowthFunction growthFunction;
	
	/**
	 * 
	 * @param m the colonization rate Unit: 1/(cell * day)
	 * @param alpha the fractal dimension of blood vessels (how good is the tumor supplied with blood)
	 */
	public ColonizationRate(double m, double alpha) {
		super();
		this.m = m;
		this.alpha = alpha;
		this.growthFunction = null;
	}
	
	/**
	 * Only used for cloning.
	 */
	private ColonizationRate(double m, double alpha, PiecewiseGrowthFunction growthFunction){
		super();
		this.m = m;
		this.alpha = alpha;
		this.growthFunction = growthFunction;
	}

	public double getM(){
		return m;
	}
	
	public double getAlpha(){
		return alpha;
	}
	
	public double f(double x) {
		if(growthFunction == null){
			throw new IllegalStateException("The growth function is 'null'. You have to set the growth function via setGrowthFunction() before you can call f().");
		}
		return m * Math.pow( growthFunction.f(x), alpha );
	}
	

	public double f_FirstDerivation(double x) {
		if(growthFunction == null){
			throw new IllegalStateException("The growth function is 'null'. You have to set the growth function via setGrowthFunction() before you can call f_FirstDerivation().");
		}
		// Chain rule: inner * outer
		return growthFunction.f_FirstDerivation(x) *  m*alpha * Math.pow( growthFunction.f(x), alpha-1);
	}
	
	public void setGrowthFunction(PiecewiseGrowthFunction growthFunction) {
	    this.growthFunction = growthFunction;
	}

	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   

	public ColonizationRate copy() {
		ColonizationRate clone = new ColonizationRate(m, alpha, growthFunction);
		return clone;
	}
	
	@Override
	public String toString() {
		return "ColonisationRate ["+ m + " * " + growthFunction + "^" + alpha + "]";
	}
	
	@Override
	public int hashCode() {
		final int prime = 91;
		int result = 1;
		result = prime * result + ((growthFunction == null) ? 0 : growthFunction.hashCode());
		long temp;
		temp = Double.doubleToLongBits(m);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(alpha);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
        if (obj == null) return false;
        if (this == obj) return true;
        if(this.getClass() != obj.getClass()) return false;
        
        ColonizationRate pFunc = (ColonizationRate) obj;
        return (this.alpha == pFunc.alpha && 
                this.m == pFunc.m &&
                ((growthFunction == null)?pFunc.growthFunction == null : growthFunction.equals(pFunc.growthFunction)));
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
}
