/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.distributions.UniformDistribution;

/**
 * Util class that copies the given function and replaces the parameter that models the start size with the given value.
 * Furthermore, it provides a functions to convert time values from day to minute
 * @author Anja Bethge
 *
 */
public class FunctionUtil {
	
	/**
	 * Copies the given function and replaces the parameter that models the start size with the given value.
	 * Supports only normal growth functions. Hence, constant, linear, exponential, logarithmic, power and gompertz function.
	 * @param function the function to copy
	 * @param startSize the new value for the start size parameter
	 * @return the copied function or null if the given function type is not supported.
	 */
	public static Function copyGrowthFunctionWithNewStartSize(Function function, long startSize){
		
		//constant function
		if(function instanceof ConstantFunction){
			return new ConstantFunction(startSize);
		
		//linear function
		}else if(function instanceof LinearFunction){
			LinearFunction linear = (LinearFunction)function;
			return new LinearFunction(linear.getM(), startSize);
		
		//exponential function
		}else if(function instanceof ExponentialFunction){
			ExponentialFunction expo = (ExponentialFunction)function;
			return new ExponentialFunction(startSize, expo.getB(), 0);
		
		//log function
		}else if(function instanceof LogFunction){
			LogFunction log = (LogFunction)function;
			return new LogFunction(log.getA(), log.getB(), startSize);

		//power function
		}else if(function instanceof PowerFunction){
			PowerFunction pow = (PowerFunction)function;
			return new PowerFunction(pow.getA(), pow.getB(), startSize);

		//gompertz function
		}else if(function instanceof GompertzFunction){
			GompertzFunction gom = (GompertzFunction)function;
			return new GompertzFunction(gom.getA(), gom.getB(), startSize);
		}
		
		
		return null;
	}
	
	/**
	 * Converts the time dependent parameters from days to minutes
	 * @param function
	 * @return a copy of the given function with all time dependent parameters converted to minutes (p/24/60).
	 */
	public static Function convertToMinutes(Function function){
		
		// constant function
		if (function instanceof ConstantFunction) {
			//to time dependent parameters
			return function;

		// linear function
		} else if (function instanceof LinearFunction) {
			LinearFunction linear = (LinearFunction) function;
			//m represents the growth rate
			return new LinearFunction(linear.getM()/24/60, linear.getN());

		// exponential function
		} else if (function instanceof ExponentialFunction) {
			ExponentialFunction expo = (ExponentialFunction) function;
			//b represents the growth rate
			return new ExponentialFunction(expo.getA(), expo.getB()/24/60, expo.getC());

		// log function
		} else if (function instanceof LogFunction) {
			LogFunction log = (LogFunction) function;
			
			if(log.getN() != null){
				//b represents the growth rate
				return new LogFunction(log.getA(), log.getB()/24./60., log.getN());
			}else{
				//if n == null the log function cannot be used as a growth function
				return null;
			}

		// power function
		} else if (function instanceof PowerFunction) {
			PowerFunction pow = (PowerFunction) function;
			//b represents the growth rate
			return new PowerFunction(pow.getA(), pow.getB()/24/60, pow.getC());

		// gompertz function
		} else if (function instanceof GompertzFunction) {
			GompertzFunction gom = (GompertzFunction) function;
			//b represents the growth rate constant
			return new GompertzFunction(gom.getA()/24/60, gom.getB(), gom.getN());
		} 		

		return null;
	}
	
	/**
	 * Converts the time dependent parameters from days to minutes
	 * @param function the ColonizationRate
	 * @return a copy of the given colonization rate with all time dependent parameters converted to minutes (p/24/60).
	 */
	public static ColonizationRate convertToMinutes(ColonizationRate function){

		//colonizationRate
		if (function instanceof ColonizationRate){
			ColonizationRate rate = (ColonizationRate) function;
			//m is time dependent (colonization rate)
			//convert from 1/(cell*day) to 1/(cell*minute)
			return new ColonizationRate(rate.getM()/24/60, rate.getAlpha());
		}
		
		return null;
	}
	
	/**
	 * Converts the time dependent parameters from days to minutes
	 * @param distribtion
	 * @return a copy of the given distribution with all time dependent parameters converted to minutes (p*24*60).
	 */
	public static Distribution convertToMinutes(Distribution distribtion){
		
		// gaussian distribution
		if (distribtion instanceof GaussianDistribution) {
			GaussianDistribution gauss = (GaussianDistribution) distribtion;
			return new GaussianDistribution(gauss.getMean()*24*60, gauss.getStddev()*24*60);

		// uniform distribution
		} else if (distribtion instanceof UniformDistribution) {
			UniformDistribution uni = (UniformDistribution) distribtion;
			return new UniformDistribution(uni.getMin()*24*60, uni.getMax()*24*60);		
		
		}

		return distribtion;
	}

}
