/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

/**
 * Its an extended version of the Gompertz growth function with the boundary condition x(0)=n 
 * (The tumor starts with a number of cells n at time t=0)
 * 
 * f(x) = b * (b/n)^-e^(-a*t)
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class GompertzFunction implements Function {
	
	private double a;
	private double b;
	private double n;
	
	/**
	 * 
	 * @param a the growth rate. Unit: 1/day
	 * @param b the max size of the tumour
	 */
	public GompertzFunction(double a, double b) {
		this(a, b, 1);
	}
	
	/**
	 * 
	 * @param a the growth rate. Unit: 1/day
	 * @param b the max size of the tumour
	 * @param n the start size of the tumour
	 */
	public GompertzFunction(double a, double b, double n) {
		if(a <= 0 || b <= 0 || n < 1){
			throw new IllegalArgumentException("Growth rate and maximal tumor size must not be smaller or equal than zero. The start size must not be smaller than 1.");
		}
		
		this.a = a;
		this.b = b;
		this.n = n;
	}

	public double f(double x) {
		double part1 = b/n; 
		double part2 = -1.0*Math.exp(-1.0*a*x);
		
		return b * Math.pow(part1, part2);
	}

	public double f_FirstDerivation(double x) {
		// f'(x)=a*b*(ln(b)-ln(n))*e^(-at-e^(-at)*(ln(b)-ln(n)))
		double z = -1 * a * x;
		double v = Math.log( b ) - Math.log( n );
		return a * b * v * Math.exp( z - (Math.exp( z ) * v ));
	}
	
	public double getA() {
		return a;
	}
	
	public double getB() {
		return b;
	}
	
	public double getN() {
		return n;
	}

	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
    @Override
    public String toString() {
    	return "GompertzFunction ["+b+" * e^((ln("+b+")-ln("+n+"))*-e^-"+a+"*t) ]";
    }
    
	@Override
	public int hashCode() {
		final int prime = 77;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(a);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(b);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(n);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
        if (obj == null) return false;
        if (this == obj) return true;
        if(this.getClass() != obj.getClass()) return false;

        GompertzFunction gFunc = (GompertzFunction) obj;
        return (this.a == gFunc.a && 
                this.b == gFunc.b && 
                this.n == gFunc.n);
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}
