package de.fhstralsund.catsit.support.functions;


/**
 * Evaluates the function f(x) = a * ln ( b * x ) 
 * 
 * 
 * If used as a growth function, the following form is used:
 * f(x) = a * ln (b * x + e^(n / a))<br><br>
 *  
 * where
 * <li> <i>b</i> displays the growth rate. Unit: 1/day
 * <li> <i>n</i> displays the start size of the tumor and must not be smaller than 1
 * <li> in metastases <i>n</i> must be 1, since all metastases start as a single cell
 * <br><br>
 * 
 * @author Gero Wedemann
 * @author Anja Bethge
 * @author Michael Suhr
 */
public class LogFunction implements Function {
	
	private double a;
	private double b;
	private Long n;
	
	public LogFunction(double a, double b) {
		if(b <= 0){
			throw new IllegalArgumentException("The value of 'b' must not be smaller or equal than '0'");
		}
		
		this.a = a;
		this.b = b;
		this.n = null;
	}
	
	public LogFunction(double a, double b, long n) {
		if(b <= 0){
			throw new IllegalArgumentException("The value of 'b' must not be smaller or equal than '0'");
		}
		if(a == 0){
			throw new IllegalArgumentException("The value of 'a' must not be '0'");
		}
		
		this.a = a;
		this.b = b;
		this.n = new Long(n);
	}


	public double f(double x) {
		if(n!=null){
			return a * Math.log( b * x + (Math.exp(n/a)) );
		}else{			
			return a * Math.log( b * x );
		}
		
	}

	public double f_FirstDerivation(double x) {
		if(n!=null){
			//f'(x) = (a * b) / ( b * x + e^(n/a) )
			return (a*b) / (b*x + Math.exp(n/a));
		}else{
			//f'(x) = a / x
			return a/x;
		}
	}

    public double getA() {
		return a;
	}

	public double getB() {
		return b;
	}

	public Long getN() {
		return n;
	}

	
	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
	@Override
	public String toString() {
		if(n!=null){
			return "LogFunction ["+a+" * ln ( "+b+" * x + e^("+n+" / "+a+") ) ]";
		}else{			
			return "LogFunction ["+a+" * ln ( "+b+" * x ) ]";
		}
	}
	
    @Override
    public int hashCode() {
        int result = 73;
        long temp = Double.doubleToLongBits(a);
        result = 30 * result + (int)(temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(b);
        result = 7 * result + (int)(temp ^ (temp >>> 32));
        result = (n!=null)? 61 * result + (int)(Double.doubleToLongBits(n) ^ (Double.doubleToLongBits(n) >>> 32)) : 85 * result ;
        return result;
    }
    
	@Override
    public boolean equals(Object obj) {
        if (obj == null) return false;
        if (this == obj) return true;
        if(this.getClass() != obj.getClass()) return false;
        
        LogFunction func = (LogFunction) obj;
        
        return (this.a == func.a && 
                this.b == func.b &&
                ((n!=null)? this.n.equals(func.n) : func.n==null) );
    }
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
}
