/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;


/**
 * Models the changing growing behavior of a continuous compartment.
 * @author Anja Bethge
 *
 */
public class PiecewiseGrowthFunction implements Function {
	
	private class Section{
		public double time;
		public Function function;
		public Section next;
		
		public Section(double time, Function f, Section next){
			this.time = time;
			this.function = f;
			this.next = next;
		}
		
		@Override
		public String toString() {
			return "Section [Time: "+time + ", Function: "+function+"]";
		}
	}

	private Section firstSection;

	/**
	 * 
	 * @param function the first section/Function of the PiecewiseGrowthFunction 
	 */
	public PiecewiseGrowthFunction(double time, Function function){
		firstSection = new Section(time, function, null);
	}
	
	
	public double getFirstTimePoint(){
		return firstSection.time;
	}
	
	
	/**
	 * Adds the given Function to the piecewise function and sorts it into the list according to the given time
	 * @param time the time from with the function shall be valid. must be bigger than 0
	 * @param function the function to add
	 */
	public void addFunction(double time, Function function) {
		
		if (time <= firstSection.time) throw new IllegalArgumentException("The time must be bigger than the creation time "+firstSection.time+" but was "+ time +".");
		if (function == null) throw new IllegalArgumentException("The function must no be null");
		
		Section currentValidSection = findCurrentValidSection(time);
		Section newSection = new Section(time, function, currentValidSection.next);
		currentValidSection.next = newSection;		
	}
	
	public double f(double x) {
		Section s = findCurrentValidSection(x);
		return s.function.f(x-s.time);
	}

	public double f_FirstDerivation(double x) {
		Section s = findCurrentValidSection(x);
		return s.function.f_FirstDerivation(x-s.time);
	}
	
	public Function getCurrentValidFunction(double time) {
		return findCurrentValidSection(time).function;
	}

	public double getTimeOfCurrentValidSection(double time) {
		return findCurrentValidSection(time).time;		
	}
	
	public double getTimeOfNextValidFunction(double time) throws NoNextSectionException{
		Section currentValidSection = findCurrentValidSection(time);
		
		if(currentValidSection.next == null){
			throw new NoNextSectionException("No next section avaliable for the current section: "+currentValidSection);
		}else{
			return currentValidSection.next.time;
		}
		
	}
	
	/**
	 * Deletes all sections from the piecewise function that occur after the given time.
	 * @param time 
	 */
	public void deleteAllFollowingSections(double time) {
		Section currentValidSection = findCurrentValidSection(time);
		currentValidSection.next = null;
	}
	
	
	/**
	 * Finds the actual valid section according to the given time.
	 * @param time the current global time
	 * @return the actual valid section
	 */
	private Section findCurrentValidSection(double time) {
		if (time < firstSection.time) 
			throw new IllegalArgumentException("There is no current valid section at the time "+time+". "
					+ "The first section becomes valid at time " + firstSection.time+".");
		
		Section currentSection = firstSection;
		while(true){
			if(currentSection.time <= time && (currentSection.next == null || currentSection.next.time > time)){
				//either no sections are left in the list or the next section's time is bigger than the given one
				//=> currentSection is the actual valid one
				return currentSection;
				
			} else{
				//continue with the next section
				currentSection = currentSection.next;
			}
		}		
	}
	
	@Override
	public String toString() {
		String s = "";
		Section currentSection = firstSection;
		do{
			s += currentSection.time + ": " +currentSection.function+" - ";
			currentSection = currentSection.next;
		}while(currentSection != null);
		
		return s;
	}


	@Override
	public int hashCode() {
		final int prime = 21;
		int result = 1;
		
		Section cS = firstSection;
		do{
			long temp = Double.doubleToLongBits(cS.time);
			result = prime * result + + (int) (temp ^ (temp >>> 32));
			result = prime * result	+ cS.function.hashCode();
			
			cS = cS.next;
		}while(cS != null);		
		
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		PiecewiseGrowthFunction other = (PiecewiseGrowthFunction) obj;
		
		Section cS = firstSection;
		Section otherCS = other.firstSection;
		do{
			if(cS.time != otherCS.time || !cS.function.equals(otherCS.function)) return false;
			else {
				cS = cS.next;
				otherCS = otherCS.next;
				
				if((cS == null && otherCS != null) || (cS != null && otherCS == null)) return false;
			}
		}while(cS != null);		
		return true;
	}
}
