/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

/**
 * Computes f(x)= a*x^b + c
 * 
 * 
 * If used as a growth function, please notice:<br>
 * <li> <i>b</i> displays the growth rate. Unit: 1/day
 * <li> <i>a</i> serves as a scaling factor
 * <li> <i>c</i> displays the start size of the tumour and must not be smaller than 1
 * <li> in metastases <i>c</i> must be 1, since all metastases start as a single cell
 * <br><br>
 * 
 * @author Anja Bethge, Michael Suhr
 *
 */
public class PowerFunction implements Function {

	private double a;
	private double b;
	private double c;
	
	/**
	 * a*x^b + c
	 * @param a multiplication factor
	 * @param b power
	 * @param c addition factor
	 */
	public PowerFunction(double a, double b, double c){
		this.a = a;
		this.b = b;
		this.c = c;
	}
	
	
	public double f(double x) {
		return a * Math.pow(x, b) + c;
	}
	
	public double f_FirstDerivation(double x) {
		// f'(x)= a*b *x^(b-1)
		return a*b * Math.pow(x, b-1);
	}

	
	/**
	 * Needed for TestCase for EventGenerator
	 * @return
	 */
	public double getA() {
		return a;
	}

	/**
	 * Needed for TestCase for EventGenerator
	 * @return
	 */
	public double getB() {
		return b;
	}
	
	public double getC() {
		return c;
	}

	//////////// toString(), equals() & hashCode() ////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////   
	@Override
	public String toString() {
		return "PowerFunction ["+a+"*x^"+b+"+"+c+" ]";
	}
	
	@Override
	public int hashCode() {
		final int prime = 47;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(a);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(b);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(c);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}
    
    @Override
    public boolean equals(Object obj) {
        if (obj == null) return false;
        if (this == obj) return true;
        if(this.getClass() != obj.getClass()) return false;
        
        PowerFunction func = (PowerFunction) obj;
        return (this.a == func.a &&
                this.b == func.b &&
                this.c == func.c);
    }
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}
