/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.chemo;

import de.fhstralsund.catsit.support.functions.Function;

/**
 * The basic class of all chemotherapy functions. Defines the parameter for the drug concentration.
 * 
 * 
 * @author Anja Bethge
 *
 */
public abstract class BasicChemoFunction implements Function {

	private double drugDecayRate;			//decay rate of drug
	private double drugSensitivity;			//cell kill parameter
	private double cyclingCellsFraction;	//amount of cycling cells [0;1]
	
	private long chemoStartSize;			//number of cells at the beginning of chemo cycle
	
	//delay if the chemo starts influencing the tumor later than the start of the cycle (due to dormancy or creation of a new metastasis) 
	// this value describes the time span between the start of the chemo cycle and the time the chemo starts effecting the tumor
	private double delay;	
	

	/**
	 * 
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraction amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
	 * @param delay if the chemo starts influencing the tumor later than the start of the cycle (due to dormancy or creation of a new metastasis) 
	 * this value describes the time span between the start of the chemo cycle and the time the chemo starts effecting the tumor
	 */
	public BasicChemoFunction(long chemoStartSize, double cyclingCellsFraction, double drugDecayRate, double drugSensitivity, double delay) {
			
		this.drugDecayRate = drugDecayRate;
		this.drugSensitivity = drugSensitivity;
		this.cyclingCellsFraction = cyclingCellsFraction;
		
		this.chemoStartSize = chemoStartSize;
		this.delay = delay;
	}

	
	protected double getDrugDecayRate(){
		return drugDecayRate;
	}
	
	protected double getDrugSensitivity(){
		return drugSensitivity;
	}
	
	protected long getChemoStartSize(){
		return chemoStartSize;
	}
		
	protected double getCyclingCellsFraction(){
		return cyclingCellsFraction;
	}	
	
	public double getDelay() {
		return delay;
	}
	
	public void setDelay(double delay){
		this.delay = delay;
	}

	/**
	 * 
	 * @return the minimum of this function or null if no minimum exists
	 */
	abstract public Double getMinimum();
	
	/**
	 * Computes the time when the drug concentration in the system is so low, that is has practically no influence on the system anymore.
	 * 
	 * @return the time the drug has practically no influence on the system anymore
	 */
	public double getTimeOfDrugDecay() {
		//since the drug concentration won't reach zero due to the e-function we compute the time when the influence of the drug 
		//reaches the inverse value of the number of cells at the start of the chemo
		return (Math.log(cyclingCellsFraction * drugSensitivity * chemoStartSize)/drugDecayRate) - delay;
	}
	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 5;
		long temp;
		temp = Double.doubleToLongBits(drugDecayRate);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(drugSensitivity);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(chemoStartSize);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(cyclingCellsFraction);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(delay);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		
		BasicChemoFunction other = (BasicChemoFunction) obj;
		
		return(drugDecayRate == other.drugDecayRate && drugSensitivity == other.drugSensitivity && 
				cyclingCellsFraction == other.cyclingCellsFraction && chemoStartSize == other.chemoStartSize &&
				delay == other.delay);
	}	
	
}
