/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.chemo;


/**
 * This function describes the influence of a chemotherapy. 
 * A constant cell number was used as the basic "growth" rate.
 * 
 * @author Anja Bethge
 *
 */
public class ConstantChemoFunction extends BasicChemoFunction {	
	
	/**
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit: 1/day 
	 */
	public ConstantChemoFunction(long chemoStartSize, double cyclingCellsFraktion, double drugDecayRate, double drugSensitivity) {
		this(chemoStartSize, cyclingCellsFraktion, drugDecayRate, drugSensitivity, 0.);
	}
	
	/**
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
 	 * @param delay if the chemo starts influencing the tumor later than the start of the cycle (due to dormancy or creation of a new metastasis) 
	 * this value describes the time span between the start of the chemo cycle and the time the chemo starts effecting the tumor
	 */
	public ConstantChemoFunction(long chemoStartSize, double cyclingCellsFraktion, double drugDecayRate, double drugSensitivity, double delay) {
		super(chemoStartSize, cyclingCellsFraktion, drugDecayRate, drugSensitivity, delay);
	}
	
	
	public double f(double x) {
		double t = x;
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double n = getChemoStartSize();
		double l = getDelay();
		
		double z = c * s * Math.exp(-1.*l*d) * (Math.exp(-1.*t*d) - 1);
		return n * Math.exp(z/d);
	}

	public double f_FirstDerivation(double x) {
		double t = x;
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double n = getChemoStartSize();
		double l = getDelay();
		
		double z = c * s * Math.exp(-1.*l*d) * (Math.exp(-1.*t*d) - 1);
		return -1. * n * c * s * Math.exp((z/d) - d*(t+l));
	}	
	
	@Override
	public Double getMinimum() {
		//the constant chemo has no minimum, but a limit to which it converges
		//this limit is only interesting if it is smaller 1 (=> tumor dies during treatment), otherwise we don't care
		double limit = getChemoStartSize() * Math.exp( (-1. * getCyclingCellsFraction() * getDrugSensitivity() * Math.exp(-1.*getDrugDecayRate()*getDelay())) 
				/ getDrugDecayRate() );
		
		if(limit >= 1) {
			return null; //since the tumor doesn't die during the chemo, we don't care about the limit
			
		}else{			
			if( f(0)>=1 ){//make sure, the function starts above the limit
				//since no analytical solution is available we compute the time point for the limit (I chose 0.9999999) numerically via Newton method
				long counter=0;
				double oldValue = 0;
				double newValue = 0;
				do{	
					//the testcases took under 10 iteration to compute the time point with a precision of 1e-10, so 1000 iteration should be enough 
					if(counter > 1000){
						//not accurate enough after 1000 iterations? => seems like the function does not reach this value
						return null;
					}
					oldValue=newValue;
					newValue = oldValue-((f(oldValue)-0.9999999)/f_FirstDerivation(oldValue)); 
					counter++;
				}while((Math.abs(newValue-oldValue)>1e-10));
				
				return newValue;
			}else{
				return null;
			}	
		}
	}


	@Override
	public String toString() {
		return "ConstantCycleNonSpecificChemoFunction [CyclingCellsFraktion=" + getCyclingCellsFraction() 
				+ ", DrugDecayRate=" + getDrugDecayRate() + ", DrugSensitivity=" + getDrugSensitivity()
				+ ", ChemoStartSize=" + getChemoStartSize() + "]";
	}
	
}
