/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.chemo;


/**
 * This function describes the influence of a chemotherapy.
 * An exponential growth was used as the basic growth rate.
 * 
 * @author Anja Bethge
 *
 */
public class ExponentialChemoFunction extends BasicChemoFunction {

	private double b;		//growth rate of tumour
		
		
	/**
	 * 
	 * @param b growth rate of the tumour. Unit 1/day
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
	 */
	public ExponentialChemoFunction(double b, long chemoStartSize, double cyclingCellsFraktion, double drugDecayRate, double drugSensitivity) {
		this(b, chemoStartSize, cyclingCellsFraktion, drugDecayRate, drugSensitivity, 0.);
	}
	
	/**
	 * 
	 * @param b growth rate of the tumour. Unit 1/day
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
	 * @param delay if the chemo starts influencing the tumor later than the start of the cycle (due to dormancy or creation of a new metastasis) 
	 * this value describes the time span between the start of the chemo cycle and the time the chemo starts effecting the tumor
	 */
	public ExponentialChemoFunction(double b, long chemoStartSize, double cyclingCellsFraktion, double drugDecayRate, double drugSensitivity, double delay) {
		super(chemoStartSize, cyclingCellsFraktion, drugDecayRate, drugSensitivity, delay);
		this.b = b;

	}
	
	public double getB(){
		return b;
	}
	

	public double f(double x) {
		double t = x;
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double n = getChemoStartSize();
		double l = getDelay();
		
		double z = ( c * s * (Math.exp(t*d) - 1) * Math.exp(-1.*d*(t+l)) );
		return n * Math.exp((b * t) - (z/d));
	}

	public double f_FirstDerivation(double x) {
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double l = getDelay();
		
		return f(x) * (b - (c*s*Math.exp(-1.*d* (x + l) )) );
	}

	
	@Override
	public Double getMinimum() {				
		double minimum = ( (-1.*getDrugDecayRate()*getDelay()) + Math.log(( getCyclingCellsFraction()*getDrugSensitivity() ) / b) ) / getDrugDecayRate();
		
		//if the minimum lies before the start of the chemo then is has no influence => return null
		if(minimum < 0.) return null;
		else return minimum;
	}

	@Override
	public String toString() {
		return "ExponentialCycleNonSpecificChemoFunction [b=" + b + ", cyclingCellsFraktion=" + getCyclingCellsFraction() 
				+ ", drugDecayRate=" + getDrugDecayRate() + ", drugSensitivity=" + getDrugSensitivity()
				+ ", n0=" + getChemoStartSize() + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(b);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;

		ExponentialChemoFunction other = (ExponentialChemoFunction) obj;
		
		return (b == other.b);
	}
	
}
