/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.chemo;

import de.fhstralsund.catsit.support.functions.BasicFunction;
import de.fhstralsund.catsit.support.functions.chemo.derivations.BasicChemoFunctionDerivation;
import de.fhstralsund.catsit.support.functions.chemo.derivations.GompertzChemoFunctionFirstDerivation;
import de.fhstralsund.catsit.support.functions.chemo.derivations.GompertzChemoFunctionSecondDerivation;

/**
 * This function describes the influence of a chemotherapy.
 * The Gompertz growth was used as the basic growth rate.
 * 
 * @author Anja Bethge
 *
 */
public class GompertzChemoFunction extends BasicChemoFunction {

	private double a;		//growth rate
	private double b;		//max size of the tumour
	
	/**
	 * 
	 * @param a growth rate of the tumour. Unit 1/day
	 * @param b max size of the tumour
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
	 */
	public GompertzChemoFunction(double a, double b, long chemoStartSize, 
			double cyclingCellsFraction, double drugDecayRate, double drugSensitivity) {
		this(a, b, chemoStartSize, cyclingCellsFraction, drugDecayRate, drugSensitivity, 0.);
	}
	
	/**
	 * 
	 * @param a growth rate of the tumour
	 * @param b max size of the tumour
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug
	 * @param drugSensitivity cell kill parameter
	 * @param delay if the chemo starts influencing the tumor later than the injection (due to dormancy or creation of a new metastasis) this value describes the time span 
	 * between the start of the chemo cycle and the time the chemo starts effecting the tumor
	 */
	public GompertzChemoFunction(double a, double b, long chemoStartSize, 
			double cyclingCellsFraction, double drugDecayRate, double drugSensitivity, double delay) {
		super(chemoStartSize, cyclingCellsFraction, drugDecayRate, drugSensitivity, delay);
		this.a = a;
		this.b = b;
	}


	public double f(double x) {
		double t = x;
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double n = getChemoStartSize();
		double l = getDelay();
		
		double z = -1.*c*s * Math.exp( (-1.*a*t) - (l*d) ) * (Math.exp(t*(a-d))-1.);
		return b * Math.pow((b/n), -1.*Math.exp(-1.*a*t)) * Math.exp(z/(a-d));
	}

	public double f_FirstDerivation(double x) {
		double t = x;
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double n = getChemoStartSize();
		double l = getDelay();
		
		double o = a-d;
		double p = b/n;
		double q = l*d;
		
		double part1 = -1.*a*t - q - (c*s*Math.exp(-1.*a*t - q)*(Math.exp(t*o) - 1))/o;
		double part2 = c*s*(-1.*a + d*Math.exp(t*o)) + a*o*Math.exp(q)*Math.log(p);
		
		return b/o * Math.exp(part1) * Math.pow(p, -1.*Math.exp(-1.*a*t)) * part2; 
	}

	public double getA() {
		return a;
	}


	public double getB() {
		return b;
	}

	
	@Override
	public Double getMinimum() {
		BasicChemoFunctionDerivation firstD = new GompertzChemoFunctionFirstDerivation(a, b, getCyclingCellsFraction(), getDrugDecayRate(), 
				getDrugSensitivity(), getChemoStartSize(), getDelay());
		BasicFunction secondD = new GompertzChemoFunctionSecondDerivation(a, b, getCyclingCellsFraction(), getDrugDecayRate(), 
				getDrugSensitivity(), getChemoStartSize(), getDelay());
		
		double minimum = firstD.findRoot(secondD, 0);//start searching for the minimum at the start of the chemo
		
		//if the minimum lies before the start of the chemo then is has no influence => return null
		if(minimum < 0.) return null;
		else return minimum;
	}


	@Override
	public String toString() {
		return "GompertzCycleNonSpecificChemoFunction [a=" + a + ", b=" + b
				+ ", cyclingCellsFraktion=" + getCyclingCellsFraction() + ", drugDecayRate=" + getDrugDecayRate() + ", drugSensitivity=" + getDrugSensitivity()
				+ ", chemoStartSize=" + getChemoStartSize() + "]";
	}


	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(a);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(b);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}


	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj)) return false;


		GompertzChemoFunction other = (GompertzChemoFunction) obj;
		return (a == other.a && b == other.b);
	}	
}
