/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.chemo;

import numal.Special_functions;
import de.fhstralsund.catsit.support.functions.BasicFunction;
import de.fhstralsund.catsit.support.functions.chemo.derivations.BasicChemoFunctionDerivation;
import de.fhstralsund.catsit.support.functions.chemo.derivations.LinearChemoFunctionFirstDerivation;
import de.fhstralsund.catsit.support.functions.chemo.derivations.LinearChemoFunctionSecondDerivation;

/**
 * This function describes the influence of a chemotherapy.
 * A linear growth was used as the basic growth rate.
 * 
 * @author Anja Bethge
 *
 */
public class LinearChemoFunction extends BasicChemoFunction {

	private double m;		//growth rate of tumour
	
	private LinearChemoFunctionFirstDerivation firstDerivation;
	
	
	/**
	 * 
	 * @param m growth rate of the tumor. Unit 1/day
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
	 */
	public LinearChemoFunction(double m, long chemoStartSize, double cyclingCellsFraktion, double drugDecayRate, double drugSensitivity) {
		this(m, chemoStartSize, cyclingCellsFraktion, drugDecayRate, drugSensitivity, 0.);
	}
	
	/**
	 * 
	 * @param m growth rate of the tumour
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param cyclingCellsFraktion amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug
	 * @param drugSensitivity cell kill parameter	 
	 * @param delay if the chemo starts influencing the tumor later than the start of the cycle (due to dormancy or creation of a new metastasis) 
	 * this value describes the time span between the start of the chemo cycle and the time the chemo starts effecting the tumor
	 */
	public LinearChemoFunction(double m, long chemoStartSize, double cyclingCellsFraktion, double drugDecayRate, double drugSensitivity, double delay) {
		super(chemoStartSize, cyclingCellsFraktion, drugDecayRate, drugSensitivity, delay);
		this.m = m;
	}
	
	public double getM(){
		return m;
	}
	
	public double f(double x) {
		double t = x;
		double c = getCyclingCellsFraction();
		double s = getDrugSensitivity();
		double d = getDrugDecayRate();
		double n = getChemoStartSize();
		double l = getDelay();
		
		double z = (c*s*Math.exp(-1.*l*d))/d;
		double v = Math.exp(-1.*t*d);
		
		return (1/d) * Math.exp(z * (v-1)) * ( (n * d) + (Math.exp(z) * m * (Special_functions.ei(-1.*z) - Special_functions.ei(-1.*z*v)) ) );
	}

	public double f_FirstDerivation(double x) {
		if(firstDerivation == null){
			firstDerivation = new LinearChemoFunctionFirstDerivation(m, getCyclingCellsFraction(), getDrugDecayRate(), getDrugSensitivity(), getChemoStartSize(), getDelay());
		}
		return firstDerivation.f(x);
	}

	
	@Override
	public Double getMinimum() {
		BasicChemoFunctionDerivation firstD = new LinearChemoFunctionFirstDerivation(m, getCyclingCellsFraction(), getDrugDecayRate(), 
				getDrugSensitivity(), getChemoStartSize(), getDelay()); 
		BasicFunction secondD = new LinearChemoFunctionSecondDerivation(m, getCyclingCellsFraction(), getDrugDecayRate(), 
				getDrugSensitivity(), getChemoStartSize(), getDelay());
		
		double minimum = firstD.findRoot(secondD, 0);//start searching for the minimum at the start of the chemo
		
		//if the minimum lies before the start of the chemo then is has no influence => return null
		if(minimum < 0.) return null;
		else return minimum;
	}

	@Override
	public String toString() {
		return "LinearChemoFunction [m=" + m + ", cyclingCellsFraktion=" + getCyclingCellsFraction() 
				+ ", drugDecayRate=" + getDrugDecayRate() + ", drugSensitivity=" + getDrugSensitivity()
				+ ", chemoStartSize=" + getChemoStartSize() + "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		long temp;
		temp = Double.doubleToLongBits(m);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (!super.equals(obj))	return false;

		LinearChemoFunction other = (LinearChemoFunction) obj;

		return(m == other.m);
	}	
	
}
