/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.chemo.derivations;

import de.fhstralsund.catsit.support.functions.BasicFunction;

/**
 * Basic super class for all chemotherapy function derivations.
 * @author Anja
 *
 */
abstract public class BasicChemoFunctionDerivation implements BasicFunction {
	
	private double drugDecayRate;			//decay rate of drug
	private double drugSensitivity;			//cell kill parameter
	private double cyclingCellsFraction;	//amount of cycling cells [0;1]
	
	private long chemoStartSize;			//number of cells at the beginning of chemo cycle
	
	private double delay;	//
	
	/**
	 * 
	 * @param cyclingCellsFraction amount of cycling cells [0;1]
	 * @param drugDecayRate decay rate of drug. Unit: 1/day
	 * @param drugSensitivity cell kill parameter. Unit 1/day
	 * @param chemoStartSize number of cells at the beginning of chemo cycle
	 * @param delay delay if the chemo starts influencing the tumor later than the start of the cycle (due to dormancy or creation of a new metastasis) 
	 * this value describes the time span between the start of the chemo cycle and the time the chemo starts effecting the tumor
	 */
	public BasicChemoFunctionDerivation(double cyclingCellsFraction, double drugDecayRate, double drugSensitivity, 
			long chemoStartSize, double delay) {
		this.cyclingCellsFraction = cyclingCellsFraction;
		this.drugDecayRate = drugDecayRate;
		this.drugSensitivity = drugSensitivity;
		this.chemoStartSize = chemoStartSize;
		this.delay = delay;
	}

	/**
	 * Tries to find the root numerical via Newton method.  
	 * @param derivation the first derivation of the function whose root is wanted
	 * @param start the start value, where the numerical estimation shall start
	 * @return the root with a precision of 1e10 or null if no root is find with the required precision after 1000 iterations 
	 */
	public Double findRoot(BasicFunction derivation, double start){
		long counter=0;
		double oldValue = start;
		double newValue = start;
		do{	
			//the testcases took under 10 iteration to compute the root with a precision of 1e-10, so 1000 iteration should be enough 
			if(counter > 1000){				
				//not accurate enough after 1000 iterations? => seems like there is no root
				return null;
			}
			oldValue=newValue;
			newValue = oldValue-(f(oldValue)/derivation.f(oldValue)); 
			counter++;
		}while((Math.abs(newValue-oldValue)>1e-10));
		
		return newValue;
	}

	protected double getCyclingCellsFraction(){
		return cyclingCellsFraction;
	}
	
	protected double getDrugDecayRate(){
		return drugDecayRate;
	}
	
	protected double getDrugSensitivity(){
		return drugSensitivity;
	}
	
	protected double getChemoStartSize(){
		return chemoStartSize;
	}
	
	protected double getDelay() {
		return delay;
	}

}
