/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions.dosimetry;

import de.fhstralsund.catsit.support.functions.BasicFunction;

/**
 * Computes the mean absorbed dose.
 * f(x)= A0/m * E * T/ln(2) 
 * 
 * @author Anja Bethge
 *
 */
public class MIRDMeanAbsorbedDose implements BasicFunction {

	//calibrated activity (A0) (in GBq)
	private double activity;
	
	//mass of the tissue, the radiation in absorbed within (in kg)
	private double tissueMass;
	
	//average energy released during decay (in (Gy*kg)/(GBq*h))
	private double energy;
	
	//the effective half life of the injected radionuclide (a combination of biological and physical half life -> Teff = (Tp*Tb)/(Tp+Tb) ) (in h)
	//if no biological half life is given, the physical half life is used as effective half life 
	private double effectiveHalflife;
	
	
	/**
	 * 
	 * @param activity the injected activity. Unit: GBq
	 * @param energy the mean energy of radionuclide (can be a single emission or the sum of multiple emissions). Unit: MeV/(Bq*s) 
	 * @param effectiveHalflife the effective half life of the injected radionuclide (a combination of biological and physical half life 
	 * -> Teff = (Tp*Tb)/(Tp+Tb)). If the biological half life is not given or irrelevant (e.g. radioembolisation), the physical half life 
	 * is used as effective half live. Unit: h
	 */
	public MIRDMeanAbsorbedDose(double activity, double energy, double effectiveHalflife, double tissueMass) {
		
		if(tissueMass <= 0){
			throw new IllegalArgumentException("The tissue mass must not be zero.");
		}
		if(activity <= 0){
			throw new IllegalArgumentException("The activity must not be zero.");
		}
		
		this.activity = activity;
		//convert from MeV/(Bq*s) to Gy*kg / (GBq*h) => 
		this.energy = energy * 1.602177e-13 /* MeV -> Gy*kg */ * 1e9 /* Bq -> GBq */ * 3600 /* s -> h */;
		this.effectiveHalflife = effectiveHalflife;
		this.tissueMass = tissueMass;
	}

	public double f(double x) {		
		return (activity/tissueMass)*energy*(effectiveHalflife/Math.log(2));
	}
	
	public double getValue(){
		return f(0);
	}

}
