/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.randoms;


/**
 * abstract superclass for all random number generators
 * 
 * @author Gero Wedemann
 */
public abstract class RandomG {
	private static RandomG instance;
	
	/** 
	 * use this, if you want only one instance in your program
	 * 
	 * @return an instance
	 */
	public static RandomG getInstance() {
		if (instance==null) {
			instance = new RandomMT();
		}
		return instance;
	}
	
	/**
	 * @return a random double range [0,1)
	 */
	public abstract double getDouble();
	
	/**
	 * Computes a random double value according to standard normal distribution N[0,1]. Hence, about 68.27% of
	 * the values lie in the interval [-1,1] and 95.45% in the interval [-2,2].
	 * @return random double by standard normal distribution 
	 */
	public double getNormal() {
		// the Marsaglia polar method is used to create a normal distributed random value
		double v1,v2,s;
	    do {
	    	double rand1;
	    	double rand2;
	    	synchronized (this) {
	    		rand1=getDouble(); // [0,1)
	    		rand2=getDouble(); // [0,1)
	    	}
	        v1 = 2.0 * rand1 - 1.0; // make it [-1,1)
	        v2 = 2.0 * rand2 - 1.0; // make it [-1,1)
	        
	        s = v1*v1 + v2*v2;
	    } while (s==0 || s > 1.0);
	    
	    double a = Math.sqrt(-2.0*Math.log(s)/s);
	    double n = v1*a;
	    return n;
	}
    
    @SuppressWarnings("static-access")
    @Override
    public boolean equals(Object obj) {
        if (obj == null) 
            return false;
        if (this == obj)
            return true;
        if(this.getClass() != obj.getClass())
            return false;

        RandomG rand = (RandomG) obj;
        return (instance==null? rand.getInstance()==null : instance.equals(rand.getInstance()));
    }
}