/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.mockobjects.MockCellProgressionAction;
import de.fhstralsund.catsit.base.mockobjects.MockCompartment;
import de.fhstralsund.catsit.base.mockobjects.MockDiscreteCompartment;
import de.fhstralsund.catsit.base.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;

public class TestAbstractCompartment extends TestCase {

	
	@Override
	public void setUp() throws MetastaseException{		
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();	
	}
	
	public void testConstructor() throws MetastaseException{
		
		HostOrgan organ = new MockHostOrgan();
		Compartment testCompartment = new MockCompartment("test", organ, true);
		
		assertTrue(testCompartment.getMyId() == 1);
		assertTrue(testCompartment.isPrimaryTumor());
		assertEquals(organ, testCompartment.getHostOrgan());
		assertEquals("test", testCompartment.getName());
		
		assertTrue(CompartmentInventory.getCompartments().size() == 1);
		assertEquals(testCompartment, CompartmentInventory.getCompartments().get(0));
		
		try {
			testCompartment = new MockCompartment("test", null, false);
			fail("An IllegalArgumentException should have been thrown due to hostorgan==null");
		} catch (IllegalArgumentException e) {
			// we are supposed to be here
		}		
	}
	
	
	public void testDormancy() throws MetastaseException{
		
		//test dormancy///////////////////////////////////////////////////////////////////////
		HostOrgan organ = new HostOrgan("test1", new GaussianDistribution(50/60./24., 0), false);
		Compartment c = new MockCompartment("test compartment", organ, true);
		
		assertEquals("test1", c.getHostOrganName());
		assertEquals(organ, c.getHostOrgan());
		
		//check dormancy
		assertTrue(c.hasDormancy());
		assertFalse(c.hasLateDormancy());
		
		//check late dormancy
		organ = new HostOrgan("test1", null,new GaussianDistribution(100, 0), new GaussianDistribution(50/60./24., 0),false);
		c = new MockCompartment("test compartment", organ, true);
		
		assertTrue(c.hasLateDormancy());
		assertEquals(100, c.getLateDormancyStartSize());
		assertEquals(50.0, c.getLateDormancyDuration());
		assertFalse(c.hasDormancy());			
	}
	
	public void testDelayEvent() throws MetastaseException{
		HostOrgan organ = new HostOrgan("test1", null, new GaussianDistribution(100., 0), new GaussianDistribution(20/60./24., 0),false);
		Compartment testCompartment = new MockCompartment("test", organ, true);
		
		//check delay events
		CellProgressionEventAction action = new MockCellProgressionAction();
		testCompartment.delayEvent(action, 456.0);
		
		Event e = EventList.getInstance().nextEvent();
		assertTrue(e instanceof LocalEvent);
		//the action should be the same
		assertEquals(action, ((LocalEvent)e).getEventAction());
		assertEquals(476.0, e.getTime());
				
	}
	
	public void testGetterAndSetter() throws MetastaseException{

		Compartment compartment1 = new MockCompartment("compartment1", true);
		Compartment compartment2 = new MockCompartment("compartment2", false);
		
		//myID
		assertTrue(compartment1.getMyId() < compartment2.getMyId());
			
		//successor
		assertNull(compartment1.getSuccessor());
		
		DiscreteCompartment successor = new MockDiscreteCompartment(); //FIXME eigene mocks im base package
		compartment1.setSuccessor(successor);
		
		assertNotNull(compartment1.getSuccessor());
		assertTrue(successor.sameAs(compartment1.getSuccessor()));
			
		try {
			DiscreteCompartment dc = new MockDiscreteCompartment();
			dc.setSuccessor(dc);
			fail("An IllegalArgumentException should have been thrown");
		} catch (IllegalArgumentException e) {
			// we should be here
		}
		
		//name
		assertEquals("compartment1", compartment1.getName());
		assertEquals("compartment2", compartment2.getName());
		
		//host organ name
		assertEquals("mock organ", compartment1.getHostOrganName());
		assertEquals("mock organ", compartment2.getHostOrganName());
		
		//primary tumor
		assertTrue(compartment1.isPrimaryTumor());
		assertFalse(compartment2.isPrimaryTumor());
		
		//blood stream
		assertFalse(compartment1.isBloodStream());
		assertFalse(compartment2.isBloodStream());
		
		compartment1.setBloodStream(true);
		compartment2.setBloodStream(true);
		assertTrue(compartment1.isBloodStream());
		assertTrue(compartment2.isBloodStream());
		
		compartment1.setBloodStream(false);
		compartment2.setBloodStream(false);
		assertFalse(compartment1.isBloodStream());
		assertFalse(compartment2.isBloodStream());
		
		//dormancy
		assertFalse(compartment1.isDormant());
		compartment1.setDormant(true);
		assertTrue(compartment1.isDormant());
		assertFalse(compartment1.isInLateDormancy());
		compartment1.setDormant(false);
		assertFalse(compartment1.isDormant());
		
		//late dormancy
		
		assertFalse(compartment2.isInLateDormancy());
		compartment2.setInLateDormancy(true);
		assertTrue(compartment2.isInLateDormancy());
		assertFalse(compartment1.isDormant());
		compartment2.setInLateDormancy(false);
		assertFalse(compartment2.isInLateDormancy());
		assertFalse(compartment2.isDormant());
	}
	
	public void testToString() throws MetastaseException{
		
		Compartment compartment1 = new MockCompartment("compartment1", true);
		
		String expected = "myId=1, name=compartment1, isPrimaryTumor=true, isBloodStream=false, "
				+ "hostOrgan=HostOrgan [name=mock organ, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], "
				+ "successor=null, isDormant=false, isInLateDormancy=false, lateDormancyStartSize=0, lateDormancyDuration=0.0";
		assertEquals(expected, compartment1.toString());
			
		compartment1.setSuccessor(new MockDiscreteCompartment());
		expected = "myId=1, name=compartment1, isPrimaryTumor=true, isBloodStream=false, "
				+ "hostOrgan=HostOrgan [name=mock organ, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], "
				+ "successor=DiscreteCompartment [myId=2, name=mock type, isPrimaryTumor=false, isBloodStream=false, "
				+ "hostOrgan=HostOrgan [name=mock organ, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], "
				+ "successor=null, isDormant=false, isInLateDormancy=false, lateDormancyStartSize=0, lateDormancyDuration=0.0, numberOfCells=0, "
				+ "alreadyUndergoneLateDormancy=false, timeScheduledCompartmentTypes=[ScheduledCompartmentType [condition=time>=0.0, "
				+ "compartmentType=DiscreteCompartmentType [name=mock type, possibleEventActions=[]] ]], sizeScheduledCompartmentTypes=[]], "
				+ "isDormant=false, isInLateDormancy=false, lateDormancyStartSize=0, lateDormancyDuration=0.0";
		assertEquals(expected, compartment1.toString());
			
		
	}
	
	public void testEqualsAndHashCode() throws MetastaseException{
		
		HostOrgan organ = new HostOrgan("organ", false);
		Compartment compartment1 = new MockCompartment("compartment", organ, true);
		Compartment compartment2 = new MockCompartment("compartment", organ, true);
		
		
		assertTrue(compartment1.equals(compartment1));
		assertTrue(compartment1.hashCode() == compartment1.hashCode());
		assertTrue(compartment1.equals(compartment2));
		assertTrue(compartment1.hashCode() == compartment2.hashCode());

		assertFalse(compartment1.equals(null));
		assertFalse(compartment1.equals(new ConstantFunction()));
		assertFalse(compartment1.hashCode() == new ConstantFunction().hashCode());

		Compartment compartment3 = new MockCompartment("another compartment", organ, true);
		
		//name
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		//isPrimaryTumor
		compartment3 = new MockCompartment("compartment", organ, false);
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		//isBloodStream
		compartment3 = new MockCompartment("compartment", organ, true);
		compartment3.setBloodStream(true);
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		//host organ
		compartment3 = new MockCompartment("compartment", new HostOrgan("another organ", false), true);
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		//successor
		compartment2.setSuccessor(new MockDiscreteCompartment("successor1"));
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());

		compartment1.setSuccessor(new MockDiscreteCompartment("successor2"));
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());

		compartment1.setSuccessor(new MockDiscreteCompartment("successor1"));
		assertTrue(compartment1.equals(compartment2));
		assertTrue(compartment1.hashCode() == compartment2.hashCode());
		
		compartment1.setSuccessor(new MockDiscreteCompartment());
		compartment2.setSuccessor(null);
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());

		//dormancy
		compartment1.setSuccessor(null);
		compartment2.setSuccessor(null);
		compartment1.setDormant(true);
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());
		compartment1.setDormant(false);

		//late dormancy
		compartment1.setInLateDormancy(true);
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());
		compartment1.setInLateDormancy(false);
		
		HostOrgan organ1 = new HostOrgan("organ", null, new GaussianDistribution(10, 0), new GaussianDistribution(20, 0), false);
		HostOrgan organ2 = new HostOrgan("organ", null, new GaussianDistribution(15, 0), new GaussianDistribution(20, 0), false);
		compartment1 = new MockCompartment("compartment", organ1, true);
		compartment2 = new MockCompartment("compartment", organ2, true);
		
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());
		
		organ1 = new HostOrgan("organ", null, new GaussianDistribution(10, 0), new GaussianDistribution(20, 0), false);
		organ2 = new HostOrgan("organ", null, new GaussianDistribution(10, 0), new GaussianDistribution(25, 0), false);
		compartment1 = new MockCompartment("compartment", organ1, true);
		compartment2 = new MockCompartment("compartment", organ2, true);
		
		assertFalse(compartment1.equals(compartment2));
		assertFalse(compartment1.hashCode() == compartment2.hashCode());
	}

	public void testSame() throws MetastaseException{
		Compartment c1 = new MockCompartment();
		Compartment c2 = new MockCompartment();
		
		assertTrue(c1.sameAs(c1));
		assertTrue(c2.sameAs(c2));
		
		assertFalse(c1.sameAs(c2));
		assertFalse(c2.sameAs(c1));
	}
	
}
