/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StartLateDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopLateDormancy;
import de.fhstralsund.catsit.base.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.base.mockobjects.MockDiscreteCompartment;
import de.fhstralsund.catsit.base.mockobjects.MockEvent;
import de.fhstralsund.catsit.base.mockobjects.MockExternalBeamTreatment;
import de.fhstralsund.catsit.base.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.base.mockobjects.MockRadiationTreatment;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.ExternalBeamTreatment;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.ResectionTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.machine.NoEventLeftException;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.LogFunction;
import de.fhstralsund.catsit.support.functions.chemo.BasicChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.GompertzChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.LinearChemoFunction;

public class TestContinuousCompartment extends TestCase {

	
	@Override
	protected final void setUp() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
		TreatmentList.reset();
	}
	
	@Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}
	
	
	public void testGetNumberOfCells() throws MetastaseException{
		
		HostOrgan organ = new HostOrgan("organ", false);
		HostOrgan organWithD = new HostOrgan("organWithD", new GaussianDistribution(100/60./24., 0), false);
		HostOrgan organWithLD = new HostOrgan("organWithLD", null, new GaussianDistribution(100, 0), new GaussianDistribution(50/60./24., 0), false);
		
		//normal growth function
		ContinuousCompartment compartment1 = new ContinuousCompartment(0., "c1", organ, new LinearFunction(2*60.*24., 1), null);
		assertEquals(1, compartment1.getNumberOfCells(0));
		assertEquals(11, compartment1.getNumberOfCells(5));
		assertEquals(141, compartment1.getNumberOfCells(70));
		assertEquals(601, compartment1.getNumberOfCells(300));
		
		//dormancy before the normal growth function
		ContinuousCompartment compartment2 = new ContinuousCompartment(20., "c2", organWithD, new LinearFunction(2*60.*24., 1), null);
		assertTrue(compartment2.isDormant());
		assertEquals(1, compartment2.getNumberOfCells(20));
		assertEquals(1, compartment2.getNumberOfCells(25));
		assertEquals(1, compartment2.getNumberOfCells(90));
		assertEquals(1, compartment2.getNumberOfCells(120));
		assertEquals(11, compartment2.getNumberOfCells(125));
		assertEquals(141, compartment2.getNumberOfCells(190));
		assertEquals(601, compartment2.getNumberOfCells(420));
		
		//late dormancy 
		ContinuousCompartment compartment3 = new ContinuousCompartment(0., "c3", organWithLD, new LinearFunction(2*60.*24., 1), null);
		assertEquals(1, compartment3.getNumberOfCells(0));
		assertEquals(11, compartment3.getNumberOfCells(5));
		assertEquals(99, compartment3.getNumberOfCells(49));
		assertEquals(100, compartment3.getNumberOfCells(49.5));
		assertEquals(100, compartment3.getNumberOfCells(50));
		assertEquals(100, compartment3.getNumberOfCells(70));
		assertEquals(100, compartment3.getNumberOfCells(99));
		assertEquals(100, compartment3.getNumberOfCells(99.5));
		assertEquals(501, compartment3.getNumberOfCells(300));
	}
	
	
	public void testDormancyAndLateDormancy() throws MetastaseException{
		
		EventList eventList = EventList.getInstance();
		
		HostOrgan organWithD = new HostOrgan("organWithD", new GaussianDistribution(100/60./24., 0), false);
		ContinuousCompartment compartment = new ContinuousCompartment(31., "c1", organWithD, new LinearFunction(2*60.*24., 1), null);
		
		//a StopDormancy event should have been added to the event list
		Event e = eventList.nextEvent();
		assertTrue(e instanceof LocalEvent);
		assertEquals(131., e.getTime());
		assertTrue(((LocalEvent)e).getEventAction() instanceof StopDormancy);
		
		//check the number of cells
		assertEquals(1, compartment.getNumberOfCells(31));
		assertEquals(1, compartment.getNumberOfCells(40));
		assertEquals(1, compartment.getNumberOfCells(100));
		assertEquals(1, compartment.getNumberOfCells(130));
		assertEquals(1, compartment.getNumberOfCells(131));
		assertEquals(41, compartment.getNumberOfCells(151));
		assertEquals(139, compartment.getNumberOfCells(200));
		
		
		HostOrgan organWithLD = new HostOrgan("organWithD", null, new GaussianDistribution(100., 0), new GaussianDistribution(50/60./24., 0), false);
		compartment = new ContinuousCompartment(10., "c1", organWithLD, new LinearFunction(2*60.*24., 1), null);
		
		assertEquals(1, compartment.getNumberOfCells(10));
		assertEquals(11, compartment.getNumberOfCells(15));
		assertEquals(99, compartment.getNumberOfCells(59));
		assertEquals(100, compartment.getNumberOfCells(59.5));
		assertEquals(100, compartment.getNumberOfCells(60));
		assertEquals(100, compartment.getNumberOfCells(80));
		assertEquals(100, compartment.getNumberOfCells(109));
		assertEquals(100, compartment.getNumberOfCells(109.5));
		assertEquals(501, compartment.getNumberOfCells(310));

		//a StartLateDormancy event should have been added to the event list
		e = eventList.nextEvent();
		assertTrue(e instanceof LocalEvent);
		assertEquals(59.5, e.getTime(), 1e-2);
		//the action should be the same
		assertTrue(((LocalEvent)e).getEventAction() instanceof StartLateDormancy);
		
		//a StopLateDormancy event should have been added to the event list
		e = eventList.nextEvent();
		assertTrue(e instanceof LocalEvent);
		assertEquals(109.5, e.getTime(), 1e-2);
		//the action should be the same
		assertTrue(((LocalEvent)e).getEventAction() instanceof StopLateDormancy);
	}
	
	
	
	
	public void testCreateFirstAndNextEvent() throws MetastaseException{		
		EventList eventList = EventList.getInstance();
		HostOrgan organ = new HostOrgan("organ", false);
		ContinuousCompartment compartment = new ContinuousCompartment(10., "name", organ, new ConstantFunction(5), null);
		
		//no colonization rate
		try {
			compartment.createFirstEvent(10.);
			//since no colonization rate was specified no event should have been added to the event list
			Event e = eventList.nextEvent();
			fail("No event should have been in the event list! Found: " + e);
		} catch (NoEventLeftException nele) {
			// we are supposed to be here
		}
		
		//normal colonization rate
		compartment = new ContinuousCompartment(10., "name", organ, new ConstantFunction(5), new ColonizationRate(1, 1));
		
		compartment.createFirstEvent(10.);
		compartment.createNextEvent(15.);
		//two intravasation events should have been added to the event list
		Event e = eventList.nextEvent();
		assertTrue(((LocalEvent)e).getEventAction() instanceof AddCellToDiscreteCompartment);
		e = eventList.nextEvent();
		assertTrue(((LocalEvent)e).getEventAction() instanceof AddCellToDiscreteCompartment);
		
		compartment.setDormant(true);
		try {
			compartment.createNextEvent(20.);
			fail("An exception should have been thrown.");
		} catch (IllegalStateException e2) {
			// we are supposed to be here
		}
		
		compartment.setDormant(false);
		compartment.setInLateDormancy(true);
		try {
			compartment.createNextEvent(20.);
			fail("An exception should have been thrown.");
		} catch (IllegalStateException e2) {
			// we are supposed to be here
		}
		
		
		//tumor is dead
		compartment = new ContinuousCompartment(10., "name", organ, new ConstantFunction(0), new ColonizationRate(1, 1));
		try {
			compartment.createFirstEvent(10.);
			//since the tumor is dead no event should have been added to the event list
			e = eventList.nextEvent();
			fail("No event should have been in the event list! Found: " + e);
		} catch (NoEventLeftException nele) {
			// we are supposed to be here
		}
	}
	
	public void testBuildTreatments() throws MetastaseException{
		
		HostOrgan organ = new HostOrgan("organ", false);
		HostOrgan organ2 = new HostOrgan("organ2", false);
		
		TreatmentList treatmentList = TreatmentList.getInstance();
		
		//add three chemo cycles every month
		treatmentList.addTreatment(300./24./60., new ChemoTreatment(0.5, 0.81*60.*24., 0.6*60.*24.));
		treatmentList.addTreatment(314./24./60., new ChemoTreatment(0.5, 0.81*60.*24., 0.6*60.*24.));
		treatmentList.addTreatment(328./24./60., new ChemoTreatment(0.5, 0.81*60.*24., 0.6*60.*24.));
		treatmentList.addTreatment(350/24./60., new RadioimmunoTreatment(0, 0, 0, 0, 0, 0, 0, true));//radioimunotherapy should have no influence in buildTreatments 
		//add resection
		treatmentList.addTreatment(400/24./60., new ResectionTreatment(0.));
		//add radiation 
		treatmentList.addTreatment(450/24./60., new ExternalBeamTreatment(0.35, 0.023, 2));
		treatmentList.addTreatment(500/24./60., new RadioembolizationTreatment(0.35, 0.023, 0.001, 0.01, 1, organ, 0.0001));	
		
		//try to create a compartment with a growth function that is not supported for chemo
		try {
			new ContinuousCompartment(0, "name", organ, new LogFunction(1, 2, 1), null);
			fail("an exception should have been thrown due to a growth function that is not supported for chemo therapy");
		} catch (MetastaseException e) {
			// we are supposed to be here
		}
		
		//if the tumor is dead, it should stay dead ;) hence further treatments have no influence
		ContinuousCompartment compartment1 = new ContinuousCompartment(0., "name", organ, new ConstantFunction(0), null, true);
		assertEquals(new ConstantFunction(0), compartment1.getCurrentValidFunction(10));
		assertEquals(new ConstantFunction(0), compartment1.getCurrentValidFunction(100));
		assertEquals(new ConstantFunction(0), compartment1.getCurrentValidFunction(1000));
		
		
		ContinuousCompartment primary = new ContinuousCompartment(0., "name", organ, new GompertzFunction(0.00286*60.*24., 7.3e8), null, true);
		ContinuousCompartment noPrimary = new ContinuousCompartment(0., "name", organ, new GompertzFunction(0.00286*60.*24., 7.3e8), null, false);
		ContinuousCompartment noPrimary2 = new ContinuousCompartment(0., "name", organ2, new GompertzFunction(0.00286*60.*24., 7.3e8), null, false);
		
		//primary
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(10));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(100));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(200));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(299));
		
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 127406, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(300));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 127406, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(307));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 127406, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(313));
		
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 93275, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(314));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 93275, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(320));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 93275, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(327));
		
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(328));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(370));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), primary.getCurrentValidFunction(399));
		
		assertEquals(new ConstantFunction(0), primary.getCurrentValidFunction(400));
		assertEquals(new ConstantFunction(0), primary.getCurrentValidFunction(600));
		assertEquals(new ConstantFunction(0), primary.getCurrentValidFunction(700));
		assertEquals(new ConstantFunction(0), primary.getCurrentValidFunction(800));
		
		//no primary
		assertEquals(new GompertzFunction(0.00286, 7.3e8), noPrimary.getCurrentValidFunction(10));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), noPrimary.getCurrentValidFunction(100));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), noPrimary.getCurrentValidFunction(200));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), noPrimary.getCurrentValidFunction(299));
		
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 127406, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(300));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 127406, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(307));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 127406, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(313));
		
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 93275, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(314));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 93275, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(320));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 93275, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(327));
		
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(328));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(370));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(399));
		
		//non primary tumor shoudn't be affected by resection and external radiation therapy
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(400));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(450));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary.getCurrentValidFunction(499));

		//after the radioembolization the original growth function should have been restored
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 165268), noPrimary.getCurrentValidFunction(500));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 165268), noPrimary.getCurrentValidFunction(600));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 165268), noPrimary.getCurrentValidFunction(700));
		
		//if the compartment does not belong to the right host organ the radioembolization should have no effect
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary2.getCurrentValidFunction(500));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary2.getCurrentValidFunction(600));
		assertEquals(new GompertzChemoFunction(0.00286, 7.3e8, 69128, 0.5, 0.6, 0.81), noPrimary2.getCurrentValidFunction(700));
		
		//create some compartments after the chemo treatments
		ContinuousCompartment compartment2 = new ContinuousCompartment(400., "name", organ, new LinearFunction(1e4 *60.*24., 1), null, false);
		ContinuousCompartment compartment3 = new ContinuousCompartment(400., "name", organ, new ConstantFunction(99), null, false);
		ContinuousCompartment compartment4 = new ContinuousCompartment(400., "name", organ, new LinearFunction(100 *60.*24., 1), null, false);
		
		//compartment 2 should only be effected by the radioembolization
		assertEquals(new LinearFunction(1e4, 1), compartment2.getCurrentValidFunction(400));
		assertEquals(new LinearFunction(1e4, 1), compartment2.getCurrentValidFunction(450));
		assertEquals(new LinearFunction(1e4, 11053), compartment2.getCurrentValidFunction(500));
		assertEquals(new LinearFunction(1e4, 11053), compartment2.getCurrentValidFunction(600));
		
		//compartment 3 should not be effected, since it is too small
		assertEquals(new ConstantFunction(99), compartment3.getCurrentValidFunction(400));
		assertEquals(new ConstantFunction(99), compartment3.getCurrentValidFunction(450));
		assertEquals(new ConstantFunction(99), compartment3.getCurrentValidFunction(500));
		assertEquals(new ConstantFunction(99), compartment3.getCurrentValidFunction(600));
		
		//compartment 4 should be killed by the radioembolization
		assertEquals(new LinearFunction(100, 1), compartment4.getCurrentValidFunction(400));
		assertEquals(new LinearFunction(100, 1), compartment4.getCurrentValidFunction(450));
		assertEquals(new ConstantFunction(0), compartment4.getCurrentValidFunction(500));
		assertEquals(new ConstantFunction(0), compartment4.getCurrentValidFunction(600));
		
		//create a primary after the resection that is influenced by the external beam radiation
		primary = new ContinuousCompartment(401., "name", organ, new GompertzFunction(0.00286 *60.*24., 7.3e8), null, true);
		
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(402));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 6), primary.getCurrentValidFunction(450));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 6), primary.getCurrentValidFunction(500)); //to small for radioembolization
		
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//add a fake treatment to the list
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(300./24./60., new MockRadiationTreatment());
		
		try {
			new ContinuousCompartment(0, "bla", organ, new LinearFunction(2, 1), null);
			fail("an exception should have been thrown due to the fake treatment in the treatment list");
		} catch (MetastaseException e) {
			// we are supposed to be here
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//add radiation treatment shortly after a chemo therapy
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(300./24./60., new ChemoTreatment(0.5, 0.8 *60.*24., 0.6 *60.*24.));
		TreatmentList.getInstance().addTreatment(310./24./60., new RadioembolizationTreatment(0.35, 0.023, 0.001, 0.01, 1, organ, 0.0001)); 
		
		try {
			new ContinuousCompartment(0, "bla", organ, new GompertzFunction(0.00286 *60.*24., 7.3e8), null, true);
			fail("an exception should have been thrown due to the small intervall between chemo and radiationt");
		} catch (MetastaseException e) {
			// we are supposed to be here
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//add two chemo cycles
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(300./24./60., new ChemoTreatment(0.5, 0.8 *60.*24., 0.6 *60.*24.));
		TreatmentList.getInstance().addTreatment(310./24./60., new ChemoTreatment(0.5, 0.8 *60.*24., 0.6 *60.*24.));
		
		//create a compartment that dies during the first chemo therapy
		ContinuousCompartment compartment5 = new ContinuousCompartment(0., "name", organ, new LinearFunction(0.001 *60.*24., 1), null, false);
		assertEquals(new ConstantFunction(0), compartment5.getCurrentValidFunction(300));
		assertEquals(new ConstantFunction(0), compartment5.getCurrentValidFunction(600));
		
		//add two chemo cycles
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(300./24./60., new ChemoTreatment(0.5, 0.81 *60.*24., 0.6 *60.*24.));
		TreatmentList.getInstance().addTreatment(301./24./60., new ChemoTreatment(0.5, 0.81 *60.*24., 0.6 *60.*24.)); 
		//create a compartment that dies during the second chemo therapy
		ContinuousCompartment compartment6 = new ContinuousCompartment(0., "name", organ, new LinearFunction(0.005 *60.*24., 1), null, false);
		assertEquals(new LinearChemoFunction(0.005, 2, 0.5, 0.6, 0.81), compartment6.getCurrentValidFunction(300));
		assertEquals(new ConstantFunction(0), compartment6.getCurrentValidFunction(310));
		
		//create a compartment directly after the chemo cycle
		compartment6 = new ContinuousCompartment(301.1, "name", organ, new LinearFunction(0.003 *60.*24., 1), null, false);
		Function firstGrowthFunction = compartment6.getCurrentValidFunction(301.1);
		assertTrue(firstGrowthFunction instanceof BasicChemoFunction);
		
		
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// test radiation during dormancy
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(200./24./60., new MockExternalBeamTreatment(100));
		
		//late dormancy that starts at size 200 and lasts 200 days
		organ = new HostOrgan("organ", null, new GaussianDistribution(200, 0), new GaussianDistribution(200 /60./24., 0), false);
		//create a compartment that that is radiated during late dormancy
		ContinuousCompartment compartment7 = new ContinuousCompartment(0., "name", organ, new LinearFunction(2. *60.*24., 0), null, true);
		ContinuousCompartment compartment8 = new ContinuousCompartment(50., "name", organ, new LinearFunction(2. *60.*24., 0), null, true);
		
		assertEquals(new LinearFunction(2, 0), compartment7.getCurrentValidFunction(0));
		assertEquals(new LinearFunction(2, 0), compartment7.getCurrentValidFunction(50));
		assertEquals(new LinearFunction(2, 0), compartment7.getCurrentValidFunction(99));//normal growth
		assertEquals(new ConstantFunction(200), compartment7.getCurrentValidFunction(100));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment7.getCurrentValidFunction(150));
		assertEquals(new ConstantFunction(200), compartment7.getCurrentValidFunction(199));
		assertEquals(new ConstantFunction(100), compartment7.getCurrentValidFunction(200));//late dormancy after radiation
		assertEquals(new ConstantFunction(100), compartment7.getCurrentValidFunction(250));
		assertEquals(new ConstantFunction(100), compartment7.getCurrentValidFunction(299));
		assertEquals(new LinearFunction(2, 100), compartment7.getCurrentValidFunction(300));//end of late dormancy
		assertEquals(new LinearFunction(2, 100), compartment7.getCurrentValidFunction(500));
		assertEquals(new LinearFunction(2, 100), compartment7.getCurrentValidFunction(1000));		
		
		assertEquals(new LinearFunction(2., 0), compartment8.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment8.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment8.getCurrentValidFunction(149.));//normal growth
		assertEquals(new ConstantFunction(200), compartment8.getCurrentValidFunction(150.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment8.getCurrentValidFunction(170.));
		assertEquals(new ConstantFunction(200), compartment8.getCurrentValidFunction(199.));
		assertEquals(new ConstantFunction(100), compartment8.getCurrentValidFunction(200.));//late dormancy after radiation
		assertEquals(new ConstantFunction(100), compartment8.getCurrentValidFunction(250.));
		assertEquals(new ConstantFunction(100), compartment8.getCurrentValidFunction(349.));
		assertEquals(new LinearFunction(2., 100), compartment8.getCurrentValidFunction(350.));//end of late dormancy
		assertEquals(new LinearFunction(2., 100), compartment8.getCurrentValidFunction(500.));
		assertEquals(new LinearFunction(2., 100), compartment8.getCurrentValidFunction(1000.));
		
		///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// test chemo during dormancy
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(200./24./60., new ChemoTreatment(0.2, 0.9 *60.*24., 0.6 *60.*24.));
		//late dormancy that starts at size 200 and lasts 200 or 101 days
		organ = new HostOrgan("organ", null, new GaussianDistribution(200, 0), new GaussianDistribution(200 /60./24., 0), false);
		organ2 = new HostOrgan("organ", null, new GaussianDistribution(200, 0), new GaussianDistribution(101 /60./24., 0), false);
		//create a compartment that that is in late dormancy at the start of the chemo
		ContinuousCompartment compartment9 = new ContinuousCompartment(0., "name", organ, new LinearFunction(2 *60.*24., 0), null, true);
		ContinuousCompartment compartment10 = new ContinuousCompartment(50., "name", organ, new LinearFunction(2 *60.*24., 0), null, true);
		ContinuousCompartment compartment11 = new ContinuousCompartment(0., "name", organ2, new LinearFunction(2 *60.*24., 0), null, true);
		ContinuousCompartment compartment12 = new ContinuousCompartment(50., "name", organ2, new LinearFunction(2 *60.*24., 0), null, true);
		
		//compartment 9 is not influenced by the chemo, since the drug concentration is too low after the dormancy phase
		assertEquals(new LinearFunction(2., 0), compartment9.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment9.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment9.getCurrentValidFunction(99.));//normal growth
		assertEquals(new ConstantFunction(200), compartment9.getCurrentValidFunction(100.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment9.getCurrentValidFunction(200.));
		assertEquals(new ConstantFunction(200), compartment9.getCurrentValidFunction(299.));
		assertEquals(new LinearFunction(2., 200), compartment9.getCurrentValidFunction(300.));//end of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment9.getCurrentValidFunction(500.));
		assertEquals(new LinearFunction(2., 200), compartment9.getCurrentValidFunction(1000.));
		
		//compartment 10 is not influenced by the chemo, since the drug concentration is too low after the dormancy phase
		assertEquals(new LinearFunction(2., 0), compartment10.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment10.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment10.getCurrentValidFunction(149.));//normal growth
		assertEquals(new ConstantFunction(200), compartment10.getCurrentValidFunction(150.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment10.getCurrentValidFunction(250.));
		assertEquals(new ConstantFunction(200), compartment10.getCurrentValidFunction(349.));
		assertEquals(new LinearFunction(2., 200), compartment10.getCurrentValidFunction(350.));//end of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment10.getCurrentValidFunction(500.));
		assertEquals(new LinearFunction(2., 200), compartment10.getCurrentValidFunction(1000.));
		
		
		//compartment 11 is influenced by the chemo
		assertEquals(new LinearFunction(2., 0), compartment11.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment11.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment11.getCurrentValidFunction(99.));//normal growth
		assertEquals(new ConstantFunction(200), compartment11.getCurrentValidFunction(100.));//start of late dormancy
		assertEquals(new LinearChemoFunction(2, 200, 0.2, 0.6, 0.9, 1), compartment11.getCurrentValidFunction(201.));//end of late dormancy
		assertEquals(new LinearChemoFunction(2, 200, 0.2, 0.6, 0.9, 1), compartment11.getCurrentValidFunction(500.));
		assertEquals(new LinearChemoFunction(2, 200, 0.2, 0.6, 0.9, 1), compartment11.getCurrentValidFunction(1000.));
		
		//compartment 12 is not influenced by the chemo
		assertEquals(new LinearFunction(2., 0), compartment12.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment12.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment12.getCurrentValidFunction(149.));//normal growth
		assertEquals(new ConstantFunction(200), compartment12.getCurrentValidFunction(150.));//start of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment12.getCurrentValidFunction(251.));//end of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment12.getCurrentValidFunction(500.));
		assertEquals(new LinearFunction(2., 200), compartment12.getCurrentValidFunction(1000.));
		
		
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//test resection with left overs
		organ = new HostOrgan("organ", false);
		
		TreatmentList.reset();
		treatmentList = TreatmentList.getInstance();

		//add resection which leaves some cells in the tissue
		treatmentList.addTreatment(400/24./60., new ResectionTreatment(0.001));
		//add radiation 
		treatmentList.addTreatment(450/24./60., new ExternalBeamTreatment(0.35, 0.023, 2));
		
		primary = new ContinuousCompartment(0., "name", organ, new GompertzFunction(0.00286*60.*24., 7.3e8), null, true);
		
		//primary
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(10));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(100));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(200));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(300));
		assertEquals(new GompertzFunction(0.00286, 7.3e8), primary.getCurrentValidFunction(399));
		
		//the primary tumor should not have been resected completely
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 1096), primary.getCurrentValidFunction(400));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 1096), primary.getCurrentValidFunction(420));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 1096), primary.getCurrentValidFunction(449));
		
		//ExternalBeamTreatment
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 2963), primary.getCurrentValidFunction(450));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 2963), primary.getCurrentValidFunction(500));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 2963), primary.getCurrentValidFunction(1000));
		assertEquals(new GompertzFunction(0.00286, 7.3e8, 2963), primary.getCurrentValidFunction(5000));
	}
	
	public void testUpdateDormancySize() throws MetastaseException{
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(400./24./60., new ChemoTreatment(0.2, 0.9*60.*24., 0.6*60.*24.));
		HostOrgan organ = new HostOrgan("organ", null, new GaussianDistribution(200, 0), new GaussianDistribution(200/60./24., 0), false);		
		ContinuousCompartment compartment = new ContinuousCompartment(0., "name", organ, new LinearFunction(2*60.*24., 0), null, true);
		
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(99.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(100.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(200.));
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(299.));
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(300.));//end of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		compartment.updateDormancySize(110, 20);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(99.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(100.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(109.));
		assertEquals(new ConstantFunction(20), compartment.getCurrentValidFunction(110.));//updated size
		assertEquals(new ConstantFunction(20), compartment.getCurrentValidFunction(299.));
		assertEquals(new LinearFunction(2., 20), compartment.getCurrentValidFunction(300.));//end of late dormancy
		assertEquals(new LinearFunction(2., 20), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 220, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 220, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		
		compartment = new ContinuousCompartment(50., "name", organ, new LinearFunction(2*60.*24., 0), null, true);
		
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(149.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(150.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(250.));
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(349.));
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(350.));//end of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 300, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 300, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		
		try {
			//try to update the dormancy size at a time the compartment is not dormant
			compartment.updateDormancySize(110, 20);
			fail("An exception should have been thrown");
		} catch (MetastaseException e) {
			//we are supposed to be here
		}
		
		compartment.updateDormancySize(250, 20);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(149.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(150.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(249.));
		assertEquals(new ConstantFunction(20), compartment.getCurrentValidFunction(250.));//updated size
		assertEquals(new ConstantFunction(20), compartment.getCurrentValidFunction(349.));
		assertEquals(new LinearFunction(2., 20), compartment.getCurrentValidFunction(350.));//end of late dormancy
		assertEquals(new LinearFunction(2., 20), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 120, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 120, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		assertEquals(0, compartment.getNumberOfCells(50.));
		assertEquals(100, compartment.getNumberOfCells(100.));
		assertEquals(198, compartment.getNumberOfCells(149.));//normal growth
		assertEquals(200, compartment.getNumberOfCells(150.));//start of late dormancy
		assertEquals(200, compartment.getNumberOfCells(249.));
		assertEquals(20, compartment.getNumberOfCells(250.));//updated size
		assertEquals(20, compartment.getNumberOfCells(349.));
		assertEquals(20, compartment.getNumberOfCells(350.));//end of late dormancy
		assertEquals(287, compartment.getNumberOfCells(500.));
		assertEquals(1287, compartment.getNumberOfCells(1000.));
	}
	
	public void testUpdateTumorSize() throws MetastaseException{
		TreatmentList.reset();
		TreatmentList.getInstance().addTreatment(400./24./60., new ChemoTreatment(0.2, 0.9*60.*24., 0.6*60.*24.));
		HostOrgan organ = new MockHostOrgan();		
		ContinuousCompartment compartment = new ContinuousCompartment(0., "name", organ, new LinearFunction(2*60.*24., 0), null, true);
		
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(300.));
		assertEquals(new LinearChemoFunction(2., 800, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.)); //chemo
		assertEquals(new LinearChemoFunction(2., 800, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(500.));
		assertEquals(new LinearChemoFunction(2., 800, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.)); 
		
		compartment.updateTumorSize(250, 100);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 100), compartment.getCurrentValidFunction(300.));//size was changed
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.)); //chemo
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(500.));
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		compartment.updateTumorSize(550, 300);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 100), compartment.getCurrentValidFunction(300.));//size was changed
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.)); //chemo
		assertEquals(new LinearChemoFunction(2., 400, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(500.));
		assertEquals(new LinearFunction(2., 300), compartment.getCurrentValidFunction(550.));//size was changed
		
		compartment.updateTumorSize(80, 0);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(00.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(79.));
		assertEquals(new ConstantFunction(0), compartment.getCurrentValidFunction(300.));
		assertEquals(new ConstantFunction(0), compartment.getCurrentValidFunction(400.));
		assertEquals(new ConstantFunction(0), compartment.getCurrentValidFunction(500.));
		assertEquals(new ConstantFunction(0), compartment.getCurrentValidFunction(550.));
		
		//compartment with late dormancy
		organ = new HostOrgan("organ", null, new GaussianDistribution(200, 0), new GaussianDistribution(200/60./24., 0), false);		
		compartment = new ContinuousCompartment(50., "name", organ, new LinearFunction(2*60.*24., 0), null, true);
		
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(100.));
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(149.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(150.));//start of late dormancy
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(250.));
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(349.));
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(350.));//end of late dormancy
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 300, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 300, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		compartment.updateTumorSize(100, 10);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 10), compartment.getCurrentValidFunction(100.));//size was changed
		assertEquals(new LinearFunction(2., 10), compartment.getCurrentValidFunction(194.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(195.));//start of late dormancy should have changed
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(250.));
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(394.));
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(395.));//end of late dormancy should have changed
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 210, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 210, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(1000.));
		
		compartment.updateTumorSize(550, 10);
		assertEquals(new LinearFunction(2., 0), compartment.getCurrentValidFunction(50.));
		assertEquals(new LinearFunction(2., 10), compartment.getCurrentValidFunction(100.));//size was changed
		assertEquals(new LinearFunction(2., 10), compartment.getCurrentValidFunction(194.));//normal growth
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(195.));//start of late dormancy should have changed
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(250.));
		assertEquals(new ConstantFunction(200), compartment.getCurrentValidFunction(394.));
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(395.));//end of late dormancy should have changed
		assertEquals(new LinearFunction(2., 200), compartment.getCurrentValidFunction(399.));
		assertEquals(new LinearChemoFunction(2., 210, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(400.));
		assertEquals(new LinearChemoFunction(2., 210, 0.2, 0.6, 0.9), compartment.getCurrentValidFunction(549.));
		assertEquals(new LinearFunction(2., 10), compartment.getCurrentValidFunction(550.));
	}
	
	
	public void testGetCreationTime() throws MetastaseException{
		HostOrgan organ = new HostOrgan("organ", false);
		ContinuousCompartment compartment = new ContinuousCompartment(0., "name", organ, new ConstantFunction(0), null, true);
		assertEquals(0., compartment.getCreationTime());
		
		compartment = new ContinuousCompartment(400., "name", organ, new ConstantFunction(0), null, true);
		assertEquals(400., compartment.getCreationTime());
		
		compartment = new ContinuousCompartment(123., "name", organ, new LinearFunction(2, 7058), null, true);
		assertEquals(123., compartment.getCreationTime());
		
		compartment = new ContinuousCompartment(456., "name", organ, new ExponentialFunction(1, 2, 3), null, true);
		assertEquals(456., compartment.getCreationTime());
		
		compartment = new ContinuousCompartment(789., "name", organ, new GompertzFunction(4, 5), null, true);
		assertEquals(789., compartment.getCreationTime());
	}
	
	
	
	public void testDeleteAllEvents() throws MetastaseException{
		
		EventList eventList = EventList.getInstance();
		ContinuousCompartment compartment1 = new MockContinuousCompartment("c1");
		ContinuousCompartment compartment2 = new MockContinuousCompartment("c2");
		DiscreteCompartment discreteCompartment = new MockDiscreteCompartment();
		
		AddCellToDiscreteCompartment addCell1 = new AddCellToDiscreteCompartment("test1", compartment1);
		AddCellToDiscreteCompartment addCell2 = new AddCellToDiscreteCompartment("test2", compartment2);
		StopDormancy dormancy = new StopDormancy();
		StartLateDormancy startLateDormancy = new StartLateDormancy();
		StopLateDormancy stopLateDormancy = new StopLateDormancy();
		
		Event e1 = new LocalEvent(addCell1, 1.0, discreteCompartment);
		Event e2 = new LocalEvent(addCell1, 2.0, discreteCompartment);
		Event e3 = new LocalEvent(addCell1, 3.0, discreteCompartment);
		Event e4 = new LocalEvent(addCell1, 4.0, discreteCompartment);
		Event e5 = new LocalEvent(addCell2, 5.0, discreteCompartment); //
		Event e6 = new LocalEvent(addCell2, 6.0, discreteCompartment);//
		Event e7 = new LocalEvent(dormancy, 7.0, compartment1);
		Event e8 = new LocalEvent(dormancy, 8.0, compartment1);
		Event e9 = new LocalEvent(dormancy, 9.0, compartment2);//
		Event e10 = new LocalEvent(startLateDormancy, 10.0, compartment1);
		Event e11 = new LocalEvent(startLateDormancy, 11.0, compartment2);//
		Event e12 = new LocalEvent(startLateDormancy, 12.0, compartment2);//
		Event e13 = new LocalEvent(stopLateDormancy, 13.0, compartment1);
		Event e14 = new LocalEvent(stopLateDormancy, 14.0, compartment2);//
		Event e15 = new LocalEvent(stopLateDormancy, 15.0, compartment1);
		Event e16 = new MockEvent(16);//
		
		eventList.addEvent(e1);
		eventList.addEvent(e2);
		eventList.addEvent(e3);
		eventList.addEvent(e4);
		eventList.addEvent(e5);
		eventList.addEvent(e6);
		eventList.addEvent(e7);
		eventList.addEvent(e8);
		eventList.addEvent(e9);
		eventList.addEvent(e10);
		eventList.addEvent(e11);
		eventList.addEvent(e12);
		eventList.addEvent(e13);
		eventList.addEvent(e14);
		eventList.addEvent(e15);
		eventList.addEvent(e16);
				
		compartment1.deleteAllEvents();
		
		try {
			assertEquals(e5, eventList.nextEvent());
			assertEquals(e6, eventList.nextEvent());
			assertEquals(e9, eventList.nextEvent());
			assertEquals(e11, eventList.nextEvent());
			assertEquals(e12, eventList.nextEvent());
			assertEquals(e14, eventList.nextEvent());
			assertEquals(e16, eventList.nextEvent());
			
			assertTrue(eventList.isEmpty());
			
		} catch (NoEventLeftException e) {
			e.printStackTrace();
			fail("Unexpected eception");
		}
	}
	
	public void testDeleteLastIntravasationEvent() throws MetastaseException{
		EventList eventList = EventList.getInstance();
		ContinuousCompartment compartment1 = new MockContinuousCompartment("c1");
		ContinuousCompartment compartment2 = new MockContinuousCompartment("c2");
		DiscreteCompartment discreteCompartment = new MockDiscreteCompartment();
		
		AddCellToDiscreteCompartment addCell1 = new AddCellToDiscreteCompartment("test1", compartment1);
		AddCellToDiscreteCompartment addCell2 = new AddCellToDiscreteCompartment("test2", compartment2);
		StopDormancy dormancy = new StopDormancy();
		StartLateDormancy startLateDormancy = new StartLateDormancy();
		StopLateDormancy stopLateDormancy = new StopLateDormancy();
		
		Event e1 = new LocalEvent(addCell1, 1.0, discreteCompartment);
		Event e2 = new LocalEvent(addCell1, 2., discreteCompartment);
		Event e3 = new LocalEvent(addCell1, 3.0, discreteCompartment);
		Event e4 = new LocalEvent(addCell1, 4.0, discreteCompartment);
		Event e5 = new LocalEvent(addCell2, 0.8, discreteCompartment); //
		Event e6 = new LocalEvent(addCell2, 6.0, discreteCompartment);//
		Event e7 = new LocalEvent(dormancy, 7.0, compartment1);
		Event e8 = new LocalEvent(dormancy, 8.0, compartment1);
		Event e9 = new LocalEvent(dormancy, 9.0, compartment2);//
		Event e10 = new LocalEvent(startLateDormancy, 10.0, compartment1);
		Event e11 = new LocalEvent(startLateDormancy, 11.0, compartment2);//
		Event e12 = new LocalEvent(startLateDormancy, 12.0, compartment2);//
		Event e13 = new LocalEvent(stopLateDormancy, 13.0, compartment1);
		Event e14 = new LocalEvent(stopLateDormancy, 14.0, compartment2);//
		Event e15 = new LocalEvent(stopLateDormancy, 0.5, compartment1);
		Event e16 = new MockEvent(0.6);//
		
		eventList.addEvent(e1);
		eventList.addEvent(e2);
		eventList.addEvent(e3);
		eventList.addEvent(e4);
		eventList.addEvent(e5);
		eventList.addEvent(e6);
		eventList.addEvent(e7);
		eventList.addEvent(e8);
		eventList.addEvent(e9);
		eventList.addEvent(e10);
		eventList.addEvent(e11);
		eventList.addEvent(e12);
		eventList.addEvent(e13);
		eventList.addEvent(e14);
		eventList.addEvent(e15);
		eventList.addEvent(e16);
				
		AddCellToDiscreteCompartment eventAction = compartment1.deleteLastIntravasationEvent();
		assertEquals(addCell1, eventAction);
		
		try {
			//all other events should still be in the list
			assertEquals(e15, eventList.nextEvent());
			assertEquals(e16, eventList.nextEvent());
			assertEquals(e5, eventList.nextEvent());
			assertEquals(e2, eventList.nextEvent());
			assertEquals(e3, eventList.nextEvent());
			assertEquals(e4, eventList.nextEvent());			
			assertEquals(e6, eventList.nextEvent());
			assertEquals(e7, eventList.nextEvent());
			assertEquals(e8, eventList.nextEvent());
			assertEquals(e9, eventList.nextEvent());
			assertEquals(e10, eventList.nextEvent());
			assertEquals(e11, eventList.nextEvent());
			assertEquals(e12, eventList.nextEvent());
			assertEquals(e13, eventList.nextEvent());
			assertEquals(e14, eventList.nextEvent());			
			
			
			assertTrue(eventList.isEmpty());
			
		} catch (NoEventLeftException e) {
			e.printStackTrace();
			fail("Unexpected eception");
		}
		
		
		eventList.addEvent(e5);
		eventList.addEvent(e7);
		eventList.addEvent(e10);
		eventList.addEvent(e13);
		
		eventAction = compartment1.deleteLastIntravasationEvent();
		assertNull(eventAction);
	}
	
	public void testEqualsAndHashCode() throws MetastaseException{
		ContinuousCompartment compartment1 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), null);
		ContinuousCompartment compartment2 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), null);
		
		assertTrue(compartment1.equals(compartment1));
		assertTrue(compartment1.hashCode() == compartment1.hashCode());
		assertTrue(compartment1.equals(compartment2));
		assertTrue(compartment1.hashCode() == compartment2.hashCode());

		assertFalse(compartment1.equals(null));
		assertFalse(compartment1.equals(new ConstantFunction()));
		assertFalse(compartment1.hashCode() == new ConstantFunction().hashCode());
		
		//growth function
		ContinuousCompartment compartment3 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 10), null);
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		//colonization rate
		compartment3 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), new ColonizationRate(1, 2));
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		compartment1 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), new ColonizationRate(1, 2));
		compartment3 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), null);
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		compartment1 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), new ColonizationRate(1, 2));
		compartment3 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), new ColonizationRate(2, 1));
		assertFalse(compartment1.equals(compartment3));
		assertFalse(compartment1.hashCode() == compartment3.hashCode());
		
		compartment1 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), new ColonizationRate(1, 2));
		compartment3 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2, 1), new ColonizationRate(1, 2));
		assertTrue(compartment1.equals(compartment3));
		assertTrue(compartment1.hashCode() == compartment3.hashCode());
	}
	
	public void testToString() throws MetastaseException{
		ContinuousCompartment compartment1 = new ContinuousCompartment(0, "name", new MockHostOrgan(), new LinearFunction(2*60.*24., 1), null);
		String expected = "ContinuousCompartment [myId=1, name=name, isPrimaryTumor=false, isBloodStream=false, "
				+ "hostOrgan=HostOrgan [name=mock organ, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], "
				+ "successor=null, isDormant=false, isInLateDormancy=false, lateDormancyStartSize=0, lateDormancyDuration=0.0, "
				+ "growthFunction=0.0: LinearFunction [2.0*x+1.0 ] - , colonizationRate=null, eventTimeCalculator=null]";
		assertEquals(expected, compartment1.toString());

	}
}
