/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.conditions.Condition;
import de.fhstralsund.catsit.base.conditions.SizeCondition;
import de.fhstralsund.catsit.base.conditions.TimeCondition;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDeath;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDivision;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopLateDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNextCompartment;
import de.fhstralsund.catsit.base.mockobjects.MockCellProgressionAction;
import de.fhstralsund.catsit.base.mockobjects.MockDiscreteCompartmentType;
import de.fhstralsund.catsit.base.mockobjects.MockGenerationInfo;
import de.fhstralsund.catsit.base.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.machine.NoEventLeftException;
import de.fhstralsund.catsit.base.mockobjects.MockGlobalEvent;
import de.fhstralsund.catsit.support.functions.ConstantFunction;

public class TestDiscreteCompartment extends TestCase {
	 
	@Override
	public void setUp() {
		TreatmentList.reset();
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
	}
	
	@Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}
	
	public void testConstructor() throws NoEventLeftException{
		
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		DiscreteCompartment compartment2 = new DiscreteCompartment(0, organ, type, false);
		DiscreteCompartment compartment3 = new DiscreteCompartment(0, organ, type);	
		
		//check the IDs
		assertEquals(1, compartment1.getMyId());
		assertEquals(2, compartment2.getMyId());
		assertEquals(3, compartment3.getMyId());		
		
		//set the successors
		compartment1.setSuccessor(compartment3);
		compartment2.setSuccessor(compartment3);		
		
		assertEquals(compartment3, compartment1.getSuccessor());
		assertEquals(compartment3, compartment2.getSuccessor());
		assertNull(compartment3.getSuccessor());		
		
		//check the initial number of cells (should be zero)
		assertEquals(0, compartment1.getNumberOfCells(0.));
		assertEquals(0, compartment2.getNumberOfCells(0.));
		assertEquals(0, compartment3.getNumberOfCells(0.));
		
		//check, if primary tumor
		assertTrue(compartment1.isPrimaryTumor());
		assertFalse(compartment2.isPrimaryTumor());
		assertFalse(compartment3.isPrimaryTumor());
		
		//check, if bloodstream
		assertFalse(compartment1.isBloodStream());
		assertFalse(compartment2.isBloodStream());
		assertFalse(compartment3.isBloodStream());
		
		compartment3.setBloodStream(true);
		assertTrue(compartment3.isBloodStream());
		
		
		organ = new HostOrgan("test", new GaussianDistribution(2/60./24., 0), false);
		compartment1 = new DiscreteCompartment(0, organ, type, false);
		
		//check the host organ
		assertEquals(organ, compartment1.getHostOrgan());
		assertEquals("test", compartment1.getHostOrganName());
		
		//the compartment should be dormant...
		assertTrue(compartment1.isDormant());
		//...but not in late dormancy
		assertFalse(compartment1.isInLateDormancy());
		
		Event e = compartment1.getEventList().nextEvent();
		assertTrue(e instanceof LocalEvent);
		//a StopTumorDomancy Event should have been added to the event list
		assertTrue(((LocalEvent)e).getEventAction() instanceof StopDormancy);
		//the should be 2.0
		assertEquals(2.0, e.getTime());
		
		//test late dormancy
		organ = new HostOrgan("test", null, new GaussianDistribution(50, 0), new GaussianDistribution(10/60./24., 0), false);
		compartment1 = new DiscreteCompartment(0, organ, type, false);
		
		//check the start size and the duration
		assertEquals(50, compartment1.getLateDormancyStartSize());
		assertEquals(10.0, compartment1.getLateDormancyDuration());
	}
	
	public void testGetNumberOfCells() throws Exception {
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		
		compartment1.setNumberOfCells(12345);
		assertEquals(12345, compartment1.getNumberOfCells(20.0));
		
		compartment1.setNumberOfCells(54321);
		assertEquals(54321, compartment1.getNumberOfCells(20.0));
		
		compartment1.setDormant(true);
		assertEquals(54321, compartment1.getNumberOfCells(20.0));
		assertEquals(54321, compartment1.getNumberOfCells(50.0));
		assertEquals(54321, compartment1.getNumberOfCells(70.0));
		
		
		compartment1 = new DiscreteCompartment(0, organ, type, 150, false);
		compartment1.setDormant(true);
		assertEquals(150, compartment1.getNumberOfCells(20.0));
	}
	
	
	
	public void testAddCell() throws NoEventLeftException {
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		
		int numberOfCells = 0;

		assertEquals(numberOfCells, compartment1.getNumberOfCells(0.));
		for (int i=0; i<4; i++){
			compartment1.addCell(0);
			assertEquals(++numberOfCells, compartment1.getNumberOfCells(0.));
		}
		
		try {
			//adding cells shouldn't be possible when dormant
			compartment1.setDormant(true);
			compartment1.addCell(0.0);
			fail("An IllegalStateException should have been thrown.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
		
		try {
			//adding cells shouldn't be possible when in late dormancy
			compartment1.setDormant(false);
			compartment1.setInLateDormancy(true);
			compartment1.addCell(0.0);
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
		
		
		organ = new HostOrgan("test", null, new GaussianDistribution(200, 0), new GaussianDistribution(10/60./24., 0), false);
		compartment1 = new DiscreteCompartment(0, organ, type, true);
		
		//the late dormancy starts at size 200
		compartment1.setNumberOfCells(198);
		compartment1.addCell(0);
		assertEquals(199, compartment1.getNumberOfCells(0.));
		assertFalse(compartment1.isInLateDormancy());
		compartment1.addCell(140.);
		assertEquals(200, compartment1.getNumberOfCells(0.));
		assertTrue(compartment1.isInLateDormancy());
		
		// a StopLateDormancy Event should have been added to the List
		Event e = compartment1.getEventList().nextEvent();
		assertEquals(150.0, e.getTime());
		assertTrue(e instanceof LocalEvent);
		assertTrue(((LocalEvent)e).getEventAction() instanceof StopLateDormancy);		
		
		//if the tumor looses cells in the course of a treatment, no second late dormancy shall occur if the start size is reached a second, third, fourth ... time
		compartment1.setInLateDormancy(false);
		compartment1.setNumberOfCells(199);
		
		compartment1.addCell(0);
		assertEquals(200, compartment1.getNumberOfCells(0.));
		assertFalse(compartment1.isInLateDormancy());
	}

	public void testDeleteCell() {
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		
		int numberOfCells = 10;
		for (int i=0; i<numberOfCells; i++) {
			compartment1.addCell(0);
		}
		
		assertEquals(numberOfCells, compartment1.getNumberOfCells(0.));
		
		for (int i=0; i<4; i++){
			compartment1.deleteCell(0.);
			assertEquals(--numberOfCells, compartment1.getNumberOfCells(0.));
		}
		
		try {
			compartment1.setNumberOfCells(0);
			compartment1.deleteCell(0.);
			fail("An IllegalStateException should have been thrown.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}		
	}

	public void testTransferCellToNextCompartment() {
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		DiscreteCompartment compartment2 = new DiscreteCompartment(0, organ, type, false);
		compartment1.setSuccessor(compartment2);
		
		// prepare compartments
		int numberOfCellsCompartment1 = 7;
		for (int i=0; i<numberOfCellsCompartment1; i++){
			compartment1.addCell(0);
		}
		assertEquals(numberOfCellsCompartment1, compartment1.getNumberOfCells(0.));
	
		int numberOfCellsCompartment2 = 5;
		for (int i=0; i<numberOfCellsCompartment2; i++){
			compartment2.addCell(0);
		}
		assertEquals(numberOfCellsCompartment2, compartment2.getNumberOfCells(0.));
		
		// test transferCellToNextCompartment
		for (int i=0; i<3; i++) {
			compartment1.transferCellToNextCompartment(0);
			assertEquals(--numberOfCellsCompartment1, compartment1.getNumberOfCells(0.));
			assertEquals(++numberOfCellsCompartment2, compartment2.getNumberOfCells(0.));
		}
		
		try {
			//no cells left to transfer
			compartment1.setNumberOfCells(0);
			compartment1.transferCellToNextCompartment(0.0);
			fail("An IllegalStateException should have been thrown.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
		
		try {
			//no successor defined to where to transfer the cell
			compartment1.setSuccessor(null);
			compartment1.setNumberOfCells(5);
			compartment1.transferCellToNextCompartment(0.0);
			fail("An IllegalStateException should have been thrown.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
	}

	public void testTransferCellToNewDiscreteCompartment() {

		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		
		int numberOfCellsCompartment1 = 22;
		for (int i=0; i<numberOfCellsCompartment1; i++){
			compartment1.addCell(0);
		}
		assertEquals(numberOfCellsCompartment1, compartment1.getNumberOfCells(0.));
		
		DiscreteCompartment newCompartment = compartment1.transferCellToNewDiscreteCompartment(0, new MockHostOrgan(), new MockDiscreteCompartmentType());
		//one cell less in the old compartment
		assertEquals(--numberOfCellsCompartment1, compartment1.getNumberOfCells(0.));
		//one cell in the new compartment
		assertEquals(1, newCompartment.getNumberOfCells(0.));
		//new compartment should be registered in the CompartmentInventory
		assertTrue(CompartmentInventory.getCompartments().contains(newCompartment));
		//the new compartment is neither primary nor blood stream
		assertFalse(newCompartment.isPrimaryTumor());
		assertFalse(newCompartment.isBloodStream());
			
		try {
			//no cells left to transfer
			compartment1.setNumberOfCells(0);
			compartment1.transferCellToNewDiscreteCompartment(0.0, new MockHostOrgan(), type);
			fail("An IllegalStateException should have been thrown.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
	}
	
	public void testTransferCellToNewContinuousCompartment() throws MetastaseException{
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);		
		
		int numberOfCellsCompartment1 = 22;
		for (int i=0; i<numberOfCellsCompartment1; i++){
			compartment1.addCell(0);
		}
		assertEquals(numberOfCellsCompartment1, compartment1.getNumberOfCells(0.));
		
		ContinuousCompartment newCompartment = compartment1.transferCellToNewContinuousCompartment(
				0., "new continuous compartment", new MockHostOrgan(), new ConstantFunction(), null);
		//one cell less in the old compartment
		assertEquals(--numberOfCellsCompartment1, compartment1.getNumberOfCells(0.));
		//new compartment should be registered in the CompartmentInventory
		assertTrue(CompartmentInventory.getCompartments().contains(newCompartment));
			
		try {
			//no cells left to transfer
			compartment1.setNumberOfCells(0);
			compartment1.transferCellToNewContinuousCompartment(0.0, "name", new MockHostOrgan(), new ConstantFunction(), null);
			fail("An IllegalStateException should have been thrown.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
	}
	
	public void testAddCompartmentType() throws MetastaseException{
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type1 = new MockDiscreteCompartmentType();
		DiscreteCompartmentType type2 = new MockDiscreteCompartmentType();
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type1, true);
		
		compartment1.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 100.), type2);
		assertEquals(type1, compartment1.getActualCompartmentType(10.));
		assertEquals(type2, compartment1.getActualCompartmentType(100.));		
	}
	
	public void testAddScheduledCompartmentType() throws MetastaseException{
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type1 = new MockDiscreteCompartmentType("type 1");
		DiscreteCompartmentType type2 = new MockDiscreteCompartmentType("type 2");
		DiscreteCompartmentType type3 = new MockDiscreteCompartmentType("type 3");
		DiscreteCompartmentType type4 = new MockDiscreteCompartmentType("type 4");
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type1, true);
		
		compartment1.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 100/60./24.), type2);
		compartment1.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 200/60./24.), type3);
		compartment1.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 50/60./24.), type4);
		
		assertEquals("type 1", compartment1.getActualCompartmentType(0.).getName());
		assertEquals("type 1", compartment1.getActualCompartmentType(1.).getName());
		assertEquals("type 4", compartment1.getActualCompartmentType(50.).getName());
		assertEquals("type 4", compartment1.getActualCompartmentType(99.).getName());
		assertEquals("type 2", compartment1.getActualCompartmentType(100.).getName());
		assertEquals("type 2", compartment1.getActualCompartmentType(111.).getName());
		assertEquals("type 3", compartment1.getActualCompartmentType(200.).getName());
		assertEquals("type 3", compartment1.getActualCompartmentType(2000.).getName());
	}
	
	public void testGetActualScheduledCompartmentType(){
	
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType("type 1");		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		DiscreteCompartment compartment2 = new DiscreteCompartment(0, organ, type, false);
		
		DiscreteCompartmentType type2 = new DiscreteCompartmentType("type 2");
		DiscreteCompartmentType type3 = new DiscreteCompartmentType("type 3");
		DiscreteCompartmentType type4 = new DiscreteCompartmentType("type 4");
		DiscreteCompartmentType type5 = new DiscreteCompartmentType("type 5");
		
		compartment1.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 10./60./24.), type2);
		compartment1.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 12./60./24.), type3);
				
		assertEquals("type 1", compartment1.getActualCompartmentType(0.).getName());
		assertEquals("type 1", compartment1.getActualCompartmentType(1.).getName());
		assertEquals("type 2", compartment1.getActualCompartmentType(10.).getName());
		assertEquals("type 2", compartment1.getActualCompartmentType(11.).getName());
		assertEquals("type 3", compartment1.getActualCompartmentType(12.).getName());
		assertEquals("type 3", compartment1.getActualCompartmentType(111.).getName());
		assertEquals("type 1", compartment1.getActualCompartmentType(5.).getName());
		
		//test the size conditions
		compartment2.addCompartmentType(new SizeCondition(Condition.GREATER_EQUALS, 100000), type2);
		compartment2.addCompartmentType(new SizeCondition(Condition.LESSER, 100), type3);
		compartment2.addCompartmentType(new SizeCondition(Condition.GREATER_EQUALS, 1000), type4);
		compartment2.addCompartmentType(new SizeCondition(Condition.LESSER, 500), type5);
		
		assertEquals("type 1", compartment2.getActualCompartmentType(0.).getName());
		assertEquals("type 1", compartment2.getActualCompartmentType(10000.).getName());
		//add some cells
		compartment2.setNumberOfCells(1000);
		compartment2.addCell(50.);
		assertEquals("type 1", compartment2.getActualCompartmentType(10.).getName());
		
		//add some more cells
		compartment2.setNumberOfCells(99999);
		compartment2.addCell(100.);
		assertEquals("type 2", compartment2.getActualCompartmentType(10.).getName());
		
		//add some more cells
		compartment2.setNumberOfCells(111000);
		compartment2.addCell(101.);
		assertEquals("type 2", compartment2.getActualCompartmentType(10.).getName());
		
		//delete some cells
		compartment2.setNumberOfCells(11000);
		compartment2.deleteCell(110.);
		compartment2.deleteCell(111.);
		compartment2.deleteCell(112.);
		assertEquals("type 2", compartment2.getActualCompartmentType(10.).getName());
		
		//delete some more cells
		compartment2.setNumberOfCells(100);
		compartment2.deleteCell(120.);
		assertEquals("type 3", compartment2.getActualCompartmentType(10.).getName());
		
		//add some cells again
		compartment2.setNumberOfCells(998);
		compartment2.addCell(150.);
		assertEquals("type 3", compartment2.getActualCompartmentType(10.).getName());
		
		//add another cell
		compartment2.addCell(151.);
		assertEquals("type 4", compartment2.getActualCompartmentType(10.).getName());
		
		//delete some more cells
		compartment2.setNumberOfCells(500);
		compartment2.deleteCell(170.);
		assertEquals("type 5", compartment2.getActualCompartmentType(10.).getName());
		
		//add some cells again
		compartment2.setNumberOfCells(999);
		compartment2.addCell(180.);
		assertEquals("type 5", compartment2.getActualCompartmentType(10.).getName());
		
		//add both time and size conditions
		DiscreteCompartment compartment3 = new DiscreteCompartment(0, organ, type, false);
			
		compartment3.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 10./60./24.), type2);
		compartment3.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 400./60./24.), type3);
		compartment3.addCompartmentType(new SizeCondition(Condition.GREATER_EQUALS, 100000), type4);
		compartment3.addCompartmentType(new SizeCondition(Condition.LESSER, 100), type5);
		
		//add some cells
		compartment3.setNumberOfCells(50);
		compartment3.addCell(1.);		
		
		assertEquals("type 1", compartment3.getActualCompartmentType(2.).getName());
		assertEquals("type 1", compartment3.getActualCompartmentType(5.).getName());
		
		//add some cells
		compartment3.setNumberOfCells(50);
		compartment3.addCell(9.);
		assertEquals("type 2", compartment3.getActualCompartmentType(10.).getName());
		assertEquals("type 2", compartment3.getActualCompartmentType(100.).getName());
		
		compartment3.setNumberOfCells(99999);
		compartment3.addCell(300.);
		assertEquals("type 4", compartment3.getActualCompartmentType(302.).getName());
		assertEquals("type 4", compartment3.getActualCompartmentType(399.).getName());
		

		compartment3.setNumberOfCells(1000000);
		compartment3.addCell(405.);
		compartment3.addCell(450.);
		assertEquals("type 3", compartment3.getActualCompartmentType(400.).getName());
		assertEquals("type 3", compartment3.getActualCompartmentType(450.).getName());
		
		compartment3.setNumberOfCells(50);
		compartment3.setNumberOfCells(100);
		compartment3.deleteCell(500.);
		assertEquals("type 5", compartment3.getActualCompartmentType(550.).getName());
	}

	
	public void testCreateFirstEvent() throws NoEventLeftException{
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();
		type.addPossibleEventAction(new MockCellProgressionAction());
		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
			
		//create the first event
		compartment1.createFirstEvent(0);
		
		Event e = compartment1.getEventList().nextEvent();
		
		assertTrue(e instanceof LocalEvent);
		assertTrue(((LocalEvent)e).getEventAction() instanceof CellProgressionEventAction);
	}
	
	public void testCreateNextEvent() throws NoEventLeftException{
		
		//create the mock event list and action event
		EventList eventList = EventList.getInstance();
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();
		type.addPossibleEventAction(new MockCellProgressionAction());
		
		DiscreteCompartment compartment = new DiscreteCompartment(0, organ, type, true);
		
		compartment.createNextEvent(0);
		compartment.createNextEvent(1);
		compartment.createNextEvent(2);
		compartment.createNextEvent(3);
				
		LocalEvent e1 = (LocalEvent)eventList.nextEvent();
		LocalEvent e2 = (LocalEvent)eventList.nextEvent();
		LocalEvent e3 = (LocalEvent)eventList.nextEvent();
		LocalEvent e4 = (LocalEvent)eventList.nextEvent();
		
		assertTrue(eventList.isEmpty());
			
		//check, if the time was computed correctly
		assertEquals(10.0, e1.getTime());
		assertEquals(11.0, e2.getTime());
		assertEquals(12.0, e3.getTime());
		assertEquals(13.0, e4.getTime());
			
		//check, if the correct compartment was given to the new event
		assertSame(compartment, e1.getCompartment());
		assertSame(compartment, e2.getCompartment());
		assertSame(compartment, e3.getCompartment());
		assertSame(compartment, e4.getCompartment());
		
		
		//check, whether an exception is thrown when trying to create a new event while in late dormancy 
		compartment.setInLateDormancy(true);		
		try {
			compartment.createNextEvent(30);
			fail("An exception should have been thrown => creating the next event while in late dormany phase is not allowed.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
		
		//check, whether an exception is thrown when trying to create a new event while in dormancy 
		compartment.setInLateDormancy(false);
		compartment.setDormant(true);
		try {
			compartment.createNextEvent(30);
			fail("An exception should have been thrown => creating the next event while in dormany phase is not allowed.");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
	}

	public void testDeleteEvents() throws NoEventLeftException{
		
		EventList eventList = EventList.getInstance();
		HostOrgan organ = new MockHostOrgan();
		DiscreteCompartmentType type = new MockDiscreteCompartmentType();		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, organ, type, true);
		DiscreteCompartment compartment2 = new DiscreteCompartment(0, organ, type, false);
		
		CellDivision cp_action = new CellDivision("cell proliferation", new MockGenerationInfo());
		CellDeath cd_action = new CellDeath("cell death", new MockGenerationInfo());
		TransferToNextCompartment ttnc_action = new TransferToNextCompartment("transfer to next", new MockGenerationInfo());
		TransferToNewDiscreteCompartment ttndc_action = new TransferToNewDiscreteCompartment("transfer to new discrete", new MockGenerationInfo(), organ, type, null);
		TransferToNewContinuousCompartment ttncc_action = new TransferToNewContinuousCompartment("transfer to new continuous", new MockGenerationInfo(), 
																							"new Compartment", organ, new ConstantFunction(), null, null);		
		AddCellToDiscreteCompartment ac_action = new AddCellToDiscreteCompartment("test1", null);
		CellProgressionEventAction random_action = new MockCellProgressionAction();

		Event e1 = new LocalEvent(cp_action, 1.0, compartment1);
		Event e2 = new LocalEvent(cp_action, 2.0, compartment1);
		Event e3 = new LocalEvent(cp_action, 2.0, compartment1);
		
		Event e4 = new LocalEvent(cd_action, 4.0, compartment1);
		Event e5 = new LocalEvent(cd_action, 5.0, compartment1);
		Event e6 = new LocalEvent(cd_action, 6.0, compartment1);
		
		Event e7 = new LocalEvent(ttnc_action, 7.0, compartment1);
		Event e8 = new LocalEvent(ttnc_action, 8.0, compartment1);
		Event e9 = new LocalEvent(ttnc_action, 9.0, compartment1);
		
		Event e10 = new LocalEvent(ttndc_action, 10.0, compartment1);
		Event e11 = new LocalEvent(ttndc_action, 11.0, compartment1);
		Event e12 = new LocalEvent(ttndc_action, 12.0, compartment1);
		
		Event e13 = new LocalEvent(ttncc_action, 13.0, compartment1);
		Event e14 = new LocalEvent(ttncc_action, 14.0, compartment1);
		Event e15 = new LocalEvent(ttncc_action, 15.0, compartment1);
		
		Event e16 = new LocalEvent(ttncc_action, 16.0, compartment2);
		Event e17 = new LocalEvent(ac_action, 17.0, compartment1);
		Event e18 = new LocalEvent(random_action, 18.0, compartment1);
		
		Event e19 = new MockGlobalEvent(19.0);
		
		eventList.addEvent(e1);		eventList.addEvent(e2);		eventList.addEvent(e3);		eventList.addEvent(e4);
		eventList.addEvent(e5);		eventList.addEvent(e6);		eventList.addEvent(e7);		eventList.addEvent(e8);
		eventList.addEvent(e9);		eventList.addEvent(e10);	eventList.addEvent(e11);	eventList.addEvent(e12);
		eventList.addEvent(e13);	eventList.addEvent(e14);	eventList.addEvent(e15);	eventList.addEvent(e16);
		eventList.addEvent(e17);	eventList.addEvent(e18);	eventList.addEvent(e19);
				
		compartment1.deleteEvents(1.0);

		try {
			assertEquals(e16, eventList.nextEvent());
			assertEquals(e18, eventList.nextEvent());
			assertEquals(e19, eventList.nextEvent());
			
			assertTrue(eventList.isEmpty());
		} catch (NoEventLeftException e) {
			e.printStackTrace();
			fail("Unexpected eception");
		}		
		
		
		eventList.addEvent(e1);		eventList.addEvent(e2);		eventList.addEvent(e3);		eventList.addEvent(e4);
		eventList.addEvent(e5);		eventList.addEvent(e6);		eventList.addEvent(e7);		eventList.addEvent(e8);
		eventList.addEvent(e9);		eventList.addEvent(e10);	eventList.addEvent(e11);	eventList.addEvent(e12);
		eventList.addEvent(e13);	eventList.addEvent(e14);	eventList.addEvent(e15);	eventList.addEvent(e16);
		eventList.addEvent(e17);	eventList.addEvent(e18);	eventList.addEvent(e19);
				
		compartment1.deleteEvents(0.8);
		
		try {
			// one event should be left in the event list of each type
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof CellDivision);
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof CellDeath);
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof TransferToNextCompartment);
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof TransferToNewDiscreteCompartment);
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof TransferToNewContinuousCompartment);
			
			assertEquals(e16, eventList.nextEvent());
			assertEquals(e17, eventList.nextEvent());
			assertEquals(e18, eventList.nextEvent());
			assertEquals(e19, eventList.nextEvent());
			
			assertTrue(eventList.isEmpty());
		} catch (NoEventLeftException e) {
			e.printStackTrace();
			fail("Unexpected eception");
		}		
		
		//try to apply a wrong value for the rate
		eventList.addEvent(e1);		eventList.addEvent(e2);		eventList.addEvent(e3);		eventList.addEvent(e4);
		
		try {
			compartment1.deleteEvents(1.001);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we are supposed to be here
		}
		
		try {
			compartment1.deleteEvents(-0.01);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we are supposed to be here
		}
		
		//no events to delete
		compartment1.deleteEvents(0.);
		
		//rate to small to delete any events
		compartment1.deleteEvents(0.1);
		try {
			//all added events should still be in the event list
			assertEquals(e1, eventList.nextEvent());
			assertEquals(e2, eventList.nextEvent());
			assertEquals(e3, eventList.nextEvent());
			assertEquals(e4, eventList.nextEvent());
			assertTrue(eventList.isEmpty());
		} catch (NoEventLeftException e) {
			e.printStackTrace();
			fail("Unexpected eception");
		}
	}
	
	
	
	public void testToString(){
//		String expected = "DiscreteCompartment [myId=1, name=sample type 1, creationTime=0.0, hostOrgan=HostOrgan [name=mockOrgan, mass=0.0, " +
//				"dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], " +
//				"successor=DiscreteCompartment [myId=2, name=sample type 1, creationTime=0.0, hostOrgan=HostOrgan [name=mockOrgan, mass=0.0, " +
//				"dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], " +
//				"successor=DiscreteCompartment [myId=3, name=sample type 2, creationTime=0.0, hostOrgan=HostOrgan [name=mockOrgan, mass=0.0, " +
//				"dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], " +
//				"successor=null, timeScheduledCompartmentTypes=[ScheduledCompartmentType [condition=time>=0.0, compartmentType=DiscreteCompartmentType " +
//				"[name=sample type 2, possibleEventActions=[name=some name, genererationInfo=GenerationInfo [probability=1.0, " +
//				"meanTimeToNextEvent=ConstantFunction [returnValue=1.0], distribution=UniformDistribution [min=1.0, interval=1.0]]]] ]], " +
//				"sizeScheduledCompartmentTypes=[], isDormant=false, dormancyTime=0.0, isInLateDormancy=false, lateDormancyStartSize=0, " +
//				"lateDormancyDuration=0.0, alreadyUndergoneLateDormancy=false, startSize=0, numberOfCells=0, deleteCelldeathImmediately=false], " +
//				"timeScheduledCompartmentTypes=[ScheduledCompartmentType [condition=time>=0.0, compartmentType=DiscreteCompartmentType [name=sample type 1, " +
//				"possibleEventActions=[name=some name, genererationInfo=GenerationInfo [probability=1.0, meanTimeToNextEvent=ConstantFunction [returnValue=1.0], " +
//				"distribution=UniformDistribution [min=1.0, interval=1.0]]]] ]], sizeScheduledCompartmentTypes=[], isDormant=false, dormancyTime=0.0, " +
//				"isInLateDormancy=false, lateDormancyStartSize=0, lateDormancyDuration=0.0, alreadyUndergoneLateDormancy=false, startSize=0, numberOfCells=0, " +
//				"deleteCelldeathImmediately=false], timeScheduledCompartmentTypes=[ScheduledCompartmentType [condition=time>=0.0, " +
//				"compartmentType=DiscreteCompartmentType [name=sample type 1, possibleEventActions=[name=some name, genererationInfo=GenerationInfo " +
//				"[probability=1.0, meanTimeToNextEvent=ConstantFunction [returnValue=1.0], distribution=UniformDistribution [min=1.0, interval=1.0]]]] ]], " +
//				"sizeScheduledCompartmentTypes=[], isDormant=false, dormancyTime=0.0, isInLateDormancy=false, lateDormancyStartSize=0, lateDormancyDuration=0.0, " +
//				"alreadyUndergoneLateDormancy=false, startSize=0, numberOfCells=0, deleteCelldeathImmediately=false]";
//		assertEquals(expected, compartment1.toString());
	}
	
	public void testEqualsAndHashCode(){
//		assertEquals(compartment1, compartment1);
//		assertEquals(compartment1.hashCode(), compartment1.hashCode());
//		
//		assertFalse(compartment1.equals(compartment2));
//		assertFalse(compartment1.hashCode() == compartment2.hashCode());
//		assertFalse(compartment1.equals(null));
//		assertFalse(compartment1.equals(new ConstantFunction()));
//		assertFalse(compartment1.hashCode() == new ConstantFunction().hashCode());
//		
//		compartment1.setSuccessor(compartment3);
//		compartment2.setSuccessor(compartment3);
//		assertEquals(compartment1, compartment2);
//		assertEquals(compartment1.hashCode(), compartment2.hashCode());
//		
//		assertFalse(compartment1.equals(compartment2));
//		assertFalse(compartment1.hashCode() == compartment2.hashCode());
//		
//		compartment1.addCell(0);
//		assertFalse(compartment1.equals(compartment2));
//		assertFalse(compartment1.hashCode() == compartment2.hashCode());
//		
//		compartment2.addCell(0);
//		assertTrue(compartment1.equals(compartment2));
//		assertEquals(compartment1.hashCode(), compartment2.hashCode());
//		
//		
//		try {
//			compartment1 = new MockDiscreteCompartment(100);
//			compartment1.addCell(0);
//			assertFalse(compartment1.equals(new MockDiscreteCompartment(101)));
//			assertFalse(compartment1.hashCode() == compartment2.hashCode());
//			
//		} catch (MetastaseException e) {
//			fail(e.getMessage());
//			e.printStackTrace();
//		}
	}
}
