/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.LocalEventAction;
import de.fhstralsund.catsit.base.mockobjects.MockCellProgressionAction;
import de.fhstralsund.catsit.support.functions.ConstantFunction;

/**
 * Tests for {@link metastase.base.DiscreteCompartmentType}
 * 
 * @author Gero Wedemann, Anja Bethge, Michael Suhr
 */
public class TestDiscreteCompartmentType extends TestCase {
	private static final long NUMBER_TEST_ITERATIONS = 1000000;
	private static final double DELTA = 0.002;
	
	private DiscreteCompartmentType compartmentType;
	private CellProgressionEventAction eventAction1;
	private CellProgressionEventAction eventAction2;
	private CellProgressionEventAction eventAction3;
	private CellProgressionEventAction eventAction4;
	
	@Override
	public void setUp() {
		compartmentType = new DiscreteCompartmentType("an example");
		
		eventAction1 = new MockCellProgressionAction("a1", 0.3);
		eventAction2 = new MockCellProgressionAction("a2", 0.5);
		eventAction3 = new MockCellProgressionAction("a3", 0.2);
		eventAction4 = new MockCellProgressionAction("a4", 0.05);
	}
	
	
	public void testGetName() {
		assertEquals("an example", compartmentType.getName());
	}
	
	
	/**
	 * Test method for {@link metastase.base.DiscreteCompartmentType#checkPossibleEventActions()}.
	 */
	public void testCheckPossibleEventActions() {
		
		//no action event added
		try {
			compartmentType.checkPossibleEventActions();
			fail("expected DiscreteCompartmentTypeException not thrown");
		} catch (DiscreteCompartmentTypeException e) {
			// we should be here
		}
		
		//sum of all probabilities < 1
		compartmentType.addPossibleEventAction(eventAction1);
		try {
			compartmentType.checkPossibleEventActions();
			fail("expected DiscreteCompartmentTypeException not thrown");
		} catch (DiscreteCompartmentTypeException e) {
			// we should be here
		}
		
		//sum of all probabilities = 1
		compartmentType.addPossibleEventAction(eventAction2);
		compartmentType.addPossibleEventAction(eventAction3);
		try {
			compartmentType.checkPossibleEventActions();
		} catch (DiscreteCompartmentTypeException e) {
			fail("unexpected exception: " + e);
		}
		
		//sum of all probabilities > 1
		compartmentType.addPossibleEventAction(eventAction4);
		try {
			compartmentType.checkPossibleEventActions();
			fail("expected DiscreteCompartmentTypeException not thrown");
		} catch (DiscreteCompartmentTypeException e) {
			// we should be here
		}		
	}

	/**
	 * Test method for {@link metastase.base.DiscreteCompartmentType#getNextEventAction()}.
	 */
	public void testGetNextEventAction() {
		compartmentType.addPossibleEventAction(eventAction1);
		compartmentType.addPossibleEventAction(eventAction2);
		compartmentType.addPossibleEventAction(eventAction3);
		
		try {
			compartmentType.checkPossibleEventActions();
		} catch (DiscreteCompartmentTypeException e) {
			fail("unexpected exception: " + e);
		}
		
		long numberEA1 = 0;
		long numberEA2 = 0;
		long numberEA3 = 0;
		for (int i=0; i<NUMBER_TEST_ITERATIONS; i++) {
			LocalEventAction nextEA = compartmentType.getNextEventAction();
			if (nextEA == eventAction1) numberEA1++;
			if (nextEA == eventAction2) numberEA2++;
			if (nextEA == eventAction3) numberEA3++;
		}
		assertEquals(NUMBER_TEST_ITERATIONS, numberEA1+numberEA2+numberEA3);
		double rateNumber1 = (double)numberEA1/(double)NUMBER_TEST_ITERATIONS;
		double rateNumber2 = (double)numberEA2/(double)NUMBER_TEST_ITERATIONS;

		assertEquals(0.3, rateNumber1, DELTA);
		assertEquals(0.5, rateNumber2, DELTA);
	}

	public void testToString() {
		String expected = "DiscreteCompartmentType [name=an example, possibleEventActions=[]]";
		assertEquals(expected, compartmentType.toString());
	}

    public void testEquals() {
    	assertTrue(compartmentType.equals(compartmentType));
    	
        DiscreteCompartmentType newCompartmentType = new DiscreteCompartmentType(compartmentType.getName());
        assertTrue(compartmentType.equals(newCompartmentType));
        
        compartmentType.addPossibleEventAction(eventAction1);
        assertFalse(compartmentType.equals(newCompartmentType));
        
        newCompartmentType.addPossibleEventAction(eventAction1);
        assertTrue(compartmentType.equals(newCompartmentType));        
		
		assertFalse(compartmentType.equals(null));
		assertFalse(compartmentType.equals(new ConstantFunction()));
		assertFalse(compartmentType.equals(new DiscreteCompartmentType("123")));        
    }


    public void testHashCode() {
        DiscreteCompartmentType newCompartmentType = new DiscreteCompartmentType(compartmentType.getName());
        assertEquals(compartmentType.hashCode(), newCompartmentType.hashCode());
        compartmentType.addPossibleEventAction(eventAction1);
        newCompartmentType.addPossibleEventAction(eventAction1);
        assertEquals(compartmentType.hashCode(), newCompartmentType.hashCode());
    }

}
