/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.distribution;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.support.functions.ConstantFunction;


public class TestGaussianDistribution extends TestCase {

	public void testGetDouble() {
		
		//check the constructor
		try{
			new GaussianDistribution(2, -2);
			fail("An IllegalArgumentException should have been thrown.");
		}catch(IllegalArgumentException e){
			assertEquals("The standard derivation must not be smaller than '0'", e.getMessage());
		}
		
		try{
			new GaussianDistribution(0, 0);
			fail("An IllegalArgumentException should have been thrown.");
		}catch(IllegalArgumentException e){
			assertEquals("At least one value (mean or stdev) must be bigger than '0'", e.getMessage());
		}
		
		
		double meanGiven = 3.;
		double stdGiven = 1.;
		GaussianDistribution distri = new GaussianDistribution(meanGiven, stdGiven);
		
		assertEquals(meanGiven, distri.getMean());
		assertEquals(stdGiven, distri.getStddev());
		
		long[] buckets = new long[100];	 // -5 .. 5
		int numberOfLoops = 10000000;
		
		for (int i=0; i<numberOfLoops; i++) {
			double rand = distri.getDouble();
			int index = (int) (50. + 10. * (rand-meanGiven));
			if (index>99) index = 99;
			if (index<0) index = 0;
			buckets[index]++;
		}
		
		
		double normFactor = 1./Math.sqrt(2*stdGiven*Math.PI);
		for (int i=40; i<=60; i++){ // 1 .. 5
			double x = ((double)i-50.)/10. + meanGiven;
			double expectedProbability = normFactor * Math.exp((-1.*(x-meanGiven)*(x-meanGiven)) / (2*stdGiven*stdGiven));
			// the factor 5. in the next equation is the inverse interval 0.2
			double isProbability = 10.*(double)buckets[i]/(double)numberOfLoops;					
			assertEquals("at i=" +i, expectedProbability, isProbability, 0.015 );
		}
	}
	
	public void testEquals(){
		GaussianDistribution dis = new GaussianDistribution(60., 7.);
		
		assertFalse(dis.equals(null));
		assertFalse(dis.equals(new ConstantFunction()));
		assertFalse(dis.equals(new GaussianDistribution(60, 7.1)));
		assertFalse(dis.equals(new GaussianDistribution(60.4, 7.)));
		assertFalse(dis.equals(new GaussianDistribution(-60, 7.)));
		assertFalse(dis.equals(new GaussianDistribution(-3, 1)));
		
		assertTrue(dis.equals(dis));
		assertTrue(dis.equals(new GaussianDistribution(60., 7.)));
	}
	
	public void testHashCode() {
		GaussianDistribution dis = new GaussianDistribution(60., 7.);

		assertFalse(dis.hashCode() == new ConstantFunction().hashCode());
		assertFalse(dis.hashCode() == new GaussianDistribution(60, 7.1).hashCode());
		assertFalse(dis.hashCode() == new GaussianDistribution(60.4, 7.).hashCode());
		assertFalse(dis.hashCode() == new GaussianDistribution(-60, 7.).hashCode());
		assertFalse(dis.hashCode() == new GaussianDistribution(-3, 1).hashCode());
		
		assertTrue(dis.hashCode() == dis.hashCode());
		assertTrue(dis.hashCode() == new GaussianDistribution(60., 7.).hashCode());
	}
	
	public void testToString(){		
		GaussianDistribution dis = new GaussianDistribution(60., 7.);
		assertEquals("GaussianDistribution [mean=60.0, stddev=7.0]", dis.toString());

		dis = new GaussianDistribution(60.4, 7.);
		assertEquals("GaussianDistribution [mean=60.4, stddev=7.0]", dis.toString());

		dis = new GaussianDistribution(-60, 8.);
		assertEquals("GaussianDistribution [mean=-60.0, stddev=8.0]", dis.toString());
	}
}
