/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.globaleventactions;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.GlobalEventAction;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.Radioembolization;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.mockobjects.MockDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.mockobjects.MockRadioembolizationTreatment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;

public class TestRadioembolization extends TestCase {

	protected void setUp() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
		TreatmentList.reset();
	}

	@Override
	protected void tearDown() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
		TreatmentList.reset();
	}	
	
	public void testExecute() throws MetastaseException{
		
		DiscreteCompartment dc = new MockDiscreteCompartment(true, "organ1");
		DiscreteCompartment dc2 = new MockDiscreteCompartment(true, "organ1");
		DiscreteCompartment dc3 = new MockDiscreteCompartment(false, "organ1");
		DiscreteCompartment dc4 = new MockDiscreteCompartment(false, "organ1");
		DiscreteCompartment dc5 = new MockDiscreteCompartment(false, "organ2");
		ContinuousCompartment cc = new MockContinuousCompartment(true);
		ContinuousCompartment cc2 = new MockContinuousCompartment(false);
		
		dc.setNumberOfCells(10000000); 	//big enough
		dc2.setNumberOfCells(0); 		//already dead
		dc3.setNumberOfCells(1000000); 	//right at the limit
		dc4.setNumberOfCells(100000);	//to small
		dc5.setNumberOfCells(10000000); //big enough
		
		
		assertEquals(10000000, dc.getNumberOfCells(100));
		assertEquals(0, dc2.getNumberOfCells(100));
		assertEquals(1000000, dc3.getNumberOfCells(100));
		assertEquals(100000, dc4.getNumberOfCells(100));
		assertEquals(10000000, dc5.getNumberOfCells(100));
		assertEquals(100, cc.getNumberOfCells(100)); //the mock compartment has a linear function as growth function x=1t+0
		assertEquals(100, cc2.getNumberOfCells(100));
		
		GlobalEventAction eventAction = new Radioembolization("name", new MockRadioembolizationTreatment("organ1"));
		eventAction.execute(150);
		
		assertEquals(20, dc.getNumberOfCells(100));		//the MockRadioembolizationTreatment always returns 20 cells for computeNumberOfSurvivingCells
		assertEquals(0, dc2.getNumberOfCells(100)); 	//already dead => stays dead
		assertEquals(20, dc3.getNumberOfCells(100)); 	
		assertEquals(100000, dc4.getNumberOfCells(100));//to small => no change
		assertEquals(10000000, dc5.getNumberOfCells(100));//wrong organ => no change
		assertEquals(100, cc.getNumberOfCells(100)); 	// no discrete compartment => no change
		assertEquals(100, cc2.getNumberOfCells(100)); 	// no discrete compartment => no change		
	}
	
	public void testEqualsAndHashcode(){
		Radioembolization re1 = new Radioembolization("name", new MockRadioembolizationTreatment("organ"));
		Radioembolization re2 = new Radioembolization("name", new MockRadioembolizationTreatment("organ"));
		Radioembolization re3 = new Radioembolization("name", new MockRadioembolizationTreatment("organ2"));
		
		assertTrue(re1.equals(re1));
		assertTrue(re1.hashCode() == re1.hashCode());
		assertTrue(re1.equals(re2));
		assertTrue(re1.hashCode() == re2.hashCode());

		assertFalse(re1.equals(null));
		assertFalse(re1.equals(new ConstantFunction()));
		assertFalse(re1.hashCode() == new ConstantFunction().hashCode());
		
		assertFalse(re1.equals(re3));
		assertFalse(re1.hashCode() == re3.hashCode());
	}
	
	public void testToString(){
		Radioembolization re = new Radioembolization("name", new RadioembolizationTreatment(1, 2, 3, 4, 5, new MockHostOrgan(), 6));
		assertEquals("Radioembolization [name=name, treatmentData=RadioembolizationTreatment [alpha=1.0, beta=2.0, activity=3.0, "
				+ "energy=4.0, halflife=5.0, treatedOrgan=HostOrgan [name=mock, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, "
				+ "printFileForHostOrgan=false], minVolume=6.0]]", re.toString());
	}
}
