/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.globaleventactions;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.Event;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.GlobalEventAction;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.Radioimmunotherapy;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.mockobjects.MockDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.globaleventactions.mockobjects.MockRadioimmunoTreatment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.mockobjects.MockCompartment;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;

public class TestRadioimmunotherapy extends TestCase {

	protected void setUp() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
		TreatmentList.reset();
	}

	@Override
	protected void tearDown() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
		TreatmentList.reset();
	}	
	
	public void testExecute() throws MetastaseException{
		
		DiscreteCompartment dc = new MockDiscreteCompartment();
		DiscreteCompartment dc2 = new MockDiscreteCompartment();
		DiscreteCompartment dc3 = new MockDiscreteCompartment();
		dc3.setBloodStream(true);
		ContinuousCompartment cc = new MockContinuousCompartment(new LinearFunction(2 *60.*24., 1)); 
		ContinuousCompartment cc2 = new MockContinuousCompartment(new LinearFunction(2 *60.*24., 100));
		
		dc.setNumberOfCells(10000000);
		dc2.setNumberOfCells(0); 		//already dead
		dc3.setNumberOfCells(1000); 	//blood stream
		
		
		assertEquals(10000000, dc.getNumberOfCells(100));
		assertEquals(0, dc2.getNumberOfCells(100));
		assertEquals(1000, dc3.getNumberOfCells(100));
		assertEquals(201, cc.getNumberOfCells(100));
		assertEquals(300, cc2.getNumberOfCells(100));
		
		GlobalEventAction eventAction = new Radioimmunotherapy("name", new MockRadioimmunoTreatment());
		eventAction.execute(150);
		
		assertEquals(20, dc.getNumberOfCells(150));		//the MockRadioembolizationTreatment always returns 20 cells for computeNumberOfSurvivingCells
		assertEquals(0, dc2.getNumberOfCells(150)); 	//already dead => stays dead
		assertEquals(1000, dc3.getNumberOfCells(100));	//blood stream is ignored
		assertEquals(20, cc.getNumberOfCells(150));
		assertEquals(320, cc.getNumberOfCells(300));
		assertEquals(20, cc2.getNumberOfCells(150));
		assertEquals(320, cc2.getNumberOfCells(300));
		
		eventAction = new Radioimmunotherapy("name", new MockRadioimmunoTreatment(0));
		eventAction.execute(200);
		
		assertEquals(20, cc.getNumberOfCells(150));
		assertEquals(0, cc.getNumberOfCells(200));
		assertEquals(20, cc2.getNumberOfCells(150));
		assertEquals(0, cc2.getNumberOfCells(200));
		
		//test dormancy
		HostOrgan organ1 = new HostOrgan("organ1", new GaussianDistribution(100 /60./24., 0), false);
		HostOrgan organ2 = new HostOrgan("organ2", null, new GaussianDistribution(200, 0), new GaussianDistribution(100. /60./24., 0), false);
		cc = new ContinuousCompartment(0, "name", organ1, new LinearFunction(2 *60.*24., 1), null);
		cc2 = new ContinuousCompartment(0, "name", organ2, new LinearFunction(2 *60.*24., 100), null);
		
		
		assertEquals(1, cc.getNumberOfCells(0));
		assertEquals(1, cc.getNumberOfCells(20));
		assertEquals(1, cc.getNumberOfCells(50));
		assertEquals(100, cc2.getNumberOfCells(0));
		assertEquals(140, cc2.getNumberOfCells(20));
		cc2.setInLateDormancy(true);
		
		eventAction = new Radioimmunotherapy("name", new MockRadioimmunoTreatment());
		eventAction.execute(75);		
		
		assertEquals(20, cc.getNumberOfCells(75));
		assertEquals(20, cc.getNumberOfCells(100));
		assertEquals(120, cc.getNumberOfCells(150));		
		
		assertEquals(200, cc2.getNumberOfCells(50));
		assertEquals(200, cc2.getNumberOfCells(74));
		assertEquals(20, cc2.getNumberOfCells(75));
		assertEquals(20, cc2.getNumberOfCells(150));
		assertEquals(320, cc2.getNumberOfCells(300));
		
		//test renew AddCell event
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
		cc = new MockContinuousCompartment(new LinearFunction(2 *60.*24., 1));
		cc2 = new MockContinuousCompartment(new LinearFunction(2 *60.*24., 100));
		AddCellToDiscreteCompartment addCell1 = new AddCellToDiscreteCompartment("add cell", cc);
		AddCellToDiscreteCompartment addCell2 = new AddCellToDiscreteCompartment("add cell", cc2);
		addCell2.setTimeOfLastExecutedEventAction(50);
		
		EventList list = EventList.getInstance();
		list.addEvent(new LocalEvent(addCell1, 80, dc));
		list.addEvent(new LocalEvent(addCell2, 80, dc));
		
		eventAction.execute(75);
		
		Event event1 = list.nextEvent();
		assertTrue(event1 instanceof LocalEvent);
		assertTrue(((LocalEvent)event1).getEventAction() instanceof AddCellToDiscreteCompartment);
		assertSame(cc, ((AddCellToDiscreteCompartment)((LocalEvent)event1).getEventAction()).getTumorCompartment());
		assertEquals(300., event1.getTime());
		
		Event event2 = list.nextEvent();
		assertTrue(event2 instanceof LocalEvent);
		assertTrue(((LocalEvent)event2).getEventAction() instanceof AddCellToDiscreteCompartment);
		assertSame(cc2, ((AddCellToDiscreteCompartment)((LocalEvent)event2).getEventAction()).getTumorCompartment());
		assertEquals(350., event2.getTime());
		
		//test chemo function still active
		TreatmentList.getInstance().addTreatment(100 /60./24., new ChemoTreatment(0.5, 0.8 *60.*24., 0.4 *60.*24.));
		cc = new MockContinuousCompartment(new LinearFunction(2 *60.*24., 1));
		try {
			eventAction.execute(101);
			fail("An exception schould have been thrown");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		}
		
		eventAction.execute(150);
		assertEquals(20, cc.getNumberOfCells(150));
		
		//wrong Compartment (neither discrete nor continuous)
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		new MockCompartment();

		try {
			eventAction.execute(101);
			fail("An exception schould have been thrown");
		} catch (IllegalStateException e) {
			assertEquals("Unknown Compartment: MockCompartment", e.getMessage());
		}
		
	}
	
	public void testEqualsAndHashcode(){
		Radioimmunotherapy rt1 = new Radioimmunotherapy("name", new MockRadioimmunoTreatment());
		Radioimmunotherapy rt2 = new Radioimmunotherapy("name", new MockRadioimmunoTreatment());
		
		assertTrue(rt1.equals(rt1));
		assertTrue(rt1.hashCode() == rt1.hashCode());
		assertTrue(rt1.equals(rt2));
		assertTrue(rt1.hashCode() == rt2.hashCode());

		assertFalse(rt1.equals(null));
		assertFalse(rt1.equals(new ConstantFunction()));
		assertFalse(rt1.hashCode() == new ConstantFunction().hashCode());
		
		assertFalse(rt1.equals(new Radioimmunotherapy("name", new MockRadioimmunoTreatment(1))));
		assertFalse(rt1.hashCode() == new Radioimmunotherapy("name", new MockRadioimmunoTreatment(1)).hashCode());
	}
	
	public void testToString(){
		Radioimmunotherapy rt = new Radioimmunotherapy("name", new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, true));
		assertEquals("Radioimmunotherapy [treatmentData=RadioimmunoTreatment [alpha=1.0, beta=2.0, activity=3.0, "
				+ "energy=4.0, halflife=5.0, biologicalHalflife=NaN, effectiveHalflife=5.0, clearanceFraction=6.0, "
				+ "excludeBloodStream=true]]", rt.toString());
	}
}
