/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.distributions.UniformDistribution;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockCellProgressionAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;


/**
 * Tests the AddCellToDiscreteCompartment event action.
 * @author Anja Bethge
 *
 */
public class TestAddCellToDiscreteCompartment extends TestCase {

	@Override
	protected void setUp() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
	}

	public void testExecute() throws MetastaseException{
		
		EventList eventList = EventList.getInstance();
		ContinuousCompartment mcc = new ContinuousCompartment(0., "mock", new MockHostOrgan(), new LinearFunction(1, 1), new ColonizationRate(1, 0.6));
		
		AddCellToDiscreteCompartment eventAction = new AddCellToDiscreteCompartment("add cell action", mcc);
		assertEquals(mcc, eventAction.getTumorCompartment());
		
		//create the compartment type and the discrete compartment, that is needed for the execute-method
		HostOrgan organ = new HostOrgan("tissue", null, new GaussianDistribution(50, 20), new UniformDistribution(10, 100), false);
		DiscreteCompartmentType compartmentType = new DiscreteCompartmentType("sample type");
		compartmentType.addPossibleEventAction(new MockCellProgressionAction());
		
		DiscreteCompartment compartment = new DiscreteCompartment(0.0, organ, compartmentType);
		assertEquals(0, compartment.getNumberOfCells(0));
		
		
		eventAction.execute(compartment, 0);
		//one cell has had to be added to the discrete compartment 
		assertEquals(1, compartment.getNumberOfCells(0));
		
		//the first event's action has to be a MockCellProgressionAction => describes what happens next to the cell in the discrete compartment
		//the second event's action has to be a AddCellToDiscreteCompartment => created by the event generator to model the next intravasation
		assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof MockCellProgressionAction); 
		assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof AddCellToDiscreteCompartment);
		
		assertTrue(eventList.isEmpty());
		
		//now, try to execute the event with a wrong compartment type
		ContinuousCompartment cc = new MockContinuousCompartment();
		try {
			eventAction.execute(cc, 1.0);
			fail("An IllegalArgumentException should have been thrown.");
		} catch (IllegalArgumentException e) {
			// everything is just fine
		}
		
		//check, whether the event is delayed when the compartment is in the late dormancy state
		double duration = compartment.getLateDormancyDuration();
		compartment.setInLateDormancy(true);
		eventAction.execute((Compartment)compartment, 0);
		
		//get the next event from the list (it should be the delayed cell division event)
		LocalEvent le = (LocalEvent)eventList.nextEvent();
		assertEquals(duration, le.getTime());
		assertEquals(eventAction, le.getEventAction());
	
	}
	
	public void testEqualsAndHashcode() throws MetastaseException{
		
		AddCellToDiscreteCompartment action = new AddCellToDiscreteCompartment("name", new MockContinuousCompartment("mcc"));
		AddCellToDiscreteCompartment action2 = new AddCellToDiscreteCompartment("name", new MockContinuousCompartment("mcc"));
		
		assertTrue(action.equals(action));
		assertTrue(action.hashCode() == action.hashCode());
		assertTrue(action.equals(action2));
		assertTrue(action.hashCode() == action2.hashCode());
		
		assertFalse(action.equals(null));
		assertFalse(action.equals(new ConstantFunction()));
		assertFalse(action.hashCode() == new ConstantFunction().hashCode());
		assertFalse(action.equals(new AddCellToDiscreteCompartment("other name", new MockContinuousCompartment("mcc"))));
		assertFalse(action.hashCode() == new AddCellToDiscreteCompartment("other name", new MockContinuousCompartment("mcc")).hashCode());
		assertFalse(action.equals(new AddCellToDiscreteCompartment("name", new MockContinuousCompartment("mcc2"))));
		assertFalse(action.hashCode() == new AddCellToDiscreteCompartment("name", new MockContinuousCompartment("mcc2")).hashCode());
		
		action2.setTimeOfLastExecutedEventAction(50.);
		assertEquals(50., action2.getTimeOfLastExecutedEventAction());
		assertFalse(action.equals(action2));
		assertFalse(action.hashCode() == action2.hashCode());
	}
	
	public void testToString() throws MetastaseException{

		AddCellToDiscreteCompartment eventAction = new AddCellToDiscreteCompartment("add cell action", new MockContinuousCompartment());
		
		String expected = "AddCellToDiscreteCompartment [name=add cell action, tumorCompartment=ContinuousCompartment [myId=1, name=mock, "
				+ "isPrimaryTumor=false, isBloodStream=false, hostOrgan=HostOrgan [name=mock, dormancy=null, lateDormancySize=null, "
				+ "lateDormancyDuration=null, printFileForHostOrgan=false], successor=null, isDormant=false, isInLateDormancy=false, "
				+ "lateDormancyStartSize=0, lateDormancyDuration=0.0, growthFunction=0.0: LinearFunction [1.0*x+0.0 ] - , colonizationRate=null, "
				+ "eventTimeCalculator=null]]";
		assertEquals(expected, eventAction.toString());		
	}

}
