/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.distributions.UniformDistribution;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDivision;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockCellProgressionAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.machine.EventList;

/**
 * 
 * @author Anja Bethge
 *
 */
public class TestCellDivision extends TestCase {

	@Override
	protected void setUp() throws Exception {
		EventList.reset();
	}
	
	public void testExecute() {
		
		try{
			EventList eventList = EventList.getInstance();
			
			CellDivision eventAction = new CellDivision("cell division action", null);
			
			//create the compartment type and the discrete compartment, that is needed for the execute-method
			HostOrgan organ = new HostOrgan("tissue", null, new GaussianDistribution(50, 20), new UniformDistribution(99, 100), false);
			DiscreteCompartmentType compartmentType = new DiscreteCompartmentType("sample type");
			compartmentType.addPossibleEventAction(new MockCellProgressionAction());
			
			DiscreteCompartment compartment = new DiscreteCompartment(0.0, organ, compartmentType);
			
			int numberOfCells = 10;
			for (int i=0; i<numberOfCells; i++) {
				compartment.addCell(0.0);
			}
			assertEquals(numberOfCells, compartment.getNumberOfCells(0));
			
			for (int i = 0; i < 6; i++) {
				eventAction.execute(compartment, 0);
				//one cell has had to be added
				assertEquals(++numberOfCells, compartment.getNumberOfCells(0));
								
				//both events have to be created from the compartment type
				assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof MockCellProgressionAction);
				assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof MockCellProgressionAction);
				
				assertTrue(eventList.isEmpty());
			}
			
			//now, try to execute the event with a wrong compartment type
			ContinuousCompartment cc = new MockContinuousCompartment();
			try {
				eventAction.execute(cc, 1.0);
				fail("An IllegalArgumentException should have been thrown.");
			} catch (IllegalArgumentException e) {
				// everything is just fine
			}
			
			//check, whether the event is delayed when the compartment is in the late dormancy state
			double duration = compartment.getLateDormancyDuration();
			compartment.setInLateDormancy(true);
			eventAction.execute((Compartment)compartment, 0);
			
			
			//get the next event from the list (it should be the delayed cell decision event)
			LocalEvent le = (LocalEvent)eventList.nextEvent();
			assertEquals(duration, le.getTime());
			assertEquals(eventAction, le.getEventAction());
		
		}catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exception: "+e);
		}
	}
}
