/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.AddCellToDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockColonizationRate;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;

public class TestTransferToNewContinuousCompartment extends TestCase {

	@Override
	protected void setUp() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
	}
	
	public void testExecute() throws MetastaseException{
		
		EventList eventList = EventList.getInstance();
		
		//create the compartment type and the discrete compartment, that is needed for the execute-method
		DiscreteCompartmentType compartmentType = new DiscreteCompartmentType("sample discrete type");
		
		DiscreteCompartment compartment1 = new DiscreteCompartment(0, new MockHostOrgan(), compartmentType);
		DiscreteCompartment newSuccessor = new DiscreteCompartment(0, new MockHostOrgan(), compartmentType);
		int numberOfCells = 10;
		for (int i=0; i<numberOfCells; i++) {
			compartment1.addCell(0);
			newSuccessor.addCell(0);
		}
		assertEquals(numberOfCells, compartment1.getNumberOfCells(0));
		assertEquals(numberOfCells, newSuccessor.getNumberOfCells(0));
		
		
		TransferToNewContinuousCompartment eventAction = new TransferToNewContinuousCompartment("event action", null, 
				"new compartment", new MockHostOrgan(), new ConstantFunction(), new ColonizationRate(1, 2), newSuccessor);
		
		
		eventAction.execute((Compartment)compartment1, 0.);
		//one cell has had to be deleted
		assertEquals(numberOfCells-1, compartment1.getNumberOfCells(0.));
		//a new compartment has had to be registered
		assertEquals(3, CompartmentInventory.getCompartments().size());
		//get the new Compartment
		Compartment newCompartment = CompartmentInventory.getCompartments().get(2);
		assertEquals(3, newCompartment.getMyId());
		assertTrue(newCompartment instanceof ContinuousCompartment);
		
		ContinuousCompartment newConCompartment = (ContinuousCompartment)newCompartment;
		assertNotSame(compartment1, newConCompartment.getSuccessor());
		assertSame(newSuccessor, newConCompartment.getSuccessor());
		
		//the new event action has to be AddCellToDiscreteCompartment
		assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof AddCellToDiscreteCompartment);
		assertTrue(eventList.isEmpty());
		
		
		//test the dormancy of new created compartment
		HostOrgan organ = new HostOrgan("organ", new GaussianDistribution(20, 5), false);
		eventAction = new TransferToNewContinuousCompartment("event action", null, 
				"new compartment", organ, new ConstantFunction(), new MockColonizationRate(), newSuccessor);
		eventAction.setSuccessorOfNewCompartment(newSuccessor);
		
		eventAction.execute(compartment1, 0.);
		//one cell has had to be deleted
		assertEquals(8, compartment1.getNumberOfCells(0.));
		//a new compartment has had to be registered
		assertEquals(4, CompartmentInventory.getCompartments().size());
		//get the new Compartment
		newCompartment = CompartmentInventory.getCompartments().get(3);
		assertEquals(4, newCompartment.getMyId());
		assertTrue(newCompartment instanceof ContinuousCompartment);
		
		newConCompartment = (ContinuousCompartment)newCompartment;
		assertNotSame(compartment1, newConCompartment.getSuccessor());
		assertSame(newSuccessor, newConCompartment.getSuccessor());
		
		//the new event action has to be StopTumorDormancy
		assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof StopDormancy);
		assertTrue(eventList.isEmpty());
		
		//now, try to execute the event with a wrong compartment type
		ContinuousCompartment cc = new MockContinuousCompartment();
		try {
			eventAction.execute(cc, 1.0);
			fail("An IllegalArgumentException should have been thrown.");
		} catch (IllegalArgumentException e) {
			// everything is just fine
		}
	}

	public void testEqualsAndHashCode() throws MetastaseException{		
		
		//create successor
		HostOrgan newOrgan = new MockHostOrgan();
		Function newGrowthF = new ConstantFunction(5);
		ColonizationRate newCol = new ColonizationRate(1, 2);
		DiscreteCompartment newSuccessor = new MockDiscreteCompartment("successor");
		
		GenerationInfo gi = new GenerationInfo(1, new ConstantFunction(), new GaussianDistribution(10, 0));
		
		TransferToNewContinuousCompartment ea = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, newSuccessor);
		TransferToNewContinuousCompartment ea2 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, newSuccessor);
		TransferToNewContinuousCompartment ea3 = new TransferToNewContinuousCompartment("321", gi, "newName", newOrgan, newGrowthF, newCol, newSuccessor);
		
		assertTrue(ea.equals(ea));
		assertEquals(ea.hashCode(), ea.hashCode());
		assertTrue(ea.equals(ea2));
		assertEquals(ea.hashCode(), ea2.hashCode());
		
		assertFalse(ea.equals(null));
		assertFalse(ea.equals(new ConstantFunction()));
		assertFalse(ea.hashCode() == new ConstantFunction().hashCode());
		
		//name
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());		
		
		//new name
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName2", newOrgan, newGrowthF, newCol, newSuccessor);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());
		
		//new organ
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", new MockHostOrgan("organ2"), newGrowthF, newCol, newSuccessor);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());
		
		//new growth function
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, new ConstantFunction(20), newCol, newSuccessor);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());
		
		//new colonization rate
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, new ColonizationRate(2, 1), newSuccessor);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());
		
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, null, newSuccessor);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());		
		assertFalse(ea3.equals(ea));
		assertFalse(ea3.hashCode() == ea.hashCode());
		
		ea = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, null, newSuccessor);
		assertTrue(ea3.equals(ea));
		assertTrue(ea3.hashCode() == ea.hashCode());
		
		//successor
		ea = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, newSuccessor);
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, null);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());
		
		ea = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, null);
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, newSuccessor);
		assertFalse(ea.equals(ea3));
		assertFalse(ea.hashCode() == ea3.hashCode());
		
		ea3 = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, null);
		assertTrue(ea.equals(ea3));
		assertTrue(ea.hashCode() == ea3.hashCode());
		
		
	
		ea = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, newSuccessor);
		String expected = "TransferToNewContinuousCompartment [name=123, genererationInfo=GenerationInfo [probability=1.0, "
				+ "meanTimeToNextEvent=ConstantFunction [returnValue=1.0], distribution=GaussianDistribution [mean=10.0, stddev=0.0]], "
				+ "newName=newName, newHostOrgan=HostOrgan [name=mock, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, "
				+ "printFileForHostOrgan=false], newGrowthFunction=ConstantFunction [returnValue=5.0], "
				+ "newColonizationRate=ColonisationRate [1.0 * null^2.0], successorOfNewCompartment=successor]";
		assertEquals(expected, ea.toString());
		
		ea = new TransferToNewContinuousCompartment("123", gi, "newName", newOrgan, newGrowthF, newCol, null);
		expected = "TransferToNewContinuousCompartment [name=123, genererationInfo=GenerationInfo [probability=1.0, "
				+ "meanTimeToNextEvent=ConstantFunction [returnValue=1.0], distribution=GaussianDistribution [mean=10.0, stddev=0.0]], "
				+ "newName=newName, newHostOrgan=HostOrgan [name=mock, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, "
				+ "printFileForHostOrgan=false], newGrowthFunction=ConstantFunction [returnValue=5.0], "
				+ "newColonizationRate=ColonisationRate [1.0 * null^2.0], successorOfNewCompartment=null]";
		assertEquals(expected, ea.toString());
	}
}
