/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.eventactions.localeventactions;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.CompartmentInventory;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.LocalEvent;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.StopDormancy;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockCellProgressionAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.base.mockobjects.MockDiscreteCompartmentType;
import de.fhstralsund.catsit.machine.EventList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;

public class TestTransferToNewDiscreteCompartment extends TestCase {

	@Override
	protected void setUp() throws Exception {
		CompartmentInventory.reset();
		CompartmentInventory.resetCompartmentCounter();
		EventList.reset();
	}
	
	public void testExecute() {		
		try {
			EventList eventList = EventList.getInstance();
			
			//create the compartment type and the discrete compartment, that is needed for the execute-method
			DiscreteCompartmentType compartmentType = new DiscreteCompartmentType("sample discrete type");
			compartmentType.addPossibleEventAction(new MockCellProgressionAction());
			
			DiscreteCompartment compartment1 = new DiscreteCompartment(0, new MockHostOrgan(), compartmentType);
			DiscreteCompartment newSuccessor = new DiscreteCompartment(0, new MockHostOrgan(), compartmentType);
			int numberOfCells = 10;
			for (int i=0; i<numberOfCells; i++) {
				compartment1.addCell(0);
				newSuccessor.addCell(0);
			}
			assertEquals(numberOfCells, compartment1.getNumberOfCells(0));
			assertEquals(numberOfCells, newSuccessor.getNumberOfCells(0));
			
			
			TransferToNewDiscreteCompartment eventAction = new TransferToNewDiscreteCompartment(
					"event action", null, new MockHostOrgan(), compartmentType, newSuccessor);
			
			
			eventAction.execute((Compartment)compartment1, 0.);
			//one cell has had to be deleted
			assertEquals(numberOfCells-1, compartment1.getNumberOfCells(0.));
			//a new compartment has had to be registered
			assertEquals(3, CompartmentInventory.getCompartments().size());
			//get the new Compartment
			Compartment newCompartment = CompartmentInventory.getCompartments().get(2);
			assertEquals(3, newCompartment.getMyId());
			assertTrue(newCompartment instanceof DiscreteCompartment);
			
			DiscreteCompartment newDisCompartment = (DiscreteCompartment)newCompartment;
			assertNotSame(compartment1, newDisCompartment.getSuccessor());
			assertSame(newSuccessor, newDisCompartment.getSuccessor());
						
			//the new event action has to be AddCellToDiscreteCompartment
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof CellProgressionEventAction);
			assertTrue(eventList.isEmpty());

			//test the dormancy of new created compartment
			HostOrgan organ = new HostOrgan("organ", new GaussianDistribution(20, 5), false);
			eventAction = new TransferToNewDiscreteCompartment("event action", null, organ, compartmentType, newSuccessor);
			DiscreteCompartmentType newCompartmentType = new DiscreteCompartmentType("sampe continuous type");		
			eventAction.setNewCompartmentType(newCompartmentType);
			eventAction.setSuccessorOfNewCompartment(newSuccessor);
			
			eventAction.execute(compartment1, 0.);
			//one cell has had to be deleted
			assertEquals(8, compartment1.getNumberOfCells(0.));
			//a new compartment has had to be registered
			assertEquals(4, CompartmentInventory.getCompartments().size());
			//get the new Compartment
			newCompartment = CompartmentInventory.getCompartments().get(3);
			assertEquals(4, newCompartment.getMyId());
			assertTrue(newCompartment instanceof DiscreteCompartment);
			
			newDisCompartment = (DiscreteCompartment)newCompartment;
			assertNotSame(compartment1, newDisCompartment.getSuccessor());
			assertSame(newSuccessor, newDisCompartment.getSuccessor());
			
			//the new event action has to be StopTumorDormancy
			assertTrue(((LocalEvent)eventList.nextEvent()).getEventAction() instanceof StopDormancy);
			assertTrue(eventList.isEmpty());
			
			//now, try to execute the event with a wrong compartment type
			ContinuousCompartment cc = new MockContinuousCompartment();
			try {
				eventAction.execute(cc, 1.0);
				fail("An IllegalArgumentException should have been thrown.");
			} catch (IllegalArgumentException e) {
				// everything is just fine
			}
			
		} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
	}

	public void testEqualsAndHashCode() throws MetastaseException{
		
		DiscreteCompartmentType newDCT = new DiscreteCompartmentType("sample discrete type");
		DiscreteCompartment newSuccessor = new MockDiscreteCompartment("successor");
		GenerationInfo gi = new GenerationInfo(1, new ConstantFunction(), new GaussianDistribution(10, 0));
		
		TransferToNewDiscreteCompartment ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, newSuccessor);
		TransferToNewDiscreteCompartment ea2 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, newSuccessor);
		TransferToNewDiscreteCompartment ea3 = new TransferToNewDiscreteCompartment("321", gi, new MockHostOrgan(), newDCT, newSuccessor);
		
		assertEquals(ea1, ea1);
		assertEquals(ea1, ea2);
		assertEquals(ea1.hashCode(), ea1.hashCode());
		assertEquals(ea1.hashCode(), ea2.hashCode());
		
		assertFalse(ea1.equals(null));
		assertFalse(ea1.equals(new ConstantFunction()));
		assertFalse(ea1.hashCode() == new ConstantFunction().hashCode());
		
		assertFalse(ea1.equals(ea3));
		assertFalse(ea1.hashCode() == ea3.hashCode());
		
		//successor
		ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, null);
		assertFalse(ea1.equals(ea2));
		assertFalse(ea1.hashCode() == ea3.hashCode());
		
		ea2 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, null);
		assertEquals(ea1, ea2);
		assertEquals(ea1.hashCode(), ea2.hashCode());
		
		ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, new MockDiscreteCompartment("successor"));
		ea2 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, new MockDiscreteCompartment("successor2"));
		assertFalse(ea1.equals(ea2));
		assertFalse(ea1.hashCode() == ea2.hashCode());

		//host organ		
		ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan("lung"), newDCT, newSuccessor);
		ea2 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan("liver"), newDCT, newSuccessor);
		assertFalse(ea1.equals(ea2));
		assertFalse(ea1.hashCode() == ea2.hashCode());		
		
		//compartment type		
		ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, newSuccessor);
		ea2 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), new MockDiscreteCompartmentType(), newSuccessor);
		assertFalse(ea1.equals(ea2));
		assertFalse(ea1.hashCode() == ea2.hashCode());
		
		
		ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, null);
		assertEquals("TransferToNewDiscreteCompartment [name=123, genererationInfo=GenerationInfo [probability=1.0, "
				+ "meanTimeToNextEvent=ConstantFunction [returnValue=1.0], distribution=GaussianDistribution [mean=10.0, stddev=0.0]], "
				+ "successorNewCompartment=null, newCompartmentType=sample discrete type, newHostOrgan=HostOrgan [name=mock, dormancy=null, "
				+ "lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false]]", 
				ea1.toString());
		
		ea1 = new TransferToNewDiscreteCompartment("123", gi, new MockHostOrgan(), newDCT, new MockDiscreteCompartment("successor"));
		assertEquals("TransferToNewDiscreteCompartment [name=123, genererationInfo=GenerationInfo [probability=1.0, "
				+ "meanTimeToNextEvent=ConstantFunction [returnValue=1.0], distribution=GaussianDistribution [mean=10.0, stddev=0.0]], "
				+ "successorNewCompartment=successor, newCompartmentType=sample discrete type, newHostOrgan=HostOrgan [name=mock, dormancy=null, "
				+ "lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false]]", 
				ea1.toString());

	}
	
}
