/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.LogFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;
import de.fhstralsund.catsit.support.functions.chemo.ConstantChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.ExponentialChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.GompertzChemoFunction;
import de.fhstralsund.catsit.support.functions.chemo.LinearChemoFunction;

public class TestChemoTreatment extends TestCase {

	protected void setUp() throws Exception {
		TreatmentList.reset();
	}
	
	@Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}
	
	public void testGetters(){
		ChemoTreatment ct = new ChemoTreatment(1, 2*60.*24., 3*60.*24.);
		assertEquals(3., ct.getDrugDecayRate());		
	}
	
	public void testGetCycleNonSpecificChemoFunction(){
		//gompertz
		//////////////////////
		Function initialGF = new GompertzFunction(0.5, 1e8, 1e4);
		Function expectedChemoGF = new GompertzChemoFunction(0.5, 1e8, 1000000, 0.2, 0.9, 2.5);
			
		Function chemoGF = new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 1000000);
		assertEquals(expectedChemoGF, chemoGF);
		
		//constant
		//////////////////////
		initialGF = new ConstantFunction(1e8);
		expectedChemoGF = new ConstantChemoFunction(100000000, 0.2, 0.9, 2.5);
		
		chemoGF = new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 100000000);
		assertEquals(expectedChemoGF, chemoGF);	
		
		//no chemo function if the tumour is already dead
		initialGF = new ConstantFunction(0);	
		assertNull(new ChemoTreatment(0.2, 2.5, 0.64).getChemoFunction(initialGF, 100000000));
		
		//linear
		//////////////////////
		initialGF = new LinearFunction(0.5, 1e4);
		expectedChemoGF = new LinearChemoFunction(0.5, 1000000, 0.2, 0.9, 2.5);
		
		chemoGF = new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 1000000);
		assertEquals(expectedChemoGF, chemoGF);
		
		//exponential
		/////////////////////
		initialGF = new ExponentialFunction(1e4, 0.5, 0);
		expectedChemoGF = new ExponentialChemoFunction(0.5, 1000000, 0.2, 0.9, 2.5);
			
		chemoGF = new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 1000000);
		assertEquals(expectedChemoGF, chemoGF);
		
		
		//no chemo function for log and power functions
		initialGF = new LogFunction(0.5, 1);	
		assertNull(new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 100000000));
		
		initialGF = new PowerFunction(1, 2, 1e4);	
		assertNull(new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 100000000));
		
		//no chemo function for any other function
		initialGF = new LinearChemoFunction(2, 100, 1, 2, 3);
		assertNull(new ChemoTreatment(0.2, 2.5*60.*24., 0.9*60.*24.).getChemoFunction(initialGF, 100000000));
	}
	
	public void testToString(){
		ChemoTreatment ct = new ChemoTreatment(0.2, 2.5*60.*24., 0.64*60.*24.);
		assertEquals("ChemoTreatment [cyclingCellFraction=0.2, drugSensitivity=2.5, drugDecayRate=0.6399999999999999]", ct.toString());
	}
	
	public void testEqualsAndHashCode(){
		ChemoTreatment ct = new ChemoTreatment(0.2, 2.5, 0.64);
		ChemoTreatment ct2 = new ChemoTreatment(0.2, 2.5, 0.64);
		
		assertEquals(ct, ct);
		assertEquals(ct.hashCode(), ct.hashCode());
		
		assertEquals(ct, ct2);		
		assertEquals(ct.hashCode(), ct2.hashCode());
		
		assertFalse(ct.equals(new ChemoTreatment(0.1, 2.5, 0.64)));
		assertFalse(ct.hashCode() == new ChemoTreatment(0.1, 2.5, 0.64).hashCode());
		assertFalse(ct.equals(new ChemoTreatment(0.2, 1, 0.64)));
		assertFalse(ct.hashCode() == new ChemoTreatment(0.2, 1, 0.64).hashCode());
		assertFalse(ct.equals(new ChemoTreatment(0.2, 2.5, 0.94)));
		assertFalse(ct.hashCode() == new ChemoTreatment(0.2, 2.5, 0.94).hashCode());
		
		assertFalse(ct.equals(new ConstantFunction()));
		assertFalse(ct.hashCode() == new ConstantFunction().hashCode());
		
		assertFalse(ct.equals(null));
	}

}
