/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.mockobjects.MockHostOrgan;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import junit.framework.TestCase;

public class TestRadioembolizationTreatment extends TestCase {

	protected void setUp() throws Exception {
		TreatmentList.reset();
	}
	
	@Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}
	
	public void testGetters(){
		HostOrgan organ = new MockHostOrgan();
		RadioembolizationTreatment ret = new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 6);
		
		assertEquals(1., ret.getAlpha());
		assertEquals(2., ret.getBeta());
		assertEquals(3., ret.getActivity());
		assertEquals(4., ret.getEnergy());
		assertEquals(5., ret.getHalflife());
		assertEquals(6., ret.getMinVolume());
		assertEquals((long)6e6, ret.getMinSize());
		assertEquals(organ, ret.getTreatedOrgan());
	}
	
	public void testComputeNumberOfSurvivingCells(){
		HostOrgan organ = new MockHostOrgan();
		RadioembolizationTreatment ret = new RadioembolizationTreatment(0.3, 0.002, 0.001, 0.5, 5, organ, 6);
		
		assertEquals(0, ret.computeNumberOfSurvivingCells(1));
		assertEquals(0, ret.computeNumberOfSurvivingCells(100000));
		assertEquals(81979, ret.computeNumberOfSurvivingCells(100000000));
		assertEquals(4411757985L, ret.computeNumberOfSurvivingCells(5000000000L));
	}
	
	public void testToString(){
		HostOrgan organ = new MockHostOrgan();
		RadioembolizationTreatment ret = new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 6);
		assertEquals("RadioembolizationTreatment [alpha=1.0, beta=2.0, activity=3.0, energy=4.0, halflife=5.0, "
				+ "treatedOrgan=HostOrgan [name=mock organ, dormancy=null, lateDormancySize=null, lateDormancyDuration=null, printFileForHostOrgan=false], "
				+ "minVolume=6.0]", ret.toString());
	}
	
	public void testEqualsAndHashCode(){
		HostOrgan organ = new MockHostOrgan();
		RadioembolizationTreatment ret = new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 6);
		RadioembolizationTreatment ret2 = new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 6);
		
		assertEquals(ret, ret);
		assertEquals(ret.hashCode(), ret.hashCode());
		
		assertEquals(ret, ret2);		
		assertEquals(ret.hashCode(), ret2.hashCode());
		
		assertFalse(ret.equals(new RadioembolizationTreatment(2, 2, 3, 4, 5, organ, 6)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(2, 2, 3, 4, 5, organ, 6).hashCode());
		assertFalse(ret.equals(new RadioembolizationTreatment(1, 3, 3, 4, 5, organ, 6)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(1, 3, 3, 4, 5, organ, 6).hashCode());
		assertFalse(ret.equals(new RadioembolizationTreatment(1, 2, 4, 4, 5, organ, 6)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(1, 2, 4, 4, 5, organ, 6).hashCode());
		assertFalse(ret.equals(new RadioembolizationTreatment(1, 2, 3, 5, 5, organ, 6)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(1, 2, 3, 5, 5, organ, 6).hashCode());
		assertFalse(ret.equals(new RadioembolizationTreatment(1, 2, 3, 4, 6, organ, 6)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(1, 2, 3, 4, 6, organ, 6).hashCode());
		assertFalse(ret.equals(new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 7)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 7).hashCode());
		assertFalse(ret.equals(new RadioembolizationTreatment(1, 2, 3, 4, 5, new MockHostOrgan("organ2"), 6)));
		assertFalse(ret.hashCode() == new RadioembolizationTreatment(1, 2, 3, 4, 5, new MockHostOrgan("organ2"), 6).hashCode());
		
		
		assertFalse(ret.equals(new ConstantFunction()));
		assertFalse(ret.hashCode() == new ConstantFunction().hashCode());
		
		assertFalse(ret.equals(null));
	}
}
