/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.base.treatments;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.support.functions.ConstantFunction;

public class TestRadioimmunoTreatment extends TestCase {

	protected void setUp() throws Exception {
		TreatmentList.reset();
	}
	
	@Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}
	
	public void testGetters(){
		RadioimmunoTreatment rit = new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 7, true);
		
		assertEquals(1., rit.getAlpha());
		assertEquals(2., rit.getBeta());
		assertEquals(3., rit.getActivity());
		assertEquals(4., rit.getEnergy());
		assertEquals(5., rit.getHalflife());
		assertEquals(7., rit.getClearanceFraction());
		assertEquals(true, rit.isExcludeBloodStream());
		
	}
	
	public void testComputeNumberOfSurvivingCells(){
		RadioimmunoTreatment rit = new RadioimmunoTreatment(0.3, 0.002, 0.001, 0.5, 5, 6, 0.2, true);
		
		assertEquals(0, rit.computeNumberOfSurvivingCells(1,5000000000L));
		assertEquals(94, rit.computeNumberOfSurvivingCells(100,5000000000L));
		assertEquals(94692805, rit.computeNumberOfSurvivingCells(100000000,5000000000L));
		assertEquals(6689, rit.computeNumberOfSurvivingCells(3000000,50000000L));
		
		rit = new RadioimmunoTreatment(0.3, 0.002, 0.001, 0.5, 5, 6, 0, true);
		assertEquals(93408690, rit.computeNumberOfSurvivingCells(100000000,5000000000L));
		assertEquals(1182, rit.computeNumberOfSurvivingCells(3000000,50000000L));
		
		rit = new RadioimmunoTreatment(0.3, 0.002, 0.001, 0.5, 5, 0, true);
		assertEquals(88235159, rit.computeNumberOfSurvivingCells(100000000,5000000000L));
		assertEquals(0, rit.computeNumberOfSurvivingCells(3000000,50000000L));
	}
	
	public void testToString(){
		RadioimmunoTreatment rit = new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 7, true);
		assertEquals("RadioimmunoTreatment [alpha=1.0, beta=2.0, activity=3.0, energy=4.0, halflife=5.0, biologicalHalflife=6.0, "
				+ "effectiveHalflife=2.727272727272727, clearanceFraction=7.0, excludeBloodStream=true]", rit.toString());
	}
	
	public void testEqualsAndHashCode(){
		RadioimmunoTreatment rit = new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 7, true);
		RadioimmunoTreatment rit2 = new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 7, true);
		
		assertEquals(rit, rit);
		assertEquals(rit.hashCode(), rit.hashCode());
		
		assertEquals(rit, rit2);		
		assertEquals(rit.hashCode(), rit2.hashCode());
		
		assertFalse(rit.equals(new RadioimmunoTreatment(2, 2, 3, 4, 5, 6, 7, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(2, 2, 3, 4, 5, 6, 7, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 3, 3, 4, 5, 6, 7, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 3, 3, 4, 5, 6, 7, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 2, 4, 4, 5, 6, 7, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 2, 4, 4, 5, 6, 7, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 2, 3, 5, 5, 6, 7, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 2, 3, 5, 5, 6, 7, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 2, 3, 4, 6, 6, 7, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 2, 3, 4, 6, 6, 7, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 2, 3, 4, 5, 7, 7, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 2, 3, 4, 5, 7, 7, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 8, true)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 8, true).hashCode());
		assertFalse(rit.equals(new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 7, false)));
		assertFalse(rit.hashCode() == new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 7, false).hashCode());
		
		
		assertFalse(rit.equals(new ConstantFunction()));
		assertFalse(rit.hashCode() == new ConstantFunction().hashCode());
		
		assertFalse(rit.equals(null));
	}
}
