/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.excel;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.io.excel.writer.ExcelHeaderEntry;
import de.fhstralsund.catsit.io.excel.writer.ExcelRecordMain;
import de.fhstralsund.catsit.io.excel.writer.ExcelRecordSingleHostOrgan;
import de.fhstralsund.catsit.io.excel.writer.ExcelWriter;
import de.fhstralsund.catsit.io.excel.writer.ExcelWriterException;
import de.fhstralsund.catsit.support.HistogramEntry;
import de.fhstralsund.catsit.support.LogSampleHistogram;


/**
 * Test for the ExcelWriter.
 * @author Sebastian Gl�de, Anja Bethge
 * 
 */
public class TestExcelWriter extends TestCase{

	/**
	 * excel-file used in this tests.
	 */
	private static final String TEST_XLS = "test.xlsx";

	/**
	 * Test method for
	 * {@link metastase.io.excel.writer.ExcelWriter#ExcelWriter(java.lang.String)}.
	 * 
	 * @throws Exception
	 */
	public void testExcelWriter() throws Exception {
		
		// Tests empty organ list
		try {
			new ExcelWriter(new ArrayList<HostOrgan>());
			fail("empty string was accepted");
		} catch (IllegalArgumentException ex) {
			Assert.assertEquals("hostOrgans must not be empty", ex.getMessage());
		}
		
		// Tests empty organ list
		try {
			new ExcelWriter(null);
			fail("empty string was accepted");
		} catch (IllegalArgumentException ex) {
			Assert.assertEquals("hostOrgans must not be null", ex.getMessage());
		}

	}

	/**
	 * Test method for
	 * {@link metastase.io.excel.writer.ExcelWriter#bufferMainRecord(ExcelRecordMain record)} and
	 * {@link metastase.io.excel.writer.ExcelWriter#bufferHostOrganRecord(String organName, ExcelRecordSingleHostOrgan record)}
	 * .
	 * 
	 * @throws Exception
	 */
	public void testBufferRecord() throws Exception {
		List<HostOrgan> organs = new ArrayList<HostOrgan>();
		organs.add(new MockHostOrgan("liver", true));
		organs.add(new MockHostOrgan("lung", true));
		
		ExcelWriter excelWriter = new ExcelWriter(organs);
		Assert.assertEquals(0, excelWriter.getMainBufferSize());
		
		Assert.assertEquals(0, excelWriter.getOrganBufferSize("liver"));
		Assert.assertEquals(0, excelWriter.getOrganBufferSize("lung"));
		
		excelWriter.bufferMainRecord(new ExcelRecordMain(0, 0, 0, 0, 0, 0, generateSampleHistogram()));
		excelWriter.bufferMainRecord(new ExcelRecordMain(0, 0, 0, 0, 0, 0, generateSampleHistogram()));

		Assert.assertEquals(2, excelWriter.getMainBufferSize());
		Assert.assertEquals(0, excelWriter.getOrganBufferSize("liver"));
		Assert.assertEquals(0, excelWriter.getOrganBufferSize("lung"));
		
		excelWriter.bufferHostOrganRecord("liver", new ExcelRecordSingleHostOrgan(0, 0, 0, 0, generateSampleHistogram()));
		excelWriter.bufferHostOrganRecord("lung", new ExcelRecordSingleHostOrgan(0, 0, 0, 0, generateSampleHistogram()));
		excelWriter.bufferHostOrganRecord("lung", new ExcelRecordSingleHostOrgan(0, 0, 0, 0, generateSampleHistogram()));
		
		Assert.assertEquals(2, excelWriter.getMainBufferSize());
		Assert.assertEquals(1, excelWriter.getOrganBufferSize("liver"));
		Assert.assertEquals(2, excelWriter.getOrganBufferSize("lung"));
		
		try {
			new ExcelRecordMain(-1, 0, 0, 0, 0, 0, generateSampleHistogram());
			fail("negative value was accepted");
		} catch (IllegalArgumentException e) {
			Assert.assertEquals("At least one argument is negative", e.getMessage());
		}

	}

	/**
	 * Test method for
	 * {@link metastase.io.excel.writer.ExcelWriter#isSheetNameAvailable(java.lang.String)}.
	 * 
	 * @throws Exception
	 */
	public void testIsSheetNameAvalible() throws Exception {
		String fileName = TEST_XLS;
		File excelFile = new File(fileName);
		File lungFile = new File("test_lung.xlsx");
		File liverFile = new File("test_liver.xlsx");

		if (excelFile.exists()) {
			excelFile.delete();
		}
		if (lungFile.exists()) {
			lungFile.delete();
		}
		if (liverFile.exists()) {
			liverFile.delete();
		}

		List<HostOrgan> organs = new ArrayList<HostOrgan>();
		organs.add(new MockHostOrgan("liver", true));
		organs.add(new MockHostOrgan("lung", true));
		
		ExcelWriter excelWriter = new ExcelWriter(organs);

		// No sheets in file
		String sheetName = "test";
		Assert.assertTrue(excelWriter.isSheetNameAvailable(TEST_XLS, sheetName));

		ExcelRecordMain record = new ExcelRecordMain(0, 0, 0, 0, 0, 0, generateSampleHistogram());
		ExcelRecordSingleHostOrgan organRecord = new ExcelRecordSingleHostOrgan(1, 2, 3, 4, generateSampleHistogram());
		
		excelWriter.bufferMainRecord(record);
		excelWriter.bufferHostOrganRecord("liver", organRecord);
		excelWriter.bufferHostOrganRecord("lung", organRecord);
		
		try {
			excelWriter.writeAllBuffers(TEST_XLS, sheetName);

			Assert.assertFalse(excelWriter.isSheetNameAvailable(TEST_XLS, sheetName));

		} finally { // cleanup
			if (excelFile.exists()) {
				excelFile.delete();
			}
			if (lungFile.exists()) {
				lungFile.delete();
			}
			if (liverFile.exists()) {
				liverFile.delete();
			}
		}
	}

	/**
	 * Test method for
	 * {@link metastase.io.excel.writer.ExcelWriter#writeAllBuffers(String filePath, String sheetName)}.
	 * 
	 * @throws Exception
	 */
	public void testWriteAllBuffers() throws Exception {
		String fileName = TEST_XLS;
		File excelFile = new File(fileName);
		if (excelFile.exists()) {
			excelFile.delete(); 
		}
		File lungFile = new File("test_lung.xlsx");
		if (lungFile.exists()) {
			lungFile.delete();
		}

		List<HostOrgan> organs = new ArrayList<HostOrgan>();
		organs.add(new MockHostOrgan("liver"));
		organs.add(new MockHostOrgan("lung", true));
		ExcelWriter excelWriter = new ExcelWriter(organs);
		Assert.assertFalse(excelFile.exists());
		
		try {
			//Norecords
			String sheetNameNoHeader = "test3";
			excelWriter.writeAllBuffers(TEST_XLS, sheetNameNoHeader);
			
			fail("written with no records");
		} catch (ExcelWriterException e) {
			Assert.assertEquals("No records in buffer", e.getMessage());
		}
		
		//no correct file name
		try {
			excelWriter.writeAllBuffers(null, "test");//null			
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			//we are supposed to be here
		}
		try {
			excelWriter.writeAllBuffers("", "test");//empty			
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			//we are supposed to be here
		}
		try {
			excelWriter.writeAllBuffers("test", "test");//wrong extension		
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			//we are supposed to be here
		}
		
		
		// Header and record test
		ExcelRecordMain record = new ExcelRecordMain(1, 2, 3, 4, 5, 6,generateSampleHistogram());
		ExcelRecordSingleHostOrgan organRecord = new ExcelRecordSingleHostOrgan(1, 2, 3, 4, generateSampleHistogram());
		excelWriter.bufferMainRecord(record);
		excelWriter.bufferHostOrganRecord("lung", organRecord);
		
		try {
			String sheetName = "test";
			excelWriter.writeAllBuffers(TEST_XLS, sheetName); 

			validateHeader(fileName, sheetName, record);
			validateRecord(fileName, sheetName, record, 3);

			// Histogram test
			record = new ExcelRecordMain(7, 8, 9, 10, 11, 12,generateSampleHistogram());
			organRecord = new ExcelRecordSingleHostOrgan(5, 6, 7, 8, generateSampleHistogram());

			excelWriter.bufferMainRecord(record);
			excelWriter.bufferHostOrganRecord("lung", organRecord);
			
			sheetName = "test2";
			excelWriter.writeAllBuffers(TEST_XLS, sheetName);

			validateHistogram(fileName, sheetName, record, 3);
					

			if (!new File("test_lung.xlsx").exists()) {
				fail("An excel file for the lung should have been created");
			}
			
			if (new File("test_liver.xlsx").exists()) {
				fail("No excel file for the liver should have been created");
			}
			
			//try to write a second time into the same sheet
			try{
				excelWriter.writeAllBuffers(TEST_XLS, sheetName);
				fail("An exception should have been thrown");
			}catch(ExcelWriterException e){
				//we are supposed to be here
				assertEquals("java.lang.IllegalArgumentException: sheetname allready in use", e.getMessage());
			}
			
		} finally { // cleanup
			if (excelFile.exists()) {
				excelFile.delete();
			}
			if (lungFile.exists()) {
				lungFile.delete();
			}
		}
	}

	/**
	 * Validates the histogram-data written in excel file.
	 * 
	 * @param fileName the excel file
	 * @param sheetName the name of the sheet containing the histogram under test
	 * @param record the excel-record written in the excel file
	 * @param recordIndex the index of the record
	 * @throws FileNotFoundException the excel file was not found
	 * @throws IOException the excel file could not been read
	 */
	private void validateHistogram(String fileName,	String sheetName, ExcelRecordMain record, int recordIndex) throws FileNotFoundException, IOException {
		Sheet sheet = openSheet(fileName, sheetName);
		Row row = sheet.getRow(recordIndex); // first row contains header

		LogSampleHistogram histogram = record.getHistogram();
		Assert.assertNotNull(histogram);

		ArrayList<Double> expectedData = new ArrayList<Double>();
		for (HistogramEntry he : histogram.getBuckets()) {
			expectedData.add((double) he.getUpperLimit());
			expectedData.add((double) he.getN());
		}

		for (int cellCount = 0; cellCount < expectedData.size(); cellCount++) {
			Cell cell = row.getCell(cellCount + 6); // the first 6 cells are
			// validated in
			// validateRecord
			Assert.assertEquals(Cell.CELL_TYPE_NUMERIC, cell.getCellType());
			Assert.assertEquals(expectedData.get(cellCount), cell
					.getNumericCellValue(), 0);
		}
	}

	/**
	 * Generates a sample histogram for testing.
	 * 
	 * @return a sample histogram
	 */
	private LogSampleHistogram generateSampleHistogram() {
		int decades = 3;
		int bucketsPerDecade = 10;

		LogSampleHistogram lsh = null;
		try {
			lsh = new LogSampleHistogram(decades, bucketsPerDecade);

			for (int d = 0; d < decades; d++) {
				for (int b = 1; b < bucketsPerDecade; b++) {
					double val = 0.5 + Math.pow(10, d) * b;
					lsh.add(val);
				}
			}
			lsh.add(0.5);
			lsh.add(Math.pow(10, decades) + 0.5);
		} catch (Exception e) {
			fail("histogram generation failed");
		}
		return lsh;
	}

	/**
	 * Validates the first six data-cells written in the excel file.
	 * 
	 * @param fileName the excel file
	 * @param sheetName the name of the sheet containing the data under test
	 * @param record the excel-record written in the excel file
	 * @param recordIndex the index of the record
	 * @throws FileNotFoundException the excel file was not found
	 * @throws IOException the excel file could not been read
	 */
	private void validateRecord(String fileName, String sheetName,
			ExcelRecordMain record, int recordIndex)
			throws FileNotFoundException, IOException {

		Sheet sheet = openSheet(fileName, sheetName);
		Row row = sheet.getRow(recordIndex);

		ArrayList<Double> expectedData = new ArrayList<Double>();
		expectedData.add((double) record.getTime());
		expectedData.add((double) record.getTimeInDays());
		expectedData.add((double) record.getCellsInPrimaryTumor());
		expectedData.add((double) record.getCellsInBloodStream());
		expectedData.add((double) record.getNumberMetastases());
		expectedData.add((double) record.getCellsInMetastases());

		for (int cellCount = 0; cellCount < expectedData.size(); cellCount++) {
			Cell cell = row.getCell(cellCount);
			Assert.assertEquals(Cell.CELL_TYPE_NUMERIC, cell.getCellType());
			Assert.assertEquals(expectedData.get(cellCount), cell
					.getNumericCellValue(), 0);
		}
	}

	/**
	 * Validates the header written in the excel file.
	 * 
	 * @param fileName
	 *            the excel file
	 * @param sheetName
	 *            the name of the sheet containing the data under test
	 * @throws FileNotFoundException
	 *             the excel file was not found
	 * @throws IOException
	 *             the excel file could not been read
	 */
	private void validateHeader(String fileName, String sheetName,
			ExcelRecordMain record) throws FileNotFoundException, IOException {

		Sheet sheet = openSheet(fileName, sheetName);
		Row rowTitle = sheet.getRow(0); // first row contains header
		Row rowUnit = sheet.getRow(1);
		Row rowComment = sheet.getRow(2);

		ArrayList<ExcelHeaderEntry> headers = new ArrayList<ExcelHeaderEntry>();

		headers.add(new ExcelHeaderEntry("time", "minutes", ""));
		headers.add(new ExcelHeaderEntry("time", "days", ""));
		headers.add(new ExcelHeaderEntry("# cells primary", "", ""));
		headers.add(new ExcelHeaderEntry("# cells blood stream", "", ""));
		headers.add(new ExcelHeaderEntry("# metastases", "", ""));
		headers.add(new ExcelHeaderEntry("# cells all metastases", "", ""));

		if (record.getHistogram() != null) {
			for (int i = 0; i < record.getHistogram().getBuckets().size(); i++) {
				headers.add(new ExcelHeaderEntry("uL_" + i, "", ""));
				headers.add(new ExcelHeaderEntry("n_" + i, "", ""));
			}
		}

		for (int cellCount = 0; cellCount < headers.size(); cellCount++) {
			Cell cell = rowTitle.getCell(cellCount); // title
			Assert.assertEquals(Cell.CELL_TYPE_STRING, cell.getCellType());
			Assert.assertEquals(headers.get(cellCount).getTitle(), cell
					.getRichStringCellValue().getString());

			cell = rowUnit.getCell(cellCount); // unit
			Assert.assertEquals(Cell.CELL_TYPE_STRING, cell.getCellType());
			Assert.assertEquals(headers.get(cellCount).getUnit(), cell
					.getRichStringCellValue().getString());

			cell = rowComment.getCell(cellCount); // comment
			Assert.assertEquals(Cell.CELL_TYPE_STRING, cell.getCellType());
			Assert.assertEquals(headers.get(cellCount).getComment(), cell
					.getRichStringCellValue().getString());
		}
	}

	/**
	 * Opens a spreadsheet from an excel file.
	 * 
	 * @param fileName
	 *            the excel file
	 * @param sheetName
	 *            the name of the sheet to open
	 * @return the sheet
	 * @throws IOException
	 *             the excel file could not been read
	 * @throws FileNotFoundException
	 *             the excel file was not found
	 */
	private Sheet openSheet(String fileName, String sheetName)
			throws IOException, FileNotFoundException {
		Workbook workbook = new XSSFWorkbook(new FileInputStream(new File(fileName)));

		return workbook.getSheet(sheetName);
	}

}
