/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml;

import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.conditions.Condition;
import de.fhstralsund.catsit.base.conditions.TimeCondition;
import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDeath;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.treatments.ResectionTreatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.io.xml.MPMLParser;
import de.fhstralsund.catsit.io.xml.MPMLParserImpl;
import de.fhstralsund.catsit.io.xml.ParserException;
import de.fhstralsund.catsit.machine.Technical;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;

/**
 * Tests the {@link MPMLParser} interface
 * 
 * @author Michael Suhr
 * 
 */
public class TestMPMLParser extends TestCase {

    private MPMLParser parser = new MPMLParserImpl();
    private String config1 = "src/test/resources/xml/Config1_ResectionTime.xml";
    private String config2 = "src/test/resources/xml/Config2_ResectionTimeAndTreatment.xml";
    private String config2_Multi = "src/test/resources/xml/Config2_Multi.xml";
    private String config2_Inconsitence = "src/test/resources/xml/Config2_MultiError.xml";
    private String config2_InvalidEnd = "src/test/resources/xml/Config2_MultiInvalidEnd.xml";
    private String config2_InvalidProbability = "src/test/resources/xml/Config2_MultiInvalidProbability.xml";
    private String config2_MultiEnd = "src/test/resources/xml/Config2_MultiEnd.xml";
    private String config3 = "src/test/resources/xml/Config3_StartSize.xml";
    private String config3_InvalidStartSize = "src/test/resources/xml/Config3_InvalidStartSize.xml";
    private String invalidFile = "src/test/resources/xml/Invalid.xml";
    private String technical = "src/test/resources/xml/technical.xml";
    private String invalidTechnical = "src/test/resources/xml/invalid_technical.xml";
    private Compartment compartment1;
    private Compartment compartment2;

    @Override
    protected final void setUp() throws Exception {
        TreatmentList.reset();
        compartment1 = buildConfig1();
        TreatmentList.reset();
        compartment2 = buildConfig2();
        TreatmentList.reset();
        Logger.getLogger("metastase.support.xml").setLevel(Level.WARNING);
    }
    
    @Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}

    private Compartment buildConfig1() throws MetastaseException{
    	
    	//add the resection treatment to the treatment list
    	TreatmentList.getInstance().addTreatment(600, new ResectionTreatment(0.));
        HostOrgan organ = new HostOrgan("organ", null, false);

        double tumorGompertzA = 0.00286; // (Iwata)
        double tumorGompertzB = 7.3e10; // (Iwata)
        double tumorGompertzC = 1.;
        double colonizationRateAlpha = 0.663; // (Iwata)
        double colonizationCoefficientM = 5.3e-8 ; // Iwata

        // ** primary tumor **
        Function primaryTumourGrowthFunction = new GompertzFunction(tumorGompertzA, tumorGompertzB, tumorGompertzC);
        ColonizationRate primaryTumourEventFunction = new ColonizationRate(colonizationCoefficientM, colonizationRateAlpha);
	        
        Compartment primaryTumor = new ContinuousCompartment(0, "primary tumor", organ, primaryTumourGrowthFunction, primaryTumourEventFunction, true);
        
        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
        
        // ** bloodstream **
        DiscreteCompartment bloodStream = new DiscreteCompartment(0, organ, bloodStreamType);
        primaryTumor.setSuccessor(bloodStream);
        // add event action
        Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
       	CellProgressionEventAction metastasisEventAction = new TransferToNewContinuousCompartment(
	       			"bloodstream - metastasis", new GenerationInfo(1., new ConstantFunction(), metastasisDistribution), 
	       			"metastasis", organ, primaryTumourGrowthFunction, primaryTumourEventFunction, bloodStream);
        bloodStreamType.addPossibleEventAction(metastasisEventAction);
	
        return primaryTumor;
    }

    private Compartment buildConfig2() throws MetastaseException{
       	//add the resection treatment to the treatment list
       	TreatmentList.getInstance().addTreatment(100, new ResectionTreatment(0.));
        
        HostOrgan organ = new HostOrgan("organ", null, false);
        Function identity = new ConstantFunction();

        // == construct all compartments ==
        // ** data **
        double tumorGompertzA = 0.00286; // = (Iwata)
        double tumorGompertzB = 7.3e10; // (Iwata)
        double tumorGompertzC = 1.;
        double colonizationRateAlpha = 0.663; // (Iwata)
        double colonizationCoefficientM = 5.3e-8; 
        // Iwata
        double beginTreatmentAt = 100; // time in minutes
        double treatmentDurationInMin = 150;

        // ** primary tumor **
        Function primaryTumourGrowthFunction = new GompertzFunction(tumorGompertzA, tumorGompertzB, tumorGompertzC);
        ColonizationRate primaryTumourEventFunction = new ColonizationRate(colonizationCoefficientM, colonizationRateAlpha);
	
        Compartment primaryTumor = new ContinuousCompartment(0, "primary tumor", organ, primaryTumourGrowthFunction, primaryTumourEventFunction, true);

        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
        DiscreteCompartment bloodStream = new DiscreteCompartment(0, organ, bloodStreamType);
        primaryTumor.setSuccessor(bloodStream);

        // ** bloodstream **
        Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
        
        CellProgressionEventAction metastasisEventAction = new TransferToNewContinuousCompartment("bloodstream - metastasis",
                new GenerationInfo(1., identity, metastasisDistribution), "metastasis", organ, primaryTumourGrowthFunction, primaryTumourEventFunction, bloodStream);
        bloodStreamType.addPossibleEventAction(metastasisEventAction);

        // ** bloodstream treated that only 10% survive for 2 months**
        DiscreteCompartmentType bloodStreamTreatedType = new DiscreteCompartmentType("blood stream treated");
        CellProgressionEventAction metastasisEventAction2 = new TransferToNewContinuousCompartment("metastasis - bloodstream",
                new GenerationInfo(.1, identity, metastasisDistribution), "metastasis", organ, primaryTumourGrowthFunction, primaryTumourEventFunction, bloodStream);
        
        Distribution cellDeathBloodStreamDistribution = new GaussianDistribution(720., 180.);
        CellProgressionEventAction cellDeathBloodStream = new CellDeath("cell death - bloodStream", new GenerationInfo(.9,
                identity, cellDeathBloodStreamDistribution));
        
        bloodStreamTreatedType.addPossibleEventAction(metastasisEventAction2);
        bloodStreamTreatedType.addPossibleEventAction(cellDeathBloodStream);

        bloodStream.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, beginTreatmentAt), bloodStreamTreatedType);
        bloodStream.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, beginTreatmentAt + treatmentDurationInMin), bloodStreamType);    
        
        return primaryTumor;
    }
    
    private Compartment buildConfig3() throws MetastaseException{

		HostOrgan organ = new HostOrgan("organ", null, false);

		long startSize = 10000;
		Function dummy2 = new LinearFunction(5, startSize);
		ColonizationRate dummy = new ColonizationRate(0, 0);

		// ** primary tumor **
		Compartment primaryTumor = new ContinuousCompartment(0,	"primary tumor", organ, dummy2, dummy, true);

		// ** bloodstream **
		DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
		DiscreteCompartment bloodStream = new DiscreteCompartment(0, organ,	bloodStreamType);

		CellProgressionEventAction cellDeathEA = new CellDeath("cell death", new GenerationInfo(1., dummy, new GaussianDistribution(60., 20.)));
		bloodStreamType.addPossibleEventAction(cellDeathEA);

		primaryTumor.setSuccessor(bloodStream);

		return primaryTumor;
    }

    public void testParseFromFileName() {
        Compartment actual;
        try {
            actual = parser.getCompartment(config1);
            assertEquals(compartment1, actual);
        } catch (ParserException e) {
        	e.printStackTrace();
            fail("Unexpected exception: "+e);
        }
    }

    public void testInvalidFile() {

        try {
            parser.getCompartment(invalidFile);
            fail("An Exception should be thrown!");
        } catch (ParserException pe) {
        	assertTrue(pe.getMessage().startsWith("Couldn't parse the file: src/test/resources/xml/Invalid.xml"));
        }
    }

    public void testParseConfig1() throws ParserException{
       Compartment actual = parser.getCompartment(config1);
       assertEquals(compartment1, actual);
    }

    public void testParseConfig2() throws ParserException{
        Compartment actual = parser.getCompartment(config2);
        assertEquals(compartment2, actual);
    }
    
    public void testParseConfig3() throws MetastaseException{
        Compartment actual = parser.getCompartment(config3);
        Compartment expected = buildConfig3();
        assertEquals(expected, actual);        
    }
    
    public void testParseInvalidStartSize() {
        try {
            parser.getCompartment(config3_InvalidStartSize);
            fail("An exception should have been thrown.");
        } catch (ParserException pe) {
        	assertEquals("Couldn't parse the file: src/test/resources/xml/Config3_InvalidStartSize.xml\n"
        				+ "The growth function LinearFunction [5.0*x+0.0 ] is not correctly parametrisized. "
        				+ "To be used as a growth function the parameter 'n' (represents the start size of the tumour) "
        				+ "has to be an integer equal or greater than 1!", 
        				pe.getMessage());
        }
    }

    public void testParseConfigWithMultipleTreatment() throws ParserException{
        parser.getCompartment(config2_Multi);
        //FIXME test?

    }

    public void testParseConfigWithMultipleTreatmentWithInconsitence() {
        try {
            parser.getCompartment(config2_Inconsitence);
            fail("Inconsitence treatments. An Exception should be thrown.");
        } catch (ParserException pe) {
            String expected = "Couldn't parse the file: src/test/resources/xml/Config2_MultiError.xml\nInconsistent treatment was found at:\"start:\"time>=" + 200.0 * 60 * 24
            + "\" + end:\"time>=" + 300.0 * 60 * 24 + "\"\"";
            assertEquals(expected, pe.getMessage());
        }
    }

    public void testParseConfigWithMultipleTreatmentWithInvalidEnd() {
        try {
            parser.getCompartment(config2_InvalidEnd);
            fail("End is not correct. An Exception should be thrown.");
        } catch (ParserException pe) {
            String expected = "Couldn't parse the file: src/test/resources/xml/Config2_MultiInvalidEnd.xml\nInvalid <end> in treatment.\n<start>time>=" + 100.0 * 60 * 24
            + "</start><end>time>=" + 50.0 * 60 * 24 + "</end>";
            assertEquals(expected, pe.getMessage());
        }
    }

    public void testParseConfigWithMultipleTreatmentWithNoEnd() {
        try {
            parser.getCompartment(config2_MultiEnd);
            fail("A treatment with no <end> but it follows other treatments. An Exception should be thrown.");
        } catch (ParserException pe) {
            String expected = "Couldn't parse the file: src/test/resources/xml/Config2_MultiEnd.xml\n" +
            		"A treatment with no <end> was found but it follows other treatments!";
            assertEquals(expected, pe.getMessage());
        }
    }

    public void testParseConfigWithInvalidProbability() {
        try {
            parser.getCompartment(config2_InvalidProbability);
            fail("Invalid Probabilities. An Exception should be thrown.");
        } catch (ParserException pe) {
            String expected = "Couldn't parse the file: src/test/resources/xml/Config2_MultiInvalidProbability.xml\n"
            		+ "An error occured while creating a discrete compartment type: de.fhstralsund.catsit.base."
            		+ "DiscreteCompartmentTypeException: Sum of probabilities of all EventActions must be 1 but is 1.1";
            assertEquals(expected, pe.getMessage());
        }
    }

    public void testParseSimControllerNull() {
        try {
            parser.getCompartment("noFile");
            fail("An Exception should be thrown!");
        } catch (ParserException pe) {
            String expected = "Couldn't parse the file: noFile";
            assertTrue(pe.getMessage().startsWith(expected));
        }
    }

    public void testParseTechnical() throws ParserException{
        Technical actual = parser.getTechnical(technical);
        Technical expected = new Technical(2100, 50, true);
        assertEquals(expected.toString(), actual.toString());        
    }

    public void testParseInvalidTechnical() {
        try {
            parser.getTechnical(invalidTechnical);
            fail("An Exception should be thrown!");
        } catch (ParserException pe) {
            assertTrue(pe.getMessage().startsWith("cvc-complex-type.2.4.d:"));
        }
    }

    public void testParseTechnicalNull() {
        try {
            parser.getTechnical("noFile");
            fail("An Exception should be thrown!");
        } catch (ParserException pe) {
            String expected = "Couldn't parse the file: noFile";
            assertTrue(pe.getMessage().startsWith(expected));
        }
    }

    public void testEquals() throws MetastaseException{
        Compartment actual = buildConfig2();
        assertEquals(compartment2, actual);
        actual = buildConfig1();
        assertFalse(compartment2.equals(actual));
    }

    public void testHash() throws MetastaseException{
        int actual = buildConfig2().hashCode();
        int expected = compartment2.hashCode();
        assertEquals(actual, expected);
        actual = buildConfig1().hashCode();
        assertTrue(expected != actual);
    }
}
