/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.w3c.dom.Element;

import de.fhstralsund.catsit.base.Compartment;
import de.fhstralsund.catsit.base.ContinuousCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.conditions.Condition;
import de.fhstralsund.catsit.base.conditions.SizeCondition;
import de.fhstralsund.catsit.base.conditions.TimeCondition;
import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDeath;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDivision;
import de.fhstralsund.catsit.base.eventactions.localeventactions.LocalEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.io.xml.MPMLParserImpl;
import de.fhstralsund.catsit.io.xml.ParserException;
import de.fhstralsund.catsit.io.xml.ReferenceMap.RefType;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;


/**
 * Tests the {@link MPMLParserImpl}
 * 
 * @author Michael Suhr
 * 
 */
public class TestMPMLParserImpl extends TestCase {

    private MockMPMLParser parser = new MockMPMLParser();

    @Override
    protected final void setUp() throws Exception {
        super.setUp();
        Logger.getLogger("metastase.support.xml").setLevel(Level.WARNING);
    }

//    public void testResolveFunction() {
//
//        double tumorGompertzA = 0.00286 / 24. / 60.;
//        double tumorGompertzB = 7.3e10;
//        double tumorGompertzC = 1.;
//        double colonizationRateAlpha = 0.663;
//        double colonizationCoefficientM = 5.3e-8 / 24. / 60.;
//        Function primaryTumourGrowthFunction = new GompertzFunction(tumorGompertzA, tumorGompertzB, tumorGompertzC);
//
//        Function expected = new ColonizationRate(colonizationCoefficientM, colonizationRateAlpha, primaryTumourGrowthFunction);
//        Function actual = new ColonizationRate(colonizationCoefficientM, colonizationRateAlpha, null);
//        parser.maps.putUnreferenced("gompertz", RefType.FUNCTION, actual);
//
//        assertEquals(1, parser.maps.getUnreferencedSize());
//        assertNotSame(expected, actual);
//        parser.resolveRefFunction(primaryTumourGrowthFunction, parser.maps.getUnreferenced("gompertz;function"));
//        assertEquals(0, parser.maps.getUnreferencedSize());
//        assertEquals(expected, actual);
//    }

    public void testResolveCompartment() {
    	try{
	        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	        DiscreteCompartment compartment = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	
	        Function identity = new ConstantFunction();
	        double tumorGompertzA = 0.00286 / 24. / 60.;
	        double tumorGompertzB = 7.3e10;
	        double tumorGompertzC = 1.;
	        double colonizationRateAlpha = 0.663;
	        double colonizationCoefficientM = 5.3e-8 / 24. / 60.;
	        Function primaryTumourGrowthFunction = new GompertzFunction(tumorGompertzA, tumorGompertzB, tumorGompertzC);
	        ColonizationRate primaryTumourEventFunction = new ColonizationRate(colonizationCoefficientM, colonizationRateAlpha);

	        Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
	
	        LocalEventAction transferCst = new TransferToNewContinuousCompartment("bloodstream - metastasis", new GenerationInfo(1.,
	                identity, metastasisDistribution), "metastasis", new MockHostOrgan(), primaryTumourGrowthFunction, primaryTumourEventFunction, null);
	        LocalEventAction expectedTransferCst = new TransferToNewContinuousCompartment("bloodstream - metastasis", new GenerationInfo(1.,
	                identity, metastasisDistribution), "metastasis", new MockHostOrgan(), primaryTumourGrowthFunction, primaryTumourEventFunction, compartment);
	        parser.maps.putUnreferenced("blood", RefType.COMPARTMENT, transferCst);
	
	        LocalEventAction transferDst = new TransferToNewDiscreteCompartment("bloodstream - metastasis", new GenerationInfo(1., identity,
	                metastasisDistribution), new MockHostOrgan(), bloodStreamType, null);
	        LocalEventAction expectedTransferDst = new TransferToNewDiscreteCompartment("bloodstream - metastasis", new GenerationInfo(1., identity,
	                metastasisDistribution), new MockHostOrgan(), bloodStreamType, compartment);
	        parser.maps.putUnreferenced("blood", RefType.COMPARTMENT, transferDst);
	        
	        Compartment successor = new ContinuousCompartment(0, "primary tumor", new MockHostOrgan(), primaryTumourGrowthFunction, primaryTumourEventFunction);
	        parser.maps.putUnreferenced("blood", RefType.COMPARTMENT, successor);
	        Compartment expectedSuccessor = new ContinuousCompartment(0, "primary tumor", new MockHostOrgan(), primaryTumourGrowthFunction, primaryTumourEventFunction);
	        expectedSuccessor.setSuccessor(compartment);
	
	        assertEquals(3, parser.maps.getUnreferencedSize());
	        assertNotSame(expectedTransferCst.toString(), transferCst.toString());
	        assertNotSame(expectedTransferDst.toString(), transferDst.toString());
	        assertNotSame(expectedSuccessor, successor);
	        parser.resolveRefCompartment(compartment, parser.maps.getUnreferenced("blood;compartment"));
	        assertEquals(0, parser.maps.getUnreferencedSize());
	        assertEquals(expectedTransferCst.toString(), transferCst.toString());
	        assertEquals(expectedTransferDst.toString(), transferDst.toString());
	        assertEquals(expectedSuccessor, successor);
        
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    public void testResolveCompartmentType() {
    	try{
	        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	        DiscreteCompartment compartment = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	
	        Function identity = new ConstantFunction();
	        Distribution metastasisDistribution = new GaussianDistribution(60., 20.);	
	
	        LocalEventAction transferDst = new TransferToNewDiscreteCompartment("bloodstream - metastasis", new GenerationInfo(1., identity,
	                metastasisDistribution), new MockHostOrgan(), null, compartment);
	        LocalEventAction expectedTransferDst = new TransferToNewDiscreteCompartment("bloodstream - metastasis", new GenerationInfo(1., identity,
	                metastasisDistribution), new MockHostOrgan(), bloodStreamType, compartment);
	        parser.maps.putUnreferenced("bloodTest", RefType.TYPE, transferDst);
	
	        
	        assertNotSame(expectedTransferDst, transferDst);	       
	        assertEquals(1, parser.maps.getUnreferencedSize());
	        
	        parser.resolveRefCompartmentType(bloodStreamType, parser.maps.getUnreferenced("bloodTest;type"));
	        
	        assertEquals(0, parser.maps.getUnreferencedSize());
	        assertEquals(expectedTransferDst, transferDst);
	        
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    public void testDiscreteCompartment() {
    	try{
	        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	        DiscreteCompartment expected = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);

	        parser.maps.putCompartmentType("blood_Type", bloodStreamType);
	        parser.maps.putHostOrgan("blood_organ", new MockHostOrgan());
	
	        String xml = "<discreteCompartment compartmentID=\"blood\">" + "<hostOrgan organRef=\"blood_organ\"/>" + 
	        			"<discreteCompartmentType typeRef=\"blood_Type\"/>" + "</discreteCompartment>";
	        Element doc = MockParser.parseString(xml);
        
            Compartment actual = parser.discreteCompartment(doc);
            assertEquals(expected, actual);
            
        } catch (MetastaseException e) {
            fail("No Exception should be thrown");
        }
    }

    public void testDiscreteCompartmentWithNoReference() {
        String xml = "<discreteCompartment compartmentID=\"blood\">" + "<celldeathImmediately>true</celldeathImmediately>"
        + "<discreteCompartmentType typeRef=\"blood_Type\"/>" + "</discreteCompartment>";
        Element doc = MockParser.parseString(xml);
        try {
            parser.discreteCompartment(doc);
            fail("An Exception should be thrown because the reference is missing");
        } catch (ParserException pe) {
            assertEquals("Could not find the DiscreteCompartmentType with id: blood_Type Maybe it's a misspelling?", pe.getMessage());
        }
    }

    public void testAddTreatmentsTime() {
    	try{
	        String xml = "<localTreatments>" + "<localTreatment> <start>" + "<timeCondition>100</timeCondition>" 
	        		+ "</start>" + "<end>" + "<timeCondition>250</timeCondition>" + "</end>"
	        		+ "<compartmentType typeRef=\"treated\"/>" + "</localTreatment>" + "</localTreatments>";
	        Element doc = MockParser.parseString(xml);
	
	        // initialize the expected compartment
	        Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
	        Function identity = new ConstantFunction();
	        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	        bloodStreamType.addPossibleEventAction(new CellDivision("test division", new GenerationInfo(1., identity, metastasisDistribution)));
	        DiscreteCompartment expected = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	
	        DiscreteCompartmentType bloodStreamTreatedType = new DiscreteCompartmentType("blood stream treated");
	        bloodStreamTreatedType.addPossibleEventAction(new CellDeath("test death", new GenerationInfo(1., identity, metastasisDistribution)));
	
	        expected.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 100), bloodStreamTreatedType);
	        expected.addCompartmentType(new TimeCondition(Condition.GREATER_EQUALS, 250), bloodStreamType);
	
	        parser.maps.putCompartmentType("treated", bloodStreamTreatedType);
	        DiscreteCompartment actual = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);

        
            parser.addTreatments(doc, actual);
            assertEquals(expected, actual);
        } catch (MetastaseException pe) {
        	pe.printStackTrace();
            fail("No Exception should be thrown");
        }
    }

    public void testAddTreatmentsSize() throws MetastaseException{
    	
    	String xml = "<localTreatments>" + "<localTreatment> <start>" + "<sizeCondition conditionTest=\"&lt;\">"
	        + "100</sizeCondition>" + "</start>" + "<end>"
	        + "<sizeCondition conditionTest=\"&lt;\">50</sizeCondition>" + "</end>"
	        + "<compartmentType typeRef=\"treated\"/>" + "</localTreatment>" + "</localTreatments>";
	    Element doc = MockParser.parseString(xml);
	
	    // initialize the expected compartment
	    Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
	    Function identity = new ConstantFunction();
	    DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	    bloodStreamType.addPossibleEventAction(new CellDivision("test division", new GenerationInfo(1., identity, metastasisDistribution)));
	    DiscreteCompartment expected = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	
	    DiscreteCompartmentType bloodStreamTreatedType = new DiscreteCompartmentType("blood stream treated");
	    bloodStreamTreatedType.addPossibleEventAction(new CellDeath("test death", new GenerationInfo(1., identity, metastasisDistribution)));
	
	    expected.addCompartmentType(new SizeCondition(Condition.LESSER, 100), bloodStreamTreatedType);
	    expected.addCompartmentType(new SizeCondition(Condition.LESSER, 50), bloodStreamType);
	
	    parser.maps.putCompartmentType("treated", bloodStreamTreatedType);
	    DiscreteCompartment actual = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);

        
        parser.addTreatments(doc, actual);
        assertEquals(expected, actual);

    }

    public void testAddTreatmentsWithoutStart() throws MetastaseException{
    	
	    String xml = "<localTreatments>" + "<localTreatment> <end>" + "<timeCondition>250</timeCondition>" 
	        		+ "</end>" + "<compartmentType typeRef=\"treated\"/>" + "</localTreatment>"
	        		+ "</localTreatments>";
	    Element doc = MockParser.parseString(xml);
	
	    DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	    DiscreteCompartment actual = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	    parser.maps.putCompartmentType("treated", new DiscreteCompartmentType("blood stream treated"));
	
	    try {
	        parser.addTreatments(doc, actual);
	        fail("An exception should be thrown.");
	    } catch (ParserException pe) {
	        assertEquals("No start for local treatment found.", pe.getMessage());
	    }
    	
    }

    public void testAddTreatmentsWithoutCompartmentTypeReference() {
    	try{
	        String xml = "<localTreatments>" + "<localTreatment> <start>" 
	        		+ "<timeCondition>100</timeCondition>" + "</start>" + "<end>"
			        + "<timeCondition >250</timeCondition>" + "</end>"
			        + "<compartmentType typeRef=\"treated_invalid\"/>" 
			        + "</localTreatment>" + "</localTreatments>";
	        Element doc = MockParser.parseString(xml);
	
	        // initialize the expected compartment
	        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	        DiscreteCompartment actual = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	
	        try {
	            parser.addTreatments(doc, actual);
	            fail("An exception should be thrown.");
	        } catch (ParserException pe) {
	            assertEquals("No new compartment type for local treatment found.", pe.getMessage());
	        }
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    public void testAddTreatmentsWithInvalidEnd() throws MetastaseException{
	    
	    String xml = "<localTreatments>" + "<localTreatment> "
	        		+ "<start>" + "<timeCondition>250</timeCondition>" + "</start>" 
	        		+ "<end>" + "<timeCondition>100</timeCondition>" + "</end>"
	        		+ "<compartmentType typeRef=\"treated\"/>" + "</localTreatment>" + "</localTreatments>";
	    Element doc = MockParser.parseString(xml);
	
	    DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
	    DiscreteCompartment actual = new DiscreteCompartment(0, new MockHostOrgan(), bloodStreamType);
	    parser.maps.putCompartmentType("treated", new DiscreteCompartmentType("blood stream treated"));
	
	    try {
	        parser.addTreatments(doc, actual);
	        fail("An exception should be thrown.");
	    } catch (ParserException pe) {
	        assertEquals("Invalid <end> in treatment.\n" + "<start>" + "time>=" + 250d * 60 * 24 + "</start>"
	                + "<end>" + "time>=" + 100d * 60 * 24 + "</end>", pe.getMessage());
	    }
    }

    /**
     * Tests the {@link MPMLParserImpl#getCondition(org.w3c.dom.Element)}. First
     * it tests a valid {@link TimeCondition} than the invalid form.
     * 
     * @see this{@link #testSizeCondition()}
     */
    public void testTimeCondition() {
        // first a valid TimeCondition
        String condition = "<timeCondition>100</timeCondition>";
        Element doc = MockParser.parseString(condition);
        Condition expected = new TimeCondition(Condition.GREATER_EQUALS, 100);
        try {
            assertEquals(expected.toString(), parser.getCondition(doc).toString());
        } catch (ParserException pe) {
            fail("No exception should be thrown.");
        }
        try {
            condition = "<timeCondition>-100</timeCondition>";
            doc = MockParser.parseString(condition);
            parser.getCondition(doc);
            fail("An exception should be thrown.");
        } catch (ParserException pe) {
            assertEquals("TimeCondition must have a positive value.", pe.getMessage());
        }
    }

    /**
     * Tests the {@link MPMLParserImpl#getCondition(org.w3c.dom.Element)}. First
     * it tests a valid {@link SizeCondition} than the invalid form.
     * 
     * @see this{@link #testTimeCondition()}
     */
    public void testSizeCondition() {
        // first a valid SizeCondition
        String condition = "<sizeCondition conditionTest=\"&lt;\">100</sizeCondition>";
        Element doc = MockParser.parseString(condition);
        Condition expected = new SizeCondition(Condition.LESSER, 100);
        try {
            assertEquals(expected.toString(), parser.getCondition(doc).toString());
        } catch (ParserException pe) {
            fail("No exception should be thrown.");
        }
        try {
            condition = "<sizeCondition conditionTest=\"&lt;\">-100</sizeCondition>";
            doc = MockParser.parseString(condition);
            parser.getCondition(doc);
            fail("An exception should be thrown.");
        } catch (ParserException pe) {
            assertEquals("SizeCondition must have a positive value.", pe.getMessage());
        }
    }

    /**
     * Tests if an exception is thrown when both different types of Condition
     * occurs.
     */
    public void testMixedCondition() {
        String condition = "<timeCondition>100</timeCondition>";
        Element doc = MockParser.parseString(condition);
        try {
            parser.getCondition(doc);
        } catch (ParserException pe) {
            fail("No exception should be thrown.");
        }
        try {
            condition = "<sizeCondition conditionTest=\"&lt;\">100</sizeCondition>";
            doc = MockParser.parseString(condition);
            parser.getCondition(doc);
            fail("An exception should be thrown.");
        } catch (ParserException pe) {
            assertEquals("Either timeCondtion or sizeCondition is allowed!", pe.getMessage());
        }
        
        parser = new MockMPMLParser();
        condition = "<sizeCondition conditionTest=\"&lt;\">100</sizeCondition>";
        doc = MockParser.parseString(condition);
        try {
            parser.getCondition(doc);
        } catch (ParserException pe) {
            fail("No exception should be thrown.");
        }
        try {
            condition = "<timeCondition>100</timeCondition>";
            doc = MockParser.parseString(condition);
            parser.getCondition(doc);
            fail("An exception should be thrown.");
        } catch (ParserException pe) {
            assertEquals("Either timeCondtion or sizeCondition is allowed!", pe.getMessage());
        }
    }

    public void testGetConditionValue() {
        Condition cond = new TimeCondition(Condition.GREATER_EQUALS, 100 /60./24.);
        assertEquals(100d, parser.getConditionValue(cond));
        cond = new SizeCondition(Condition.GREATER_EQUALS, 150);
        assertEquals(150l, (long) parser.getConditionValue(cond));
    }

    public void testCheckConsistenceAndSortSize() {
        // unsorted List
        List<Condition> unsorted = new ArrayList<Condition>();
        List<Condition> sorted = new ArrayList<Condition>();

        Condition start = new SizeCondition(Condition.LESSER, 200);
        Condition end = new SizeCondition(Condition.LESSER, 150);
        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
        parser.addTreatment(start, end, bloodStreamType);
        unsorted.add(start);
        sorted.add(start);
        start = new SizeCondition(Condition.LESSER, 100);
        end = new SizeCondition(Condition.LESSER, 50);
        parser.addTreatment(start, end, bloodStreamType);
        unsorted.add(start);
        // add to beginning of list
        sorted.add(start);
        assertEquals(unsorted, parser.getStartCondtions());

        try {
            parser.checkConsistentOfTreatments();
            assertEquals(sorted, parser.getStartCondtions());
        } catch (ParserException pe) {
            fail("No exception should be thrown.");
        }
    }

    public void testCheckConsistenceAndSortTime() {
        // unsorted List
        List<Condition> unsorted = new ArrayList<Condition>();
        List<Condition> sorted = new ArrayList<Condition>();

        Condition start = new TimeCondition(Condition.GREATER_EQUALS, 200);
        Condition end = new TimeCondition(Condition.GREATER_EQUALS, 300);
        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
        parser.addTreatment(start, end, bloodStreamType);
        unsorted.add(start);
        sorted.add(start);
        start = new TimeCondition(Condition.GREATER_EQUALS, 50);
        end = new TimeCondition(Condition.GREATER_EQUALS, 100);
        parser.addTreatment(start, end, bloodStreamType);
        unsorted.add(start);
        // add to beginning of list
        sorted.add(0, start);
        assertEquals(unsorted, parser.getStartCondtions());

        try {
            parser.checkConsistentOfTreatments();
            assertEquals(sorted, parser.getStartCondtions());
        } catch (ParserException pe) {
            fail("No exception should be thrown.");
        }
    }

    public void testCheckConsistenceWithNoEndTag() {
        Condition start = new TimeCondition(Condition.GREATER_EQUALS, 50);
        Condition end = null;
        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
        parser.addTreatment(start, end, bloodStreamType);
        start = new TimeCondition(Condition.GREATER_EQUALS, 100);
        end = new TimeCondition(Condition.GREATER_EQUALS, 250);
        parser.addTreatment(start, end, bloodStreamType);
        try {
            parser.checkConsistentOfTreatments();
            fail("An Exception should be thrown because missing <end> but other treatments following.");
        } catch (ParserException pe) {
            assertEquals("A treatment with no <end> was found but it follows other treatments!", pe.getMessage());
        }
    }

    public void testCheckConsistenceWithInconsistence() {
        Condition start = new TimeCondition(Condition.GREATER_EQUALS, 50);
        Condition end = new TimeCondition(Condition.GREATER_EQUALS, 200);
        DiscreteCompartmentType bloodStreamType = new DiscreteCompartmentType("blood stream");
        parser.addTreatment(start, end, bloodStreamType);
        start = new TimeCondition(Condition.GREATER_EQUALS, 100);
        end = new TimeCondition(Condition.GREATER_EQUALS, 250);
        parser.addTreatment(start, end, bloodStreamType);
        try {
            parser.checkConsistentOfTreatments();
            fail("Exception should be thrown because inconsitent treatment start times.");
        } catch (ParserException pe) {
            assertEquals("Inconsistent treatment was found at:\"start:\"" + start.toString() + "\" + end:\"" + end.toString() + "\"\"", pe.getMessage());
        }
    }

    public void testContinuousCompartment() throws MetastaseException{

		double tumorGompertzA = 0.00286 / 24. / 60.; // = 0.00286/day (Iwata)
		double tumorGompertzB = 7.3e10; // (Iwata)
		double tumorGompertzC = 1.;
		double colonizationRateAlpha = 0.663; // (Iwata)
		double colonizationCoefficientM = 5.3e-8 / 24. / 60.; // =5.3e-8
		// /(cell*day)
		// Iwata

		// ** primary tumor **
		Function primaryTumourGrowthFunction = new GompertzFunction(tumorGompertzA, tumorGompertzB, tumorGompertzC);
		ColonizationRate primaryTumourEventFunction = new ColonizationRate(colonizationCoefficientM, colonizationRateAlpha);
		
		parser.maps.putFunction("gompertz", primaryTumourGrowthFunction);
		parser.maps.putFunction("colonizationRate", primaryTumourEventFunction);
		
		parser.maps.putHostOrgan("liver", new MockHostOrgan());
		
		DiscreteCompartment blood = new DiscreteCompartment(0, new MockHostOrgan(), new DiscreteCompartmentType("blood"));
		parser.maps.putCompartment("blood", blood);

		String xml = "<continuousCompartment compartmentID=\"tumor\">" + "<hostOrgan organRef=\"liver\"/>" + "<successor compartmentRef=\"blood\"/>" 
					+ "<isPrimaryTumor/>" + "<name>primary tumor</name>"+ "<growthFunction functionRef=\"gompertz\"/>" 
					+ "<colonizationRate functionRef=\"colonizationRate\"/>" + "</continuousCompartment>";
		Element doc = MockParser.parseString(xml);

		Compartment expected = new ContinuousCompartment(0, "primary tumor", new MockHostOrgan(), primaryTumourGrowthFunction, primaryTumourEventFunction);
		expected.setSuccessor(blood);

		Compartment actual = parser.continuousCompartment(doc);
		assertEquals(expected, actual);

    }

    public void testContinuousCompartmentWithNoReference() {
    	String xml = "<continuousCompartment compartmentID=\"tumor\">" + "<hostOrgan organRef=\"liver\"/>" + "<successor compartmentRef=\"blood\"/>" 
						+ "<isPrimaryTumor/>" + "<name>primary tumor</name>"+ "<growthFunction functionRef=\"gompertz\"/>" 
						+ "<colonizationRate functionRef=\"colonizationRate\"/>" + "</continuousCompartment>";
        Element doc = MockParser.parseString(xml);

        try {
            parser.continuousCompartment(doc);
            fail("An Exception should be thrown because the reference is missing");
        } catch (ParserException pe) {
            assertEquals("Could not find the growth function with id: gompertz. Maybe a misspelling?", pe.getMessage());
        }
    }

}
