/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.w3c.dom.Element;

import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.DiscreteCompartmentTypeException;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.Distribution;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.io.xml.MockHostOrgan;
import de.fhstralsund.catsit.io.xml.MockParser;
import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.io.xml.ParserException;
import de.fhstralsund.catsit.io.xml.builder.CompartmentTypeBuilder;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;


/**
 * Tests the {@link CompartmentTypeBuilder}.
 * 
 * @author Michael Suhr
 * 
 */
public class TestCompartmentTypeBuilder extends TestCase {

    private final ObjectBuffer maps = new ObjectBuffer(); 

    String discrete = "<discreteCompartmentType typeID=\"blood_Type\">" + "<name>blood stream</name>" + "<eventActions>"
            + "<transferToNewContinuousCompartment>" + "<name>bloodstream - metastasis</name>" + "<generationInfo>"
            + "<meanTimeToNextEvent functionRef=\"identity\"/>" + "<distribution distRef=\"normal\"/>"
            + "<probability>1</probability>" + "</generationInfo>" +"<newName>metastasis</newName>" 
            + "<newHostOrgan organRef=\"liver\"/>" + "<newGrowthFunction functionRef=\"gompertz\"/>" 
            + "<newColonizationRate functionRef=\"colonization\"/>" + "<newSuccessor compartmentRef=\"blood\"/>" 
            + "</transferToNewContinuousCompartment>" + "</eventActions>" + "\t\n\n\t</discreteCompartmentType>";

    String invalidDiscrete = "<discreteCompartmentType typeID=\"blood_Type\">" + "<name>blood stream</name>"
            + "<eventActions>" + "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream</name>"
            + "<generationInfo>" + "<meanTimeToNextEvent functionRef=\"identity\"/>"
            + "<distribution distRef=\"normal\"/>" + "<probability>.8</probability>" + "</generationInfo>"
            + "<newName>metastasis</newName>" + "<newHostOrgan organRef=\"liver\"/>" 
            + "<newGrowthFunction functionRef=\"gompertz\"/>" + "<newColonizationRate functionRef=\"colonization\"/>" 
            + "<newSuccessor compartmentRef=\"blood\"/>" + "</transferToNewContinuousCompartment>" 
            + "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream test</name>" + "<generationInfo>"
            + "<meanTimeToNextEvent functionRef=\"identity\"/>" + "<distribution distRef=\"normal\"/>"
            + "<probability>.3</probability>" + "</generationInfo>" + "<newName>metastasis</newName>" 
            + "<newHostOrgan organRef=\"liver\"/>" + "<newGrowthFunction functionRef=\"gompertz\"/>" 
            + "<newColonizationRate functionRef=\"colonization\"/>" + "<newSuccessor compartmentRef=\"blood\"/>"
            + "</transferToNewContinuousCompartment>" + "</eventActions>" + "</discreteCompartmentType>";

    String continuous = "<continuousCompartmentType typeID=\"tumor_Type\">" + "<name>primary tumor</name>"
            + "<growthFunction functionRef=\"gompertz\"/>" + "<eventRateFunction functionRef=\"colonization\"/>"
            + "</continuousCompartmentType>";
    
    String continuous2 = "<continuousCompartmentType typeID=\"tumor_Type\">" + "<name>primary tumor</name>"
            + "<growthFunction functionRef=\"gompertz\"/>" + "<eventRateFunction functionRef=\"colonization\"/>"
            + "<activityConcentration>0.12345</activityConcentration>" 
            + "</continuousCompartmentType>";


    @Override
    protected final void setUp() throws Exception {
        super.setUp();
        Logger.getLogger("metastase.support.xml").setLevel(Level.WARNING);
        // Initialize all needed other classes
        Function gompertz = new GompertzFunction(0.00286d, 7.3e10, 1d);
        ColonizationRate colonization = new ColonizationRate(5.3e-8, 0.663);
//        colonization.setGrowthFunction(new PiecewiseGrowthFunction(0, gompertz));
        Function identity = new ConstantFunction();
        // All functions used by compartment type
        maps.putFunction("gompertz", gompertz);
        maps.putFunction("colonization", colonization);
        maps.putFunction("identity", identity);
        maps.putHostOrgan("liver", new MockHostOrgan("liver"));
        maps.putHostOrgan("blood", new MockHostOrgan("blood"));
    }

    /**
     * Test to create a compartment typ if compartment references cannot be resolved.
     * 
     * @throws DiscreteCompartmentTypeException
     */
    public void testDiscreteCompartmentTypeWithoutReference() throws DiscreteCompartmentTypeException {

    	try{
	        // Initialize all needed other classes
	        Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
	        maps.putDistribution("normal", metastasisDistribution);
	        assertEquals(1, maps.getDistributionSize());
	
	        DiscreteCompartmentType expected = new DiscreteCompartmentType("blood stream");
	        
	        // generated compartment type	
	        CellProgressionEventAction metastasisEventAction = new TransferToNewContinuousCompartment("bloodstream - metastasis",
	                new GenerationInfo(1., maps.getFunction("identity"), metastasisDistribution), 
	                "metastasis", new MockHostOrgan("liver"), maps.getFunction("gompertz"), maps.getColonisationRate("colonization"), null);
	        expected.addPossibleEventAction(metastasisEventAction);
	
	        Element doc = MockParser.parseString(discrete);
	        Object actual = CompartmentTypeBuilder.parse(doc, maps);
	        
	        assertTrue(actual instanceof DiscreteCompartmentType);
	        assertEquals(expected, actual);
	        
	        assertEquals(1, maps.getCompartmentTypeSize());
	        assertEquals(1, maps.getUnreferencedKeySize());
	        
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    /**
     * Test to create a compartment typ if all references can be resolved. In
     * simulation it is not possible beacuase the compartments can be created with an
     * early created compartment type. Only for test purpose!
     * 
     * @throws DiscreteCompartmentTypeException
     */
    public void testDiscreteCompartmentTypeWithReference() throws MetastaseException {
    	
		// Initialize all needed other classes
		Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
		maps.putDistribution("normal", metastasisDistribution);
		assertEquals(1, maps.getDistributionSize());

		DiscreteCompartmentType expected = new DiscreteCompartmentType("blood stream");
		DiscreteCompartment bloodStream = new DiscreteCompartment(0, new MockHostOrgan("blood"), expected);

		maps.putCompartment("blood", bloodStream);

		CellProgressionEventAction metastasisEventAction = new TransferToNewContinuousCompartment(
				"bloodstream - metastasis", new GenerationInfo(1., maps.getFunction("identity"), metastasisDistribution),
				"metastasis", new MockHostOrgan("liver"), maps.getFunction("gompertz"), maps.getColonisationRate("colonization"), bloodStream);
		expected.addPossibleEventAction(metastasisEventAction);

		Element doc = MockParser.parseString(discrete);
		Object actual = CompartmentTypeBuilder.parse(doc, maps);

		assertTrue(actual instanceof DiscreteCompartmentType);
		assertEquals(expected, actual);

		assertEquals(1, maps.getCompartmentTypeSize());
		assertEquals(0, maps.getUnreferencedKeySize());

    }

    /**
     * Test if the sum of all eventAction in this type is 1.
     */
    public void testSumEventActionsInDiscreteCompartmentType() throws MetastaseException{
    	
	    // Initialize all needed other classes
	    Distribution metastasisDistribution = new GaussianDistribution(60., 20.);
	    maps.putDistribution("normal", metastasisDistribution);
	    assertEquals(1, maps.getDistributionSize());
	    
	    DiscreteCompartmentType expected = new DiscreteCompartmentType("blood stream");
	    DiscreteCompartment bloodStream = new DiscreteCompartment(0, new MockHostOrgan("blood"), expected);
	    // adds the compartment to list. This compartment can normally created with the
	    // generated compartment type
	    assertEquals(0, maps.getCompartmentSize());
	    maps.putCompartment("blood", bloodStream);
	    assertEquals(1, maps.getCompartmentSize());
	        
	    try {
	    	Element doc = MockParser.parseString(invalidDiscrete);
		    CompartmentTypeBuilder.parse(doc, maps);
		    fail("An Exception should be thrown.");
		} catch (ParserException pe) {
		    assertEquals("Sum of probabilities of all EventActions must be 1 but is 1.1", pe.getCause().getMessage());
		}    	
    }
}
