/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.w3c.dom.Element;

import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.DiscreteCompartmentType;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.distributions.GaussianDistribution;
import de.fhstralsund.catsit.base.distributions.UniformDistribution;
import de.fhstralsund.catsit.base.eventactions.GenerationInfo;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDeath;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellDivision;
import de.fhstralsund.catsit.base.eventactions.localeventactions.CellProgressionEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.LocalEventAction;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewContinuousCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNewDiscreteCompartment;
import de.fhstralsund.catsit.base.eventactions.localeventactions.TransferToNextCompartment;
import de.fhstralsund.catsit.io.xml.MockParser;
import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.io.xml.ParserException;
import de.fhstralsund.catsit.io.xml.MockHostOrgan;
import de.fhstralsund.catsit.io.xml.builder.EventActionBuilder;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.GompertzFunction;


/**
 * Tests the {@link EventActionBuilder}.
 * 
 * @author Michael Suhr, Anja Bethge
 * 
 */
public class TestEventActionBuilder extends TestCase {

	private final ObjectBuffer maps = new ObjectBuffer();

    String transferCont = "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream</name>"
            + "<generationInfo>" + "<meanTimeToNextEvent functionRef=\"identity\"/>"
            + "<distribution distRef=\"normal\"/>" + "<probability>1</probability>" + "</generationInfo>"
            + "<newName>metastasis</newName>" + "<newHostOrgan organRef=\"liver\"/>" 
            + "<newGrowthFunction functionRef=\"gompertz\"/>" + "<newColonizationRate functionRef=\"colonization\"/>" 
            + "<newSuccessor compartmentRef=\"blood\"/>" + "</transferToNewContinuousCompartment>";

    String transferDisc = "<transferToNewDiscreteCompartment>" + "<name>discrete test</name>" + "<generationInfo>"
            + "<meanTimeToNextEvent functionRef=\"identity\"/>" + "<distribution distRef=\"normal\"/>"
            + "<probability>1</probability>" + "</generationInfo>" + "<newHostOrgan organRef=\"liver\"/>" 
            + "<newCompartmentType typeRef=\"blood_Type\"/>" + "<newSuccessor compartmentRef=\"blood\"/>" 
            + "</transferToNewDiscreteCompartment>";

    String cellDeath = "<cellDeath>" + "<name>cell death test</name>" + "<generationInfo>"
            + "<meanTimeToNextEvent functionRef=\"identity\"/>" + "<distribution distRef=\"normal\"/>"
            + "<probability>1</probability>" + "</generationInfo>" + "</cellDeath>";

    String cellDivision = "<cellDivision>" + "<name>cell division test</name>" + "<generationInfo>"
            + "<meanTimeToNextEvent functionRef=\"identity\"/>" + "<distribution distRef=\"normal\"/>"
            + "<probability>1</probability>" + " </generationInfo>" + "</cellDivision>";

    String transferToNext = "<transferToNext>" + "<name>transfer to next test</name>" + "<generationInfo>"
            + "<meanTimeToNextEvent functionRef=\"identity\"/>" + "<distribution distRef=\"treated\"/>"
            + "<probability>1</probability>" + "</generationInfo>" + " </transferToNext>";

    @Override
    protected final void setUp() throws Exception {
        super.setUp();
        Logger.getLogger("metastase.support.xml").setLevel(Level.WARNING);
        // Initialize all needed other classes
        maps.putFunction("gompertz", new GompertzFunction(0.00286d, 7.3e10, 1d));
        maps.putFunction("colonization", new ColonizationRate(5.3e-8, 0.663));
        maps.putFunction("identity", new ConstantFunction());
        maps.putDistribution("normal", new GaussianDistribution(60, 20));
        maps.putDistribution("treated", new UniformDistribution(10, 15));
        maps.putHostOrgan("liver", new MockHostOrgan("liver"));
        maps.putHostOrgan("blood", new MockHostOrgan("blood"));
    }

    /**
     * Test to create a compartment type if the <code>compartmentRef</code> and
     * <code>typeRef</code> references cannot be resolved.
     */
    public void testTransferToNewContinuousCompartmentWithoutReferences() throws MetastaseException{
    	
	    Element doc = MockParser.parseString(transferCont);

	    LocalEventAction expected = new TransferToNewContinuousCompartment("metastasis - bloodstream", 
	        		new GenerationInfo(1., maps.getFunction("identity"), maps.getDistribution("normal")), 
	        		"metastasis", new MockHostOrgan("liver"), maps.getFunction("gompertz"), maps.getColonisationRate("colonization"), null);
	
	    LocalEventAction actual = EventActionBuilder.parse(doc, maps);
	    assertEquals(expected, actual);
	    assertNotNull(actual);
        
	    //test with wrong ref for growth function
	    transferCont = "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream</name>"
	            + "<generationInfo>" + "<meanTimeToNextEvent functionRef=\"identity\"/>"
	            + "<distribution distRef=\"normal\"/>" + "<probability>1</probability>" + "</generationInfo>"
	            + "<newName>metastasis</newName>" + "<newHostOrgan organRef=\"liver\"/>" 
	            + "<newGrowthFunction functionRef=\"gombertz\"/>" + "<newColonizationRate functionRef=\"colonization\"/>" 
	            + "<newSuccessor compartmentRef=\"blood\"/>" + "</transferToNewContinuousCompartment>";
	    
	    doc = MockParser.parseString(transferCont);
	    try {
	    	EventActionBuilder.parse(doc, maps);
	    	fail("An exception should have been thrown.");
	    	
		} catch (ParserException e) {
			assertEquals("Could not find the growth function 'gombertz'. Maybe a mispelling?", e.getMessage());
		}
	    
	   //test with wrong ref for colonization rate
	    transferCont = "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream</name>"
	            + "<generationInfo>" + "<meanTimeToNextEvent functionRef=\"identity\"/>"
	            + "<distribution distRef=\"normal\"/>" + "<probability>1</probability>" + "</generationInfo>"
	            + "<newName>metastasis</newName>" + "<newHostOrgan organRef=\"liver\"/>" 
	            + "<newGrowthFunction functionRef=\"gompertz\"/>" + "<newColonizationRate functionRef=\"colonisation\"/>" 
	            + "<newSuccessor compartmentRef=\"blood\"/>" + "</transferToNewContinuousCompartment>";
	    
	    doc = MockParser.parseString(transferCont);
	    try {
	    	EventActionBuilder.parse(doc, maps);
	    	fail("An exception should have been thrown.");
	    	
		} catch (ParserException e) {
			assertEquals("Could not find the colonization rate 'colonisation'. Maybe a mispelling?", e.getMessage());
		}
	    
	    //test with no colonization rate
	    transferCont = "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream</name>"
	            + "<generationInfo>" + "<meanTimeToNextEvent functionRef=\"identity\"/>"
	            + "<distribution distRef=\"normal\"/>" + "<probability>1</probability>" + "</generationInfo>"
	            + "<newName>metastasis</newName>" + "<newHostOrgan organRef=\"liver\"/>" 
	            + "<newGrowthFunction functionRef=\"gompertz\"/>" + "<newSuccessor compartmentRef=\"blood\"/>" 
	            + "</transferToNewContinuousCompartment>";
	    
	    doc = MockParser.parseString(transferCont);
	    expected = new TransferToNewContinuousCompartment("metastasis - bloodstream", 
        		new GenerationInfo(1., maps.getFunction("identity"), maps.getDistribution("normal")), 
        		"metastasis", new MockHostOrgan("liver"), maps.getFunction("gompertz"), null, null);

	    actual = EventActionBuilder.parse(doc, maps);
	    assertEquals(expected, actual);
	    assertNotNull(actual);
	    
	    //test with no colonization rate and no successor
	    transferCont = "<transferToNewContinuousCompartment>" + "<name>metastasis - bloodstream</name>"
	            + "<generationInfo>" + "<meanTimeToNextEvent functionRef=\"identity\"/>"
	            + "<distribution distRef=\"normal\"/>" + "<probability>1</probability>" + "</generationInfo>"
	            + "<newName>metastasis</newName>" + "<newHostOrgan organRef=\"liver\"/>" 
	            + "<newGrowthFunction functionRef=\"gompertz\"/>" + "</transferToNewContinuousCompartment>";
	    
	    doc = MockParser.parseString(transferCont);
	    expected = new TransferToNewContinuousCompartment("metastasis - bloodstream", 
        		new GenerationInfo(1., maps.getFunction("identity"), maps.getDistribution("normal")), 
        		"metastasis", new MockHostOrgan("liver"), maps.getFunction("gompertz"), null, null);

	    actual = EventActionBuilder.parse(doc, maps);
	    assertEquals(expected, actual);
	    assertNotNull(actual);
    }

    /**
     * Test to create a compartment type if the <code>compartmentRef</code> and
     * <code>typeRef</code> references cannot be resolved.
     */
    public void testTransferToNewDiscreteCompartmentWihtoutReferences() {
    	try{
	        Element doc = MockParser.parseString(transferDisc);
	
	        DiscreteCompartmentType compartmentType = new DiscreteCompartmentType("sample discrete type");
	
	        DiscreteCompartment newSuccessor = new DiscreteCompartment(0, new MockHostOrgan("blood"), compartmentType);
	        LocalEventAction expected = new TransferToNewDiscreteCompartment("event action", new GenerationInfo(1., 
	        		maps.getFunction("identity"), maps.getDistribution("normal")), new MockHostOrgan("liver"), compartmentType, newSuccessor);
	
	        LocalEventAction actual = EventActionBuilder.parse(doc, maps);
	        assertNotSame(expected, actual);
	        assertEquals(2, maps.getUnreferencedKeySize());
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    /**
     * Test to create a compartment type if all references can be resolved. In
     * simulation it is not possible beacuase the compartments can be created with an
     * early created compartment type. Only for test purpose!
     */
    public void testTransferToNewContinuousCompartment() {
    	try{
	        Element doc = MockParser.parseString(transferCont);

	        DiscreteCompartmentType discreteType = new DiscreteCompartmentType("blood stream");
	        DiscreteCompartment bloodStream = new DiscreteCompartment(0, new MockHostOrgan("blood"), discreteType);
	        CellProgressionEventAction expected = new TransferToNewContinuousCompartment("metastasis - bloodstream", 
	        		new GenerationInfo(1., maps.getFunction("identity"), maps.getDistribution("normal")), 
	        		"metastasis", new MockHostOrgan("liver"), maps.getFunction("gompertz"),	maps.getColonisationRate("colonization"), bloodStream);
	
	        assertEquals(0, maps.getCompartmentSize());
	        maps.putCompartment("blood", bloodStream);
	        assertEquals(1, maps.getCompartmentSize());
	        assertEquals(0, maps.getCompartmentTypeSize());
	        
	        CellProgressionEventAction actual = EventActionBuilder.parse(doc, maps);
	        assertEquals(expected, actual);
	        assertEquals(0, maps.getUnreferencedKeySize());
	        
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    /**
     * Test to create a compartment type if all references can be resolved. In
     * simulation it is not possible because the compartments can be created with an
     * early created compartment type. Only for test purpose!
     */
    public void testTransferToNewDiscreteCompartment() {
    	try{
	        Element doc = MockParser.parseString(transferDisc);
	
	        DiscreteCompartmentType compartmentType = new DiscreteCompartmentType("sample discrete type");
	
	        DiscreteCompartment newSuccessor = new DiscreteCompartment(0, new MockHostOrgan("blood"), compartmentType);
	        TransferToNewDiscreteCompartment expected = new TransferToNewDiscreteCompartment("discrete test", 
	        		new GenerationInfo(1., maps.getFunction("identity"), maps.getDistribution("normal")), new MockHostOrgan("liver"), 
	        		compartmentType, newSuccessor);
	        assertEquals(0, maps.getCompartmentSize());
	        maps.putCompartment("blood", newSuccessor);
	        assertEquals(1, maps.getCompartmentSize());
	        assertEquals(0, maps.getCompartmentTypeSize());
	        maps.putCompartmentType("blood_Type", compartmentType);
	        assertEquals(1, maps.getCompartmentTypeSize());
	
	        CellProgressionEventAction actual = EventActionBuilder.parse(doc, maps);
	        assertEquals(expected, actual);
	        assertEquals(0, maps.getUnreferencedKeySize());
    	} catch (MetastaseException e) {
			e.printStackTrace();
			fail("unexpected exeption");
		}
    }

    public void testCellDeath() throws ParserException{
        Element doc = MockParser.parseString(cellDeath);

        CellDeath expected = new CellDeath("cell death test", new GenerationInfo(1., 
        		maps.getFunction("identity"), maps.getDistribution("normal")));

        LocalEventAction actual = EventActionBuilder.parse(doc, maps);
        assertEquals(expected.toString(), actual.toString());
        assertEquals(0, maps.getUnreferencedKeySize());
    }

    public void testCellDivision() throws ParserException{
        Element doc = MockParser.parseString(cellDivision);

        CellDivision expected = new CellDivision("cell division test", new GenerationInfo(1., 
        		maps.getFunction("identity"), maps.getDistribution("normal")));

        LocalEventAction actual = EventActionBuilder.parse(doc, maps);
        assertEquals(expected.toString(), actual.toString());
        assertEquals(0, maps.getUnreferencedKeySize());
    }

    public void testTransferToNext() throws ParserException{
        Element doc = MockParser.parseString(transferToNext);

        TransferToNextCompartment expected = new TransferToNextCompartment("transfer to next test", 
        		new GenerationInfo(1., maps.getFunction("identity"), maps.getDistribution("treated")));

        LocalEventAction actual = EventActionBuilder.parse(doc, maps);
        assertEquals(expected.toString(), actual.toString());
        assertEquals(0, maps.getUnreferencedKeySize());
    }

}
