/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;

import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.TestCase;

import org.w3c.dom.Element;

import de.fhstralsund.catsit.io.xml.MockParser;
import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.io.xml.builder.FunctionBuilder;
import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.LogFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;


/**
 * Tests the {@link FunctionBuilder}.
 * 
 * @author Michael
 * 
 */
public class TestFunctionBuilder extends TestCase {

	private final ObjectBuffer maps = new ObjectBuffer();

    String constant = "<constantFunction functionID=\"identity\">" + "<constant>1</constant></constantFunction>";

    String gompertz = "<gompertzFunction functionID=\"gompertz\">" + "<growthRate>0.00286</growthRate>"
            + "<tumorSize>7.3e10</tumorSize>" + "<c>1</c>" + "</gompertzFunction>";

    String colon = "<colonizationRate functionID=\"colonization\">"
            + "<colonizationCoefficient>5.3e-8</colonizationCoefficient>" + "<fractalDim>0.663</fractalDim>"
            + "</colonizationRate>";

    String log1 = "<logFunction functionID=\"log\">" + "<a>1</a><b>2</b></logFunction>";
    String log2 = "<logFunction functionID=\"log\">" + "<a>1</a><b>2</b><startsize>200</startsize></logFunction>";
    String power = "<powerFunction functionID=\"pow\">" + "<a>1</a><b>2</b><c>3</c></powerFunction>";
    String linear = "<linearFunction functionID=\"lin\">" + "<m>1</m><n>2</n></linearFunction>";
    String exp = "<exponentialFunction functionID=\"exp\">" + "<a>1</a>" + "<b>2</b>" + "<c>3</c>"
            + "</exponentialFunction>";

    @Override
    protected final void setUp() throws Exception {
        super.setUp();
        Logger.getLogger("metastase.support.xml").setLevel(Level.WARNING);
    }

    public void testConstantFunction() {
        assertEquals(0, maps.getFunctionSize());
        Element doc = MockParser.parseString(constant);

        Function expected = new ConstantFunction();
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(1, maps.getFunctionSize());
        assertEquals(expected, actual);
    }

    public void testGompertzFunction() {
        assertEquals(0, maps.getFunctionSize());
        Element doc = MockParser.parseString(gompertz);

        Function expected = new GompertzFunction(0.00286, 7.3e10, 1d);
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(expected, actual);
        assertEquals(1, maps.getFunctionSize());
    }

    public void testColonizationFunction() {
        Element doc = MockParser.parseString(gompertz);
        FunctionBuilder.parse(doc, maps);
        doc = MockParser.parseString(colon);
        Function expected = new ColonizationRate(5.3e-8, 0.663);
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(expected, actual);
    }

    public void testLogFunction() {
        Element doc = MockParser.parseString(log1);
        assertEquals(0, maps.getUnreferencedKeySize());

        Function expected = new LogFunction(1, 2);
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(0, maps.getUnreferencedKeySize());
        assertEquals(expected, actual);
        
        //with start size
        doc = MockParser.parseString(log2);
        assertEquals(0, maps.getUnreferencedKeySize());

        expected = new LogFunction(1, 2, 200);
        actual = FunctionBuilder.parse(doc, maps);
        assertEquals(0, maps.getUnreferencedKeySize());
        assertEquals(expected, actual);
    }

    public void testPowerFunction() {
        Element doc = MockParser.parseString(power);
        assertEquals(0, maps.getUnreferencedKeySize());

        Function expected = new PowerFunction(1, 2, 3);
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(0, maps.getUnreferencedKeySize());
        assertEquals(expected, actual);
    }

    public void testLinearFunction() {
        Element doc = MockParser.parseString(linear);
        assertEquals(0, maps.getUnreferencedKeySize());

        Function expected = new LinearFunction(1, 2);
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(0, maps.getUnreferencedKeySize());
        assertEquals(expected, actual);
    }

    public void testExponentialFunction() {
        Element doc = MockParser.parseString(exp);
        assertEquals(0, maps.getUnreferencedKeySize());

        Function expected = new ExponentialFunction(1, 2, 3);
        Function actual = FunctionBuilder.parse(doc, maps);
        assertEquals(0, maps.getUnreferencedKeySize());
        assertEquals(expected, actual);
    }

}
