/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.io.xml.builder;


import java.util.Iterator;

import junit.framework.TestCase;

import org.w3c.dom.Element;

import de.fhstralsund.catsit.base.HostOrgan;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.base.treatments.ChemoTreatment;
import de.fhstralsund.catsit.base.treatments.ExternalBeamTreatment;
import de.fhstralsund.catsit.base.treatments.RadioembolizationTreatment;
import de.fhstralsund.catsit.base.treatments.RadioimmunoTreatment;
import de.fhstralsund.catsit.base.treatments.Treatment;
import de.fhstralsund.catsit.base.treatments.TreatmentList;
import de.fhstralsund.catsit.base.treatments.TreatmentList.TreatmentEntry;
import de.fhstralsund.catsit.io.xml.MockHostOrgan;
import de.fhstralsund.catsit.io.xml.MockParser;
import de.fhstralsund.catsit.io.xml.ObjectBuffer;
import de.fhstralsund.catsit.io.xml.ParserException;
import de.fhstralsund.catsit.io.xml.builder.GlobalTreatmentBuilder;

public class TestGlobalTreatmentBuilder extends TestCase {

	protected void setUp() throws Exception {
		TreatmentList.reset();
	}
	
	@Override
	protected void tearDown() throws Exception {
		TreatmentList.reset();
	}

	public void testRadioimmunotherapy() throws ParserException{
		TreatmentList.reset();		
		ObjectBuffer maps = new ObjectBuffer();
		
		//test Radioimmunotherapy
		String xml = "<radioimmunotherapy> <time>600</time> <quantity>5</quantity> <interval>10</interval> "
				+ "<alpha>1</alpha> <beta>2</beta> <activity>3</activity> <clearanceFraction>0.7</clearanceFraction> <energy>4</energy> "
				+ "<halflife>5</halflife> <biologicalHalflife>6</biologicalHalflife> <excludeBlood>true</excludeBlood> </radioimmunotherapy>";
		Element doc = MockParser.parseString(xml);
		
		GlobalTreatmentBuilder.parse(doc, maps);
		Iterator<TreatmentEntry> it = TreatmentList.getInstance().iterator();
		
		//get the first entry
		TreatmentEntry treatmentEntry = it.next();
		Treatment exptectedTreatment = new RadioimmunoTreatment(1, 2, 3, 4, 5, 6, 0.7, true);
		//check time and treatment
		assertEquals(864000., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		treatmentEntry = it.next();	
		//check time and treatment
		assertEquals(878400., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		treatmentEntry = it.next();
		//check time and treatment
		assertEquals(892800., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		treatmentEntry = it.next();
		//check time and treatment
		assertEquals(907200., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		treatmentEntry = it.next();
		//check time and treatment
		assertEquals(921600., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);	
		
		
		//no further treatments should be in the treatment list
		assertFalse(it.hasNext());
		
		
		//test Radioimmunotherapy without biological halflife
		TreatmentList.reset();
		xml = "<radioimmunotherapy> <time>600</time> <alpha>1</alpha> <beta>2</beta> <activity>3</activity> "
				+ "<energy>4</energy> <halflife>5</halflife> <excludeBlood>true</excludeBlood> </radioimmunotherapy>";
		doc = MockParser.parseString(xml);
				
		GlobalTreatmentBuilder.parse(doc, maps);
		it = TreatmentList.getInstance().iterator();
				
		//get the first entry
		treatmentEntry = it.next();
		exptectedTreatment = new RadioimmunoTreatment(1, 2, 3, 4, 5, 0, true);
		//check time and treatment
		assertEquals(864000., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		//no further treatments should be in the treatment list
		assertFalse(it.hasNext());
	}
	
	public void testRadioembolization() throws MetastaseException{
		TreatmentList.reset();		
		ObjectBuffer maps = new ObjectBuffer();
		HostOrgan organ = new MockHostOrgan();
		maps.putHostOrgan("organ", organ);
		
		//test Radioembolization with min diameter
		String xml = "<radioembolization> <time>600</time> <alpha>1</alpha> <beta>2</beta> <hostOrgan organRef=\"organ\"/> "
				+ "<activity>3</activity> <energy>4</energy> <halflife>5</halflife> <minDiameter>4</minDiameter> </radioembolization>";
		Element doc = MockParser.parseString(xml);
		
		GlobalTreatmentBuilder.parse(doc, maps);
		Iterator<TreatmentEntry> it = TreatmentList.getInstance().iterator();
		
		//get the first entry
		TreatmentEntry treatmentEntry = it.next();
		Treatment exptectedTreatment = new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 33.510321638291124);
		//check time and treatment
		assertEquals(864000., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		//no further treatments should be in the treatment list
		assertFalse(it.hasNext());
		
		
		//test Radioembolization with min diameter with min volume
		TreatmentList.reset();
		xml = "<radioembolization> <time>600</time> <alpha>1</alpha> <beta>2</beta> <hostOrgan organRef=\"organ\"/> "
				+ "<activity>3</activity> <energy>4</energy> <halflife>5</halflife> <minVolume>5</minVolume> </radioembolization>";
		doc = MockParser.parseString(xml);
				
		GlobalTreatmentBuilder.parse(doc, maps);
		it = TreatmentList.getInstance().iterator();
				
		//get the first entry
		treatmentEntry = it.next();
		exptectedTreatment = new RadioembolizationTreatment(1, 2, 3, 4, 5, organ, 5);
		//check time and treatment
		assertEquals(864000., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
		
		//no further treatments should be in the treatment list
		assertFalse(it.hasNext());
		
		//test Radioembolization with no host organ
		TreatmentList.reset();
		xml = "<radioembolization> <time>600</time> <alpha>1</alpha> <beta>2</beta>"
				+ "<activity>3</activity> <energy>4</energy> <halflife>5</halflife> <minVolume>5</minVolume> </radioembolization>";
		doc = MockParser.parseString(xml);
		
		try {
			GlobalTreatmentBuilder.parse(doc, maps);
		} catch (ParserException e) {
			assertEquals("Please specify the host organ in which the radioembolisation is executed.", e.getMessage());
		}
		
		//test Radioembolization with wrong host organ id
		TreatmentList.reset();
		xml = "<radioembolization> <time>600</time> <alpha>1</alpha> <beta>2</beta> <hostOrgan organRef=\"organ3\"/> "
				+ "<activity>3</activity> <energy>4</energy> <halflife>5</halflife> <minVolume>5</minVolume> </radioembolization>";
		doc = MockParser.parseString(xml);
		
		try {
			GlobalTreatmentBuilder.parse(doc, maps);
		} catch (ParserException e) {
			assertEquals("Could not find host organ with the id 'organ3'. Maybe a misspelling?", e.getMessage());
		}
		
		//test Radioembolization with no min diameter and min volume
		TreatmentList.reset();
		xml = "<radioembolization> <time>600</time> <alpha>1</alpha> <beta>2</beta> <hostOrgan organRef=\"organ\"/> "
				+ "<activity>3</activity> <energy>4</energy> <halflife>5</halflife> </radioembolization>";
		doc = MockParser.parseString(xml);
		
		try {
			GlobalTreatmentBuilder.parse(doc, maps);
		} catch (ParserException e) {
			assertEquals("Please state a minimum size (mm³) or minimum diameter (mm) for metastases to be treated.", e.getMessage());
		}
		
		//test Radioembolization with both min diameter and min volume
		TreatmentList.reset();
		xml = "<radioembolization> <time>600</time> <alpha>1</alpha> <beta>2</beta> <hostOrgan organRef=\"organ\"/> "
				+ "<activity>3</activity> <energy>4</energy> <halflife>5</halflife> <minDiameter>4</minDiameter> "
				+ "<minVolume>5</minVolume> </radioembolization>";
		doc = MockParser.parseString(xml);
		
		try {
			GlobalTreatmentBuilder.parse(doc, maps);
		} catch (ParserException e) {
			assertEquals("Please state either a minimum size (mm³) or minimum diameter (mm) for metastases to be treated. " +
							"You cannot state both at the same time.", e.getMessage());
		}

	}	
	
	
	public void testExternalBeamRadiation() throws MetastaseException{
		TreatmentList.reset();
		ObjectBuffer maps = new ObjectBuffer();
		HostOrgan organ = new MockHostOrgan();
		maps.putHostOrgan("organ", organ);

		// test ExternalBeamRadiation
		String xml = "<externalBeamRadiation> <time>600</time> <alpha>1</alpha> <beta>2</beta> <dose>3</dose> </externalBeamRadiation>";
		Element doc = MockParser.parseString(xml);

		GlobalTreatmentBuilder.parse(doc, maps);
		Iterator<TreatmentEntry> it = TreatmentList.getInstance().iterator();

		// get the first entry
		TreatmentEntry treatmentEntry = it.next();
		Treatment exptectedTreatment = new ExternalBeamTreatment(1, 2, 3);
		// check time and treatment
		assertEquals(864000., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);

		// no further treatments should be in the treatment list
		assertFalse(it.hasNext());

		// test with quantity but no interval
		TreatmentList.reset();
		xml = "<externalBeamRadiation> <time>600</time> <quantity>5</quantity> <alpha>1</alpha> <beta>2</beta> <dose>3</dose> </externalBeamRadiation>";
		doc = MockParser.parseString(xml);

		try {
			GlobalTreatmentBuilder.parse(doc, maps);
		} catch (ParserException e) {
			assertEquals("Please specify an interval, if the therapy is applied more than once.", e.getMessage());
		}
		
		//try parsing a wrong treatment
		TreatmentList.reset();
		xml = "<externalBeamRadiaton> <time>600</time> <alpha>1</alpha> <beta>2</beta> <dose>3</dose> </externalBeamRadiaton>";
		doc = MockParser.parseString(xml);
		
		try {
			GlobalTreatmentBuilder.parse(doc, maps);
		} catch (ParserException e) {
			assertEquals("Could not parse the given systemic treatment. Please check the spelling of the XML tag.", e.getMessage());
		}	
	}
	
	public void testChemotherapy() throws MetastaseException{
		TreatmentList.reset();
		ObjectBuffer maps = new ObjectBuffer();
		HostOrgan organ = new MockHostOrgan();
		maps.putHostOrgan("organ", organ);

		// test ExternalBeamRadiation
		String xml = "<chemotherapy> <time>600</time> <cyclingCellsFraction>1</cyclingCellsFraction> <drugSensitivity>0.8</drugSensitivity> "
				+ "<drugDecay>0.4</drugDecay> </chemotherapy>";
		Element doc = MockParser.parseString(xml);

		GlobalTreatmentBuilder.parse(doc, maps);
		Iterator<TreatmentEntry> it = TreatmentList.getInstance().iterator();

		// get the first entry
		TreatmentEntry treatmentEntry = it.next();
		Treatment exptectedTreatment = new ChemoTreatment(1, 0.8, 0.4);
		// check time and treatment
		assertEquals(864000., treatmentEntry.time);
		assertEquals(exptectedTreatment, treatmentEntry.treatment);
			
	}
}
