/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.machine;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import de.fhstralsund.catsit.base.DiscreteCompartment;
import de.fhstralsund.catsit.base.MetastaseException;
import de.fhstralsund.catsit.machine.AdaptiveIntegrationCalculator;
import de.fhstralsund.catsit.machine.EventTimeCalculator;
import de.fhstralsund.catsit.machine.TumorIsDeadException;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.PiecewiseGrowthFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;

/**
 * @author Anja Bethge
 *
 */
public class TestAdaptiveIntegrationCalculator extends TestCase {

	private static final boolean PRINT_EVENTS = false;
	private static final double SURFACE_AREA = 1.0;
	
	/**
	 * Test method for {@link AdaptiveIntegrationCalculator#execute(DiscreteCompartment, double)}.
	 */
	public void testExecute() throws TumorIsDeadException{
		
		double[] computedValues = new double[20000];

		// the event rate functions and appropriate integrals
		List<Function> eventRateFunctions = new ArrayList<Function>();
		List<Function> integralFunctions = new ArrayList<Function>();
		
		//constant event rate functions (0.5, 1, 2)
		eventRateFunctions.add(new PiecewiseGrowthFunction(0., new ConstantFunction(0.5)));
		eventRateFunctions.add(new ConstantFunction());
		eventRateFunctions.add(new ConstantFunction(2.));
		//the appropriate integrals
		integralFunctions.add(new LinearFunction(0.5, 0.));
		integralFunctions.add(new LinearFunction(1., 0.));
		integralFunctions.add(new LinearFunction(2., 0.));
		
		//linear event rate function (0.5*x, x, 2x)
		eventRateFunctions.add(new LinearFunction(0.5, 0.));
		eventRateFunctions.add(new LinearFunction(1., 0.));
		eventRateFunctions.add(new LinearFunction(2., 0.));
		//the appropriate integrals
		integralFunctions.add(new PowerFunction(0.25, 2., 0.));
		integralFunctions.add(new PowerFunction(0.5, 2., 0.));
		integralFunctions.add(new PowerFunction(1., 2., 0.));
		
		//power event rate function (x^0.5, x^0.663 and 0.75*x^-3, x^2+4)
		eventRateFunctions.add(new PowerFunction(1., 0.5, 0.));
		eventRateFunctions.add(new PowerFunction(1., 0.663, 0.));
		//the appropriate integrals
		integralFunctions.add(new PowerFunction(2./3., 3./2., 0.));
		integralFunctions.add(new PowerFunction(1./1.663, 1.663, 0.));
		
		//exponential event rate function (e^x, 2*e^x, e^2x)
		eventRateFunctions.add(new PiecewiseGrowthFunction(0., new ExponentialFunction(1., 1., 0.)));
		eventRateFunctions.add(new ExponentialFunction(2., 1., 0.));
		eventRateFunctions.add(new ExponentialFunction(1., 2., 0.));

		//the appropriate integrals
		integralFunctions.add(new ExponentialFunction(1., 1., 0.));
		integralFunctions.add(new ExponentialFunction(2., 1., 0.));
		integralFunctions.add(new ExponentialFunction(1., 2., 0.));

		
		
		//file names array
		String[] fileNames = new String[]{
				"constant_0.5", "constant_1", "constant_2",  
				"linear_0.5", "linear_1", "linear_2", 
				"power_1_0.5","power_1_0.663",
				"expo_eHochx", "expo_2eHochx", "expo_eHoch2x", "expo_eHoch0.5x",
				};
		
		for (int index = 0; index < eventRateFunctions.size(); index++) {
			
			Function eventRateFunction = eventRateFunctions.get(index);
			//reset the array and the event-generator
			computedValues = new double[20000];
			EventTimeCalculator eventTimeCalculator = new AdaptiveIntegrationCalculator(eventRateFunction);
			//set a mock randomizer (divide by 2, since the EventGenerator multiplies by 2 to
			//get a range from 0..2)
			eventTimeCalculator.setRandomG(new MockRandom(SURFACE_AREA/2));	
			
			//generate the first event starting at time 0 resp. 1 for linear or power 
			//event rate function
			if(eventRateFunction instanceof LinearFunction || eventRateFunction instanceof PowerFunction){
				computedValues[0] = eventTimeCalculator.computeTimeToNextEvent(1);
			}else{
				computedValues[0] = eventTimeCalculator.computeTimeToNextEvent(0);
			}
			//generate the 19999 other events
			for (int i = 1; i < 20000; i++) {
				computedValues[i] = eventTimeCalculator.computeTimeToNextEvent(computedValues[i-1]);
			}
					
			//compute the expected reference values
			//starting at time 0
			double[] expectedValues = new double[20000];
			double lowerBound = 0.0;
			double upperBound = 0.0;
			
			Function integralFunction = integralFunctions.get(index);
			
			if(integralFunction instanceof LinearFunction){
				double m = ((LinearFunction)integralFunction).getM();
				for (int i = 0; i < 20000; i++) {
					upperBound = (SURFACE_AREA + integralFunction.f(lowerBound))/m;
					expectedValues[i]=upperBound;
					lowerBound = upperBound;
				}
			}else if(integralFunction instanceof PowerFunction){
				//start from 1.0 like the eventgenerator
				lowerBound=1.0;
				double a = ((PowerFunction)integralFunction).getA();
				double b = ((PowerFunction)integralFunction).getB();				
				for (int i = 0; i < 20000; i++) {
					upperBound = Math.pow((SURFACE_AREA + integralFunction.f(lowerBound))/a, 1/b);
					expectedValues[i]=upperBound;
					lowerBound = upperBound;
				}
			}else if(integralFunction instanceof ExponentialFunction){
				double a = ((ExponentialFunction)integralFunction).getA();
				double b = ((ExponentialFunction)integralFunction).getB();				
				for (int i = 0; i < 20000; i++) {
					upperBound = Math.log((SURFACE_AREA + integralFunction.f(lowerBound))/a)/b;
					expectedValues[i]=upperBound;
					lowerBound = upperBound;
				}
			}

			//the values can be either printed or checked via assertions
			if(PRINT_EVENTS){
				printEvents(computedValues, fileNames[index], index);
				printExpectedValues(expectedValues, fileNames[index], index);
			}else{
				//now compare the values from the EventGenerator and the expected values
				for (int i = 0; i < 20000; i++) {
					double computedValue = computedValues[i];
					double expectedValue = expectedValues[i];
					assertEquals(index+": "+integralFunction+": at step "+i+" =>  ", expectedValue, computedValue, 0.5);
				}
			}
		}
		
		
		//check illegal arguments
		try {
			new AdaptiveIntegrationCalculator(new ConstantFunction()).computeTimeToNextEvent(-1);
			fail("An exception should have been thrown");
		} catch (IllegalStateException e) {
			// we are supposed to be here
		} catch (TumorIsDeadException e) {
			fail("Wrong exception has been thrown");
		}
		
		//check illegal arguments
		try {
			new AdaptiveIntegrationCalculator(new ConstantFunction(0)).computeTimeToNextEvent(0);
			fail("An exception should have been thrown");
		} catch (TumorIsDeadException e) {
			// we are supposed to be here
		}		
	}
	
	
	public void testPiecewiseFunction() throws MetastaseException{
		// constant and positive linear
		PiecewiseGrowthFunction function = new PiecewiseGrowthFunction(0, new ConstantFunction(0.01));
		EventTimeCalculator eventTimeCalculator = new AdaptiveIntegrationCalculator(function);
		eventTimeCalculator.setRandomG(new MockRandom(SURFACE_AREA/2));
		
		assertEquals(130., eventTimeCalculator.computeTimeToNextEvent(30), 1e-8);
		
		//add another growth function (linear with positive m)
		function.addFunction(50., new LinearFunction(0.003, 0.01));
		assertEquals(70., eventTimeCalculator.computeTimeToNextEvent(30), 1e-8);
		
		// constant and negative linear
		function = new PiecewiseGrowthFunction(0, new ConstantFunction(0.1));
		eventTimeCalculator = new AdaptiveIntegrationCalculator(function);
		eventTimeCalculator.setRandomG(new MockRandom(SURFACE_AREA/2));
		
		assertEquals(58., eventTimeCalculator.computeTimeToNextEvent(48), 1e-8);
		
		//add another growth function (linear with negative m)
		function.addFunction(50., new LinearFunction(-0.004, 0.1));
		assertEquals(60., eventTimeCalculator.computeTimeToNextEvent(48), 1e-8);
		
		// extend some time
	}
	
	public void printEvents(double[] times, String fileName, int index) {
		//analyse the events
		//get the files
		File timeHistogramm = new File("testFiles/simplesum_A"+SURFACE_AREA+"_"+fileName+".dat");
			
		assert(timeHistogramm.canWrite());
		
		try {
			//create the file-writer
			FileWriter timeHistoWriter = new FileWriter(timeHistogramm);

			for (int i=0; i<times.length; i++) {
				//write the time and the event number into the file
				timeHistoWriter.write(""+times[0]+"\t"+(i+1)+"\n");
			}
		
			timeHistoWriter.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void printExpectedValues(double[] expectedValues, String fileName, int index) {
		//get the files
		File timeHistogramm = new File("testFiles/simplesum_A"+SURFACE_AREA+"_"+fileName+"_expectedVaues.dat");
			
		assert(timeHistogramm.canWrite());
		
		try {
			//create the file-writer
			FileWriter timeHistoWriter = new FileWriter(timeHistogramm);
			for (int i=0; i<expectedValues.length; i++) {
				//write the time and the event number into the file
				timeHistoWriter.write(""+expectedValues[i]+"\t"+(i+1)+"\n");
			}
						
			timeHistoWriter.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
