/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

import de.fhstralsund.catsit.support.functions.ColonizationRate;
import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.ExponentialFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import de.fhstralsund.catsit.support.functions.LinearFunction;
import de.fhstralsund.catsit.support.functions.PiecewiseGrowthFunction;
import de.fhstralsund.catsit.support.functions.PowerFunction;
import junit.framework.TestCase;

public class TestColonisationRate extends TestCase {

	public void testF() {
		// ** data **
		double a = 5.;
		double b = 7.;
		Function innerFunction = new ConstantFunction(5.4);
		ColonizationRate function = new ColonizationRate(a, b);
		
		try {
			function.f(20);
			fail("An exception should have been thrown.");
		} catch (IllegalStateException e) {
			assertEquals("The growth function is 'null'. You have to set the growth function via setGrowthFunction() before you can call f().", e.getMessage());
		}		
		
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
		
		assertEquals(7.0, function.getAlpha());
		assertEquals(5.0, function.getM());
		
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a*Math.pow(5.4, b);
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
		innerFunction = new LinearFunction(0.6, 0);
		function = new ColonizationRate(a, b);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a*Math.pow(0.6*i, b);
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
		innerFunction = new PowerFunction(6., 0.8, 7);
		function = new ColonizationRate(a, b);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a*Math.pow(6*Math.pow(i, 0.8)+7,b);
			assertEquals("at step "+i, correctValue, function.f(i));
		}
	}

	public void testF_FirstDerivation() {
		// ** data **
		double a = 5.;
		double b = 0.6;
		Function innerFunction = new ConstantFunction(5.4);
		ColonizationRate function = new ColonizationRate(a, b);
		
		try {
			function.f_FirstDerivation(20);
			fail("An exception should have been thrown.");
		} catch (IllegalStateException e) {
			assertEquals("The growth function is 'null'. You have to set the growth function via setGrowthFunction() before you can call f_FirstDerivation().", e.getMessage());
		}		
		
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
		
		double correctValue = 0;
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {			
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		innerFunction = new LinearFunction(0.5, 0);
		function = new ColonizationRate(a, b);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			correctValue = 0.5 * a*b*Math.pow(0.5*i, b-1);
			double computedValue = function.f_FirstDerivation(i);
			assertEquals("at step "+i + ": expected="+correctValue+", but was="+computedValue+", dev="+Math.abs(correctValue-computedValue), 
					correctValue, computedValue, 1e-20);
		}
		
		innerFunction = new PowerFunction(6., 1.8, 7);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			correctValue = 6.*1.8*Math.pow(i, 0.8) * a*b*Math.pow(6*Math.pow(i, 1.8)+7,b-1); //part1 * part2;
			double computedValue = function.f_FirstDerivation(i);
			assertEquals("at step "+i + ": expected="+correctValue+", but was="+computedValue+", dev="+Math.abs(correctValue-computedValue), 
					correctValue, computedValue, 1e-20);
		}		
		
		innerFunction = new ConstantFunction(5.4);
		function = new ColonizationRate(7, 1.0);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
		
		correctValue = 0;
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {			
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = 5;
		b = 0.6;
		double c = 1.;
		double d = 78.;
		
		innerFunction = new GompertzFunction(c, d);
		function = new ColonizationRate(a, b);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			correctValue = a * c * b * Math.log(d) * Math.exp(-c * i) * Math.pow(Math.pow(d, 1. - Math.exp(-c * i)), b);
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i), 1e-10);
		}
		
		//check whether the function returns 'NaN' if the growth function has no derivation
		//since every Function provides a derivation and BasicFunctions are not allowed, we use an anonymous implementation here
		innerFunction = new Function() {	
			public double f(double x) {return 5;}
			public double f_FirstDerivation(double x) {return Double.NaN;}
		};
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, innerFunction));
		assertEquals(Double.NaN, function.f_FirstDerivation(8));
	}
	
	public void testToString() {
		ColonizationRate function = new ColonizationRate(1, 2);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, new ExponentialFunction(1, 2, 3)));
		assertEquals("ColonisationRate [1.0 * 0.0: ExponentialFunction [1.0*e^(2.0x) +3.0] - ^2.0]", function.toString());
		
	}

	public void testEquals() {
		ColonizationRate function1 = new ColonizationRate(1, 2);
		function1.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(1, 2)));
		ColonizationRate function2 = new ColonizationRate(1, 2);
		function2.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(1, 2)));
		
		assertTrue(function1.equals(function1));
		assertTrue(function1.equals(function2));
		
		assertFalse(function1.equals(null));
		assertFalse(function1.equals(new ConstantFunction()));
		
		//no growth function in 2
		assertFalse(function1.equals(new ColonizationRate(1, 2)));
		//other growth functon
		function2.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(2, 1)));
		assertFalse(function1.equals(function2));
		//no growth function in 1 and function in 2
		function1.setGrowthFunction(null);
		assertFalse(function1.equals(function2));
		//no growth function in both
		function2.setGrowthFunction(null);
		assertTrue(function1.equals(function2));
		
		//m
		assertFalse(function1.equals(new ColonizationRate(2, 2)));
		//alpha
		assertFalse(function1.equals(new ColonizationRate(1, 0.5)));
	}
	
	public void testHashCode() {
		ColonizationRate function1 = new ColonizationRate(1, 2);
		function1.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(1, 2)));
		ColonizationRate function2 = new ColonizationRate(1, 2);
		function2.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(1, 2)));
		
		assertTrue(function1.hashCode() == function1.hashCode());
		assertTrue(function1.hashCode() == function2.hashCode());
		
		assertFalse(function1.hashCode() == new ConstantFunction().hashCode());
		
		//no growth function in 2
		assertFalse(function1.hashCode() == new ColonizationRate(1, 2).hashCode());
		//other growth functon
		function2.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(2, 1)));
		assertFalse(function1.hashCode() == function2.hashCode());
		//no growth function in 1 and function in 2
		function1.setGrowthFunction(null);
		assertFalse(function1.hashCode() == function2.hashCode());
		//no growth function in both
		function2.setGrowthFunction(null);
		assertTrue(function1.hashCode() == function2.hashCode());
		
		//m
		assertFalse(function1.hashCode() == new ColonizationRate(2, 2).hashCode());
		//alpha
		assertFalse(function1.hashCode() == new ColonizationRate(1, 0.5).hashCode());
		
		
		ColonizationRate function = new ColonizationRate(1, 2);
		function.setGrowthFunction(new PiecewiseGrowthFunction(0, new LinearFunction(1, 2)));
	}
	
}
