/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.GompertzFunction;
import junit.framework.TestCase;

public class TestGompertzFunction extends TestCase {

	public void testF() {
		double a = 2.4;
		double b = 1.8;
		double n = 1.3;
		GompertzFunction function = new GompertzFunction(a, b, n);
		
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = b * Math.exp(-1.0 * (Math.exp(-1.0*a*i)) * (Math.log(b) -Math.log(n)));
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
				
		//check constructor
		a = -5.;
		try {
			function = new GompertzFunction(a, b, n);
			fail("An exception should have been thrown, since a < 0.");
		} catch (IllegalArgumentException e) {
			// we are supposed to be here
		}
		
		a = 0.5;
		b = -0.8;
		try {
			function = new GompertzFunction(a, b, n);
			fail("An exception should have been thrown, since b < 0.");
		} catch (IllegalArgumentException e) {
			// we are supposed to be here
		}
		
		b = 8;
		n = 0;
		try {
			function = new GompertzFunction(a, b, n);
			fail("An exception should have been thrown, since n < 1.");
		} catch (IllegalArgumentException e) {
			// we are supposed to be here
		}
		
		
		a = 0.5;
		b = 0.8;
		n = 8.;
		function = new GompertzFunction(a, b, n);
		
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = b * Math.exp(-1.0 * (Math.exp(-1.0*a*i)) * (Math.log(b) -Math.log(n)));
			assertEquals("at step "+i, correctValue, function.f(i), 1e-1);
		}
		
		a = 105.;
		b = 20;
		n = 8.;		
		function = new GompertzFunction(a, b, n);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = b * Math.exp(-1.0 * (Math.exp(-1.0*a*i)) * (Math.log(b) -Math.log(n)));
			assertEquals("at step "+i, correctValue, function.f(i), 1e-10);
		}
		
		a = 105.;
		b = 20;
		function = new GompertzFunction(a, b);
		assertEquals(1.0, function.getN());
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = b * Math.exp(-1.0 * (Math.exp(-1.0*a*i)) * (Math.log(b) -Math.log(1)));
			assertEquals("at step "+i, correctValue, function.f(i), 1e-15);
		}
		
	}
	
	public void testF_FirstDerivation() {
		// ** data **
		double a = 5.;
		double b = 7.;
		double n = 8.;		
		Function function = new GompertzFunction(a, b, n);
					
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * b * (Math.log( b ) - Math.log( n )) * Math.exp( (-1 * a * i) - (Math.exp( -1 * a * i ) * (Math.log( b ) - Math.log( n )) ));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
				
		a = 5.;
		b = 0.6;
		n = 8.;		
		function = new GompertzFunction(a, b, n);
						
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * b * (Math.log( b ) - Math.log( n )) * Math.exp( (-1 * a * i) - (Math.exp( -1 * a * i ) * (Math.log( b ) - Math.log( n )) ));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
				
		a = 295.;
		b = 60;
		n = 8.;
		function = new GompertzFunction(a, b, n);
					
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * b * (Math.log( b ) - Math.log( n )) * Math.exp( (-1 * a * i) - (Math.exp( -1 * a * i ) * (Math.log( b ) - Math.log( n )) ));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		
		// ** data **
		a = 0.5;
		b = 7;
		n = 1.;		
				
		//correct values according to Mathematica => to test the implementation
		double[] correctValues1 = {
					0.972955, 1.26901, 1.22462, 0.984424, 0.708321, 0.476524, 0.307774, 0.193928, 0.120374, 0.0740419, 0.0452923, 0.0276132, 0.0168008, 
					0.0102095, 0.00619953, 0.00376283, 0.00228324, 0.00138521, 0.000840304, 0.000509718, 0.000309177, 0.000187532, 0.000113746, 0.0000689915, 
					0.0000418458, 0.0000253809, 0.0000153943, 0.00000933715, 0.00000566327, 0.00000343495, 0.0000020834, 0.00000126365,	0.000000766442, 
					0.00000046487, 0.000000281958, 0.000000171016, 0.000000103727, 0.0000000629134, 0.0000000381589, 0.0000000231445, 0.0000000140379, 
					0.0000000085144, 0.00000000516424, 0.00000000313227, 0.00000000189982, 0.0000000011523, 0.000000000698904, 0.000000000423907,
					0.000000000257113, 0.000000000155947, 0.0000000000945864, 0.0000000000573696, 0.0000000000347964, 0.0000000000211051, 0.0000000000128009, 
					0.00000000000776413, 0.00000000000470918, 0.00000000000285626, 0.00000000000173241, 0.00000000000105076,	0.000000000000637318, 
					0.000000000000386553, 0.000000000000234456, 0.000000000000142205, 0.0000000000000862516, 0.0000000000000523143, 0.0000000000000317302, 
					0.0000000000000192453,	0.0000000000000116729, 0.00000000000000707997, 0.00000000000000429422, 0.00000000000000260457,	0.00000000000000157975, 
					0.000000000000000958169, 0.000000000000000581159, 0.000000000000000352491, 0.000000000000000213796, 0.000000000000000129674, 0.0000000000000000786513, 
					0.0000000000000000477044, 0.0000000000000000289342, 0.0000000000000000175495, 0.0000000000000000106443, 0.00000000000000000645609, 
					0.00000000000000000391582, 0.00000000000000000237506, 0.00000000000000000144055, 0.000000000000000000873737, 0.000000000000000000529948, 
					0.00000000000000000032143, 0.000000000000000000194957, 0.000000000000000000118247, 0.0000000000000000000717207, 0.0000000000000000000435008, 
					0.0000000000000000000263846, 0.0000000000000000000160031, 0.00000000000000000000970634, 0.0000000000000000000058872, 0.00000000000000000000357076, 
					0.00000000000000000000216578, 
				};
						
		function = new GompertzFunction(a, b, n);
						
		for (int t = 0; t < 100; t++) {
				assertEquals("Differenz zum Zeitpunkt "+t+" = " + (correctValues1[t]-function.f((double)t)),
						correctValues1[t], 
						function.f_FirstDerivation((double)t), 0.5);//FIXME den abweichwert anpassen => dynamisch an t		
		}
	}

	
	public void testToString() {
		Function function = new GompertzFunction(1, 2, 3);
		assertEquals("GompertzFunction [2.0 * e^((ln(2.0)-ln(3.0))*-e^-1.0*t) ]", function.toString());
		
		function = new GompertzFunction(0.76, 2, 876);
		assertEquals("GompertzFunction [2.0 * e^((ln(2.0)-ln(876.0))*-e^-0.76*t) ]", function.toString());
	}

	public void testEquals() {
		Function function = new GompertzFunction(1, 2, 3);
		
		assertFalse(function.equals(null));
		assertFalse(function.equals(new ConstantFunction()));
		assertFalse(function.equals(new GompertzFunction(1, 2, 4)));
		assertFalse(function.equals(new GompertzFunction(1, 8, 3)));
		assertFalse(function.equals(new GompertzFunction(0.65, 2, 3)));
		assertFalse(function.equals(new GompertzFunction(0.076, 0.7, 875)));
		
		assertTrue(function.equals(function));
		assertTrue(function.equals(new GompertzFunction(1, 2, 3)));
		assertTrue(function.equals(new GompertzFunction(1.000, 2.00, 3.0)));
	}
	
	public void testHashCode() {
		
		assertEquals(new GompertzFunction(1, 2, 3).hashCode(), new GompertzFunction(1.0, 2.0, 3.0).hashCode());
		assertEquals(new GompertzFunction(1, 8, 3).hashCode(), new GompertzFunction(1, 8, 3).hashCode());
		assertEquals(new GompertzFunction(0.076, 0.7, 875).hashCode(), new GompertzFunction(0.076, 0.7, 875).hashCode());
		
		assertFalse(new GompertzFunction(0.64, 2, 3).hashCode() == new GompertzFunction(0.65, 2, 3).hashCode());
	}

	public void testGetA() {
		GompertzFunction function = new GompertzFunction(1, 2, 3);
		assertEquals(1, function.getA(), 0);
		
		function = new GompertzFunction(8.76, 2, 3);
		assertEquals(8.76, function.getA(), 0);
		
		function = new GompertzFunction(0.764, 2, 3);
		assertEquals(0.764, function.getA(), 0);
		
		function = new GompertzFunction(2./3., 2, 3);
		assertEquals(2./3., function.getA(), 0);
	}
	
	public void testGetB() {
		GompertzFunction function = new GompertzFunction(1, 2, 3);
		assertEquals(2, function.getB(), 0);
		
		function = new GompertzFunction(1, 8.76, 3);
		assertEquals(8.76, function.getB(), 0);
		
		function = new GompertzFunction(1, 0.764, 3);
		assertEquals(0.764, function.getB(), 0);
		
		function = new GompertzFunction(1, 2./3., 3);
		assertEquals(2./3., function.getB(), 0);		
	}
	
}
