/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.LogFunction;
import junit.framework.TestCase;

public class TestLogFunction extends TestCase {

	public void testF() {
		
		//test log function as normal function/////////////////////////////////
		///////////////////////////////////////////////////////////////////////
		//b=0
		try {
			new LogFunction(1.0, 0.0);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we should be here
		}
		//b<0
		try {
			new LogFunction(1.0, -5.0);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we should be here
		}
		
		//a=0
		try {
			new LogFunction(0.0, 5.0);
		} catch (IllegalArgumentException e) {
			// oh oh, we shouldn't be here!
			fail("Wrong Constructor! If not used as a growth function, a should be allowed to be 0.");
		}	
		
		
		// ** data **
		double a = 5.;
		double b = 7.;	
		LogFunction function = new LogFunction(a, b);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.log( b * i );
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
		a = -5.;
		b = 0.8;	
		function = new LogFunction(a, b);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.log( b * i );
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
		//test log function as growth function////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////
		// b=0
		try {
			new LogFunction(1.0, 0.0, 1L);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we should be here
		}
		// b<0
		try {
			new LogFunction(1, -5, 1L);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we should be here
		}
		
		// a=0
		try {
			new LogFunction(0, 5, 1L);
			fail("An exception should have been thrown.");
		} catch (IllegalArgumentException e) {
			// we should be here
		}

		// ** data **
		a = 5.;
		b = 7.;
		long n = 1;
		function = new LogFunction(a, b, n);

		// check if the function yields the correct values
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.log( b * i + (Math.exp(n/a)) );
			assertEquals("at step " + i, correctValue, function.f(i));
		}

		a = -5.;
		b = 0.8;
		n = 200;
		function = new LogFunction(a, b, n);
		assertEquals(200, function.getN().longValue());

		// check if the function yields the correct values
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.log( b * i + (Math.exp(n/a)) );
			assertEquals("at step " + i, correctValue, function.f(i));
		}
	}

	public void testF_FirstDerivation() {
		double a = 5.;
		double b = 7.;	
		Function function = new LogFunction(a, b);
				
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a / i;
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = -5.;
		b = 0.6;
		function = new LogFunction(a, b);
				
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a / i;
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = 5.;
		b = 7;
		long n = 1;
		function = new LogFunction(a, b, n);
				
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = (a*b) / (b*i + Math.exp(n/a));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = 5.;
		b = 0.4;
		n = 100; 
		function = new LogFunction(a, b, n);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = (a*b) / (b*i + Math.exp(n/a));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = -5.;
		b = 10;
		n = 1;
		function = new LogFunction(a, b, n);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = (a*b) / (b*i + Math.exp(n/a));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = -0.5;
		function = new LogFunction(a, b, n);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = (a*b) / (b*i + Math.exp(n/a));
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
	}
	
	public void testToString() {
		Function function = new LogFunction(1, 2);
		assertEquals("LogFunction [1.0 * ln ( 2.0 * x ) ]", function.toString());
		
		function = new LogFunction(0.76, 2);
		assertEquals("LogFunction [0.76 * ln ( 2.0 * x ) ]", function.toString());
		
		function = new LogFunction(0.76, 2, 123456L);
		assertEquals("LogFunction [0.76 * ln ( 2.0 * x + e^(123456 / 0.76) ) ]", function.toString());
	}

	public void testEquals() {
		Function function = new LogFunction(1, 2);
		
		assertFalse(function.equals(null));
		assertFalse(function.equals(new ConstantFunction()));
		assertFalse(function.equals(new LogFunction(1, 8)));
		assertFalse(function.equals(new LogFunction(0.65, 2)));
		assertFalse(function.equals(new LogFunction(-76, 0.7)));
		assertFalse(function.equals(new LogFunction(1, 2, 3L)));
		
		assertTrue(function.equals(function));
		assertTrue(function.equals(new LogFunction(1, 2)));
		assertTrue(function.equals(new LogFunction(1.000, 2.00)));
		
		function = new LogFunction(1, 2, 1L);
		assertTrue(function.equals(new LogFunction(1, 2, 1L)));
		assertFalse(function.equals(new LogFunction(1, 2)));
	}
	
	public void testHashCode() {
		assertEquals(new LogFunction(1, 2).hashCode(), new LogFunction(1, 2).hashCode());
		assertEquals(new LogFunction(1, 8, 20L).hashCode(), new LogFunction(1, 8, 20L).hashCode());
		assertEquals(new LogFunction(-76, 0.7).hashCode(), new LogFunction(-76, 0.7).hashCode());

		assertFalse(new LogFunction(-76, 0.7).hashCode() == new LogFunction(0.65, 2).hashCode());
		assertFalse(new LogFunction(0.64, 2).hashCode() == new LogFunction(0.65, 2).hashCode());
		assertFalse(new LogFunction(0.65, 2).hashCode() == new LogFunction(0.65, 2, 1L).hashCode());
		assertFalse(new LogFunction(0.65, 2, 1L).hashCode() == new LogFunction(0.65, 2).hashCode());
	}

	public void testGetA() {
		LogFunction function = new LogFunction(1, 2);
		assertEquals(1, function.getA(), 0);
		
		function = new LogFunction(8.76, 2);
		assertEquals(8.76, function.getA(), 0);
		
		function = new LogFunction(-0.764, 2);
		assertEquals(-0.764, function.getA(), 0);
		
		function = new LogFunction(2/3, 2);
		assertEquals(2/3, function.getA(), 0);
	}
	
	public void testGetB() {
		LogFunction function = new LogFunction(1, 2);
		assertEquals(2, function.getB(), 0);
		
		function = new LogFunction(1, 8.76);
		assertEquals(8.76, function.getB(), 0);
	}
	
	public void testGetN() {
		LogFunction function = new LogFunction(1, 2, 3L);
		assertEquals(3, function.getN(),0);
		
		function = new LogFunction(1, 2, 8L);
		assertEquals(8, function.getN(), 0);
		
		function = new LogFunction(1, 2, -764L);
		assertEquals(-764, function.getN(), 0);
		
		function = new LogFunction(1, 2, 50L);
		assertEquals(50, function.getN(), 0);		
	}
	
}
