/*
	--------------------------------------------------------------------------------
	CaTSiT - A simulation tool for metastatic progression and treatments
	--------------------------------------------------------------------------------
	
	Copyright (C) 2014  Gero Wedemann, Anja Bethge
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	Contact: anja.bethge@fh-stralsund.de
	         gero.wedemann@fh-stralsund.de
	
	
	--------------------------------------------------------------------------------
	Installation and documentation 
	--------------------------------------------------------------------------------
	
	Please visit http://www.bioinformatics.fh-stralsund.de/catsit/ 
 */

package de.fhstralsund.catsit.support.functions;

import de.fhstralsund.catsit.support.functions.ConstantFunction;
import de.fhstralsund.catsit.support.functions.Function;
import de.fhstralsund.catsit.support.functions.PowerFunction;
import junit.framework.TestCase;

public class TestPowerFunction extends TestCase {

	public void testF() {
		// ** data **
		double a = 5.;
		double b = 7.;
		double c = 8.;
		Function function = new PowerFunction(a, b, c);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.pow(i, b) + c;
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
		a = 0.5;
		b = -7.;
		c = 8.;
		function = new PowerFunction(a, b, c);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.pow(i, b) + c;
			assertEquals("at step "+i, correctValue, function.f(i));
		}
		
		a = -5;
		b = -0.7;
		c = 8.;
		function = new PowerFunction(a, b, c);
				
		//check if the function yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * Math.pow(i, b) + c;
			assertEquals("at step "+i, correctValue, function.f(i));
		}
	}

	public void testF_FirstDerivation() {
		double a = 5.;
		double b = 7.;
		double c = 8.;		
		Function function = new PowerFunction(a, b, c);
				
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * b * Math.pow(i, b - 1.);
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = -4.;
		b = 7.6;
		c = -8.;		
		function = new PowerFunction(a, b, c);
				
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * b * Math.pow(i, b - 1.);
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = -4.;
		b = -0.6;
		c = 5.;		
		function = new PowerFunction(a, b, c);
				
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			double correctValue = a * b * Math.pow(i, b - 1.);
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
		
		a = -4.;
		b = 1.;
		c = 5.;		
		function = new PowerFunction(a, b, c);
		
		double correctValue = a;
		//check if the Derivation yields the correct values 
		for (int i = 0; i < 1e+5; i++) {
			assertEquals("at step "+i, correctValue, function.f_FirstDerivation(i));
		}
	}
	
	
	public void testToString() {
		Function function = new PowerFunction(1, 2, 3);
		assertEquals("PowerFunction [1.0*x^2.0+3.0 ]", function.toString());
		
		function = new PowerFunction(0.76, -2, 876);
		assertEquals("PowerFunction [0.76*x^-2.0+876.0 ]", function.toString());
	}

	public void testEquals() {
		Function function = new PowerFunction(1, 2, 3);
		
		assertFalse(function.equals(null));
		assertFalse(function.equals(new ConstantFunction()));
		assertFalse(function.equals(new PowerFunction(1, 2, 4)));
		assertFalse(function.equals(new PowerFunction(1, 8, 3)));
		assertFalse(function.equals(new PowerFunction(0.65, 2, 3)));
		assertFalse(function.equals(new PowerFunction(-76, 0.7, 875)));
		
		assertTrue(function.equals(function));
		assertTrue(function.equals(new PowerFunction(1, 2, 3)));
		assertTrue(function.equals(new PowerFunction(1.000, 2.00, 3.0)));
	}
	
	public void testHashCode() {
		Function function = new PowerFunction(1, 2, 3);
		assertFalse(function.hashCode() == new ConstantFunction().hashCode());
		assertFalse(function.hashCode() == new PowerFunction(1, 2, 4).hashCode());
		assertFalse(function.hashCode() == new PowerFunction(1, 8, 3).hashCode());
		assertFalse(function.hashCode() == new PowerFunction(0.65, 2, 3).hashCode());
		assertFalse(function.hashCode() == new PowerFunction(-76, 0.7, 875).hashCode());
		
		assertTrue(function.hashCode() == function.hashCode());
		assertTrue(function.hashCode() == new PowerFunction(1, 2, 3).hashCode());
		assertTrue(function.hashCode() == new PowerFunction(1.000, 2.00, 3.0).hashCode());

	}

	public void testGetA() {
		PowerFunction function = new PowerFunction(1, 2, 3);
		assertEquals(1, function.getA(), 0);
		
		function = new PowerFunction(8.76, 2, 3);
		assertEquals(8.76, function.getA(), 0);
		
		function = new PowerFunction(-0.764, 2, 3);
		assertEquals(-0.764, function.getA(), 0);
		
		function = new PowerFunction(2/3, 2, 3);
		assertEquals(2/3, function.getA(), 0);
	}
	
	public void testGetB() {
		PowerFunction function = new PowerFunction(1, 2, 3);
		assertEquals(2, function.getB(), 0);
		
		function = new PowerFunction(1, 8.76, 3);
		assertEquals(8.76, function.getB(), 0);
		
		function = new PowerFunction(1, -0.764, 3);
		assertEquals(-0.764, function.getB(), 0);
		
		function = new PowerFunction(1, 2/3, 3);
		assertEquals(2/3, function.getB(), 0);		
	}
	
	public void testGetC() {
		PowerFunction function = new PowerFunction(1, 2, 3);
		assertEquals(3, function.getC(), 0);
		
		function = new PowerFunction(1, 2, 8.76);
		assertEquals(8.76, function.getC(), 0);
		
		function = new PowerFunction(1, 2, -0.764);
		assertEquals(-0.764, function.getC(), 0);
		
		function = new PowerFunction(1, 2, 2/3);
		assertEquals(2/3, function.getC(), 0);		
	}
}
